Package: multiSA
Title: Multi-Stock Assessment
Version: 0.2.0
Authors@R: 
    person("Quang", "Huynh", , "quang@bluematterscience.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7835-4376"))
Date: 2026-05-20
Maintainer: Quang Huynh <quang@bluematterscience.com>
Description: Implementation of a next-generation, multi-stock age-structured fisheries assessment model. 
  'multiSA' is intended for use in mixed fisheries where stock composition can not be readily identified in 
  fishery data alone, e.g., from catch and age/length composition. Models can be fitted to genetic data, 
  e.g., stock composition of catches and close-kin pairs, with seasonal stock availability and movement.
License: GPL (>= 3)
Depends: R (>= 3.5.0)
Imports: dplyr, gplots, graphics, methods, pbapply, reshape2,
        rmarkdown, snowfall, stats, tinyplot, RTMB (>= 1.9), TMB, utils
Suggests: numDeriv, RTMBdist, usethis
LazyLoad: yes
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://blue-matter.github.io/multiSA/,
        https://github.com/Blue-Matter/multiSA
BugReports: https://github.com/Blue-Matter/multiSA/issues
NeedsCompilation: no
Packaged: 2026-05-22 21:34:06 UTC; quang
Author: Quang Huynh [aut, cre] (ORCID: <https://orcid.org/0000-0001-7835-4376>)
Repository: CRAN
Date/Publication: 2026-05-22 22:10:02 UTC
Built: R 4.5.2; ; 2026-05-25 01:48:43 UTC; unix

Package: multiSA
Title: Multi-Stock Assessment
Version: 0.5.0
Authors@R: 
    person("Quang", "Huynh", , "quang@bluematterscience.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7835-4376"))
Date: 2026-09-03
Maintainer: Quang Huynh <quang@bluematterscience.com>
Description: Implementation of a next-generation, multi-stock age-structured fisheries assessment model. 
  'multiSA' is intended for use in mixed fisheries where stock composition can not be readily identified in 
  fishery data alone, e.g., from catch and age/length composition. Models can be fitted to genetic data, 
  e.g., stock composition of catches and close-kin pairs, with seasonal stock availability and movement.
License: GPL (>= 3)
Depends: R (>= 4.1.0)
Imports: gplots, graphics, methods, parallel, pbapply, reshape2,
        rmarkdown, stats, tinyplot, RTMB (>= 1.9), TMB, utils
Suggests: numDeriv, RTMBdist, usethis
LazyLoad: yes
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://blue-matter.github.io/multiSA/,
        https://github.com/Blue-Matter/multiSA
BugReports: https://github.com/Blue-Matter/multiSA/issues
NeedsCompilation: no
Packaged: 2026-09-04 06:39:48 UTC; quang
Author: Quang Huynh [aut, cre] (ORCID: <https://orcid.org/0000-0001-7835-4376>)
Repository: CRAN
Date/Publication: 2026-09-04 07:20:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,MSAretro)
S3method(report,MSAretro)
S3method(summary,MSAretro)
export(CondExpEq)
export(CondExpGe)
export(CondExpGt)
export(CondExpLe)
export(CondExpLt)
export(calc_F)
export(calc_HSP)
export(calc_LAK)
export(calc_NPR)
export(calc_POP)
export(calc_eqdist)
export(calc_fsel_age)
export(calc_growth)
export(calc_index)
export(calc_init_population)
export(calc_isel_age)
export(calc_nextN)
export(calc_phi_project)
export(calc_phi_simple)
export(calc_population)
export(calc_recruitment)
export(calc_sel_len)
export(check_data)
export(check_parameters)
export(conv_Sigma)
export(conv_mov)
export(conv_selpar)
export(fit_MSA)
export(get_MSAdata)
export(get_sdreport)
export(like_CKMR)
export(like_comp)
export(make_map)
export(make_parameters)
export(optimize_RTMB)
export(plot_B)
export(plot_CAA)
export(plot_CAL)
export(plot_Ffleet)
export(plot_Fstock)
export(plot_IAA)
export(plot_IAL)
export(plot_N)
export(plot_R)
export(plot_Rdev)
export(plot_S)
export(plot_SC)
export(plot_SRR)
export(plot_V)
export(plot_catch)
export(plot_index)
export(plot_mov)
export(plot_recdist)
export(plot_self)
export(plot_seli)
export(plot_selstock)
export(plot_tagmov)
export(posfun)
export(prior_M)
export(prior_h)
export(prior_q)
export(report)
export(retrospective)
export(sapply2)
export(softmax)
exportClasses(DCKMR)
exportClasses(Dfishery)
exportClasses(Dlabel)
exportClasses(Dmodel)
exportClasses(Dstock)
exportClasses(Dsurvey)
exportClasses(Dtag)
exportClasses(MSAassess)
exportClasses(MSAdata)
exportMethods(plot)
exportMethods(profile)
exportMethods(report)
exportMethods(residuals)
exportMethods(simulate)
exportMethods(summary)
import(RTMB)
import(snowfall)
importFrom(TMB,summary.sdreport)
importFrom(dplyr,"%>%")
importFrom(gplots,rich.colors)
importFrom(grDevices,hcl.colors)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,contour)
importFrom(graphics,filled.contour)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(methods,"slot<-")
importFrom(methods,callNextMethod)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setMethod)
importFrom(methods,setOldClass)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(pbapply,pblapply)
importFrom(reshape2,acast)
importFrom(reshape2,melt)
importFrom(rmarkdown,render)
importFrom(stats,approx)
importFrom(stats,nlminb)
importFrom(stats,optimHess)
importFrom(stats,profile)
importFrom(stats,residuals)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,simulate)
importFrom(stats,weighted.mean)
importFrom(tinyplot,tinyplot)
importFrom(tinyplot,type_barplot)
importFrom(tinyplot,type_lines)
importFrom(tinyplot,type_text)
importFrom(utils,browseURL)
importFrom(utils,packageVersion)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# multiSA <img src="man/figures/README-hex.png" align="right" height=139 width=120 />

> Multi-stock assessment with RTMB

<!-- badges: start -->

[![R-CMD-check](https://github.com/Blue-Matter/multiSA/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Blue-Matter/multiSA/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

multiSA (**Multi-Stock Assessment with Regional Spatiotemporal
dynamics**) is a multi-stock, spatially-explicit age-structured model.

With explicit seasonal stock availability and movement, multiSA is
intended for use in mixed fisheries where stock composition can not be
readily identified in fishery data alone, i.e., from catch and
age/length composition. Models can also be fitted to genetic data, e.g.,
stock composition of catches and close-kin pairs.

Funding for development of multiSA is provided by the NOAA Fisheries
Bluefin Tuna Research Program
([BTRP](https://www.fisheries.noaa.gov/grant/bluefin-tuna-research-program)
Grants NA23NMF4720184 and NA24NMFX472C0008-T1-01) in collaboration with
the Ocean Foundation.

Atlantic bluefin tuna (*Thunnus thynnus*) is the first intended case
study.

## Installation

`multiSA` is available on CRAN:

``` r
install.packages("multiSA")
```

You can also install the R package from GitHub with:

``` r
# install.packages("remotes")
remotes::install_github("Blue-Matter/multiSA")
```

#' Likelihood for composition vectors
#'
#' Returns the log-likelihood for composition data, e.g., length, age, or stock composition,
#' with various statistical distributions supported.
#'
#' @param obs A vector of observed values. Internally converted to proportions.
#' @param pred A vector of predicted values. Same length as `obs`. Internally converted to proportions.
#' @param type Character for the desired distribution
#' @param N Numeric, the sample size corresponding to `obs` for multinomial or Dirichlet multinomial distributions.
#' @param theta Numeric, the linear (`type = "dirmult1"`) or saturating (`type = "dirmult2"`) Dirichlet-multinomial parameter, respectively. See Thorson et al. (2017)
#' @param stdev Numeric or vectorized for `obs`, the likelihood standard deviation for lognormal distribution.
#' @return Numeric representing the log-likelihood.
#'
#' @details
#' Observed and predicted vectors are internally converted to proportions.
#'
#' For `type = "lognormal"` or `"logitnormal"`, zero observations are removed from the likelihood calculation.
#' @references
#' Thorson et al. 2017. Model-based estimates of effective sample size in stock assessment models using the
#' Dirichlet-multinomial distribution. Fish. Res. 192:84-93. \doi{10.1016/j.fishres.2016.06.005}
#' @examples
#' M <- 0.1
#' age <- seq(1:10)
#' pred <- exp(-M * age)
#' obs <- pred * rlnorm(10, sd = 0.05)
#' like_comp(obs, pred, N = 10, type = "multinomial")
#' like_comp(obs, pred, N = 100, type = "multinomial")
#' like_comp(obs, pred, N = 10, type = "dirmult1", theta = 1)
#' like_comp(obs, pred, N = 10, type = "dirmult1", theta = 20)
#' @importFrom stats rmultinom rnorm
#' @export
like_comp <- function(obs, pred, type = c("multinomial", "dirmult1", "dirmult2", "lognormal", "logitnormal"),
                      N = sum(obs), theta, stdev) {

  if (!inherits(obs, "simref") && (all(is.na(obs)) || !sum(obs))) { # estimation or report mode
    v <- if (inherits(pred, "advector")) advector(0) else 0
    return(v)
  }

  stopifnot(length(obs) == length(pred))
  type <- match.arg(type)

  #p_pred <- (pred + 1e-8)/sum(pred + 1e-8) # This doesn't work
  pred <- CondExpGt(pred, 1e-8, pred, 1e-8)
  p_pred <- pred/sum(pred)
  p_obs <- obs/sum(obs)

  if (type == "multinomial") {
    if (inherits(obs, "simref")) {
      if (sum(pred) && !is.na(N)) {
        obs[] <- stats::rmultinom(1, size = N, prob = p_pred)
      } else {
        obs[] <- NA
      }
    } else {
      # Do not use stats::dmultinom which rounds observations to whole numbers!
      v <- dmultinom_(N * p_obs, prob = p_pred, log = TRUE)
    }

  } else if (grepl("dirmult", type)) {

    if (type == "dirmult1") {
      alpha <- theta * N * p_pred
    } else if (type == "dirmult2") {
      alpha <- theta * p_pred
    }

    if (inherits(obs, "simref")) {
      if (!requireNamespace("RTMBdist", quietly = TRUE)) {
        stop("Need the RTMBdist package to simulate from the Dirichlet-multinomial distribution")
      }
      if (sum(pred) && !is.na(N)) {
        obs[] <- RTMBdist::rdirmult(1, size = N, alpha = alpha)
      } else {
        obs[] <- NA
      }
    } else {
      v <- ddirmult_(obs, size = N, alpha = alpha, log = TRUE)
    }

  } else if (type == "lognormal") {

    if (missing(stdev)) stdev <- 1/sqrt(p_pred)

    if (inherits(obs, "simref")) {
      if (sum(pred)) {
        obs[] <- exp(stats::rnorm(length(pred), log(p_pred), stdev))
      } else {
        obs[] <- NA
      }
    } else {
      resid <- p_obs/p_pred
      v <- dnorm(log(resid[obs > 0]), 0, stdev[obs > 0], log = TRUE) |> sum()
    }

  } else if (type == "logitnormal") {

    if (missing(stdev)) stdev <- 1/sqrt(p_pred)

    i_fit <- obs > 0
    i_ref <- rep(FALSE, length(obs))
    i_ref[which(i_fit)[1]] <- TRUE

    if (inherits(obs, "simref")) {
      if (sum(pred)) {
        xpred <- log(p_pred[!i_ref]/p_pred[i_ref])
        xsamp <- stats::rnorm(length(xpred), xpred, stdev[!i_ref])
        y <- exp(xsamp)/(1 + exp(xsamp))

        obs[!i_ref] <- y
        obs[i_ref] <- 1 - sum(y)
      } else {
        obs[] <- NA
      }
    } else {
      xobs <- log(p_obs[i_fit & !i_ref]/p_obs[i_ref])
      xpred <- log(p_pred[i_fit & !i_ref]/p_pred[i_ref])

      v <- dnorm(xobs, xpred, stdev[i_fit & !i_ref], log = TRUE) |> sum()
    }

  }

  return(v)
}

#' Likelihood for CKMR
#'
#' Returns the log-likelihood for a set of pairwise comparisons. For a parent-offspring pair, a comparison
#' is defined by the capture year of parent, capture age of parent, and birth year of offspring.
#' For a half-sibling pair, a comparison is defined by the cohort year of each sibling.
#' Binomial and Poisson distributions are supported (Conn et al. 2020).
#'
#' @param n The number of pairwise comparisons
#' @param m The number of kinship matches
#' @param p The probability of kinship match
#' @param type The statistical distribution for the likelihood calculation
#' @return Numeric representing the log-likelihood.
#' @seealso [calc_POP()] [calc_HSP()]
#' @references
#' Conn, P.B. et al. 2020. Robustness of close-kin mark-recapture estimators to dispersal
#' limitation and spatially varying sampling probabilities. Ecol. Evol. 10: 5558-5569. \doi{10.1002/ece3.6296}
#' @export
like_CKMR <- function(n, m, p, type = c("binomial", "poisson")) {
  type <- match(type)
  if (is.null(n) || all(!n)) {
    v <- 0
  } else if (type == "binomial") {
    v <- dbinom(m, n, p, log = TRUE)
  } else if (type == "poisson") {
    v <- dpois(m, n * p, log = TRUE)
  }
  return(v)
}

ddirmult_ <- function(x, size, alpha, log = FALSE) {
  x <- size * x/sum(x)
  alpha0 <- sum(alpha)
  val <- lgamma(alpha0) + lgamma(size + 1) - lgamma(alpha0 + size)
  val2 <- lgamma(x + alpha) - lgamma(alpha) - lgamma(x + 1)

  log_res <- val + sum(val2)

  if (log) log_res else exp(log_res)
}

dmultinom_ <- function(x, size = NULL, prob, log = FALSE) {
  K <- length(prob)
  if (length(x) != K)
    stop("x[] and prob[] must be equal length vectors.")
  if (any(x < 0))
    stop("'x' must be non-negative")
  N <- sum(x)
  if (is.null(size))
    size <- N
  else if (size != N)
    stop("size != sum(x), i.e. one is wrong")

  r <- lgamma(size + 1) + sum(x * log(prob) - lgamma(x + 1))
  if (log)
    r
  else exp(r)
}

#' @name profile
#' @aliases profile,MSAassess-method profile.MSAassess
#'
#' @title Profile parameters of MSA model
#'
#' @description
#' Evaluate change in objective function and likelihood components for up to 2 parameters.
#'
#' @param fitted [MSAassess-class] object returned by [fit_MSA()]
#' @param p1 Character string that represents the first parameter to be profiled,
#' including the parameter name and index of the vector/array. See "Parameters" section of [make_parameters()].
#' Additionally, this function allows users to specify `R0_s` and `h_s` (in normal units).
#' @param v1 Vector of values corresponding to `p1`
#' @param p2 Character string that represents the optional second parameter to be profiled
#' @param v2 Vector of values corresponding to `p2`
#' @param use_fitted Logical, whether to use estimated parameters or the starting values in `fitted` to start the profile
#' @param return_models Logical, whether to return fitted models in the profile
#' @param cores Integer for the number of cores to use for parallel processing
#' @param ... Not used
#' @return
#' Named list (length 2).
#'
#' First, `profile` contains a data frame of the likelihood values that correspond to
#' fixed values of `p1` and `p2`. Other columns:
#'
#' - Likelihood `loglike` refers to maximizing the probability of the observed data (higher values for better fit)
#' - Prior `logprior` refers to maximizing the probability of a parameter to their prior distribution (higher values are closer to the prior mode)
#' - Penalty `penalty` are values added to the objective function when parameters exceed model bounds (lower values are better)
#' - `fn` is the objective function returned by RTMB (lower values are better)
#' - `objective` is the objective function returned by the optimizer (lower values are better)
#'
#' Second, `fits` contains a list of the `MSAassess` objects if `return_models = TRUE`.
#' @importFrom stats profile
#' @importFrom pbapply pblapply pboptions
#' @export
setMethod("profile", signature(fitted = "MSAassess"),
          function(fitted, p1, v1, p2, v2, use_fitted = TRUE, return_models = TRUE, cores = 1, ...) {

            if (cores > 1) {
              cl <- parallel::makeCluster(cores)
              on.exit(parallel::stopCluster(cl), add = TRUE)
            } else {
              cl <- NULL
            }

            old_opts <- pbapply::pboptions()
            on.exit(pbapply::pboptions(old_opts), add = TRUE)

            pbapply::pboptions(use_lb = TRUE)

            if (missing(p2)) {
              pars <- p1
              prof_df <- expand.grid(p1 = v1)

              fits <- pbapply::pblapply(
                prof_df$p1, profile_fn, fitted, pars, use_fitted, return_models,
                cl = cl
              )

            } else {
              pars <- c(p1, p2)
              prof_df <- expand.grid(p1 = v1, p2 = v2)

              prof_list <- lapply(1:nrow(prof_df), function(i) prof_df[i, ])
              fits <- pbapply::pblapply(
                prof_list, profile_fn, fitted, pars, use_fitted, return_models,
                cl = cl
              )
            }

            if (return_models) {
              prof <- lapply(fits, get_likelihood_components)
            } else {
              prof <- fits
            }

            prof_df <- cbind(prof_df, do.call(rbind, prof))

            names(prof_df)[1] <- attr(prof_df, "p1") <- p1
            if (!missing(p2)) {
              names(prof_df)[2] <- attr(prof_df, "p2") <- p2
            }

            p <- fitted@obj$env$parList(fitted@obj$env$last.par.best)
            pval1 <- eval(parse(text = paste0("p$", p1)))
            if (is.null(pval1)) pval1 <- eval(parse(text = paste0("fitted@report$", p1)))
            if (is.null(pval1)) pval1 <- NA

            if (missing(p2)) {

              attr(prof_df, "fitted") <- cbind(
                pval1,
                get_likelihood_components(fitted)
              )
              names(attr(prof_df, "fitted"))[1] <- p1

            } else {
              pval2 <- eval(parse(text = paste0("p$", p2)))
              if (is.null(pval2)) pval2 <- eval(parse(text = paste0("fitted@report$", p2)))
              if (is.null(pval2)) pval2 <- NA

              attr(prof_df, "fitted") <- cbind(
                pval1,
                pval2,
                get_likelihood_components(fitted)
              )
              names(attr(prof_df, "fitted"))[1:2] <- c(p1, p2)
            }

            output <- list(profile = prof_df, fits = if (return_models) fits else NULL) |>
              structure(class = "MSAprof")

            return(output)
          })

profile_fn <- function(vals, fitted, pars, use_fitted = TRUE, return_models = TRUE) {
  stopifnot(length(pars) == length(vals))

  MSAdata <- get_MSAdata(fitted)
  if (use_fitted) {
    p <- fitted@obj$env$parList(fitted@obj$env$last.par.best)
  } else {
    p <- fitted@obj$env$parList(fitted@obj$par)
  }
  map <- MSAdata@Misc$map
  random <- MSAdata@Misc$random

  for (i in 1:length(pars)) {
    pi <- pars[i]
    vi <- vals[i]

    split_pi <- strsplit(pi, "\\[|\\]")[[1]] # Regex cheat sheet, split by [ or ]
    p_name <- split_pi[1]

    # Transform vi if pi = "R0_s[x]"
    if (grepl("R0_s", pi) && !grepl("t_R0_s", pi)) {
      p_name <- "t_R0_s"
      pi <- sub("R0_s", "t_R0_s", pi)
      vi <- log(vi/MSAdata@Dmodel@scale_s[as.integer(split_pi[2])])
    }

    # Transform vi if pi = "h_s[x]"
    if (grepl("h_s", pi) && !grepl("t_h_s", pi)) {
      p_name <- "t_h_s"
      pi <- sub("h_s", "t_h_s", pi)
      vi <- switch(
        MSAdata@Dstock@SRR_s[as.integer(split_pi[2])],
        "BH" = qlogis((vi - 0.2)/0.8),
        "Ricker" = log(vi - 0.2),
      )
    }

    if (is.null(map[[p_name]])) {
      map[[p_name]] <- if (is.null(dim(p[[p_name]]))) {
        1:length(p[[p_name]])
      } else {
        array(1:length(p[[p_name]]), dim(p[[p_name]]))
      }
    } else if (!is.null(dim(p[[p_name]]))) {
      map[[p_name]] <- array(map[[p_name]], dim(p[[p_name]]))
    }

    map_p <- paste0("map$", pi, " <- NA")
    eval(parse(text = map_p))
    map[[p_name]] <- factor(map[[p_name]])

    assign_p <- paste0("p$", pi, " <- ", vi)
    eval(parse(text = assign_p))
  }
  fit <- fit_MSA(MSAdata, p, map, MSAdata@Misc$random, do_sd = FALSE, silent = TRUE)

  if (return_models) {
    return(fit)
  } else {
    return(get_likelihood_components(fit))
  }
}

#' @name profile
#' @returns `get_likelihood_components()` returns a data.frame of the components to the objective function (log-likelihoods, log-priors, etc.)
#' as well as some diagnostic information: maximum gradient (`maxgrad`) and convergence (`conv`)
#' @export
get_likelihood_components <- function(fitted) {

  if (!length(fitted@report)) {
    fitted@report <- update_report(fitted@obj$report(fitted@obj$env$last.par.best), MSAdata = get_MSAdata(fitted))
  }

  nm <- names(fitted@report)
  nm_like <- grep("loglike", nm, value = TRUE)
  nm_pr <- grep("logprior", nm, value = TRUE)

  nm_out <- c(nm_like, nm_pr, "penalty", "fn")

  out <- lapply(nm_out, function(i) sum(fitted@report[[i]])) |>
    structure(names = nm_out)

  if (length(fitted@opt) && !is.character(fitted@opt)) {
    out$objective <- fitted@opt$objective
  } else {
    out$objective <- NA_real_
  }

  maxgrad <- try(max(abs(fitted@SD$gradient.fixed)), silent = TRUE)
  if (is.character(maxgrad)) maxgrad <- max(abs(fitted@obj$gr(fitted@obj$env$last.par.best)))
  out$maxgrad <- maxgrad

  out$conv <- NA
  conv <- try(fitted@SD$pdHess, silent = TRUE)
  if (is.logical(conv)) out$conv <- conv

  do.call(data.frame, out)
}

#' @rdname profile
#' @aliases plot.MSAprof
#'
#' @param x Output from [profile.MSAassess()]
#' @param component Character for the column in `x` to be plotted
#' @param rel Logical, whether the relative change in `component` is plotted (TRUE) or the raw values (FALSE)
#' @param xlab Optional character for the x-axis label
#' @param ylab Optional character for the y-axis label
#' @param main Optional character for the plot title
#' @param plot2d Character, plotting function for two-dimensional profiling (either a [contour()] or [filled.contour()] plot)
#' @param ... Other argument to the base graphics function, i.e., either plot() or contour()
#' @return
#' The accompanying plot function returns a line plot for a 1-dimensional profile or a contour plot for a two
#' dimensional profile. Will plot the negative log likelihood or negative log prior (better fit with lower values).
#'
#' Relative values are obtained by subtracting from the fitted value. See `attr(x$profile, "fitted")`
#' @importFrom graphics contour filled.contour
#' @importFrom reshape2 acast
#' @export
plot.MSAprof <- function(x, component = "objective", rel = TRUE, xlab, ylab, main,
                         plot2d = c("contour", "filled.contour"), ...) {

  prof <- x$profile

  p1 <- attr(prof, "p1")
  p2 <- attr(prof, "p2")

  if (missing(xlab)) xlab <- p1
  if (is.null(prof[[component]])) stop("\"", component, "\" not found")

  fitted <- attr(prof, "fitted")

  if (is.null(p2)) {
    xplot <- prof[[p1]]
    yplot <- prof[[component]]
    yfit <- fitted[[component]]
    if (grepl("logprior", component) || grepl("loglike", component)) {
      yplot <- -1 * yplot
      yfit <- -1 * yfit
    }
    if (rel) yplot <- yplot - yfit
    if (missing(ylab)) {
      if (grepl("logprior", component) || grepl("loglike", component)) {
        ylab <- paste("Change in negative", component)
      } else {
        ylab <- paste("Change in", component)
      }
    }
    if (missing(main)) main <- NULL

    plot(xplot, yplot, xlab = xlab, ylab = ylab, type = "o", main = main, ...)
    abline(v = fitted[[p1]], lty = 2)

  } else {

    plot2d <- match.arg(plot2d)
    plot2d <- match.fun(plot2d)

    names(prof)[names(prof) == p1] <- "p1"
    names(prof)[names(prof) == p2] <- "p2"

    zplot <- reshape2::acast(prof, list("p1", "p2"), value.var = component)
    zfit <- fitted[[component]]
    if (grepl("logprior", component) || grepl("loglike", component)) {
      zplot <- -1 * zplot
      zfit <- -1 * zfit
    }
    if (rel) zplot <- zplot - zfit
    if (missing(ylab)) ylab <- p2
    if (missing(main)) {
      if (grepl("logprior", component) || grepl("loglike", component)) {
        main <- paste("Change in negative", component)
      } else {
        main <- paste("Change in", component)
      }
    }

    plot2d(
      x = as.numeric(rownames(zplot)), y = as.numeric(colnames(zplot)), z = zplot,
      xlab = xlab, ylab = ylab, main = main,
      ...
    )
    points(fitted[[p1]], fitted[[p2]], col = "red", pch = 16)

  }
  invisible()
}

#' Quadratic penalty function
#'
#' Taped penalty function if `x < eps`
#'
#' @param x Numeric, the parameter
#' @param eps Numeric, the threshold below which a penalty will be applied
#'
#' @return
#' The penalty value is
#'
#' \deqn{
#' \textrm{penalty} =
#' \begin{cases}
#' 0.1 (x - \varepsilon)^2 & x \le \varepsilon\\
#' 0 & x > \varepsilon
#' \end{cases}
#' }
#'
#' @return Numeric
#' @export
posfun <- function(x, eps) CondExpGe(x, eps, 0, 0.01 * (x - eps) * (x - eps))

calc_selpar_penalty <- function(sel_pf, sel_f, lmid, na, map) {
  if (missing(map) || is.null(map)) {
    map <- array(1:length(sel_pf), dim(sel_pf))
  } else {
    map <- array(map, dim(sel_pf))
  }
  map <- map[-1, ]
  map_num <- as.numeric(map)

  unique_par <- !is.na(map_num) & !duplicated(map_num)

  if (inherits(sel_pf, "advector")) `[<-` <- RTMB::ADoverload("[<-")

  if (all(is.na(map))) {
    if (inherits(sel_pf, "advector")) val <- advector(0) else val <- 0
  } else {
    penalty <- array(0, c(2, ncol(sel_pf)))
    parametric_sel <- grepl("dome|logistic", sel_f)
    flen <- parametric_sel & grepl("length", sel_f)
    if (any(flen)) {
      log_binwidth <- log(0.5 * min(diff(lmid)))
      log_binrange <- log(max(lmid) - min(lmid))
      pen_len <- posfun(sel_pf[2:3, flen], log_binwidth) + posfun(log_binrange, sel_pf[2:3, flen])
      penalty[, flen] <- penalty[, flen] + pen_len
    }
    fage <- parametric_sel & grepl("age", sel_f)
    if (any(fage)) {
      pen_age <- posfun(sel_pf[2:3, fage], log(0.5)) + posfun(log(na), sel_pf[2:3, fage])
      penalty[, fage] <- penalty[, fage] + pen_age
    }
    val <- sum(penalty[unique_par])
  }

  return(val)
}

#' Softmax function
#'
#' Takes a vector of real numbers and returns the corresponding vector of probabilities
#'
#' @param eta Vector
#' @param log Logical, whether to return the value of the logarithm
#'
#' @return Numeric, vector length of `eta`: \eqn{\exp(\eta)/\sum\exp(\eta)}
#' @details Uses `multiSA:::logspace.add` for numerical stability
#' @export
softmax <- function(eta, log = FALSE) {
  den <- Reduce(logspace.add, eta)
  v <- eta - den

  if (log) {
    v
  } else {
    exp(v)
  }
}

logspace.add <- function(lx, ly) CondExpGt(lx, ly, lx, ly) + log1p(exp(-abs(lx - ly)))

#' Calculate covariance matrix
#'
#' Uses Cholesky factorization to generate a covariance matrix (or any symmetric positive definite matrix).
#'
#' @param sigma Numeric vector of marginal standard deviations (all greater than zeros)
#' @param lower_diag Numeric vector to populate the lower triangle of the correlation matrix. All real numbers.
#' Length `sum(1:(length(sigma) - 1))`
#' @examples
#' set.seed(23)
#' n <- 5
#' sigma <- runif(n, 0, 2)
#' lower_diag <- runif(sum(1:(n-1)), -10, 10)
#' Sigma <- conv_Sigma(sigma, lower_diag)
#' Sigma/t(Sigma) # Is symmetric matrix? All ones
#' cov2cor(Sigma)
#'
#' @return Numeric
#' @export
conv_Sigma <- function(sigma, lower_diag) {
  n <- length(sigma)
  stopifnot(length(lower_diag) == sum(1:(n-1)))

  # Parameterizes correlation matrix of X in terms of Cholesky factors
  # https://github.com/kaskr/RTMB/blob/master/tmb_examples/sdv_multi.R
  L <- diag(n)
  L[lower.tri(L)] <- lower_diag
  row_norms <- apply(L, 1, function(row) sqrt(sum(row * row)))
  L <- L / row_norms
  R <- L %*% t(L)  # Correlation matrix of X (guaranteed positive definite)

  V <- diag(sigma)
  Sigma <- V %*% R %*% V
  return(Sigma)
}

conv_steepness <- function(x, SRR = c("BH", "Ricker")) {
  SRR <- match.arg(SRR)
  switch(
    SRR,
    "BH" = 0.8 * plogis(x),
    "Ricker" = exp(x)
  ) + 0.2
}

conv_mat <- function(x, na) {
  a50 <- na * plogis(x[1])
  a95 <- a50 + exp(x[2])

  a <- seq(1, na) - 1
  m <- 1/(1 + exp(-log(19) * (a - a50)/(a95 - a50)))
  return(m)
}


#' Optimize RTMB model
#'
#' A convenient function to fit a RTMB model with [stats::nlminb()]
#'
#' @param obj The list returned by [RTMB::MakeADFun()]
#' @param hessian Logical, whether to pass the Hessian function `obj$he` to [stats::nlminb()]. Only used if
#' there are no random effects in the model.
#' @param restart Deprecated.
#' @param do_sd Deprecated.
#' @param control List of options passed to [stats::nlminb()]
#' @param lower Lower bounds of parameters passed to [stats::nlminb()]
#' @param upper Upper bounds of parameters passed to [stats::nlminb()]
#' @param silent Logical, whether to report progress to console
#' @return A named list, output of [stats::nlminb()]
#' @importFrom stats nlminb
#' @seealso [get_sdreport()]
#' @keywords internal
#' @export
optimize_RTMB <- function(obj, hessian = FALSE, restart = 0, do_sd = TRUE,
                          control = list(iter.max = 2e+05, eval.max = 4e+05),
                          lower = -Inf, upper = Inf, silent = FALSE) {

  if (is.null(obj$env$random) && hessian) {
    h <- obj$he
    if (!silent) message("Using hessian in optimization (can be memory-intensive)")
  } else {
    h <- NULL
  }

  if (!silent) message_info("Fitting model with stats::nlminb()..")
  opt <- tryCatch(
    nlminb(obj$par, obj$fn, obj$gr, h, control = control, lower = lower, upper = upper),
    error = function(e) as.character(e)
  )
  if (!silent) message_info("Final gradient is ", round(max(abs(obj$gr(obj$env$last.par.best))), 5))

  return(opt)
}

# Check that hessian is positive-definite
check_h <- function(h) {
  L <- try(chol(h), silent = TRUE)
  !is.character(L)
}

# Check that hessian could be marginally positive-definite: abs(det(h)) < tol
marginal_h <- function(h, tol = 0.1) {
  det_h <- determinant(h)
  !is.na(det_h$modulus) && det_h$modulus < log(tol)
}

#' Calculate standard errors
#'
#' A wrapper function to return standard errors. Various numerical techniques are employed to obtain
#' a positive-definite covariance matrix in marginal cases.
#' @inheritParams optimize_RTMB
#' @param par.fixed Numeric vector of parameters from which to calculate covariance matrix. Optional
#' @param exact Logical, whether to use autodiff or finite-difference approximation for the hessian. See details.
#' @param getReportCovariance Logical, passed to [RTMB::sdreport()]
#' @param silent Logical, whether to report progress to console. See details.
#' @param ... Other arguments to [RTMB::sdreport()] besides `par.fixed, hessian.fixed, getReportCovariance`
#' @details
#' Uses [stats::optimHess()] if `exact = FALSE`.
#' Autodiff with `exact = TRUE` is only available for TMB models without random effects, but is also memory-intensive.
#'
#' In numerically marginal cases where the determinant of the Hessian matrix is less than 0.1, the function will attempt
#' to calculate the hessian with `numDeriv::jacobian()` and the gradient from TMB.
#'
#' Finally, in other marginal cases where [chol()] identifies a positive-definite Hessian but [solve()] fails to
#' invert the matrix, the covariance matrix will be updated with `chol2inv(chol(h))`
#' @return
#' Object returned by [RTMB::sdreport()].
#'
#' A correlation matrix is generated and stored in: `get_sdreport(obj)$env$corr.fixed`
#'
#' The hessian is stored in `get_sdreport(obj)$env$hessian`
#' @importFrom stats optimHess
#' @export
get_sdreport <- function(obj, par.fixed, exact = FALSE, getReportCovariance = FALSE, silent = FALSE, ...) {
  if (missing(par.fixed)) {
    par.fixed <- obj$env$last.par.best
    if (!is.null(obj$env$random)) par.fixed <- par.fixed[-obj$env$random]
  }

  if (exact && is.null(obj$env$random)) {
    if (!silent) message_info("Calculating standard errors with hessian from obj$he()..")
    h <- obj$he(par.fixed)
  } else {
    if (!silent) message_info("Calculating standard errors with hessian from stats::optimHess()..")
    h <- optimHess(par.fixed, obj$fn, obj$gr)
  }
  res <- sdreport(obj, par.fixed = par.fixed, hessian.fixed = h,
                  getReportCovariance = getReportCovariance, ...)

  if (!res$pdHess) {
    if (!silent) message_oops("Hessian is not positive-definite.")

    if (marginal_h(h) && requireNamespace("numDeriv", quietly = TRUE)) {
      if (!silent) message_info("Calculating standard errors with hessian from numDeriv::jacobian()..")
      h <- numDeriv::jacobian(obj$gr, par.fixed)
      h <- 0.5 * (h + t(h)) # glmmTMB does this

      if (check_h(h)) {
        res <- sdreport(obj, par.fixed = par.fixed, hessian.fixed = h,
                        getReportCovariance = getReportCovariance, ...)
      } else if (!silent) {
        message_oops("Hessian is not positive-definite.")
      }
    }
  }

  if (any(is.na(res$cov.fixed)) && res$pdHess) {
    if (!silent) message_info("Calculating standard errors from chol2inv(chol(h))..")
    ch <- try(chol(h), silent = TRUE) # Not needed, this is the test for convergence in sdreport
    if (!is.character(ch)) res$cov.fixed <- chol2inv(ch)
  }

  fixed.names <- make_unique_names(res, select = "fixed")

  res$env$corr.fixed <- cov2cor(res$cov.fixed) |> round(3) |>
    structure(dimnames = list(fixed.names, fixed.names))

  res$env$hessian <- structure(h, dimnames = list(fixed.names, fixed.names))

  return(res)
}

#' @importFrom TMB summary.sdreport
sdreport_int <- function(object, select = c("all", "fixed", "random", "report"), p.value = FALSE, ...) {
  if (is.character(object)) return(object)
  select <- match.arg(select, several.ok = TRUE)
  if ("all" %in% select) select <- c("fixed", "random", "report")
  if ("report" %in% select) {
    AD <- TMB::summary.sdreport(object, "report", p.value = p.value) |> cbind("Gradient" = NA_real_)
    ADnames <- make_unique_names(object, select = "report")
  } else AD <- ADnames <- NULL

  if ("fixed" %in% select) {
    fix <- TMB::summary.sdreport(object, "fixed", p.value = p.value) |> cbind("Gradient" = as.vector(object$gradient.fixed))
    fixnames <- make_unique_names(object, select = "fixed")
  } else fix <- fixnames <- NULL

  if (!is.null(object$par.random) && "random" %in% select) {
    random <- TMB::summary.sdreport(object, "random", p.value = p.value) |> cbind("Gradient" = rep(NA_real_, length(object$par.random)))
    randomnames <- make_unique_names(object, select = "random")
  } else {
    random <- randomnames <- NULL
  }

  out <- rbind(AD, fix, random)
  out <- cbind(out, "CV" = ifelse(abs(out[, "Estimate"]) > 0, out[, "Std. Error"]/abs(out[, "Estimate"]), NA_real_))
  rownames(out) <- c(ADnames, fixnames, randomnames)
  return(out)
}


#' Retrieve data object used to fit model
#'
#' A convenient function to retrieve the data object used to fit the model. The object is embedded in an environment
#' within the RTMB object.
#'
#' @param MSAassess [MSAassess-class] object returned by `fit_MSA()`
#' @return [MSAdata-class] object
#' @export
get_MSAdata <- function(MSAassess) {
  func <- attr(MSAassess@obj$env$data, "func")
  version <- strsplit(attr(MSAassess, "version"), "multiSA ")[[1]][2]

  if (version >= "0.2.0") {
    d <- "x"
  } else {
    d <- "MSAdata"
  }
  MSAdata <- get(d, envir = environment(func), inherits = FALSE)
  return(MSAdata)
}

make_unique_names <- function(x, select = c("fixed", "random", "report")) {
  select <- match.arg(select)

  if (select == "fixed") {
    par_names <- unique(names(x$par.fixed))
    par_list <- as.list(x, what = "Estimate", report = FALSE)
  } else if (select == "report") {
    par_names <- unique(names(x$value))
    par_list <- as.list(x, what = "Estimate", report = TRUE)
  } else {
    par_names <- unique(names(x$random))
    par_list <- as.list(x, what = "Estimate", report = FALSE)
  }


  par_dims <- lapply(par_names, function(y) {
    dim_y <- dim(par_list[[y]])
    if (is.null(dim_y)) dim_y <- length(par_list[[y]])
    ind_y <- lapply(dim_y, function(i) seq(1, i))

    est_grid <- do.call(expand.grid, ind_y)

    if (select != "report") {
      map_y <- attr(x$env$parameters[[y]], "map")

      if (!is.null(map_y)) {
        est_y <- map_y >= 0 & !duplicated(map_y)
        est_grid <- est_grid[est_y, ]
      }
    }

    dim_char <- sapply(1:nrow(est_grid), function(i) {
      paste0(
        "[",
        paste0(est_grid[i, ], collapse = ", "),
        "]"
      )
    })
    paste0(y, dim_char)
  })

  do.call(c, par_dims)
}


make_yearseason <- function(year, nm = 4) {
  if (nm <= 1) return(year)
  year_long <- lapply(year, function(y) y + (1:nm - 1)/nm)
  do.call(c, year_long)
}

#' @importFrom reshape2 acast melt
collapse_yearseason <- function(x) {
  dim_x <- dim(x)
  if (length(dim_x) > 2) {
    dimnames(x)[[1]] <- 1:dim_x[1]
    dimnames(x)[[2]] <- 1:dim_x[2]

    x_df <- reshape2::melt(x)
    x_df$Y <- as.numeric(x_df$Var1) + (as.numeric(x_df$Var2) - 1)/dim_x[2]

    dims <- c("Y", paste0("Var", 3:length(dim_x))) |> as.list()
    xout <- reshape2::acast(x_df, dims, value.var = "value")
    dimnames(xout) <- NULL

    return(xout)

  } else {
    return(as.numeric(t(x)))
  }
}

message <- function(...) {
  if (requireNamespace("usethis", quietly = TRUE)) {
    dots <- list(...)
    do.call(c, dots) |> paste0(collapse = "") |> usethis::ui_done()
  } else {
    base::message(...)
  }
}


message_info <- function(...) {
  if (requireNamespace("usethis", quietly = TRUE)) {
    dots <- list(...)
    do.call(c, dots) |> paste0(collapse = "") |> usethis::ui_info()
  } else {
    base::message(...)
  }
}

message_oops <- function(...) {
  if (requireNamespace("usethis", quietly = TRUE)) {
    dots <- list(...)
    do.call(c, dots) |> paste0(collapse = "") |> usethis::ui_oops()
  } else {
    base::message(...)
  }
}

warning <- function(...) {
  if (requireNamespace("usethis", quietly = TRUE)) {
    dots <- list(...)
    do.call(c, dots) |> paste0(collapse = "") |> usethis::ui_warn()
  } else {
    base::warning(...)
  }
}


stop <- function(..., call. = TRUE, domain = NULL) {
  if (requireNamespace("usethis", quietly = TRUE)) {
    dots <- list(...)
    do.call(c, dots) |> paste0(collapse = "") |> usethis::ui_stop()
  } else {
    base::stop(..., call. = call., domain = domain)
  }
}

#' Selectivity at age and length
#'
#' @description
#' Calculate selectivity at age and length from a matrix of parameters.
#' * [conv_selpar()] converts parameters from log or logit space to real units.
#' * [calc_sel_len()] calculates selectivity at length.
#' * [calc_fsel_age()] calculates selectivity at age for fisheries, and coordinates dummy fleets.
#' * [calc_isel_age()] calculates selectivity at age for indices, and can map selectivity from fisheries
#' or population parameters (e.g, mature or total biomass).
#'
#' @param x Estimated parameters. Matrix `[3, f]`
#' @param type,fsel_type Character string to indicate the functional form of selectivity. See details below.
#' @param maxage Maximum value of the age of full selectivity
#' @param maxL Maximum value of the length of full selectivity
#' @details
#' Options for argument `type` include:
#'
#' - functional forms with respect to length: `"logistic_length", "dome_length"`
#' - functional forms with respect to age: `"logistic_age", "dome_age"`
#' - for surveys, an integer (`f`) to map index selectivity at age to fleet `f` (will be coerced to integer)
#' - `"mature"` to fix to maturity at age schedule (`"SB"` is also acceptable for backwards compatibility)
#' - `"total"` to fix selectivity to 1 for all ages (`"B"` is also acceptable for backwards compatibility)
#' - `"length_x_y"` to specify selectivity to 1 between lengths `x` and `y` (example: `"length_20_50"`)
#' - `"age_x_y"` to specify selectivity to 1 between age `x` and `y` (example: `"age_2_4"`)
#' - for surveys, `"f_x_y"` uses selectivity values from fleet `f` between bins `x` and `y`
#' (either length or age depending on definition of selectivity for `f`) (example: `"3_2_4"`)
#'
#' @section Converting selectivity parameters (conv_selpar):
#' The first row of `x` corresponds to the length or age of full selectivity: \eqn{\mu_f = L_{max}/(1 + \exp(-x_{1,f}))}
#'
#' The second row of `x` corresponds to the width of the ascending limb for selectivity:
#' \eqn{\sigma_f^{asc} = \exp(x_{2,f})}
#'
#' The third row of `x` corresponds to the width of the descending limb for selectivity (if dome-shaped):
#' \eqn{\sigma_f^{des} = \exp(x_{3,f})}
#' @return
#' [conv_selpar()] returns a matrix of the same dimensions as `x`.
#' @export
conv_selpar <- function(x, type, maxage, maxL) {
  nf <- length(type)
  stopifnot(ncol(x) == nf)
  stopifnot(nrow(x) >= 3)

  sel_par <- sapply(1:nf, function(f) {
    sd_asc <- exp(x[2, f])
    sd_desc <- exp(x[3, f])
    if (grepl("logistic_age|dome_age", type[f])) {
      Aapical <- maxage * plogis(x[1, f])
      v <- c(Aapical, sd_asc, sd_desc)
    } else if (grepl("logistic_length|dome_length", type[f])) {
      Lapical <- maxL * plogis(x[1, f])
      v <- c(Lapical, sd_asc, sd_desc)
    } else {
      v <- rep(NA_real_, 3)
    }
    return(v)
  })

  return(sel_par)
}

#' @rdname conv_selpar
#' @aliases calc_sel_len
#' @param sel_par Matrix of parameters returned by [conv_selpar()]
#' @param lmid Midpoints of length bins for calculating selectivity at length
#' @param fsel_len Selectivity at length matrix for fleets, returned by previous call to `calc_sel_len()`
#' @section Length selectivity (calc_sel_len):
#' The selectivity at length is
#' \deqn{
#' s_{\ell} =
#' \begin{cases}
#' \exp(\alpha) & L_{\ell} < \mu_f\\
#' \exp(\beta) & L_{\ell} \ge \mu_f\\
#' \end{cases}
#' }
#' where
#' \eqn{
#' \alpha = -0.5(L_\ell - \mu_f)^2/(\sigma_f^{asc})^2
#' }
#' and
#' \eqn{
#' \beta = -0.5(L_\ell - \mu_f)^2/(\sigma_f^{des})^2
#' }
#' @return
#' [calc_sel_len()] returns a matrix `[l, f]`, i.e., `[length(lmid), length(type)]`.
#' @export
calc_sel_len <- function(sel_par, lmid, type, fsel_type, fsel_len) {
  nf <- length(type)
  is_ad <- inherits(sel_par, "advector")

  sel_lf <- sapply(1:nf, function(f) {
    if (grepl("logistic_length|dome_length", type[f])) {
      ex_asc <- (lmid - sel_par[1, f])/sel_par[2, f]
      asc <- exp(-0.5 * ex_asc * ex_asc)

      if (grepl("logistic_length", type[f])) {
        desc <- 1
      } else {
        ex_desc <- (lmid - sel_par[1, f])/sel_par[3, f]
        desc <- exp(-0.5 * ex_desc * ex_desc)
      }
      v <- CondExpLt(lmid, sel_par[1, f], asc, desc)
      v <- v/max(v)
    } else if (startsWith(type[f], "length")) {
      sel_char <- strsplit(type[f], "_")[[1]]
      lmin <- as.numeric(sel_char[2])
      lmax <- as.numeric(sel_char[3])
      v <- ifelse(lmid >= lmin & lmid <= lmax, 1, 0)
    } else {
      sel_char <- strsplit(type[f], "_")[[1]]

      if (length(sel_char) == 3) {
        ff <- suppressWarnings(as.numeric(sel_char)[1])

        if (is.numeric(ff) && grepl("length", fsel_type[ff])) {
          lmin <- as.numeric(sel_char[2])
          lmax <- as.numeric(sel_char[3])
          v <- ifelse(lmid >= lmin & lmid <= lmax, fsel_len[, ff], 0)
        } else {
          v <- rep(NA_real_, length(lmid))
        }
      } else {
        v <- rep(NA_real_, length(lmid))
      }
    }
    if (is_ad) v <- advector(v)
    return(v)
  })

  return(sel_lf)
}

#' @rdname conv_selpar
#' @param sel_len Selectivity at length matrix returned by [calc_sel_len()]
#' @param LAK Length-at-age probability matrix. Matrix `[a, length(lmid)]`
#' @param sel_block Integer vector. Length `length(type)`. See details below.
#' @param mat Maturity at age vector
#' @param a Integer vector of ages corresponding to the rows of `LAK` (as well as `mat`)
#' @section Age selectivity (calc_fsel_age):
#' The equivalent selectivity at age is converted from the length values (`sel_len`) as
#' \deqn{
#' s_a = \sum_\ell s_\ell \times \textrm{Prob}(L_{\ell}|a)
#' }
#'
#' If selectivity is explicitly in age units, then the values are directly calculated
#' from parameters in `sel_par`.
#'
#' Vector `sel_block` assigns the output selectivity from a different column of the input matrix
#' and facilitates time-varying selectivity in time blocks. For example, `sel_block[1] <- 2` means
#' that selectivity values in the first column of the output is based on the second column of the
#' input matrices (`sel_len[, 2]` or `sel_par[, 2]`).
#' @return
#' [calc_fsel_age()] returns a matrix `[a, f]`, i.e., `[a, length(sel_block)]`
#' @export
calc_fsel_age <- function(sel_len, LAK, type, sel_par, sel_block = seq(1, length(type)), mat, a = seq(1, nrow(LAK)) - 1) {
  nf <- length(sel_block)

  is_ad <- inherits(sel_par, "advector")

  sel_af <- sapply(1:nf, function(ff) {
    f <- sel_block[ff]
    if (grepl("length", type[f])) {
      v <- sel_len[, f] %*% t(LAK)
      #v <- v/max(v)
    } else if (grepl("logistic_age|dome_age", type[f])) {

      ex_asc <- (a - sel_par[1, f])/sel_par[2, f]
      asc <- exp(-0.5 * ex_asc * ex_asc)

      if (grepl("logistic_age", type[f])) {
        desc <- 1
      } else {
        ex_desc <- (a - sel_par[1, f])/sel_par[3, f]
        desc <- exp(-0.5 * ex_desc * ex_desc)
      }
      v <- CondExpLt(a, sel_par[1, f], asc, desc)
      v <- v/max(v)
    } else if (startsWith(type[f], "age")) {
      sel_char <- strsplit(type[f], "_")[[1]]
      amin <- as.numeric(sel_char[2])
      amax <- as.numeric(sel_char[3])
      v <- ifelse(a >= amin & a <= amax, 1, 0)
    } else if (type[f] %in% c("SB", "mature")) {
      v <- mat
    } else if (type[f] %in% c("B", "total")) {
      v <- rep(1, length(a))
    } else if (type[f] == "free") {
      v <- plogis(sel_par[, f])
    }
    if (is_ad) v <- advector(v)
    return(v)
  })

  return(sel_af)
}

#' @rdname conv_selpar
#' @param fsel_age Matrix returned by [calc_fsel_age()]
#' @return
#' [calc_isel_age()] returns a matrix `[a, i]`, i.e., `[a, length(type)]`
#' @export
calc_isel_age <- function(sel_len, LAK, type, sel_par, fsel_age, maxage, mat,
                          a = seq(1, nrow(LAK)) - 1,
                          fsel_type, fsel_len) {
  ni <- length(type)
  is_ad <- inherits(sel_par, "advector") || inherits(fsel_age, "advector")

  sel_ai <- sapply(1:ni, function(i) {
    ti <- type[i]
    tii <- suppressWarnings(as.integer(ti))

    if (is.na(tii)) { # Not an integer
      if (grepl("length", ti)) {
        v <- sel_len[, i] %*% t(LAK)
      } else if (grepl("age", ti)) {

        if (grepl("logistic_age|dome_age", ti)) {
          ex_asc <- (a - sel_par[1, i])/sel_par[2, i]
          asc <- exp(-0.5 * ex_asc * ex_asc)

          if (grepl("logistic_age", ti)) {
            desc <- 1
          } else {
            ex_desc <- (a - sel_par[1, i])/sel_par[3, i]
            desc <- exp(-0.5 * ex_desc * ex_desc)
          }
          v <- CondExpLt(a, sel_par[1, i], asc, desc)
          v <- v/max(v)

        } else if (startsWith(ti, "age")) {

          sel_char <- strsplit(ti, "_")[[1]]
          amin <- as.numeric(sel_char[2])
          amax <- as.numeric(sel_char[3])
          v <- ifelse(a >= amin & a <= amax, 1, 0)

        }

      } else if (ti == "SB") {
        v <- mat
      } else if (ti == "B") {
        v <- rep(1, length(a))
      } else if (ti == "free") {
        v <- plogis(sel_par[, i])
      } else {
        sel_char <- strsplit(ti, "_")[[1]]
        ff <- suppressWarnings(as.numeric(sel_char[1]))
        sel_f <- fsel_type[ff]
        if (grepl("length", sel_f)) {
          v <- sel_len[, i] %*% t(LAK)
        } else {
          stop("Error: can't figure out how to calculate index selectivity")
        }
      }
    } else if (is.integer(tii)) {
      v <- fsel_age[, tii]
    } else {
      stop("Error: can't identify fleet number to mirror index selectivity")
    }
    if (is_ad) v <- advector(v)
    return(v)
  })

  return(sel_ai)
}

#' Multi-fleet, multi-area, multi-stock population dynamics model
#'
#' Project age-structured populations forward in time. Also used by [calc_phi_project()] to calculate
#' equilibrium abundance and biomass for which there is no analytic solution
#' due to seasonal movement.
#'
#' @param ny Integer, number of years for the projection
#' @param nm Integer, number of seasons
#' @param initN_ars Abundance in the first year, first season. Array `[a, r, s]`
#' @param mov_ymarrs Movement array `[y, m, a, r, r, s]`. If missing, uses a diagonal matrix (no movement among areas).
#' @param M_yas Natural mortality (per year). Array `[y, a, s]`
#' @param SRR_s Character vector by `s` for the stock recruit relationship. See [calc_recruitment()] for options
#' @param sralpha_s Numeric vector by `s` for the stock recruit alpha parameter
#' @param srbeta_s Numeric vector by `s` for the stock recruit beta parameter
#' @param mat_yas Maturity ogive. Array `[y, a, s]`
#' @param fec_yas Fecundity schedule (spawning output of mature individuals). Array `[y, a, s]`
#' @param Rdev_ys Recruitment deviations. Matrix `[y, s]`
#' @param m_spawn Integer, season of spawning
#' @param m_advanceage Integer, season at which to advance integer year age classes
#' @param delta_s Numeric vector by `s`. Fraction of season that elapses when spawning occurs, e.g., midseason spawning when `delta_s = 0.5`.
#' @param natal_rs Matrix `[r, s]`. The fraction of the mature stock `s` in region `r` that spawns at
#' time of spawning. See example in [Dstock-class].
#' @param recdist_rs Matrix `[r, s]`. The fraction of the incoming recruitment of stock `s` that settles in region `r`.
#' @param fwt_ymafs Fishery weight at age. Array `[y, m, a, f, s]`
#' @param sel_ymafs Fishery selectivity. Array `[y, m, a, f, s]`
#' @param condition Whether the fishing mortality is conditioned on the catch or specified F argument.
#' @param F_ymfr Fishing mortality (per season). Array `[y, m, f, r]`. Only used if `condition = "F"`.
#' @param Cobs_ymfr Fishery catch (weight). Array `[y, m, f, r]`. Only used if `condition = "catch"` to solve for F (see [calc_F()]).
#'
#' @examples
#' unfished_pop <- calc_population()
#' @inheritParams calc_F
#' @return
#' A named list containing:
#'
#' * `N_ymars` Stock abundance
#' * `F_ymars` Fishing mortality (summed across fleets)
#' * `F_ymfr` Fishing mortality (by fleet and region)
#' * `Z_ymars` Total mortality
#' * `F_ymafrs` Fishing mortality (disaggregated by fleet)
#' * `CN_ymafrs` Catch at age (abundance)
#' * `CB_ymfrs` Fishery catch (weight)
#' * `VB_ymfrs` Vulnerable biomass available to the fishing fleets
#' * `Nsp_yars` Spawning abundance (in the spawning season)
#' * `Npsp_yars` Potentail spawners, mature animals in the spawning season that do not spawn if outside natal regions
#' * `S_yrs` Spawning output
#' * `R_ys` Recruitment
#' * `penalty` Numeric quadratic penalty if apical fishing mortality (by fleet) exceeds `Fmax`. See [calc_F()].
#'
#' @export
calc_population <- function(ny = 10, nm = 4, na = 20, nf = 1, nr = 4, ns = 2,
                            initN_ars = array(1, c(na, nr, ns)),
                            mov_ymarrs,
                            M_yas = array(0.3, c(ny, na, ns)),
                            SRR_s = rep("BH", ns),
                            sralpha_s = rep(1e16, ns),
                            srbeta_s = rep(1e16, ns),
                            mat_yas = array(1, c(ny, na, ns)), fec_yas = array(1, c(ny, na, ns)),
                            Rdev_ys = matrix(1, ny, ns),
                            m_spawn = 1, m_advanceage = 1, delta_s = rep(0, ns), natal_rs = matrix(1, nr, ns),
                            recdist_rs = matrix(1/nr, nr, ns),
                            fwt_ymafs = array(1, c(ny, nm, na, nf, ns)),
                            q_fs = matrix(1, nf, ns), sel_ymafs = array(1, c(ny, nm, na, nf, ns)),
                            condition = c("F", "catch"),
                            F_ymfr = array(0, c(ny, nm, nf, nr)),
                            Cobs_ymfr = array(1e-8, c(ny, nm, nf, nr)),
                            Fmax = 2, nitF = 5L) {

  # Dispatch method for AD variables ----
  `[<-` <- RTMB::ADoverload("[<-")
  `c` <- RTMB::ADoverload("c")

  condition <- match.arg(condition)

  # Population array of lists ----
  delta_m <- 1/nm

  N_ym_ars <- array(list(), c(ny + 1, nm))
  Npsp_y_ars <- Nsp_y_ars <- array(list(), ny)
  F_ym_ars <-
    Z_ym_ars <- array(list(), c(ny, nm))

  S_yrs <- array(NA_real_, c(ny, nr, ns))
  R_ys <- array(NA_real_, c(ny, ns))

  if (missing(mov_ymarrs)) {
    mov_ymarrs <- diag(nr) |> array(c(nr, nr, ny, nm, na, ns)) |> aperm(c(3:5, 1:2, 6))
  }

  # Fishery arrays ----
  if (condition == "catch") {
    F_ym_fr <- array(list(), c(ny, nm))
  }
  F_ym_afrs <-
    CN_ym_afrs <- array(list(), c(ny, nm))
  CB_ym_frs <-
    VB_ym_frs <- array(list(), c(ny, nm))

  # Penalty for exceeding Fmax (if conditioning on catch)
  penalty <- 0

  N_ym_ars[[1, 1]] <- initN_ars

  # Loops over years and seasons ----
  for(y in 1:ny) {
    for(m in 1:nm) {
      ## This season's mortality ----
      if (condition == "catch") {
        Fsearch <- calc_F(
          Cobs = Cobs_ymfr[y, m, , ], N = N_ym_ars[[y, m]], sel = sel_ymafs[y, m, , , ],
          wt = fwt_ymafs[y, m, , , ], M = M_yas[y, , ], q_fs = q_fs, delta = delta_m,
          na = na, nr = nr, nf = nf, ns = ns, Fmax = Fmax, nitF = nitF, trans = "log"
        )
        penalty <- penalty + Fsearch[["penalty"]] # Report penalty for exceeding Fmax

        F_ym_afrs[[y, m]] <- Fsearch[["F_afrs"]]
        F_ym_ars[[y, m]] <- Fsearch[["F_ars"]]
        Z_ym_ars[[y, m]] <- Fsearch[["Z_ars"]]
        F_ym_fr[[y, m]] <- Fsearch[["F_index"]]

        ## This season's fishery catch, vulnerable biomass, and total biomass ----
        CN_ym_afrs[[y, m]] <- Fsearch[["CN_afrs"]]
        CB_ym_frs[[y, m]] <- Fsearch[["CB_frs"]]
        VB_ym_frs[[y, m]] <- apply(Fsearch[["VB_afrs"]], 2:4, sum)

      } else {

        ind_afrs <- as.matrix(expand.grid(y = y, m = m, a = 1:na, f = 1:nf, r = 1:nr, s = 1:ns))
        fs_afrs <- ind_afrs[, c("f", "s")]
        ymfr_afrs <- ind_afrs[, c("y", "m", "f", "r")]
        ymafs_afrs <- ind_afrs[, c("y", "m", "a", "f", "s")]
        ars_afrs <- ind_afrs[, c("a", "r", "s")]

        F_ym_afrs[[y, m]] <- array(q_fs[fs_afrs] * F_ymfr[ymfr_afrs] * sel_ymafs[ymafs_afrs], c(na, nf, nr, ns))
        F_ym_ars[[y, m]] <- array(0, c(na, nr, ns))
        for (f in 1:nf) {
          F_ym_ars[[y, m]] <- F_ym_ars[[y, m]] + array(F_ym_afrs[[y, m]][, f, , ], c(na, nr, ns))
        }

        ind_ars <- as.matrix(expand.grid(y = y, m = m, a = 1:na, r = 1:nr, s = 1:ns))
        yas_ars <- ind_ars[, c("y", "a", "s")]

        Z_ym_ars[[y, m]] <- F_ym_ars[[y, m]] + array(delta_m * M_yas[yas_ars], c(na, nr, ns))
        CN_ym_afrs[[y, m]] <- array(
          F_ym_afrs[[y, m]] * (1 - exp(-Z_ym_ars[[y,m]][ars_afrs])) * N_ym_ars[[y, m]][ars_afrs] / Z_ym_ars[[y, m]][ars_afrs],
          c(na, nf, nr, ns)
        )

        CB_afrs <- array(CN_ym_afrs[[y, m]] * fwt_ymafs[ymafs_afrs], c(na, nf, nr, ns))
        CB_ym_frs[[y, m]] <- apply(CB_afrs, 2:4, sum)

        VB_afrs <- array(
          sel_ymafs[ymafs_afrs] * fwt_ymafs[ymafs_afrs] * N_ym_ars[[y, m]][ars_afrs],
          c(na, nf, nr, ns)
        )
        VB_ym_frs[[y, m]] <- apply(VB_afrs, 2:4, sum)
      }

      ## This year's spawning and recruitment ----
      if (m == m_spawn) {
        Npsp_y_ars[[y]] <- sapply2(1:ns, function(s) {
          sapply(1:nr, function(r) {
            N_ym_ars[[y, m]][, r, s] * exp(-delta_s[s] * Z_ym_ars[[y, m]][, r, s]) * mat_yas[y, , s]
          })
        })
        Nsp_y_ars[[y]] <- sapply2(1:ns, function(s) {
          sapply(1:nr, function(r) natal_rs[r, s] * Npsp_y_ars[[y]][, r, s])
        })
        S_yrs[y, , ] <- sapply(1:ns, function(s) {
          sapply(1:nr, function(r) sum(Nsp_y_ars[[y]][, r, s] * fec_yas[y, , s]))
        })

        R_ys[y, ] <- Rdev_ys[y, ] * sapply(1:ns, function(s) {
          calc_recruitment(sum(S_yrs[y, , s]), SRR = SRR_s[s], a = sralpha_s[s], b = srbeta_s[s])
        })

        # Enter recruitment into age structure (age classes have already advanced) ----
        N_ym_ars[[y, m]][1, , ] <- sapply(1:ns, function(s) recdist_rs[, s] * R_ys[y, s])
      }

      ## Next season's abundance and total biomass ----
      ynext <- ifelse(m == nm, y+1, y)
      mnext <- ifelse(m == nm, 1, m+1)

      N_ym_ars[[ynext, mnext]] <- calc_nextN(
        N = N_ym_ars[[y, m]], surv = exp(-Z_ym_ars[[y, m]]),
        na = na, nr = nr, ns = ns,
        advance_age = mnext == m_advanceage,
        mov = mov_ymarrs[y, m, , , , ]
      )
    }
  }

  # Fill with zeros for year ny + 1
  if (nm > 1) {
    for (m in 2:nm) N_ym_ars[[ny+1, m]] <- array(0, c(na, nr, ns))
  }

  # Output
  N_ymars <- array(0, c(ny + 1, nm, na, nr, ns))
  F_ymars <-
    Z_ymars <- array(NA_real_, c(ny, nm, na, nr, ns))
  Npsp_yars <-
    Nsp_yars <- array(NA_real_, c(ny, na, nr, ns))

  F_ymafrs <-
    CN_ymafrs <- array(NA_real_, c(ny, nm, na, nf, nr, ns))
  CB_ymfrs <-
    VB_ymfrs <- array(NA_real_, c(ny, nm, nf, nr, ns))

  N_ymars[] <- do.call(c, N_ym_ars) |> array(c(na, nr, ns, ny+1, nm)) |> aperm(c(4:5, 1:3))
  F_ymars[] <- do.call(c, F_ym_ars) |> array(c(na, nr, ns, ny, nm)) |> aperm(c(4:5, 1:3))
  Z_ymars[] <- do.call(c, Z_ym_ars) |> array(c(na, nr, ns, ny, nm)) |> aperm(c(4:5, 1:3))
  F_ymafrs[] <- do.call(c, F_ym_afrs) |> array(c(na, nf, nr, ns, ny, nm)) |> aperm(c(5:6, 1:4))
  CN_ymafrs[] <- do.call(c, CN_ym_afrs) |> array(c(na, nf, nr, ns, ny, nm)) |> aperm(c(5:6, 1:4))
  CB_ymfrs[] <- do.call(c, CB_ym_frs) |> array(c(nf, nr, ns, ny, nm)) |> aperm(c(4:5, 1:3))
  VB_ymfrs[] <- do.call(c, VB_ym_frs) |> array(c(nf, nr, ns, ny, nm)) |> aperm(c(4:5, 1:3))
  Nsp_yars[] <- do.call(c, Nsp_y_ars) |> array(c(na, nr, ns, ny)) |> aperm(c(4, 1:3))
  Npsp_yars[] <- do.call(c, Npsp_y_ars) |> array(c(na, nr, ns, ny)) |> aperm(c(4, 1:3))

  if (condition == "catch") {
    F_ymfr <- do.call(c, F_ym_fr) |> array(c(nf, nr, ny, nm)) |> aperm(c(3:4, 1:2))
  } else {
    penalty <- penalty + sum(posfun(Fmax, F_ymfr))
  }

  out <- list(
    N_ymars = N_ymars,
    F_ymars = F_ymars,
    F_ymfr = F_ymfr,
    Z_ymars = Z_ymars,
    F_ymafrs = F_ymafrs,
    CN_ymafrs = CN_ymafrs,
    CB_ymfrs = CB_ymfrs,
    VB_ymfrs = VB_ymfrs,
    Nsp_yars = Nsp_yars,
    Npsp_yars = Npsp_yars,
    S_yrs = S_yrs,
    R_ys = R_ys,
    penalty = penalty
  )

  return(out)
}

#' Initial population projection
#'
#' Project a population forward in time with constant parameters (biology and F) with [calc_population()],
#' an alternative to [calc_phi_project()] to establish initial age structure.
#'
#' @inheritParams calc_phi_project
#' @inheritParams calc_population
#' @param C_mfr Equilibrium catch. Matrix `[m, f, r]`
#' @param F_mfr Equilibrium fishing mortality (per season). Matrix `[m, f, r]`
#' @param sel_mafs Selectivity by season, age, fleet, stock. Array `[m, a, f, s]`
#' @param fwt_mafs Fishery weight array by season, age, fleet, stock. Array `[m, a, r, r]`. Can be used
#' calculate yield per recruit.
#' @param mov_marrs Movement array `[m, a, r, r, s]`. If missing, uses a diagonal matrix (no movement among areas).
#' @param M_as Natural mortality. Matrix `[a, s]`
#' @param mat_as Maturity at age. Matrix `[a, s]`
#' @param fec_as Fecundity at age. Matrix `[a, s]`
#' @return A named list returned by [calc_population()].
#' @export
calc_init_population <- function(ny = 10, nm = 4, na = 20, nf = 1, nr = 4, ns = 1,
                                 initN_ars = array(1, c(na, nr, ns)),
                                 condition = c("catch", "F"),
                                 C_mfr = array(0, c(nm, nf, nr)),
                                 F_mfr = array(0, c(nm, nf, nr)),
                                 sel_mafs = array(1, c(nm, na, nf, ns)),
                                 fwt_mafs = array(1, c(nm, na, nf, ns)),
                                 q_fs = matrix(1, nf, ns),
                                 M_as, mov_marrs,
                                 mat_as, fec_as,
                                 SRR_s, sralpha_s, srbeta_s,
                                 m_spawn = 1, m_advanceage = 1,
                                 delta_s = rep(0, ns),
                                 natal_rs = matrix(1, nr, ns),
                                 recdist_rs = matrix(1/nr, nr, ns),
                                 Fmax, nitF) {

  condition <- match.arg(condition)

  delta_m <- 1/nm

  if (missing(mov_marrs)) {
    mov_ymarrs <- diag(nr) |> array(c(nr, nr, ny, nm, na, ns)) |> aperm(c(3:5, 1:2, 6))
  } else {
    mov_marrs <- array(mov_marrs, c(nm, na, nr, nr, ns))
    mov_ymarrs <- array(mov_marrs, c(nm, na, nr, nr, ns, ny)) |> aperm(c(6, 1:5))
  }
  M_yas <- array(M_as, c(na, ns, ny)) |> aperm(c(3, 1, 2))

  #SRR <- rep("BH", ns)
  #sralpha <- srbeta <- rep(1e16, ns)

  mat_yas <- array(mat_as, c(na, ns, ny)) |> aperm(c(3, 1, 2))
  fec_yas <- array(fec_as, c(na, ns, ny)) |> aperm(c(3, 1, 2))
  sel_ymafs <- array(sel_mafs, c(nm, na, nf, ns, ny)) |> aperm(c(5, 1:4))
  fwt_ymafs <- array(fwt_mafs, c(nm, na, nf, ns, ny)) |> aperm(c(5, 1:4))

  C_ymfr <- array(C_mfr, c(nm, nf, nr, ny)) |> aperm(c(4, 1:3))
  F_ymfr <- array(F_mfr, c(nm, nf, nr, ny)) |> aperm(c(4, 1:3))

  pop_phi <- calc_population(
    ny, nm, na, nf, nr, ns, initN_ars = initN_ars,
    mov_ymarrs, M_yas,
    SRR_s = SRR_s, sralpha_s = sralpha_s, srbeta_s = srbeta_s,
    mat_yas, fec_yas, Rdev_ys = matrix(1, ny, ns),
    m_spawn, m_advanceage, delta_s, natal_rs, recdist_rs,
    fwt_ymafs = fwt_ymafs, q_fs,
    sel_ymafs = sel_ymafs,
    condition = condition,
    F_ymfr = F_ymfr,
    Cobs_ymfr = C_ymfr,
    Fmax = Fmax,
    nitF = nitF
  )
  return(pop_phi)
}

#' Equilibrium spawners per recruit by projection
#'
#' Project a population forward in time using [calc_population()] with constant recruitment and
#' seasonal dynamics (growth, movement-by-season) to obtain per recruit parameters. Note that the fishing
#' mortality among fleets and stocks remain linked by matrix `q_fs`.
#'
#' @inheritParams calc_population
#'
#' @param F_mfr Equilibrium fishing mortality (per season). Matrix `[m, f, r]`
#' @param sel_mafs Selectivity by season, age, fleet, stock. Array `[m, a, f, s]`
#' @param fwt_mafs Fishery weight array by season, age, fleet, stock. Array `[m, a, r, r]`. Can be used
#' calculate yield per recruit.
#' @param mov_marrs Movement array `[m, a, r, r, s]`. If missing, uses a diagonal matrix (no movement among areas).
#' @param M_as Natural mortality. Matrix `[a, s]`
#' @param mat_as Maturity at age. Matrix `[a, s]`
#' @param fec_as Fecundity at age. Matrix `[a, s]`
#' @details
#' The initial population vector will be the survival at age evenly divided by the number of regions `nr`.
#' @return A named list returned by [calc_population()].
#' @seealso [calc_phi_simple()] [calc_init_population()]
#' @export
calc_phi_project <- function(ny, nm, na, nf = 1, nr, ns = 1,
                             F_mfr = array(0, c(nm, nf, nr)),
                             sel_mafs = array(1, c(nm, na, nf, ns)),
                             fwt_mafs = array(1, c(nm, na, nf, ns)),
                             q_fs = matrix(1, nf, ns),
                             M_as, mov_marrs,
                             mat_as, fec_as,
                             m_spawn = 1, m_advanceage = 1,
                             delta_s = rep(0, ns),
                             natal_rs = matrix(1, nr, ns),
                             recdist_rs = matrix(1/nr, nr, ns)) {
  delta_m <- 1/nm

  if (missing(mov_marrs)) {
    mov_ymarrs <- diag(nr) |> array(c(nr, nr, ny, nm, na, ns)) |> aperm(c(3:5, 1:2, 6))
  } else {
    mov_marrs <- array(mov_marrs, c(nm, na, nr, nr, ns))
    mov_ymarrs <- array(mov_marrs, c(nm, na, nr, nr, ns, ny)) |> aperm(c(6, 1:5))
  }
  M_yas <- array(M_as, c(na, ns, ny)) |> aperm(c(3, 1, 2))

  SRR <- rep("BH", ns)
  sralpha <- srbeta <- rep(1e16, ns)

  mat_yas <- array(mat_as, c(na, ns, ny)) |> aperm(c(3, 1, 2))
  fec_yas <- array(fec_as, c(na, ns, ny)) |> aperm(c(3, 1, 2))
  sel_ymafs <- array(sel_mafs, c(nm, na, nf, ns, ny)) |> aperm(c(5, 1:4))
  fwt_ymafs <- array(fwt_mafs, c(nm, na, nf, ns, ny)) |> aperm(c(5, 1:4))
  F_ymfr <- array(F_mfr, c(nm, nf, nr, ny)) |> aperm(c(4, 1:3))

  initNPR_ars <- sapply2(1:ns, function(s) {
    NPR_ar <- sapply(1:nr, function(r) {
      F_af <- lapply(1:nf, function(f) sel_ymafs[1, 1, , f, s] * q_fs[f, s] * F_ymfr[1, 1, f, r])
      Z_a <- M_yas[1, , s] + Reduce("+", F_af)
      calc_NPR(Z_a)
    })
    return(NPR_ar/nr)
  })

  pop_phi <- calc_population(
    ny, nm, na, nf, nr, ns, initN_ars = initNPR_ars,
    mov_ymarrs, M_yas,
    SRR_s = SRR, sralpha_s = sralpha, srbeta_s = srbeta,
    mat_yas, fec_yas, Rdev_ys = matrix(1, ny, ns),
    m_spawn, m_advanceage, delta_s, natal_rs, recdist_rs,
    fwt_ymafs = fwt_ymafs, q_fs,
    sel_ymafs = sel_ymafs,
    condition = "F",
    F_ymfr = F_ymfr
  )
  return(pop_phi)
}

#' Simple spawners per recruit calculation
#'
#' Calculate spawners per recruit by individual stock. Appropriate for a population model with a single
#' spatial area and an annual time steps, i.e. single season.
#'
#' @inheritParams calc_phi_project
#' @param Z Total mortality at age
#' @param mat_a Maturity at age. Vector
#' @param fec_a Fecundity at age. Vector
#' @param delta Fraction of season that elapses when spawning occurs, e.g., midseason spawning when `delta = 0.5`.
#' @return [calc_phi_simple()] returns a numeric for the spawning biomass per recruit.
#' @seealso [calc_phi_project()]
#' @export
calc_phi_simple <- function(Z, fec_a, mat_a, delta = 0) {
  NPR <- calc_NPR(Z)
  sum(NPR * exp(-Z * delta) * fec_a * mat_a)
}

#' @rdname calc_phi_simple
#' @return [calc_NPR()] returns a numeric, numbers per recruit at age
#' @param plusgroup Logical, whether the largest age class is a plusgroup accumulator age
#' @export
calc_NPR <- function(Z, na = length(Z), plusgroup = TRUE) {
  surv <- exp(-Z)
  is_ad <- inherits(Z, "advector")
  if (is_ad) {
    `[<-` <- RTMB::ADoverload("[<-")
    NPR <- advector(numeric(na))
  } else {
    NPR <- numeric(na)
  }
  NPR[1] <- 1
  for(a in 2:na) NPR[a] <- NPR[a-1] * surv[a-1]
  if (plusgroup) NPR[na] <- NPR[na]/(1 - surv[na])
  return(NPR)
}

#' @name simulate
#' @aliases simulate,MSAassess-method simulate.MSAassess
#'
#' @title Simulate data
#'
#' @description Simulate data observations from fitted MSA model.
#'
#' @param object [MSAassess-class] object returned by [fit_MSA()]
#' @param nsim Integer, number of simulations
#' @param seed Random number generator seed
#' @param ... Not used
#' @return A list of `nsim` length with data observations
#' @importFrom stats simulate
#' @export
setMethod("simulate", signature(object = "MSAassess"),
          function(object, nsim = 1, seed = NULL, ...) {
  if (!is.null(seed)) set.seed(seed)
  sims <- lapply(1:nsim, function(...) object@obj$simulate(object@obj$env$last.par.best))

  var_data <- names(object@obj$env$obs)
  out <- lapply(sims, function(x) x[var_data])
  return(out)
})

#' Check dimensions and inputs in MSAdata object
#'
#' Ensures that data inputs are of proper dimension. Whenever possible, default values are added to missing items.
#'
#' @param MSAdata S4 object containing data inputs. See [MSAdata-class]
#' @param silent Logical, whether or not to report default values to the console
#' @returns An updated [MSAdata-class] object.
#' @seealso [MSAdata-class]
#' @export
check_data <- function(MSAdata, silent = FALSE) {

  # Check S4 objects
  MSAdata@Dmodel <- update_S4(MSAdata@Dmodel)
  MSAdata@Dstock <- update_S4(MSAdata@Dstock)
  MSAdata@Dfishery <- update_S4(MSAdata@Dfishery)
  MSAdata@Dsurvey <- update_S4(MSAdata@Dsurvey)

  MSAdata@DCKMR <- update_S4(MSAdata@DCKMR)
  MSAdata@Dtag <- update_S4(MSAdata@Dtag)
  MSAdata@Dlabel <- update_S4(MSAdata@Dlabel)

  # Check entries
  MSAdata@Dmodel <- check_Dmodel(MSAdata@Dmodel, MSAdata@Dstock, MSAdata@Dfishery@nf, silent)
  MSAdata@Dstock <- check_Dstock(MSAdata@Dstock, MSAdata@Dmodel, silent)
  MSAdata@Dfishery <- check_Dfishery(MSAdata@Dfishery, MSAdata@Dstock, MSAdata@Dmodel, silent)
  MSAdata@Dsurvey <- check_Dsurvey(MSAdata@Dsurvey, MSAdata@Dmodel, silent)

  MSAdata@DCKMR <- check_DCKMR(MSAdata@DCKMR, MSAdata@Dmodel, silent)
  MSAdata@Dtag <- check_Dtag(MSAdata@Dtag, MSAdata@Dmodel, silent)
  MSAdata@Dlabel <- check_Dlabel(MSAdata@Dlabel, MSAdata@Dmodel, MSAdata@Dfishery, MSAdata@Dsurvey, silent)

  return(MSAdata)
}



#' @importFrom methods .slotNames .hasSlot slot
update_S4 <- function(x) {
  snames <- .slotNames(class(x))
  slot_check <- sapply(snames, function(i) .hasSlot(x, i))

  if (any(!slot_check)) {
    xnew <- new(class(x))
    for (i in snames) {
      if (slot_check[i]) slot(xnew, i) <- slot(x, i)
    }
    return(xnew)
  } else {
    return(x)
  }
}


check_Dmodel <- function(Dmodel, Dstock, nf, silent = FALSE) {
  ch <- as.character(substitute(Dmodel))
  if (length(ch) > 1) ch <- "Dmodel"

  if (!length(Dmodel@ny)) stop("Need ", ch, "@ny")
  if (!length(Dmodel@nyret)) Dmodel@nyret <- 0L
  y_max <- Dmodel@ny - Dmodel@nyret

  if (!length(Dmodel@nm)) {
    if (!silent) message("Setting ", ch, "@nm to 1")
    Dmodel@nm <- 1L
  }
  if (!length(Dmodel@na)) stop("Need ", ch, "@na")
  if (!length(Dmodel@nl)) {
    if (!silent) message_info("No length bins in the model.")
    Dmodel@nl <- 0L
  }
  if (!length(Dmodel@nr)) {
    if (!silent) message("Creating one region model.")
    Dmodel@nr <- 1L
  }
  if (!length(Dmodel@ns)) {
    if (!silent) message("Creating one stock model.")
    Dmodel@ns <- 1L
  }
  if (Dmodel@nl > 0) {
    if (length(Dmodel@lbin) != Dmodel@nl) stop("Need nl vector for ", ch, "@lbin")
    if (length(Dmodel@lmid) != Dmodel@nl) stop("Need nl vector for ", ch, "@lmid")
  }
  if (!length(Dmodel@Fmax)) {
    if (!silent) message("Setting ", ch, "@Fmax to 3 (per season)")
    Dmodel@Fmax <- 3
  }
  if (!length(Dmodel@y_phi)) {
    if (!silent) message("Setting ", ch, "@y_phi to year 1")
    Dmodel@y_phi <- 1L
  }
  if (!length(Dmodel@scale_s)) {
    if (!silent) message("Setting ", ch, "@scale_s = 1 for all stocks")
    Dmodel@scale_s <- rep(1, Dmodel@ns)
  } else if (length(Dmodel@scale_s) != Dmodel@ns) {
    stop(ch, "@scale_s needs to be length ", Dmodel@ns)
  }
  if (!length(Dmodel@nyinit)) {
    if (Dmodel@nm == 1 && Dmodel@nr == 1) {
      Dmodel@nyinit <- 1
    } else {
      Dmodel@nyinit <- 2 * Dmodel@na
      if (!silent) message("Setting ", ch, "@nyinit = ", Dmodel@nyinit)
    }
  }
  if (!length(Dmodel@condition)) {
    Dmodel@condition <- "F"
    if (!silent) message("Setting ", ch, "@condition = F")
  }
  Dmodel@condition <- match.arg(Dmodel@condition, choices = c("F", "catch"))
  if (Dmodel@condition == "catch" && !length(Dmodel@nitF)) {
    if (!silent) message("Setting ", ch, "@nitF to 5")
    Dmodel@nitF <- 5
  }

  if (Dmodel@condition == "F") {
    if (!length(Dmodel@y_Fmult_f)) {
      Dmodel@y_Fmult_f <- rep(round(0.5 * y_max), nf)
      if (!silent) message("Setting ", ch, "@y_Fmult_f = ", paste(Dmodel@y_Fmult_f, collapse = ", "))
    } else if (Dmodel@y_Fmult_f > y_max) {
      stop("Reduce ", ch, "@y_Fmult_f so that is less than ny - nyret")
    }
    if (!length(Dmodel@m_Fmult_f)) {
      Dmodel@m_Fmult_f <- rep(1L, nf)
      if (!silent && Dmodel@nm > 1) message("Setting ", ch, "@m_Fmult_f = ", paste(Dmodel@m_Fmult_f, collapse = ", "))
    }
    if (!length(Dmodel@r_Fmult_f)) {
      Dmodel@r_Fmult_f <- rep(1L, nf)
      if (!silent && Dmodel@nr > 1) message("Setting ", ch, "@r_Fmult_f = ", paste(Dmodel@r_Fmult_f, collapse = ", "))
    }
  }

  if (!length(Dmodel@pbc_rdev_ys)) {
    Dmodel@pbc_rdev_ys <- matrix(1, Dmodel@ny, Dmodel@ns)
  } else if (length(Dmodel@pbc_rdev_ys) == 1) {
    Dmodel@pbc_rdev_ys <- matrix(Dmodel@pbc_rdev_ys, Dmodel@ny, Dmodel@ns)
  } else {
    dim_pbc <- dim(Dmodel@pbc_rdev_ys)
    if (any(dim_pbc != c(Dmodel@ny, Dmodel@ns))) {
      stop("dim(", ch, "@pbc_rdev_ys) should be ", c(Dmodel@ny, Dmodel@ns) |> paste(collapse = ", "))
    }
  }

  na_init <- ifelse(Dmodel@nm > 1 && Dstock@m_advanceage > 1, Dmodel@na, Dmodel@na-1)
  if (!length(Dmodel@pbc_initrdev_as)) {
    Dmodel@pbc_initrdev_as <- matrix(1, na_init, Dmodel@ns)
  } else if (length(Dmodel@pbc_initrdev_as) == 1) {
    Dmodel@pbc_initrdev_as <- matrix(Dmodel@pbc_initrdev_as, na_init, Dmodel@ns)
  } else {
    dim_pbc <- dim(Dmodel@pbc_initrdev_as)
    if (any(dim_pbc != c(na_init, Dmodel@ns))) {
      stop("dim(", ch, "@pbc_initrdev_as) should be ", c(na_init, Dmodel@ns) |> paste(collapse = ", "))
    }
  }

  return(Dmodel)
}

check_Dstock <- function(Dstock, Dmodel, silent = FALSE) {
  ny <- Dmodel@ny
  nm <- Dmodel@nm
  na <- Dmodel@na
  nl <- Dmodel@nl
  nr <- Dmodel@nr
  ns <- Dmodel@ns

  ch <- as.character(substitute(Dstock))
  if (length(ch) > 1) ch <- "Dstock"

  if (!length(Dstock@m_spawn) || nm == 1L) {
    if (!silent && nm > 1) message("Setting ", ch, "@m_spawn = 1")
    Dstock@m_spawn <- 1L
  } else if (Dstock@m_spawn > nm) {
    stop(ch, "@m_spawn cannot be greater than Dmodel@nm")
  }
  if (!length(Dstock@m_advanceage) || nm == 1L) {
    if (!silent && Dmodel@nm > 1) message("Setting ", ch, "@m_advanceage = 1")
    Dstock@m_advanceage <- 1L
  } else if (Dstock@m_advanceage > nm) {
    stop(ch, "@m_advanceage cannot be greater than Dmodel@nm")
  }

  if (nl > 0) {
    if (length(Dstock@LAK_ymals)) {
      dim_LAK <- dim(Dstock@LAK_ymals) == c(ny, nm, na, nl, ns)
      if (!all(dim_LAK)) stop("dim(", ch, "@LAK_ymals) needs to be: ", c(ny, nm, na, nl, ns) |> paste(collapse = ", "))
    } else {

      dim_len <- dim(Dstock@len_ymas) == c(ny, nm, na, ns)
      if (!all(dim_len)) stop("dim(", ch, "@len_ymas) needs to be: ", c(ny, nm, na, ns) |> paste(collapse = ", "))

      dim_sdlen <- dim(Dstock@sdlen_ymas) == c(ny, nm, na, ns)
      if (!all(dim_sdlen)) stop("dim(", ch, "@sdlen_ymas) needs to be: ", c(ny, nm, na, ns) |> paste(collapse = ", "))

      if (!silent) message("Calculating ", ch, "@LAK_ymals array")
      Dstock@LAK_ymals <- sapply2(1:ns, function(s) {
        sapply2(1:nm, function(m) {
          sapply2(1:ny, function(y) calc_LAK(Dstock@len_ymas[y, m, , s], Dstock@sdlen_ymas[y, m, , s], Dmodel@lbin))
        })
      }) |> aperm(c(3, 4, 1, 2, 5))
    }
  }

  if (!length(Dstock@mat_yas)) {
    dim_mat <- dim(Dstock@mat_yas) == c(ny, na, ns)
    if (!all(dim_mat)) stop("dim(", ch, "@mat_yas) needs to be: ", c(ny, na, ns) |> paste(collapse = ", "))
  }

  dim_swt <- dim(Dstock@swt_ymas) == c(ny, nm, na, ns)
  if (!all(dim_swt)) stop("dim(", ch, "@swt_ymas) needs to be: ", c(ny, nm, na, ns) |> paste(collapse = ", "))

  if (!length(Dstock@fec_yas)) {
    if (!silent) message("Setting fecundity to stock weight at age at season of spawning")
    Dstock@fec_yas <- array(Dstock@swt_ymas[, Dstock@m_spawn, , ], c(ny, na, ns))
  } else {
    dim_fec <- dim(Dstock@fec_yas) == c(ny, na, ns)
    if (!all(dim_fec)) stop("dim(", ch, "@fec_yas) needs to be: ", c(ny, na, ns) |> paste(collapse = ", "))
  }

  if (!length(Dstock@M_yas)) {
    dim_M <- dim(Dstock@M_yas) == c(ny, na, ns)
    if (!all(dim_M)) stop("dim(", ch, "@M_yas) needs to be: ", c(ny, na, ns) |> paste(collapse = ", "))
  }

  if (length(Dstock@SRR_s) != ns) stop("SRR_s needs to be length ", ns)
  if (!length(Dstock@delta_s)) {
    Dstock@delta_s <- rep(0, ns)
  } else if (length(Dstock@delta_s) != ns) {
    stop(ch, "@delta_s needs to be length ", ns)
  }
  if (!length(Dstock@presence_rs)) {
    Dstock@presence_rs <- matrix(TRUE, nr, ns)
  } else if (any(dim(Dstock@presence_rs) != c(nr, ns))) {
    stop("dim(", ch, "@presence_rs) needs to be: ", c(nr, ns) |> paste(collapse = ", "))
  }
  if (!length(Dstock@natal_rs)) {
    Dstock@natal_rs <- matrix(1, nr, ns)
  } else if (any(dim(Dstock@natal_rs) != c(nr, ns))) {
    stop("dim(", ch, "@natal_rs) needs to be: ", c(nr, ns) |> paste(collapse = ", "))
  }

  return(Dstock)
}

check_Dfishery <- function(Dfishery, Dstock, Dmodel, silent = FALSE) {
  ny <- Dmodel@ny
  nm <- Dmodel@nm
  na <- Dmodel@na
  nl <- Dmodel@nl
  nr <- Dmodel@nr
  ns <- Dmodel@ns

  ch <- as.character(substitute(Dfishery))
  if (length(ch) > 1) ch <- "Dfishery"

  if (!length(Dfishery@nf)) stop("Need ", ch, "@nf")
  nf <- Dfishery@nf

  dim_Cobs <- dim(Dfishery@Cobs_ymfr) == c(ny, nm, nf, nr)
  if (!all(dim_Cobs)) stop("dim(", ch, "@Cobs_ymfr) needs to be: ", c(ny, nm, nf, nr) |> paste(collapse = ", "))

  if (Dmodel@condition == "F") {
    if (!length(Dfishery@Csd_ymfr)) {
      Dfishery@Csd_ymfr <- array(0.01, c(ny, nm, nf, nr))
    } else {
      dim_Csd <- dim(Dfishery@Csd_ymfr) == c(ny, nm, nf, nr)
      if (!all(dim_Csd)) stop("dim(", ch, "@Csd_ymfr) needs to be: ", c(ny, nm, nf, nr) |> paste(collapse = ", "))
    }
  }

  if (!length(Dfishery@fwt_ymafs)) {
    if (!silent) message("Setting fishery weight at age to stock weight at age")
    Dfishery@fwt_ymafs <- sapply2(1:ns, function(s) {
      sapply2(1:nf, function(f) array(Dstock@swt_ymas[, , , s], c(ny, nm, na)))
    })
  } else {
    dim_fwt <- dim(Dfishery@fwt_ymafs) == c(ny, nm, na, nf, ns)
    if (!all(dim_fwt)) stop("dim(", ch, "@fwt_ymafs) needs to be: ", c(ny, nm, na, nf, ns) |> paste(collapse = ", "))
  }

  if (!length(Dfishery@lambdaCobs_f)) {
    Dfishery@lambdaCobs_f <- rep(1, nf)
  } else if (length(Dfishery@lambdaCobs_f) == 1) {
    Dfishery@lambdaCobs_f <- rep(Dfishery@lambdaCobs_f, nf)
  } else {
    stop("Vector ", ch, "@lambdaCobs_f needs to be length ", nf)
  }

  if (length(Dfishery@CAAobs_ymafr) || length(Dfishery@CALobs_ymlfr)) {
    if (length(Dfishery@fcomp_like)) {
      Dfishery@fcomp_like <- match.arg(Dfishery@fcomp_like, choices = eval(formals(like_comp)$type))
    } else {
      if (!silent) message("Setting ", ch, "@fcomp_like = \"multinomial\"")
      Dfishery@fcomp_like <- "multinomial"
    }
  }

  if (length(Dfishery@CAAobs_ymafr)) {
    dim_CAA <- dim(Dfishery@CAAobs_ymafr) == c(ny, nm, na, nf, nr)
    if (!all(dim_CAA)) stop("dim(", ch, "@CAAobs_ymafr) needs to be: ", c(ny, nm, na, nf, nr) |> paste(collapse = ", "))

    if (!length(Dfishery@CAAN_ymfr)) {
      if (Dfishery@fcomp_like %in% c("multinomial", "ddirmult1", "ddirmult2") && !silent) {
        message("Setting ", ch, "@CAAN_ymfr from ", ch, "@CAAobs_ymafr")
      }
      Dfishery@CAAN_ymfr <- apply(Dfishery@CAAobs_ymafr, c(1, 2, 4, 5), sum)
    } else {
      dim_CAAN <- dim(Dfishery@CAAN_ymfr) == c(ny, nm, nf, nr)
      if (!all(dim_CAAN)) stop("dim(", ch, "@CAAN_ymfr) needs to be: ", c(ny, nm, nf, nr) |> paste(collapse = ", "))
    }

    if (!length(Dfishery@CAAtheta_f)) {
      if (grepl("ddirmult", Dfishery@fcomp_like) && !silent) message("Setting ", ch, "@CAAtheta_f to 1 for all fleets")
      Dfishery@CAAtheta_f <- rep(1, nf)
    } else if (length(Dfishery@CAAtheta_f) == 1) {
      Dfishery@CAAtheta_f <- rep(Dfishery@CAAtheta_f, nf)
    } else if (length(Dfishery@CAAtheta_f) != nf) {
      stop("Vector ", ch, "@CAAtheta_f needs to be length ", nf)
    }

    if (!length(Dfishery@lambdaCAA_f)) {
      Dfishery@lambdaCAA_f <- rep(1, nf)
    } else if (length(Dfishery@lambdaCAA_f) == 1) {
      Dfishery@lambdaCAA_f <- rep(Dfishery@lambdaCAA_f, nf)
    } else {
      stop("Vector ", ch, "@lambdaCAA_f needs to be length ", nf)
    }
  }

  if (length(Dfishery@CALobs_ymlfr)) {
    if (!nl) stop("Fishery length composition detected but no length bins found in Dmodel@nl")

    dim_CAL <- dim(Dfishery@CALobs_ymlfr) == c(ny, nm, nl, nf, nr)
    if (!all(dim_CAL)) stop("dim(", ch, "@CALobs_ymlfr) needs to be: ", c(ny, nm, nl, nf, nr) |> paste(collapse = ", "))

    if (!length(Dfishery@CALN_ymfr)) {
      if (Dfishery@fcomp_like %in% c("multinomial", "ddirmult1", "ddirmult2") && !silent) {
        message("Setting ", ch, "@CALN_ymfr from ", ch, "@CALobs_ymlfr")
      }
      Dfishery@CALN_ymfr <- apply(Dfishery@CALobs_ymlfr, c(1, 2, 4, 5), sum)
    } else {
      dim_CALN <- dim(Dfishery@CALN_ymfr) == c(ny, nm, nf, nr)
      if (!all(dim_CALN)) stop("dim(", ch, "@CALN_ymfr) needs to be: ", c(ny, nm, nf, nr) |> paste(collapse = ", "))
    }

    if (!length(Dfishery@CALtheta_f)) {
      if (grepl("ddirmult", Dfishery@fcomp_like) && !silent) message("Setting ", ch, "@CALtheta_f to 1 for all fleets")
      Dfishery@CALtheta_f <- rep(1, nf)
    } else if (length(Dfishery@CALtheta_f) == 1) {
      Dfishery@CALtheta_f <- rep(Dfishery@CALtheta_f, nf)
    } else if (length(Dfishery@CALtheta_f) != nf) {
      stop("Vector ", ch, "@CALtheta_f needs to be length ", nf)
    }

    if (!length(Dfishery@lambdaCAL_f)) {
      Dfishery@lambdaCAL_f <- rep(1, nf)
    } else if (length(Dfishery@lambdaCAL_f) == 1) {
      Dfishery@lambdaCAL_f <- rep(Dfishery@lambdaCAL_f, nf)
    } else {
      stop("Vector ", ch, "@lambdaCAL_f needs to be length ", nf)
    }
  }

  if (!length(Dfishery@sel_block_yf)) {
    Dfishery@sel_block_yf <- matrix(1:nf, ny, nf, byrow = TRUE)
  } else {
    dim_sb <- dim(Dfishery@sel_block_yf) == c(ny, nf)
    if (!all(dim_sb)) stop("dim(", ch, "@sel_block_yf) needs to be: ", c(ny, nf) |> paste(collapse = ", "))
  }
  nb <- max(Dfishery@sel_block_yf)
  if (length(Dfishery@sel_f) != nb) stop("Vector sel_f should be length ", nf)

  if (!length(Dfishery@Cinit_mfr)) {
    Dfishery@Cinit_mfr <- array(0, c(nm, nf, nr))
  } else {
    dim_Cinit <- dim(Dfishery@Cinit_mfr) == c(nm, nf, nr)
    if (!all(dim_Cinit)) stop("dim(", ch, "@Cinit_mfr) needs to be: ", c(nm, nf, nr) |> paste(collapse = ", "))
  }

  if (length(Dfishery@SC_ymafrs)) {
    dim_SC <- dim(Dfishery@SC_ymafrs)
    if (length(dim_SC) != 6) stop(ch, "@SC_ymafrs should be a six dimensional array")

    if (any(dim_SC[c(1, 2, 5, 6)] != c(ny, nm, nr, ns))) {
      stop("dim(", ch, "@SC_ymafrs) should be ", c(ny, nm, dim_SC[3:4], nr, ns) |> paste(collapse = ", "))
    }

    if (dim_SC[3] == na) {
      if (!length(Dfishery@SC_aa)) Dfishery@SC_aa <- diag(1, na)
    } else if (any(dim(Dfishery@SC_aa) != c(dim_SC[3], na))) {
      stop("dim(Dfishery@SC_aa) should be: ", c(Dfishery@dim_SC[3], na) |> paste(collapse = ", "))
    }

    if (dim_SC[4] == nf) {
      if (!length(Dfishery@SC_ff)) Dfishery@SC_ff <- diag(1, nf)
    } else if (any(dim(Dfishery@SC_ff) != c(dim_SC[4], nf))) {
      stop("dim(", ch, "@SC_ff) should be: ", c(dim_SC[4], nf) |> paste(collapse = ", "))
    }

    if (!length(Dfishery@SC_like)) {
      if (!silent) message("Setting ", ch, "@SC_like = \"multinomial\"")
      Dfishery@SC_like <- "multinomial"
    } else {
      Dfishery@SC_like <- match.arg(Dfishery@SC_like, choices = eval(formals(like_comp)$type))
    }

    if (!length(Dfishery@SCN_ymafr)) {
      if (Dfishery@SC_like %in% c("multinomial", "ddirmult1", "ddirmult2") && !silent) {
        message("Setting ", ch, "@SCN_ymafr from ", ch, "@SC_ymafrs")
      }
      Dfishery@SCN_ymafr <- apply(Dfishery@SC_ymafrs, 1:5, sum)
    } else {
      dim_SCN <- dim(Dfishery@SCN_ymafr) == c(ny, nm, dim_SC[3], dim_SC[4], nr)
      if (!all(dim_SCN)) stop("dim(", ch, "@SCN_ymafr) needs to be: ", c(ny, nm, dim_SC[3], dim_SC[4], nr) |> paste(collapse = ", "))
    }

    if (!length(Dfishery@SCtheta_f)) {
      if (grepl("ddirmult", Dfishery@SC_like) && !silent) message("Setting ", ch, "@SCtheta_f to 1 for all fleets")
      Dfishery@SCtheta_f <- rep(1, dim_SC[4])
    } else if (length(Dfishery@SCtheta_f) == 1) {
      Dfishery@SCtheta_f <- rep(Dfishery@SCtheta_f, dim_SC[4])
    } else if (length(Dfishery@SCtheta_f) != dim_SC[4]) {
      stop("Vector ", ch, "@SCtheta_f needs to be length ", dim_SC[4])
    }

    if (!length(Dfishery@SCstdev_ymafrs)) {
      if (grepl("log", Dfishery@SC_like) && !silent) message("Setting ", ch, "@SCstdev_ymafrs to 0.1 for all fleets")
      Dfishery@SCstdev_ymafrs <- array(0.1, c(ny, nm, dim_SC[3], dim_SC[4], nr, ns))
    } else if (length(Dfishery@SCstdev_ymafrs) == 1) {
      Dfishery@SCstdev_ymafrs <- array(Dfishery@SCstdev_ymafrs, c(ny, nm, dim_SC[3], dim_SC[4], nr, ns))
    } else if (any(dim(Dfishery@SCstdev_ymafrs) != c(ny, nm, dim_SC[3], dim_SC[4], nr, ns))) {
      stop("dim(", ch, "@SCstdev_ymafrs) needs to be: ", c(ny, nm, dim_SC[3], dim_SC[4], nr, ns) |> paste(collapse = ", "))
    }

    if (!length(Dfishery@lambdaSC_f)) {
      Dfishery@lambdaSC_f <- rep(1, dim_SC[4])
    } else if (length(Dfishery@lambdaSC_f) == 1) {
      Dfishery@lambdaSC_f <- rep(Dfishery@lambdaSC_f, dim_SC[4])
    } else if (length(Dfishery@lambdaSC_f) != dim_SC[4]) {
      stop("Vector ", ch, "@lambdaSC_f needs to be length ", dim_SC[4])
    }
  }
  return(Dfishery)
}

check_Dsurvey <- function(Dsurvey, Dmodel, silent = FALSE) {
  ny <- Dmodel@ny
  nm <- Dmodel@nm
  na <- Dmodel@na
  nl <- Dmodel@nl
  nr <- Dmodel@nr
  ns <- Dmodel@ns

  ch <- as.character(substitute(Dsurvey))
  if (length(ch) > 1) ch <- "Dsurvey"

  if (!length(Dsurvey@ni)) {
    if (length(Dsurvey@Iobs_ymi) || length(Dsurvey@IAAobs_ymai) || length(Dsurvey@IALobs_ymli)) {
      stop("Need ", ch, "@ni")
    } else {
      if (!silent) message("Setting ", ch, "@ni to zero")
      Dsurvey@ni <- 0
    }
  }
  ni <- Dsurvey@ni

  if (ni > 0) {
    dim_Iobs <- dim(Dsurvey@Iobs_ymi) == c(ny, nm, ni)
    if (!all(dim_Iobs)) stop("dim(", ch, "@Iobs_ymi) needs to be: ", c(ny, nm, ni) |> paste(collapse = ", "))

    if (!length(Dsurvey@unit_i)) {
      if (!silent) message("Setting unit_i to biomass for all indices")
      Dsurvey@unit_i <- rep("B", ni)
    }

    if (!length(Dsurvey@lambdaI_i)) {
      Dsurvey@lambdaI_i <- rep(1, ni)
    } else if (length(Dsurvey@lambdaI_i) == 1) {
      Dsurvey@lambdaI_i <- rep(Dsurvey@lambdaI_i, ni)
    } else {
      stop("Vector ", ch, "@lambdaI_i needs to be length ", ni)
    }

    if (length(Dsurvey@IAAobs_ymai) || length(Dsurvey@IALobs_ymli)) {
      if (length(Dsurvey@icomp_like)) {
        Dsurvey@icomp_like <- match.arg(Dsurvey@icomp_like, choices = eval(formals(like_comp)$type))
      } else {
        if (!silent) message("Setting ", ch, "@icomp_like = \"multinomial\"")
        Dsurvey@icomp_like <- "multinomial"
      }
    }

    if (length(Dsurvey@IAAobs_ymai)) {
      dim_IAA <- dim(Dsurvey@IAAobs_ymai) == c(ny, nm, na, ni)
      if (!all(dim_IAA)) stop("dim(IAAobs_ymai) needs to be: ", c(ny, nm, na, ni) |> paste(collapse = ", "))

      if (!length(Dsurvey@IAAN_ymi)) {
        if (Dsurvey@icomp_like %in% c("multinomial", "ddirmult1", "ddirmult2") && !silent) {
          message("Setting ", ch, "@IAAN_ymi from ", ch, "@IAAobs_ymai")
        }
        Dsurvey@IAAN_ymi <- apply(Dsurvey@IAAobs_ymai, c(1, 2, 4), sum)
      } else {
        dim_IAAN <- dim(Dsurvey@IAAN_ymi) == c(ny, nm, ni)
        if (!all(dim_IAAN)) stop("dim(", ch, "@IAAN_ymi) needs to be: ", c(ny, nm, ni) |> paste(collapse = ", "))
      }

      if (!length(Dsurvey@IAAtheta_i)) {
        if (grepl("ddirmult", Dsurvey@icomp_like) && !silent) message("Setting ", ch, "@IAAtheta_i to 1 for all indices")
        Dsurvey@IAAtheta_i <- rep(1, ni)
      } else if (length(Dsurvey@IAAtheta_i) == 1) {
        Dsurvey@IAAtheta_i <- rep(Dsurvey@IAAtheta_i, ni)
      } else if (length(Dsurvey@IAAtheta_i) != ni) {
        stop("Vector ", ch, "@IAAtheta_i needs to be length ", ni)
      }

      if (!length(Dsurvey@lambdaIAA_i)) {
        Dsurvey@lambdaIAA_i <- rep(1, ni)
      } else if (length(Dsurvey@lambdaIAA_i) == 1) {
        Dsurvey@lambdaIAA_i <- rep(Dsurvey@lambdaIAA_i, ni)
      } else {
        stop("Vector ", ch, "@lambdaIAA_i needs to be length ", ni)
      }
    }

    if (length(Dsurvey@IALobs_ymli)) {
      if (!nl) stop("Index length composition detected but no length bins found in Dmodel@nl")

      dim_IAL <- dim(Dsurvey@IALobs_ymli) == c(ny, nm, nl, ni)
      if (!all(dim_IAL)) stop("dim(", ch, "@IALobs_ymli) needs to be: ", c(ny, nm, nl, ni) |> paste(collapse = ", "))

      if (!length(Dsurvey@IALN_ymi)) {
        if (Dsurvey@icomp_like %in% c("multinomial", "ddirmult1", "ddirmult2") && !silent) {
          message("Setting ", ch, "@IALN_ymi from ", ch, "@IALobs_ymli")
        }
        Dsurvey@IALN_ymi <- apply(Dsurvey@IALobs_ymli, c(1, 2, 4), sum)
      } else {
        dim_IALN <- dim(Dsurvey@IALN_ymi) == c(ny, nm, ni)
        if (!all(dim_IALN)) stop("dim(", ch, "@IALN_ymi) needs to be: ", c(ny, nm, ni) |> paste(collapse = ", "))
      }

      if (!length(Dsurvey@IALtheta_i)) {
        if (grepl("ddirmult", Dsurvey@icomp_like) && !silent) message("Setting ", ch, "@IALtheta_i to 1 for all indices")
        Dsurvey@IALtheta_i <- rep(1, ni)
      } else if (length(Dsurvey@IALtheta_i) == 1) {
        Dsurvey@IALtheta_i <- rep(Dsurvey@IALtheta_i, ni)
      } else if (length(Dsurvey@IALtheta_i) != ni) {
        stop("Vector ", ch, "@IALtheta_i needs to be length ", ni)
      }

      if (!length(Dsurvey@lambdaIAL_i)) {
        Dsurvey@lambdaIAL_i <- rep(1, ni)
      } else if (length(Dsurvey@lambdaIAL_i) == 1) {
        Dsurvey@lambdaIAL_i <- rep(Dsurvey@lambdaIAL_i, ni)
      } else {
        stop("Vector ", ch, "@lambdaIAL_i needs to be length ", ni)
      }
    }

    if (!length(Dsurvey@samp_irs)) {
      if (nr > 1 || ns > 1) {
        if (!silent) message("Setting ", ch, "@samp_irs = 1. All indices operate in all regions and sample all stocks")
      }
      Dsurvey@samp_irs <- array(1, c(ni, nr, ns))
    } else {
      dim_samp <- dim(Dsurvey@samp_irs) == c(ni, nr, ns)
      if (!all(dim_samp)) stop("dim(", ch, "@samp_irs) needs to be: ", c(ni, nr, ns) |> paste(collapse = ", "))
    }

    if (length(Dsurvey@sel_i) == 1) {
      Dsurvey@sel_i <- rep(Dsurvey@sel_i, ni)
    } else if(length(Dsurvey@sel_i) != ni) {
      stop("Vector ", ch, "@sel_i needs to be length ", ni)
    }

    if (!length(Dsurvey@delta_i)) {
      if (!silent) message("Setting ", ch, "@delta_i = 0 for all indices (survey timing within time step)")
      Dsurvey@delta_i <- rep(0, ni)
    } else if (length(Dsurvey@delta_i) == 1) {
      Dsurvey@delta_i <- rep(Dsurvey@delta_i, ni)
    } else if (length(Dsurvey@delta_i) != ni) {
      stop("Vector ", ch, "@delta_i needs to be length ", ni)
    }

    if (!length(Dsurvey@qest_i)) {
      Dsurvey@qest_i <- rep("est", ni)
    } else if (length(Dsurvey@qest_i) != ni) {
      stop("Vector ", ch, "@qest_i needs to be length ", ni)
    }

  }

  return(Dsurvey)
}

check_DCKMR <- function(DCKMR, Dmodel, silent = FALSE) {
  ns <- Dmodel@ns

  if (length(DCKMR@POP_s)) {
    vars_POP <- c("a", "t", "y", "n", "m")
    check_POP <- sapply(1:ns, function(s) {
      if (nrow(DCKMR@POP_s[[s]])) {
        all(vars_POP %in% names(DCKMR@POP_s[[s]]))
      } else {
        TRUE
      }
    })
    if (any(!check_POP)) {
      stop("Missing columns in close-kin POP data frames. See: help(\"MSAdata-class\")")
    }
  }

  if (length(DCKMR@HSP_s)) {
    vars_HSP <- c("yi", "yj", "n", "m")
    check_HSP <- sapply(1:ns, function(s) {
      if (nrow(DCKMR@HSP_s[[s]])) {
        all(vars_HSP %in% names(DCKMR@HSP_s[[s]]))
      } else {
        TRUE
      }
    })
    if (any(!check_HSP)) {
      stop("Missing columns in close-kin HSP data frames. See: help(\"MSAdata-class\")")
    }
  }

  if (length(DCKMR@POP_s) || length(DCKMR@HSP_s)) {
    if (!length(DCKMR@CKMR_like)) {
      if (!silent) message("Setting close-kin likelihood to \"binomial\"")
      DCKMR@CKMR_like <- "binomial"
    }
  }
  return(DCKMR)
}

check_Dtag <- function(Dtag, Dmodel, silent = FALSE) {
  ny <- Dmodel@ny
  nm <- Dmodel@nm
  na <- Dmodel@na
  nr <- Dmodel@nr
  ns <- Dmodel@ns

  ch <- as.character(substitute(Dtag))
  if (length(ch) > 1) ch <- "Dtag"

  if (length(Dtag@tag_ymarrs) && length(Dtag@tag_ymars)) {
    stop("Tag data found in both Dtag@tag_ymarrs (tag movement) and Dtag@ymars (tag distribution). Only one tag setup can be used at this time.")
  }

  if (length(Dtag@tag_ymarrs) || length(Dtag@tag_ymars)) {
    if (length(Dtag@tag_like)) {
      Dtag@tag_like <- match.arg(Dtag@tag_like, choices = eval(formals(like_comp)$type))
    } else {
      if (!silent) message("Setting ", ch, "@tag_like = \"multinomial\"")
      Dtag@tag_like <- "multinomial"
    }

    if (!length(Dtag@tagtheta_s)) {
      if (grepl("ddirmult", Dtag@tag_like) && !silent) message("Setting ", ch, "@tagtheta_s to 1 for all stocks")
      Dtag@tagtheta_s <- rep(1, ns)
    } else if (length(Dtag@tagtheta_s) == 1) {
      Dtag@tagtheta_s <- rep(Dtag@tagtheta_s, ns)
    } else if (length(Dtag@tagtheta_s) != ns) {
      stop("Vector ", ch, "@tagtheta_s needs to be length ", ns)
    }

    if (!length(Dtag@tagstdev_s)) {
      if (grepl("log", Dtag@tag_like) && !silent) message("Setting ", ch, "@tagstdev_s to 0.1 for all fleets")
      Dtag@tagstdev_s <- rep(0.1, ns)
    } else if (length(Dtag@tagstdev_s) == 1) {
      Dtag@tagstdev_s <- rep(Dtag@tagstdev_s, ns)
    } else if (length(Dtag@tagstdev_s) != ns) {
      stop("Vector ", ch, "@tagstdev_s needs to be length ", ns)
    }

    if (!length(Dtag@lambdaTag_s)) {
      Dtag@lambdaTag_s <- rep(1, ns)
    } else if (length(Dtag@lambdaTag_s) == 1) {
      Dtag@lambdaTag_s <- rep(Dtag@lambdaTag_s, ns)
    } else if (length(Dtag@lambdaTag_s != ns)) {
      stop("Vector ", ch, "@lambdaTag_s needs to be length ", ns)
    }
  } else { # Sets up movement estimation
    Dtag@tag_yy <- matrix(1:ny, 1, ny)
  }

  if (length(Dtag@tag_ymarrs)) {

    dim_tag1 <- dim(Dtag@tag_ymarrs)
    if (length(dim_tag1) != 6) stop("tag_ymarrs should be a six dimensional array")

    if (any(dim_tag1[c(2, 4, 5, 6)] != c(nm, nr, nr, ns))) {
      stop("dim(", ch, "@tag_ymarrs) should be ", c(dim_tag1[1], nm, dim_tag1[2], nr, nr, ns) |> paste(collapse = ", "))
    }

    if (dim_tag1[1] == ny) {
      if (!length(Dtag@tag_yy)) Dtag@tag_yy <- diag(1, ny)
    } else if (any(dim(Dtag@tag_yy) != c(dim_tag1[1], ny))) {
      stop("dim(", ch, "@tag_yy) should be: ", c(dim_tag1[1], ny) |> paste(collapse = ", "))
    }

    if (dim_tag1[3] == na) {
      if (!length(Dtag@tag_aa)) Dtag@tag_aa <- diag(1, na)
    } else if (any(dim(Dtag@tag_aa) != c(dim_tag1[3], na))) {
      stop("dim(", ch, "@tag_aa) should be: ", c(dim_tag1[3], na) |> paste(collapse = ", "))
    }

    if (!length(Dtag@tagN_ymars)) {
      if (Dtag@tag_like %in% c("multinomial", "ddirmult1", "ddirmult2") && !silent) {
        message("Setting ", ch, "@tagN_ymars from ", ch, "@tag_ymarrs")
      }
      Dtag@tagN_ymars <- apply(Dtag@tag_ymarrs, c(1:4, 6), sum)
    } else {
      dim_tagN <- dim(Dtag@tagN_ymars) == c(dim_tag1[1], nm, dim_tag1[3], nr, ns)
      if (!all(dim_tagN)) stop("dim(", ch, "@tagN_ymars) needs to be: ", c(dim_tag1[1], nm, dim_tag1[3], nr, ns) |> paste(collapse = ", "))
    }

  }

  if (length(Dtag@tag_ymars)) {

    dim_tag2 <- dim(Dtag@tag_ymars)
    if (length(dim_tag2) != 5) stop("tag_ymars should be a five dimensional array")

    if (any(dim_tag2[c(1, 2, 4, 5)] != c(ny, nm, nr, ns))) {
      stop("dim(tag_ymars) should be ", c(ny, nm, dim_tag2[3], nr, ns) |> paste(collapse = ", "))
    }

    if (dim_tag2[3] == na) {
      if (!length(Dtag@tag_aa)) Dtag@tag_aa <- diag(1, na)
    } else if (any(dim(Dtag@tag_aa) != c(dim_tag2[3], na))) {
      stop("dim(tag_aa) should be: ", c(dim_tag2[3], na) |> paste(collapse = ", "))
    }

    if (!length(Dtag@tagN_ymas)) {
      if (Dtag@tag_like %in% c("multinomial", "ddirmult1", "ddirmult2") && !silent) {
        message("Setting ", ch, "@tagN_ymas from ", ch, "@tag_ymars")
      }
      Dtag@tagN_ymas <- apply(Dtag@tag_ymars, c(1:3, 5), sum)
    } else {
      dim_tagN <- dim(Dtag@tagN_ymas) == c(dim_tag2[1], nm, dim_tag2[3], ns)
      if (!all(dim_tagN)) stop("dim(tagN_ymas) needs to be: ", c(dim_tag2[1], nm, dim_tag2[3], ns) |> paste(collapse = ", "))
    }

  }

  return(Dtag)
}

check_Dlabel <- function(Dlabel, Dmodel, Dfishery, Dsurvey, silent = FALSE) {
  ny <- Dmodel@ny
  nm <- Dmodel@nm
  na <- Dmodel@na
  nr <- Dmodel@nr
  ns <- Dmodel@ns

  nf <- Dfishery@nf
  ni <- Dsurvey@ni

  ch <- as.character(substitute(Dlabel))
  if (length(ch) > 1) ch <- "Dlabel"

  if (!length(Dlabel@year)) {
    Dlabel@year <- 1:ny
  } else if (length(Dlabel@year) != ny) {
    stop("length(", ch, "@year) needs to be ", ny)
  }
  if (nm > 1) {
    if (!length(Dlabel@season) && nm > 1) {
      Dlabel@season <- paste("Season", 1:nm)
    } else if (length(Dlabel@season) != nm) {
      stop("length(", ch, "@season) needs to be ", nm)
    }
  }
  if (!length(Dlabel@age)) {
    Dlabel@age <- seq(1, na) - 1
  } else if (length(Dlabel@age) != na) {
    stop("length(", ch, "@age) needs to be ", na)
  }
  if (nr > 1) {
    if (!length(Dlabel@region) && nr > 1) {
      Dlabel@region <- paste("Region", 1:nr)
    } else if (length(Dlabel@region) != nr) {
      stop("length(", ch, "@region) needs to be ", nr)
    }
  }
  if (ns > 1) {
    if (!length(Dlabel@stock) && ns > 1) {
      Dlabel@stock <- paste("Stock", 1:ns)
    } else if (length(Dlabel@stock) != ns) {
      stop("length(", ch, "@stock) needs to be ", ns)
    }
  }
  if (!length(Dlabel@fleet)) {
    Dlabel@fleet <- paste("Fleet", 1:nf)
  } else if (length(Dlabel@fleet) != nf) {
    stop("length(", ch, "@fleet) needs to be ", nf)
  }
  if (Dsurvey@ni > 0) {
    if (!length(Dlabel@index)) {
      Dlabel@index <- paste("Index", 1:ni)
    } else if (length(Dlabel@index) != ni) {
      stop("length(", ch, "@index) needs to be ", ni)
    }
  }

  return(Dlabel)
}

#' Fit MSA model
#'
#' Wrapper function that calls RTMB to create the model and perform the numerical optimization
#'
#' @param x Data object. Class [MSAdata-class], validated by [check_data()]. Alternatively, [MSAassess-class] that will be fitted again.
#' @param parameters List of parameters, e.g., returned by [make_parameters()] and validated by [check_parameters()].
#' @param map List of parameters indicated whether they are fixed and how they are shared, e.g., returned by [make_parameters()].
#' See [RTMB::MakeADFun()].
#' @param random Character vector indicating the parameters that are random effects, e.g., returned by [make_parameters()].
#' @param run_model Logical, whether to fit the model through [stats::nlminb()].
#' @param do_sd Logical, whether to calculate the standard errors with [RTMB::sdreport()].
#' @param report Logical, whether to return the report list with `obj$report(obj$env$last.par.best)`.
#' @param silent Logical, whether to report progress to console. **Not passed to [TMB::MakeADFun()].** Recommend to set to `TRUE`
#' to speed up run time, e.g., when running simulations, multiple fits, etc.
#' @param control Passed to [stats::nlminb()]
#' @param ... Other arguments to [RTMB::MakeADFun()].
#' @returns A [MSAassess-class] object.
#' @details
#' When re-fitting a model, you may want to update the starting values through `x@obj$par`.
#' @importFrom methods new
#' @seealso [report()] [retrospective()] [do_jitter()]
#' @export
fit_MSA <- function(x, parameters, map = list(), random = NULL,
                    run_model = TRUE, do_sd = TRUE, report = TRUE, silent = FALSE,
                    control = list(iter.max = 2e+05, eval.max = 4e+05), ...) {

  if (inherits(x, "MSAdata")) {
    x@Misc$map <- map
    x@Misc$random <- random

    #old_comparison <- TapeConfig()["comparison"]
    #on.exit(TapeConfig(comparison = old_comparison))
    #TapeConfig(comparison = "tape")

    func <- function(p) .MSA(p, d = x)

    if (!silent) message("Building model with RTMB::MakeADFun()..")
    obj <- RTMB::MakeADFun(
      func = func, parameters = parameters,
      map = map, random = random,
      silent = TRUE,
      ...
    )
  } else if (inherits(x, "MSAassess")) {
    obj <- x@obj
  }

  if (!silent) {
    fn <- obj$fn()
    if (is.na(fn)) {
      message_oops("Objective function is NA at initial values.")
      report_start <- obj$report()
      if (x@Dmodel@condition == "catch" && any(is.na(report_start$F_ymfr))) {
        message_oops("NA's found in F array. Try increasing start value of R0.")
      }

    } else if (is.infinite(fn)) {
      message_oops("Objective function is infinite at initial values.")
    }

    gr <- obj$gr()
    if (any(is.na(gr))) {
      par_NA <- unique(names(obj$par)[!is.na(gr)])
      message_oops("Gradients of NA at initial values were found.")
      message_info(paste0(par_NA, collapse = ", "))
    }
    if (any(!gr, na.rm = TRUE)) {
      par_zero <- unique(names(obj$par)[!gr])
      message_oops("Gradients of zero at initial values for these parameters (may not be identifiable without prior):")
      message_info(paste0(par_zero, collapse = ", "))
    }
  }

  M <- new("MSAassess", obj = obj)

  if (run_model) {
    if (!silent) time <- proc.time()

    opt <- optimize_RTMB(M@obj, do_sd = do_sd, control = control, silent = silent)

    if (!silent) {
      time_fit <- c(proc.time() - time)["elapsed"]
      message_info("Run time: ", floor(time_fit/60), " minutes, ", round(time_fit %% 60), " seconds")
    }
    if (is.character(opt)) {
      message_oops("Error message from optimization:\n", opt)
    } else {
      M@opt <- opt
    }
  }
  if (do_sd) {
    if (!silent) time <- proc.time()

    M@SD <- get_sdreport(obj, silent = silent)

    if (!silent) {
      time_fit <- c(proc.time() - time)["elapsed"]
      message_info("Run time: ", floor(time_fit/60), " minutes, ", round(time_fit %% 60), " seconds")
    }
  }

  if (report) {
    if (!silent) message("Generating report list..")
    M@report <- update_report(obj$report(obj$env$last.par.best), MSAdata = get_MSAdata(M))
  }
  if (!silent) message("Complete.")
  return(M)
}

update_report <- function(r, MSAdata) {

  if (is.null(r$F_yas)) {
    nr <- MSAdata@Dmodel@nr
    nm <- MSAdata@Dmodel@nm

    ny <- MSAdata@Dmodel@ny
    na <- MSAdata@Dmodel@na
    ns <- MSAdata@Dmodel@ns

    if (nr == 1 && nm == 1) {
      r$F_yas <- array(r$F_ymars[, 1, , 1, ], c(ny, na, ns))
    } else {
      Fmax <- MSAdata@Dmodel@Fmax
      nf <- MSAdata@Dfishery@nf

      r$F_yas <- sapply2(1:ns, function(s) {
        sapply(1:na, function(a) {
          sapply(1:ny, function(y) {
            calc_summary_F(M = r$M_yas[y, a, s], N = sum(r$N_ymars[y, 1, a, , s]),
                           CN = sum(r$CN_ymafrs[y, , a, , , s]), Fmax = nf * Fmax)
          })
        })
      })

      if (any(is.na(r$F_yas))) message_oops("NA in predicted catch at age")
      if (any(is.infinite(r$F_yas))) {
        if (MSAdata@Dstock@m_spawn > 1) {
          message_info("Annual catch-at-age exceeds abundance-at-age at beginning of year, but this is mathemetically possible in seasonal models with recruitment during the year")
        } else {
          message_oops("Annual catch-at-age exceeds abundance-at-age at beginning of year")
        }
      }
    }
    r$Z_yas <- r$F_yas + r$M_yas
  }
  return(r)
}

.MSA <- function(p = list(), d) {

  # Dispatch method for AD variables ----
  is_ad <- any(sapply(p, inherits, "advector"))
  if (is_ad) {
    `[<-` <- RTMB::ADoverload("[<-")
  }

  # Assign data variables to environment, see OBS() for simulation ----
  #getAllS4(d@Dmodel, d@Dstock, d@Dfishery, d@Dsurvey, d@DCKMR, d@Dtag)
  Dmodel <- d@Dmodel
  Dstock <- d@Dstock
  Dfishery <- d@Dfishery
  Dsurvey <- d@Dsurvey
  DCKMR <- d@DCKMR
  Dtag <- d@Dtag

  ny <- Dmodel@ny
  nm <- Dmodel@nm
  na <- Dmodel@na
  nl <- Dmodel@nl
  nr <- Dmodel@nr
  ns <- Dmodel@ns

  nf <- Dfishery@nf
  ni <- Dsurvey@ni

  map <- d@Misc$map
  random <- d@Misc$random

  # Population arrays ----
  N_ymars <- array(NA_real_, c(ny + 1, nm, na, nr, ns))
  B_ymrs <- array(NA_real_, c(ny, nm, nr, ns))

  Npsp_yars <-
    Nsp_yars <- array(NA_real_, c(ny, na, nr, ns))
  S_yrs <- array(NA_real_, c(ny, nr, ns))
  R_ys <-
    Rdev_ys <- array(NA_real_, c(ny, ns))
  F_ymars <-
    Z_ymars <- array(NA_real_, c(ny, nm, na, nr, ns))
  M_yas <- mat_yas <- array(NA_real_, c(ny, na, ns))
  if (length(DCKMR@HSP_s)) F_yas <- array(NA_real_, c(ny, na, ns))
  mov_ymarrs <- array(NA_real_, c(ny, nm, na, nr, nr, ns))
  recdist_rs <- array(NA_real_, c(nr, ns))

  # Fishery arrays ----
  sel_ymafs <- array(NA_real_, c(ny, nm, na, nf, ns))

  F_ymfr <-
    log_F_ymfr <- array(NA_real_, c(ny, nm, nf, nr))
  F_ymafrs <-
    CN_ymafrs <- array(NA_real_, c(ny, nm, na, nf, nr, ns))
  CB_ymfrs <-
    VB_ymfrs <- array(NA_real_, c(ny, nm, nf, nr, ns))

  if (sum(Dfishery@CALobs_ymlfr > 0, na.rm = TRUE)) {
    has_CAL_f <- apply(Dfishery@CALobs_ymlfr, 4, function(i) sum(i, na.rm = TRUE) > 0)
  } else {
    has_CAL_f <- rep(FALSE, nf)
  }
  any_CAL <- any(has_CAL_f)

  if (any_CAL) CN_ymlfrs <- array(NA_real_, c(ny, nm, nl, nf, nr, ns))

  # Identify index lengths predicted from fishery selectivity
  any_IAL <- ni > 0 && any(Dsurvey@IALobs_ymli > 0, na.rm = TRUE)
  if (any_IAL) {
    has_IAL_i <- apply(Dsurvey@IALobs_ymli, 4, function(i) sum(i, na.rm = TRUE) > 0)
    Isel_f <- suppressWarnings(as.numeric(Dsurvey@sel_i))
    Isel_fleet <- !is.na(Isel_f) & Isel_f > 0
    is_IALfsel_i <- has_IAL_i & Isel_fleet
  } else {
    is_IALfsel_i <- FALSE
  }

  if (any_CAL || (any_IAL && any(is_IALfsel_i))) {
    LAKsel_ymalfs <- array(NA_real_, c(ny, nm, na, nl, ns, nf)) |>
      aperm(c(1:4, 6, 5))
  }

  # Index of abundance arrays ----
  if (ni > 0) {
    IN_ymais <-
      sel_ymais <- array(NA_real_, c(ny, nm, na, ni, ns))
    VI_ymi <- I_ymi <- array(NA_real_, c(ny, nm, ni))

    if (any_IAL) {
      IN_ymlis <- array(NA_real_, c(ny, nm, nl, ni, ns))
      LAKsel_ymalis <- array(NA_real_, c(ny, nm, na, nl, ns, ni)) |>
        aperm(c(1:4, 6, 5))
    }
  }

  # Transform parameters ----
  ## Maturity at age ogive ----
  if (is.null(map$mat_ps)) map$mat_ps <- matrix(TRUE, 2, ns)
  map$mat_ps <- matrix(as.character(map$mat_ps), 2, ns)
  mat_yas[] <- sapply2(1:ns, function(s) {
    if (all(is.na(map$mat_ps[, s]))) {
      Dstock@mat_yas[1:ny, , s]
    } else {
      m <- conv_mat(p$mat_ps[, s], na)
      matrix(m, ny, na, byrow = TRUE)
    }
  })

  ## Natural mortality ----
  for(s in 1:ns) {
    if (!is.null(map$log_M_s) && is.na(map$log_M_s[s])) {
      M_yas[, , s] <- Dstock@M_yas[1:ny, , s]
    } else {
      M_yas[, , s] <- matrix(exp(p$log_M_s[s]), ny, na)
    }
  }

  ## Fishery selectivity ----
  q_fs <- exp(p$log_q_fs)
  selconv_pf <- conv_selpar(p$sel_pf, type = Dfishery@sel_f, maxage = na - 1, maxL = 0.95 * max(Dmodel@lmid))
  sel_lf <- calc_sel_len(selconv_pf, Dmodel@lmid, type = Dfishery@sel_f)

  ## Fishing mortality ----
  if (Dmodel@condition == "F") {
    log_Fmult_f <- sapply(1:nf, function(f) p$log_Fdev_ymfr[Dmodel@y_Fmult_f[f], Dmodel@m_Fmult_f[f], f, Dmodel@r_Fmult_f[f]])
    log_F_ymfr[] <- sapply2(1:nr, function(r) {
      sapply2(1:nf, function(f) {
        sapply(1:nm, function(m) {
          sapply(1:ny, function(y) {
            Fmult_y <- y == Dmodel@y_Fmult_f[f]
            Fmult_m <- m == Dmodel@m_Fmult_f[f]
            Fmult_r <- r == Dmodel@r_Fmult_f[f]
            if (Fmult_y && Fmult_m && Fmult_r) {
              log_Fmult_f[f]
            } else {
              log_Fmult_f[f] + p$log_Fdev_ymfr[y, m, f, r]
            }
          })
        })
      })
    })
    F_ymfr[] <- exp(log_F_ymfr)
  }

  ## Index selectivity ----
  if (ni > 0) {
    selconv_pi <- conv_selpar(p$sel_pi, type = Dsurvey@sel_i, maxage = na - 1, maxL = 0.95 * max(Dmodel@lmid))
    sel_li <- calc_sel_len(selconv_pi, Dmodel@lmid, type = Dsurvey@sel_i, fsel_type = Dfishery@sel_f, fsel_len = sel_lf)
  }

  ## Fishery and index selectivity ----

  for (s in 1:ns) {
    # Fishery selectivity

    # Check for time-varying growth only if length-based sel
    tv_growth <- any(sapply(2:ny, function(y) max(Dstock@LAK_ymals[y, , , , s] - Dstock@LAK_ymals[1, , , , s])) > 0)
    tv_flensel_growth <- tv_growth && any(grepl("length", Dfishery@sel_f))

    # Check for time-varying sel due to tv maturity
    tv_mat <- any(sapply(2:ny, function(y) max(mat_yas[y, , s] - mat_yas[1, , s])) > 0)
    tv_fsel_mat <- tv_mat && any(Dfishery@sel_f %in% c("SB", "mature"))

    # Slower method by individual time step for now if tv growth or maturity
    if (tv_flensel_growth || tv_fsel_mat) {
      for (y in 1:ny) {
        for (m in 1:nm) {
          sel_ymafs[y, m, , , s] <- calc_fsel_age(
            sel_lf, Dstock@LAK_ymals[y, m, , , s], Dfishery@sel_f, selconv_pf, Dfishery@sel_block_yf[y, ], mat = mat_yas[y, , s], a = seq(1, na) - 1
          )
        }
      }
    } else { # Fill in sel array by unique rows of sel blocks

      sel_block_y <- sapply(1:ny, function(y) Reduce("paste0", Dfishery@sel_block_yf[y, ]))
      sel_block_unique <- unique(sel_block_y)

      for (j in 1:length(sel_block_unique)) {
        y_j <- which(sel_block_y == sel_block_unique[j])
        for (m in 1:nm) {
          sel_ymafs[y_j[1], m, , , s] <- calc_fsel_age(
            sel_lf, Dstock@LAK_ymals[y_j[1], m, , , s], Dfishery@sel_f, selconv_pf, Dfishery@sel_block_yf[y_j[1], ],
            mat = mat_yas[y_j[1], , s], a = seq(1, na) - 1
          )
        }
        if (length(y_j) > 1) {
          fsel_ind <- fsel1_ind <- as.matrix(expand.grid(y = y_j[-1], m = 1:m, a = 1:na, f = 1:nf, s = s))
          fsel1_ind[, "y"] <- y_j[1]
          sel_ymafs[fsel_ind] <- sel_ymafs[fsel1_ind]
        }
      }

    }

    # Survey selectivity
    if (ni > 0) {
      ilensel <- any(sapply(1:ni, function(i) grepl("length", Dsurvey@sel_i[i])))
      tv_iagesel_growth <- ilensel && tv_growth
      tv_isel_mat <- tv_mat && any(Dsurvey@sel_i %in% c("SB", "mature"))
      if (tv_iagesel_growth || tv_isel_mat) {
        for (y in 1:ny) {
          for (m in 1:nm) {
            sel_ymais[y, m, , , s] <- calc_isel_age(
              sel_li, Dstock@LAK_ymals[y, m, , , s], Dsurvey@sel_i, selconv_pi,
              matrix(sel_ymafs[y, m, , , s], na, nf), mat = mat_yas[y, , s], a = seq(1, na) - 1,
              fsel_type = Dfishery@sel_f,
              fsel_len = sel_lf
            )
          }
        }
      } else {
        for (m in 1:nm) {
          sel_ymais[1, m, , , s] <- calc_isel_age(
            sel_li, Dstock@LAK_ymals[1, m, , , s], Dsurvey@sel_i, selconv_pi,
            matrix(sel_ymafs[1, m, , , s], na, nf), mat = mat_yas[1, , s], a = seq(1, na) - 1,
            fsel_type = Dfishery@sel_f,
            fsel_len = sel_lf
          )
        }
        isel_ind <- isel1_ind <- as.matrix(expand.grid(y = 2:ny, m = 1:m, a = 1:na, i = 1:ni, s = s))
        isel1_ind[, "y"] <- 1
        sel_ymais[isel_ind] <- sel_ymais[isel1_ind]
      }
    }
  }

  ## Stock distribution and movement parameters ----
  recdist_rs[] <- sapply(1:ns, function(s) softmax(p$log_recdist_rs[, s]))

  # Movement
  if (nr > 1) {
    for (yy in 1:nrow(Dtag@tag_yy)) {
      yvec <- which(Dtag@tag_yy[yy, ] > 0)
      y1 <- yvec[1]

      for (m in 1:nm) {
        mov_ymarrs[y1, m, , , , ] <- sapply2(1:ns, function(s) {
          mov_arr <- array(0, c(na, nr, nr))
          r_eff <- Dstock@presence_rs[, s]
          nr_eff <- sum(Dstock@presence_rs[, s])
          mov_arr[, r_eff, r_eff] <- conv_mov(
            p$mov_x_marrs[m, , r_eff, r_eff, s], p$mov_g_ymars[y1, m, , r_eff, s], p$mov_v_ymas[y1, m, , s], na, nr_eff
          )
          return(mov_arr)
        })
      }

      if (length(yvec) > 1) {
        mov_ind <- mov1_ind <- as.matrix(expand.grid(y = yvec[-1], m = 1:m, a = 1:na, rf = 1:nr, rt = 1:nr, s = 1:ns))
        mov1_ind[, "y"] <- yvec[1]
        mov_ymarrs[mov_ind] <- mov_ymarrs[mov1_ind]
      }
    }
  } else {
    mov_ymarrs[] <- 1
  }

  ## Stock recruit parameters ----
  R0_s <- exp(p$t_R0_s) * Dmodel@scale_s
  h_s <- sapply(1:ns, function(s) conv_steepness(p$t_h_s[s], Dstock@SRR_s[s]))
  kappa_s <- sapply(1:ns, function(s) SRkconv(h_s[s], Dstock@SRR_s[s]))

  if (nm == 1 && nr == 1) {
    nyinit <- 1L
    NPR0_mars <- sapply2(1:ns, function(s) calc_NPR(M_yas[Dmodel@y_phi, , s])) |>
      array(c(na, ns, nm, nr)) |>
      aperm(c(3, 1, 4, 2))
    phi_s <- sapply(1:ns, function(s) {
      calc_phi_simple(M_yas[Dmodel@y_phi, , s], mat_a = mat_yas[Dmodel@y_phi, , s], fec_a = Dstock@fec_yas[Dmodel@y_phi, , s],
                      delta = Dstock@delta_s[s])
    })
  } else {
    nyinit <- Dmodel@nyinit
    NPR_unfished <- calc_phi_project(
      nyinit, nm, na, nf = 1, nr, ns, M_as = M_yas[Dmodel@y_phi, , ], mov_marrs = mov_ymarrs[Dmodel@y_phi, , , , , ],
      mat_as = mat_yas[Dmodel@y_phi, , ], fec_as = Dstock@fec_yas[Dmodel@y_phi, , ], m_spawn = Dstock@m_spawn, m_advanceage = Dstock@m_advanceage,
      delta_s = Dstock@delta_s, natal_rs = Dstock@natal_rs, recdist_rs = recdist_rs
    )
    NPR0_mars <- array(NPR_unfished[["N_ymars"]][nyinit, , , , ], c(nm, na, nr, ns))
    phi_s <- sapply(1:ns, function(s) sum(NPR_unfished[["S_yrs"]][nyinit, , s]))
  }
  N0_mars <- sapply2(1:ns, function(s) array(NPR0_mars[, , , s] * R0_s[s], c(nm, na, nr)))

  SB0_s <- R0_s * phi_s
  sralpha_s <- kappa_s/phi_s
  srbeta_s <- sapply(1:ns, function(s) SRbetaconv(h_s[s], R0_s[s], phi_s[s], SRR = Dstock@SRR_s[s]))

  ## Recruitment deviates ----
  sdr_s <- exp(p$log_sdr_s)
  bcr_s <- -0.5 * sdr_s * sdr_s

  par_rdev_ys <- matrix(TRUE, ny, ns)
  bcr_ys <- sapply(1:ns, function(s) bcr_s[s] * Dmodel@pbc_rdev_ys[, s])
  if (!is.null(map$log_rdev_ys)) {
    par_rdev_ys[] <- matrix(!is.na(map$log_rdev_ys) & !duplicated(map$log_rdev_ys, MARGIN = 0), ny, ns)
    bcr_ys[is.na(map$log_rdev_ys)] <- 0
  }

  na_init <- ifelse(Dstock@m_advanceage > 1, na, na-1)
  par_initrdev_as <- matrix(TRUE, na_init, ns)
  bcrinit_as <- sapply(1:ns, function(s) bcr_s[s] * Dmodel@pbc_initrdev_as[, s])
  if (!is.null(map$log_initrdev_as)) {
    par_initrdev_as[] <- matrix(!is.na(map$log_initrdev_as) & !duplicated(map$log_initrdev_as, MARGIN = 0), na_init, ns)
    bcrinit_as[is.na(map$log_initrdev_as)] <- 0
  }

  Rdev_ys[] <- exp(p$log_rdev_ys + bcr_ys)

  ## Miscellaneous penalty term, e.g., F > Fmax
  penalty <- 0

  # First year, first season initialization ----
  initRdev_as <- exp(p$log_initrdev_as + bcrinit_as)
  initF_mfr <- array(NA_real_, c(nm, nf, nr))

  initNPR_mars <- array(NA_real_, c(nm, na, nr, ns))
  initNeq_mars <- array(NA_real_, c(nm, na, nr, ns))
  initN_ars <- array(NA_real_, c(na, nr, ns))

  initZ_mars <- array(NA_real_, c(nm, na, nr, ns))

  initCN_mafrs <- array(0, c(nm, na, nf, nr, ns))
  initCB_mfrs <- array(0, c(nm, nf, nr, ns))

  if (all(Dfishery@Cinit_mfr <= 1e-8)) {
    initF_mfr[] <- 0
    initNPR_mars[] <- NPR0_mars
    initphi_s <- phi_s
    initReq_s <- R0_s
    initNeq_mars[] <- N0_mars
  } else if (nm == 1 && nr == 1) {
    initF_mfr[] <- exp(p$log_initF_mfr)
    initZ_mars[1, , 1, ] <- sapply(1:ns, function(s) {
      F_a <- lapply(1:nf, function(f) sel_ymafs[1, 1, , f, s] * q_fs[f, s] * initF_mfr[1, f, 1])
      Z_a <- M_yas[1, , s] + Reduce("+", F_a)
      return(Z_a)
    })
    initNPR_mars[1, , 1, ] <- sapply(1:ns, function(s) calc_NPR(initZ_mars[1, , 1, s]))
    initphi_s <- sapply(1:ns, function(s) {
      sum(initNPR_mars[1, , 1, s] * exp(-Dstock@delta_s[s] * initZ_mars[1, , 1, s]) * mat_yas[1, , s] * Dstock@fec_yas[1, , s])
    })
    initReq_s <- sapply(1:ns, function(s) {
      calc_recruitment(initphi_s[s], Dstock@SRR_s[s], eq = TRUE, a = sralpha_s[s], b = srbeta_s[s])
    })
    initNeq_mars[] <- sapply2(1:ns, function(s) array(initNPR_mars[, , , s] * initReq_s[s], c(nm, na, nr)))

    # Equilibrium catch
    ind <- as.matrix(expand.grid(m = 1:nm, a = 1:na, f = 1:nf, r = 1:nr, s = 1:ns))
    mfr_mafrs <- ind[, c("m", "f", "r")]
    mars_mafrs <- ind[, c("m", "a", "r", "s")]
    initCN_mafrs[] <- initF_mfr[mfr_mafrs]/initZ_mars[mars_mafrs] * (1 - exp(-initZ_mars[mars_mafrs])) * initNeq_mars[mars_mafrs]

    initCB_mfrs[] <- 0
    ind <- as.matrix(expand.grid(y = 1, m = 1:nm, a = 0, f = 1:nf, r = 1:nr, s = 1:ns))
    for (a in 1:na) {
      ind[, "a"] <- a
      mafrs_ind <- ind[, c("m", "a", "f", "r", "s")]
      ymafs_ind <- ind[, c("y", "m", "a", "f", "s")]
      initCB_mfrs[] <- initCB_mfrs[] + initCN_mafrs[mafrs_ind] * Dfishery@fwt_ymafs[ymafs_ind]
    }

  } else {

    if (Dmodel@condition == "F") initF_mfr[] <- exp(p$log_initF_mfr)

    init_proj <- calc_init_population(
      nyinit, nm, na, nf, nr, ns,
      initN_ars = array(N0_mars[Dstock@m_spawn, , , ], c(na, nr, ns)),
      condition = Dmodel@condition,
      C_mfr = Dfishery@Cinit_mfr, F_mfr = initF_mfr, sel_mafs = sel_ymafs[1, , , , ],
      fwt_mafs = Dfishery@fwt_ymafs[1, , , , ], q_fs = q_fs,
      M_as = M_yas[1, , ], mov_marrs = mov_ymarrs[Dmodel@y_phi, , , , , ],
      mat_as = mat_yas[1, , ], fec_as = Dstock@fec_yas[1, , ],
      SRR_s = Dstock@SRR_s, sralpha_s = sralpha_s, srbeta_s = srbeta_s,
      m_spawn = Dstock@m_spawn, m_advanceage = Dstock@m_advanceage,
      delta_s = Dstock@delta_s, natal_rs = Dstock@natal_rs, recdist_rs = recdist_rs,
      Fmax = Dmodel@Fmax, nitF = Dmodel@nitF
    )

    if (Dmodel@condition == "catch") initF_mfr[] <- init_proj[["F_ymfr"]][nyinit, , , ]

    initZ_mars[] <- init_proj[["Z_ymars"]][nyinit, , , , ]
    initReq_s <- init_proj[["R_ys"]][nyinit, ]
    initNeq_mars[] <- init_proj[["N_ymars"]][nyinit, , , , ]
    initNPR_mars[] <- sapply2(1:ns, function(s) initNeq_mars[, , , s]/initReq_s[s])
    initphi_s <- apply(init_proj[["S_yrs"]][nyinit, , , drop = FALSE], 3, sum)/initReq_s

    # Equilibrium catch
    initCN_mafrs[] <- init_proj$CN_ymafrs[nyinit, , , , , ]
    initCB_mfrs[] <- init_proj$CB_ymfrs[nyinit, , , , ]
  }

  # Initial abundance ----
  ind <- as.matrix(expand.grid(m = 1, a = 1:na, r = 1:nr, s = 1:ns))
  as_ind <- ind[, c("a", "s")]
  if (Dstock@m_advanceage > 1) {
    initN_ars[] <- initRdev_as[as_ind] * initNeq_mars[ind]
  } else {
    initN_ars[-1, , ] <- sapply2(1:ns, function(s) array(initRdev_as[, s] * initNeq_mars[1, -1, , s], c(na-1, nr)))
    initN_ars[1, , ] <- 0
  }

  # Run population model ----
  pop <- calc_population(
    ny, nm, na, nf, nr, ns, initN_ars, mov_ymarrs, M_yas, Dstock@SRR_s, sralpha_s, srbeta_s,
    mat_yas, Dstock@fec_yas, Rdev_ys, Dstock@m_advanceage, Dstock@m_spawn, Dstock@delta_s, Dstock@natal_rs, recdist_rs = recdist_rs,
    Dfishery@fwt_ymafs, q_fs, sel_ymafs,
    condition = Dmodel@condition, F_ymfr = F_ymfr, Cobs_ymfr = Dfishery@Cobs_ymfr, Fmax = Dmodel@Fmax, nitF = Dmodel@nitF
  )

  # Assign population arrays ----
  N_ymars[] <- pop$N_ymars
  F_ymars[] <- pop$F_ymars
  Z_ymars[] <- pop$Z_ymars
  F_ymafrs[] <- pop$F_ymafrs
  CN_ymafrs[] <- pop$CN_ymafrs
  CB_ymfrs[] <- pop$CB_ymfrs
  VB_ymfrs[] <- pop$VB_ymfrs
  Nsp_yars[] <- pop$Nsp_yars
  Npsp_yars[] <- pop$Npsp_yars
  S_yrs[] <- pop$S_yrs
  R_ys[] <- pop$R_ys
  penalty <- penalty + pop$penalty

  if (Dmodel@condition == "catch") F_ymfr[] <- pop$F_ymfr

  ind_ymars <- as.matrix(expand.grid(y = 1:ny, m = 1:nm, a = 1:na, r = 1:nr, s = 1:ns))
  ymas_ymars <- ind_ymars[, c("y", "m", "a", "s")]

  B_ymrs[] <- array(N_ymars[ind_ymars] * Dstock@swt_ymas[ymas_ymars], c(ny, nm, na, nr, ns)) |>
    apply(c(1, 2, 4, 5), sum)

  # Likelihoods ----
  y_like <- seq(1, ny - Dmodel@nyret)

  Cinit_mfr <- Dfishery@Cinit_mfr
  any_Cinit <- any(Cinit_mfr > 1e-8)
  if (any_Cinit) {
    initCB_mfr <- apply(initCB_mfrs, 1:3, sum)

    if ((nm == 1 && nr == 1) || Dmodel@condition == "F") {
      Cinit_mfr <- OBS(Cinit_mfr)

      lambda_Cinit <- array(Dfishery@lambdaCobs_f, c(nf, nm, nr)) |> aperm(c(2, 1, 3))

      loglike_Cinit_mfr <- lambda_Cinit * dnorm(log(Cinit_mfr/initCB_mfr), 0, 0.01, log = TRUE)
      loglike_Cinit_mfr[Cinit_mfr <= 1e-8] <- 0
    } else {
      loglike_Cinit_mfr <- 0
    }
  } else {
    loglike_Cinit_mfr <- 0
  }

  ## Catch ----
  Cobs_ymfr <- Dfishery@Cobs_ymfr
  if (Dmodel@condition == "F") {

    CB_ymfr <- apply(CB_ymfrs, 1:4, sum)
    Cobs_ymfr <- OBS(Cobs_ymfr)

    lambda_Cobs <- array(Dfishery@lambdaCobs_f, c(nf, ny, nm, nr)) |> aperm(c(2, 3, 1, 4))

    loglike_Cobs_ymfr <- array(0, c(ny, nm, nf, nr))
    loglike_Cobs_ymfr[] <- lambda_Cobs * dnorm(log(Cobs_ymfr/CB_ymfr), 0, Dfishery@Csd_ymfr, log = TRUE)
    loglike_Cobs_ymfr[Cobs_ymfr <= 1e-8] <- 0
    loglike_Cobs_ymfr[1:ny > max(y_like), , , ] <- 0
  } else {
    loglike_Cobs_ymfr <- 0
  }

  ## Marginal fishery age composition ----
  CAAobs_ymafr <- Dfishery@CAAobs_ymafr
  any_CAA <- any(CAAobs_ymafr > 0, na.rm = TRUE)
  if (any_CAA) {
    CN_ymafr <- apply(CN_ymafrs, 1:5, sum)
    CAAobs_ymafr <- OBS(CAAobs_ymafr)

    lambda_CAA <- array(Dfishery@lambdaCAA_f, c(nf, length(y_like), nm, nr)) |> aperm(c(2, 3, 1, 4))

    loglike_CAA_ymfr <- array(0, c(ny, nm, nf, nr))
    loglike_CAA_ymfr[y_like, , , ] <- lambda_CAA * sapply2(1:nr, function(r) {
      sapply2(1:nf, function(f) {
        sapply(1:nm, function(m) {
          sapply(y_like, function(y) {
            like_comp(obs = (Cobs_ymfr[y, m, f, r] > 1e-8) * CAAobs_ymafr[y, m, , f, r],
                      pred = CN_ymafr[y, m, , f, r], type = Dfishery@fcomp_like,
                      N = Dfishery@CAAN_ymfr[y, m, f, r], theta = Dfishery@CAAtheta_f[f])
          })
        })
      })
    })
  } else {
    loglike_CAA_ymfr <- 0
  }

  ## Marginal fishery length composition ----
  CALobs_ymlfr <- Dfishery@CALobs_ymlfr

  # Adjust LAK by selectivity function (note: values stay at LAK_ymals if no length selectivity)
  for (f in 1:nf) {
    if (has_CAL_f[f] || (any_IAL && f %in% Isel_f)) { # Has length comps from either fishery or survey (with mirrored selectivity)
      for (b in unique(Dfishery@sel_block_yf[, f])) {
        y_b <- which(Dfishery@sel_block_yf[, f] == b)
        if (grepl("length", Dfishery@sel_f[b])) {
          if (tv_growth) {
            for (y in 1:ny) {
              if (Dfishery@sel_block_yf[y, f] == b) {
                for (m in 1:nm) {
                  LAKsel_ymalfs[y, m, , , f, ] <- sapply2(1:ns, function(s) {
                    LAK_la <- t(Dstock@LAK_ymals[y, m, , , s]) * sel_lf[, b]
                    denom <- colSums(LAK_la) + 1e-8
                    t(LAK_la)/denom
                  })
                }
              }
            }
          } else {
            for (m in 1:nm) {
              LAKsel_ymalfs[y_b[1], m, , , f, ] <- sapply2(1:ns, function(s) {
                LAK_la <- t(Dstock@LAK_ymals[y_b[1], m, , , s]) * sel_lf[, b]
                denom <- colSums(LAK_la) + 1e-8
                t(LAK_la)/denom
              })
            }
            fsel_ind <- fsel1_ind <- as.matrix(expand.grid(y = y_b[-1], m = 1:m, a = 1:na, 1:nl, f = f, s = 1:ns))
            fsel1_ind[, "y"] <- y_b[1]
            LAKsel_ymalfs[fsel_ind] <- LAKsel_ymalfs[fsel1_ind]
          }
        } else {
          LAKsel_ymalfs[y_b, , , , f, ] <- Dstock@LAK_ymals[y_b, , , , s]
        }
      }
    }
  }

  if (any_CAL) {
    CN_ymalfrs <- array(NA_real_, c(ny, nm, na, nl, nf, nr, ns))
    ind_ymalfrs <- expand.grid(y = 1:ny, m = 1:nm, a = 1:na, l = 1:nl, f = 1:nf, r = 1:nr, s = 1:ns) |>
      as.matrix()

    ymafrs_ymalfrs <- ind_ymalfrs[, c("y", "m", "a", "f", "r", "s")]
    ymalfs_ymalfrs <- ind_ymalfrs[, c("y", "m", "a", "l", "f", "s")]
    CN_ymalfrs[ind_ymalfrs] <- CN_ymafrs[ymafrs_ymalfrs] * LAKsel_ymalfs[ymalfs_ymalfrs]

    CN_ymlfrs <- array(0, c(ny, nm, nl, nf, nr, ns))
    for (a in 1:na) {
      ind_ymalfrs2 <- expand.grid(y = 1:ny, m = 1:nm, a = a, l = 1:nl, f = 1:nf, r = 1:nr, s = 1:ns) |>
        as.matrix()
      ymlfrs_ymalfrs <- ind_ymalfrs2[, c("y", "m", "l", "f", "r", "s")]
      CN_ymlfrs[ymlfrs_ymalfrs] <- CN_ymlfrs[ymlfrs_ymalfrs] + CN_ymalfrs[ind_ymalfrs2]
    }

    CN_ymlfr <- array(0, c(ny, nm, nl, nf, nr))
    for (s in 1:ns) {
      ind_ymlfrs <- expand.grid(y = 1:ny, m = 1:nm, l = 1:nl, f = 1:nf, r = 1:nr, s = s) |>
        as.matrix()
      ymlfr_ymlfrs <- ind_ymlfrs[, c("y", "m", "l", "f", "r")]
      CN_ymlfr[ymlfr_ymlfrs] <- CN_ymlfr[ymlfr_ymlfrs] + CN_ymlfrs[ind_ymlfrs]
    }

    CALobs_ymlfr <- OBS(CALobs_ymlfr)

    lambda_CAL <- array(Dfishery@lambdaCAL_f, c(nf, length(y_like), nm, nr)) |> aperm(c(2, 3, 1, 4))

    loglike_CAL_ymfr <- array(0, c(ny, nm, nf, nr))
    loglike_CAL_ymfr[y_like, , , ] <- lambda_CAL * sapply2(1:nr, function(r) {
      sapply2(1:nf, function(f) {
        sapply(1:nm, function(m) {
          sapply(y_like, function(y) {
            like_comp(obs = (Cobs_ymfr[y, m, f, r] > 1e-8) * CALobs_ymlfr[y, m, , f, r],
                      pred = CN_ymlfr[y, m, , f, r], type = Dfishery@fcomp_like,
                      N = Dfishery@CALN_ymfr[y, m, f, r], theta = Dfishery@CALtheta_f[f])
          })
        })
      })
    })
  } else {
    loglike_CAL_ymfr <- 0
  }

  ## Index ----
  Iobs_ymi <- Dsurvey@Iobs_ymi
  if (ni > 0) {
    for(y in 1:ny) {
      for(m in 1:nm) {
        IN_ymais[y, m, , , ] <- calc_index(
          N = N_ymars[y, m, , , ], Z = Z_ymars[y, m, , , ], sel = sel_ymais[y, m, , , ],
          na = na, nr = nr, ns = ns, ni = ni, samp = Dsurvey@samp_irs, delta = Dsurvey@delta_i
        )
        VI_ymi[y, m, ] <- sapply(1:ni, function(i) {
          I_s <- sapply(1:ns, function(s) {
            w <- if (Dsurvey@unit_i[i] == "N") 1 else Dstock@swt_ymas[y, m, , s]
            sum(IN_ymais[y, m, , i, s] * w)
          })
          sum(I_s)
        })
      }
    }
    q_i <- sapply(1:ni, function(i) {
      q_fixed <- suppressWarnings(as.numeric(Dsurvey@qest_i[i]))
      if (!is.na(q_fixed)) {
        AD(q_fixed)
      } else {
        calc_q(Iobs_ymi[, , i], B = VI_ymi[, , i])
      }
    })
    I_ymi[] <- sapply2(1:ni, function(i) q_i[i] * VI_ymi[, , i])

    Iobs_ymi <- OBS(Iobs_ymi)

    lambda_I <- array(Dsurvey@lambdaI_i, c(ni, ny, nm)) |> aperm(c(2, 3, 1))

    loglike_I_ymi <- array(0, c(ny, nm, ni))
    loglike_I_ymi[] <- lambda_I * dnorm(log(Iobs_ymi/I_ymi), 0, Dsurvey@Isd_ymi, log = TRUE)
    loglike_I_ymi[is.na(Iobs_ymi)] <- 0
    loglike_I_ymi[1:ny > max(y_like), , ] <- 0

  } else {
    loglike_I_ymi <- 0
  }

  IAAobs_ymai <- Dsurvey@IAAobs_ymai
  any_IAA <- ni > 0 && any(IAAobs_ymai > 0, na.rm = TRUE)
  if (any_IAA) {
    IN_ymai <- apply(IN_ymais, 1:4, sum)
    IAAobs_ymai <- OBS(IAAobs_ymai)

    lambda_IAA <- array(Dsurvey@lambdaIAA_i, c(ni, length(y_like), nm)) |> aperm(c(2, 3, 1))

    loglike_IAA_ymi <- array(0, c(ny, nm, ni))
    loglike_IAA_ymi[y_like, , ] <- lambda_IAA * sapply2(1:ni, function(i) {
      sapply(1:nm, function(m) {
        sapply(y_like, function(y) {
          like_comp(obs = IAAobs_ymai[y, m, , i], pred = IN_ymai[y, m, , i], type = Dsurvey@icomp_like,
                    N = Dsurvey@IAAN_ymi[y, m, i], theta = Dsurvey@IAAtheta_i[i])
        })
      })
    })
  } else {
    loglike_IAA_ymi <- 0
  }

  IALobs_ymli <- Dsurvey@IALobs_ymli
  # Adjust LAK by selectivity function (note: values stay at LAK_ymals if no length selectivity)

  if (any_IAL) {
    for (i in 1:ni) {
      if (has_IAL_i[i]) {
        if (grepl("length", Dsurvey@sel_i[i])) {
          if (tv_growth) {
            for (y in 1:ny) {
              for (m in 1:nm) {
                LAKsel_ymalis[y, m, , , i, ] <- sapply2(1:ns, function(s) {
                  LAK_la <- t(Dstock@LAK_ymals[y, m, , , s]) * sel_li[, i]
                  denom <- colSums(LAK_la) + 1e-8
                  t(LAK_la)/denom
                })
              }
            }
          } else {
            for (m in 1:nm) {
              LAKsel_ymalis[1, m, , , i, ] <- sapply2(1:ns, function(s) {
                LAK_la <- t(Dstock@LAK_ymals[1, m, , , s]) * sel_li[, i]
                denom <- colSums(LAK_la) + 1e-8
                t(LAK_la)/denom
              })
            }

            isel_ind <- isel1_ind <- as.matrix(expand.grid(y = 2:ny, m = 1:m, a = 1:na, 1:nl, i = i, s = 1:ns))
            isel1_ind[, "y"] <- 1
            LAKsel_ymalis[isel_ind] <- LAKsel_ymalis[isel1_ind]
          }
        } else if (is_IALfsel_i[i]) {
          LAKsel_ymalis[, , , , i, ] <- LAKsel_ymalfs[, , , , Isel_f[i], ]
        } else {
          LAKsel_ymalis[, , , , i, ] <- Dstock@LAKsel_ymals
        }
      }
    }

    IN_ymalis <- array(NA_real_, c(ny, nm, na, nl, ni, ns))
    ind_ymalis <- expand.grid(y = 1:ny, m = 1:nm, a = 1:na, l = 1:nl, i = 1:ni, s = 1:ns) |>
      as.matrix()
    ymais_ymalis <- ind_ymalis[, c("y", "m", "a", "i", "s")]
    IN_ymalis[] <- IN_ymais[ymais_ymalis] * LAKsel_ymalis
    IN_ymlis[] <- apply(IN_ymalis, c(1, 2, 4:6), sum)
    IN_ymli <- apply(IN_ymalis, c(1, 2, 4:5), sum)

    IALobs_ymli <- OBS(IALobs_ymli)

    lambda_IAL <- array(Dsurvey@lambdaIAL_i, c(ni, length(y_like), nm)) |> aperm(c(2, 3, 1))

    loglike_IAL_ymi <- array(0, c(ny, nm, ni))
    loglike_IAL_ymi[y_like, , ] <- lambda_IAL * sapply2(1:ni, function(i) {
      sapply(1:nm, function(m) {
        sapply(y_like, function(y) {
          like_comp(obs = IALobs_ymli[y, m, , i], pred = IN_ymli[y, m, , i], type = Dsurvey@icomp_like,
                    N = Dsurvey@IALN_ymi[y, m, i], theta = Dsurvey@IALtheta_i[i])
        })
      })
    })
  } else {
    loglike_IAL_ymi <- 0
  }

  ## Stock composition ----
  SC_ymafrs <- Dfishery@SC_ymafrs
  any_SC <- ns > 1 && length(SC_ymafrs)
  if (any_SC) {
    SC_ymafrs <- OBS(SC_ymafrs)
    SCpred_ymafrs <- sapply2(1:nrow(Dfishery@SC_ff), function(ff) {
      fvec <- as.logical(Dfishery@SC_ff[ff, ])
      sapply2(1:nrow(Dfishery@SC_aa), function(aa) {
        avec <- as.logical(Dfishery@SC_aa[aa, ])
        apply(CN_ymafrs[, , avec, fvec, , , drop = FALSE], c(1, 2, 5, 6), sum)
      })
    }) |>
      aperm(c(1, 2, 5, 6, 3, 4))

    lambda_SC <- array(Dfishery@lambdaSC_f, c(nrow(Dfishery@SC_ff), length(y_like), nm, nrow(Dfishery@SC_aa), nr)) |>
      aperm(c(2, 3, 4, 1, 5))

    loglike_SC_ymafr <- array(0, dim(SC_ymafrs)[1:5])
    loglike_SC_ymafr[y_like, , , , ] <- lambda_SC * sapply2(1:nr, function(r) {
      sapply2(1:nrow(Dfishery@SC_ff), function(ff) {
        sapply2(1:nrow(Dfishery@SC_aa), function(aa) {
          sapply(1:nm, function(m) {
            sapply(y_like, function(y) {
              fvec <- as.logical(Dfishery@SC_ff[ff, ])
              Cobs <- sum(Cobs_ymfr[y, m, fvec, r])
              like_comp(obs = (Cobs > 1e-8) * SC_ymafrs[y, m, aa, ff, r, ],
                        pred = SCpred_ymafrs[y, m, aa, ff, r, ], type = Dfishery@SC_like,
                        N = Dfishery@SCN_ymafr[y, m, aa, ff, r], theta = Dfishery@SCtheta_f[ff],
                        stdev = Dfishery@SCstdev_ymafrs[y, m, aa, ff, r, ])
            })
          })
        })
      })
    })
  } else {
    loglike_SC_ymafr <- 0
  }

  ## CKMR ----
  if (length(DCKMR@POP_s)) {
    pPOP_s <- lapply(1:ns, function(s) {
      calc_POP(t = DCKMR@POP_s[[s]]$t, a = DCKMR@POP_s[[s]]$a, y = DCKMR@POP_s[[s]]$y,
               N = apply(Nsp_yars[, , , s], 1:2, sum), fec = Dstock@fec_yas[, , s])
    })
    loglike_POP_s <- lapply(1:ns, function(s) {
      val <- 0
      if (DCKMR@POP_s[[s]]$y %in% y_like) {
        val <- like_CKMR(n = DCKMR@POP_s[[s]]$n, m = DCKMR@POP_s[[s]]$m, p = pPOP_s[[s]], type = DCKMR@CKMR_like)
      }
      return(val)
    })
  } else {
    loglike_POP_s <- 0
  }

  if (length(DCKMR@HSP_s)) {
    ## Summary F and Z by year ----
    F_yas[] <- sapply2(1:ns, function(s) {
      sapply(1:na, function(a) {
        sapply(1:ny, function(y) {
          calc_summary_F(M = M_yas[y, a, s], N = sum(N_ymars[y, 1, a, , s]),
                         CN = sum(CN_ymafrs[y, , a, , , s]), Fmax = nf * Dmodel@Fmax)
        })
      })
    })
    Z_yas <- F_yas + M_yas
    pHSP_s <- lapply(1:ns, function(s) {
      calc_HSP(yi = DCKMR@HSP_s[[s]]$yi, yj = DCKMR@HSP_s[[s]]$yj,
               N = apply(Nsp_yars[, , , s], 1:2, sum), fec = Dstock@fec_yas[, , s], Z = Z_yas[, , s])
    })
    loglike_HSP_s <- lapply(1:ns, function(s) {
      val <- 0
      if (DCKMR@HSP_s[[s]]$yi %in% y_like) {
        val <- like_CKMR(n = DCKMR@HSP_s[[s]]$n, m = DCKMR@HSP_s[[s]]$m, p = pHSP_s[[s]], type = DCKMR@CKMR_like)
      }
      return(val)
    })
  } else {
    loglike_HSP_s <- 0
  }

  ## Tag
  tag_ymarrs <- Dtag@tag_ymarrs
  if (any(tag_ymarrs > 0, na.rm = TRUE)) {
    tag_ymarrs <- OBS(tag_ymarrs)
    tagpred_ymarrs <- array(0, dim(tag_ymarrs))
    tagpred_ymarrs[] <- sapply2(1:nrow(Dtag@tag_aa), function(aa) {
      a1 <- which(Dtag@tag_aa[aa, ] > 0)[1]
      sapply2(1:nm, function(m) {
        sapply2(1:nrow(Dtag@tag_yy), function(yy) {
          y1 <- which(Dtag@tag_yy[yy, ] > 0)[1]
          mov_ymarrs[y1, m, a1, , , ] # rrsyma
        })
      })
    }) |>
      aperm(c(4:6, 1:3))

    loglike_tag_mov_ymars <- sapply2(1:ns, function(s) { # Likelihood of where the fish are going
      Dtag@lambdaTag_s[s] * sapply2(1:nr, function(rf) {
        sapply2(1:nrow(Dtag@tag_aa), function(aa) {
          sapply(1:nm, function(m) {
            sapply(1:nrow(Dtag@tag_yy), function(yy) {
              if (any(c(1:ny)[as.logical(Dtag@tag_yy[yy, ])] %in% y_like)) {
                val <- like_comp(obs = tag_ymarrs[yy, m, aa, rf, , s],
                                 pred = tagpred_ymarrs[yy, m, aa, rf, , s], type = Dtag@tag_like,
                                 N = Dtag@tagN_ymars[yy, m, aa, rf, s], theta = Dtag@tagtheta_s[s],
                                 stdev = Dtag@tagstdev_s[s])
              } else if (inherits(tagpred_ymarrs, "advector")) {
                val <- advector(0)
              } else {
                val <- 0
              }
              return(val)
            })
          })
        })
      })
    })
  } else {
    loglike_tag_mov_ymars <- 0
  }

  tag_ymars <- Dtag@tag_ymars
  if (any(tag_ymars > 0, na.rm = TRUE)) {
    tag_ymars <- OBS(tag_ymars)

    loglike_tag_dist_ymas <- sapply2(1:ns, function(s) { # Likelihood of where the fish are present
      Dtag@lambdaTag_s[s] * sapply2(1:nrow(Dtag@tag_aa), function(aa) {
        avec <- as.logical(Dtag@tag_aa[aa, ])
        sapply(1:nm, function(m) {
          sapply(1:nrow(Dtag@tag_yy), function(yy) {
            pred <- apply(N_ymars[yvec, m, avec, , s, drop = FALSE], 3, sum)
            if (any(c(1:ny)[as.logical(Dtag@tag_yy[yy, ])] %in% y_like)) {
              val <- like_comp(obs = tag_ymars[yy, m, aa, , s],
                               pred = pred, type = Dtag@tag_like,
                               N = Dtag@tagN_ymas[yy, m, aa, s], theta = Dtag@tagtheta_s[s],
                               stdev = Dtag@tagstdev_s[s])
            } else if (inherits(pred, "advector")) {
              val <- advector(0)
            } else {
              val <- 0
            }
            return(val)
          })
        })
      })
    })
  } else {
    loglike_tag_dist_ymas <- 0
  }

  loglike <- sum(loglike_Cinit_mfr) +
    sum(loglike_Cobs_ymfr) + sum(loglike_CAA_ymfr) + sum(loglike_CAL_ymfr) +
    sum(loglike_I_ymi) + sum(loglike_IAA_ymi) + sum(loglike_IAL_ymi) +
    sum(loglike_SC_ymafr) +
    Reduce(sum, loglike_POP_s) + Reduce(sum, loglike_HSP_s) +
    sum(loglike_tag_mov_ymars) +
    sum(loglike_tag_dist_ymas)

  # Priors ----
  logprior_rdev_ys <- array(0, c(ny, ns))
  logprior_rdev_ys[] <- sapply(1:ns, function(s) {
    CondExpGt(par_rdev_ys[, s], 0, dnorm(p$log_rdev_ys[, s], 0, sdr_s[s], log = TRUE), 0)
  })
  logprior_rdev_ys[1:ny > max(y_like), ] <- 0

  logprior_initrdev_as <- sapply(1:ns, function(s) {
    CondExpGt(par_initrdev_as[, s], 0, dnorm(p$log_initrdev_as[, s], 0, sdr_s[s], log = TRUE), 0)
  })

  if (nr > 1 && "mov_g_ymars" %in% random) {
    sdg_s <- sapply2(1:ns, function(s) conv_Sigma(sigma = exp(p$log_sdg_rs[, s]), lower_diag = p$t_rhog_rs[, s]))
    par_g_ymars <- !is.na(map$mov_g_ymars) & !duplicated(map$mov_g_ymars, MARGIN = 0)

    logprior_dist_ymas <- array(0, c(ny, nm, na, ns))
    logprior_dist_ymas[y_like, , , ] <- sapply2(1:ns, function(s) {
      sapply2(1:na, function(a) {
        sapply(1:nm, function(m) {
          sapply(y_like, function(y) dmvnorm(p$mov_g_ymars[y, m, a, , s], mu = 0, Sigma = sdg_s[, , s], log = TRUE))
        })
      })
    })
  } else {
    logprior_dist_ymas <- 0
  }

  if (length(Dmodel@prior)) {
    logprior_par <- sapply(Dmodel@prior, function(x, p) eval(parse(text = x)), p = p)
  } else {
    logprior_par <- 0
  }

  logprior <- sum(logprior_par) + sum(logprior_initrdev_as) + sum(logprior_rdev_ys) + sum(logprior_dist_ymas)

  # Penalty selectivity parameters to avoid steep ascending and descending limbs
  penalty <- penalty +
    calc_selpar_penalty(p$sel_pf, Dfishery@sel_f, Dmodel@lmid, na, map$sel_pf) +
    calc_selpar_penalty(p$sel_pi, Dsurvey@sel_i, Dmodel@lmid, na, map$sel_pi)

  # Objective function ----
  fn <- -1 * (logprior + loglike) + penalty

  # Report out variables ----
  ## Parameters ----
  ADREPORT(R0_s)
  ADREPORT(h_s)
  REPORT(R0_s)
  REPORT(h_s)
  REPORT(kappa_s)
  REPORT(SB0_s)
  REPORT(sralpha_s)
  REPORT(srbeta_s)
  REPORT(sdr_s)
  REPORT(phi_s)

  REPORT(selconv_pf)
  if (length(Dmodel@lmid)) REPORT(sel_lf)
  REPORT(q_fs)

  if (ni > 0) {
    REPORT(selconv_pi)
    if (length(Dmodel@lmid)) REPORT(sel_li)

    ADREPORT(q_i)
  }

  REPORT(F_ymfr)

  REPORT(M_yas)
  REPORT(mat_yas)

  ## Initial (first year, first season) calculations ----
  REPORT(NPR0_mars)
  REPORT(initRdev_as)
  if (any_Cinit) {
    REPORT(initF_mfr)
    REPORT(initZ_mars)
    REPORT(initNPR_mars)
    REPORT(initReq_s)
    REPORT(initphi_s)
    REPORT(initCN_mafrs)
    REPORT(initCB_mfrs)
  }

  ## Population arrays ----
  REPORT(N_ymars)
  REPORT(S_yrs)
  REPORT(R_ys)
  REPORT(Rdev_ys)
  REPORT(F_ymars)
  REPORT(Z_ymars)
  REPORT(B_ymrs)
  if (nr > 1) {
    REPORT(mov_ymarrs)
    REPORT(recdist_rs)
  }

  ## Fishery arrays ----
  REPORT(sel_ymafs)
  REPORT(F_ymafrs)
  REPORT(CN_ymafrs)
  if (any_CAL) REPORT(CN_ymlfrs)
  REPORT(CB_ymfrs)
  REPORT(VB_ymfrs)
  if (any_SC) REPORT(SCpred_ymafrs)

  ## Index of abundance arrays ----
  if (ni > 0) {
    REPORT(sel_ymais)
    REPORT(I_ymi)
    if (any_IAA) REPORT(IN_ymais)
    if (any_IAL) REPORT(IN_ymlis)
    REPORT(q_i)
  }

  ## Tag ----
  if (any(tag_ymarrs > 0, na.rm = TRUE)) REPORT(tagpred_ymarrs)

  ## CKMR ----
  if (length(DCKMR@POP_s)) REPORT(pPOP_s)

  if (length(DCKMR@HSP_s)) {
    REPORT(F_yas)
    REPORT(Z_yas)
    REPORT(pHSP_s)
  }

  ## Objective function values ----
  REPORT(loglike)
  REPORT(logprior)
  REPORT(penalty)
  REPORT(fn)

  if (any_Cinit) REPORT(loglike_Cinit_mfr)
  if (Dmodel@condition == "F") REPORT(loglike_Cobs_ymfr)
  if (any_CAA) REPORT(loglike_CAA_ymfr)
  if (any_CAL) REPORT(loglike_CAL_ymfr)

  if (ni > 0) {
    REPORT(loglike_I_ymi)
    if (any_IAA) REPORT(loglike_IAA_ymi)
    if (any_IAL) REPORT(loglike_IAL_ymi)
  }

  if (any_SC) REPORT(loglike_SC_ymafr)

  if (length(DCKMR@POP_s)) REPORT(loglike_POP_s)
  if (length(DCKMR@HSP_s)) REPORT(loglike_HSP_s)

  if (any(tag_ymarrs > 0, na.rm = TRUE)) REPORT(loglike_tag_mov_ymars)
  if (any(tag_ymars > 0, na.rm = TRUE)) REPORT(loglike_tag_dist_ymas)

  if (length(Dmodel@prior)) REPORT(logprior_par)

  if (is.null(map$log_initrdev_as) || any(!is.na(map$log_initrdev_as))) REPORT(logprior_initrdev_as)
  if (is.null(map$log_rdev_ys) || any(!is.na(map$log_rdev_ys))) REPORT(logprior_rdev_ys)
  if (nr > 1 && "mov_g_ymars" %in% random) REPORT(logprior_dist_ymas)

  return(fn)
}

#' @importFrom methods slotNames
getAllS4 <- function (..., warn = TRUE) {
  fr <- parent.frame()
  dots <- list(...)

  for(i in 1:length(dots)) {
    nm <- slotNames(dots[[i]])
    for (j in nm) {
      if (warn && !is.null(fr[[j]])) warning("Object '", j, "' already defined")
      fr[[j]] <- slot(dots[[i]], j)
    }
  }
  invisible(NULL)
}

make_rmd_CAA <- function(f, r, fname, rname) {
  cap <- paste0("Observed (black) and predicted (red) catch at age for ", fname, " in ", rname)
  cap2 <- paste0("Z-score residuals for the catch at age for ", fname, " in ", rname)
  cap3 <- paste0("Residual histogram for the catch at age for ", fname, " in ", rname)

  rmd <- c(
    paste0("```{r caa-f", f, "-r", r, ", fig.cap=\"", cap, ".\"}"),
    paste0("if (length(dat@Dfishery@CAAobs_ymafr) && any(dat@Dfishery@CAAobs_ymafr[, , , ", f, ", ", r, "] > 0, na.rm = TRUE)) {"),
    paste0("  plot_CAA(x, f = ", f, ", r = ", r, ")"),
    "}",
    "```",
    "",
    paste0("```{r caa-resid-f", f, "-r", r, ", fig.cap=\"", cap2, ".\"}"),
    paste0("if (length(dat@Dfishery@CAAobs_ymafr) && any(dat@Dfishery@CAAobs_ymafr[, , , ", f, ", ", r, "] > 0, na.rm = TRUE)) {"),
    paste0("  plot_resid_CAA(x, f = ", f, ", r = ", r, ")"),
    "}",
    "```",
    "",
    paste0("```{r caa-hist-resid-f", f, "-r", r, ", fig.cap=\"", cap3, ".\"}"),
    paste0("if (length(dat@Dfishery@CAAobs_ymafr) && any(dat@Dfishery@CAAobs_ymafr[, , , ", f, ", ", r, "] > 0, na.rm = TRUE)) {"),
    paste0("  plot_resid_CAA(x, f = ", f, ", r = ", r, ", do_hist = TRUE)"),
    "}",
    "```",
    ""
  )

  return(rmd)
}

make_rmd_CAL <- function(f, r, fname, rname) {
  cap <- paste0("Catch at length for ", fname, " in ", rname)
  cap2 <- paste0("Z-score residuals for the catch at length for ", fname, " in ", rname)
  cap3 <- paste0("Residual histogram for the catch at length for ", fname, " in ", rname)

  rmd <- c(
    paste0("```{r cal-f", f, "-r", r, ", fig.cap=\"", cap, ".\"}"),
    paste0("if (length(dat@Dfishery@CALobs_ymlfr) && any(dat@Dfishery@CALobs_ymlfr[, , , ", f, ", ", r, "] > 0, na.rm = TRUE)) {"),
    paste0("  plot_CAL(x, f = ", f, ", r = ", r, ")"),
    "}",
    "```",
    "",
    paste0("```{r cal-resid-f", f, "-r", r, ", fig.cap=\"", cap2, ".\"}"),
    paste0("if (length(dat@Dfishery@CALobs_ymlfr) && any(dat@Dfishery@CALobs_ymlfr[, , , ", f, ", ", r, "] > 0, na.rm = TRUE)) {"),
    paste0("  plot_resid_CAL(x, f = ", f, ", r = ", r, ")"),
    "}",
    "```",
    "",
    paste0("```{r cal-hist-resid-f", f, "-r", r, ", fig.cap=\"", cap3, ".\"}"),
    paste0("if (length(dat@Dfishery@CALobs_ymlfr) && any(dat@Dfishery@CALobs_ymlfr[, , , ", f, ", ", r, "] > 0, na.rm = TRUE)) {"),
    paste0("  plot_resid_CAL(x, f = ", f, ", r = ", r, ", do_hist = TRUE)"),
    "}",
    "```",
    ""
  )

  return(rmd)
}



make_rmd_fishery <- function(f, fname, nm = 1, rname, nr = length(rname)) {
  Frate <- ifelse(nm > 1, "season", "year")

  rmd_CAA <- sapply(1:nr, function(i) {
    make_rmd_CAA(f = f, r = i, fname = fname, rname = rname[i])
  }) |> as.character()

  rmd_CAL <- sapply(1:nr, function(i) {
    make_rmd_CAL(f = f, r = i, fname = fname, rname = rname[i])
  }) |> as.character()

  rmd <- c(
    paste("###", fname, "{.tabset}"),
    "",
    "#### Fits and residuals",
    "",
    paste0("```{r catch-", f, ", fig.cap=\"Catch of ", fname, ".\"}"),
    paste0("plot_catch(x, f = ", f, ")"),
    "```",
    "",
    paste0("```{r catch-annual-", f, ", fig.cap=\"Catch of ", fname, ", aggregated to year.\"}"),
    paste0("if (nm > 1) plot_catch(x, f = ", f, ", annual = TRUE)"),
    "```",
    "",
    paste0("```{r catch-resid-", f, ", fig.cap=\"Catch residuals of ", fname, ".\"}"),
    paste0("plot_resid_Cobs(x, f = ", f, ")"),
    "```",
    "",
    rmd_CAA,
    rmd_CAL,
    "#### Estimates",
    "",
    paste0("```{r sel-", f, ", fig.cap=\"Selectivity by length.\"}"),
    paste0("plot_self(x, f = ", f, ")"),
    "```",
    "",
    paste0("```{r sel-age-", f, ", fig.cap=\"Selectivity by age.\"}"),
    paste0("plot_self(x, f = ", f, ", type = \"age\")"),
    "```",
    "",
    paste0("```{r Ffleet-", f, ", fig.cap=\"Apical fishing mortality of ", fname, " (instantaneous rate, per ", Frate, ").\"}"),
    paste0("plot_Ffleet(x, f = ", f, ")"),
    "```",
    "",
    paste0("```{r catch-region-prop-", f, ", fig.cap=\"Proportion catch by region from ", fname, ".\"}"),
    "if (nr > 1) {",
    paste0("  plot_catch(x, f = ", f, ", by = \"region\", prop = TRUE)"),
    "}",
    "```",
    paste0("```{r catch-region-prop-annual-", f, ", fig.cap=\"Proportion catch by region from ", fname, ", catch aggregated to year.\"}"),
    "if (nm > 1 && nr > 1) {",
    paste0("  plot_catch(x, f = ", f, ", by = \"region\", prop = TRUE, annual = TRUE)"),
    "}",
    "```",
    "",
    paste0("```{r catch-stock-", f, ", fig.cap=\"Catch by stock of ", fname, ".\"}"),
    "if (ns > 1) {",
    paste0("  plot_catch(x, f = ", f, ", by = \"stock\")"),
    "}",
    "```",
    "",
    paste0("```{r catch-stock-annual-", f, ", fig.cap=\"Catch by stock from ", fname, ", catch aggregated to year.\"}"),
    "if (ns > 1) {",
    paste0("  plot_catch(x, f = ", f, ", by = \"stock\", annual = TRUE)"),
    "}",
    "```",
    "",
    paste0("```{r catch-stock-prop-", f, ", fig.cap=\"Proportion catch by stock from ", fname, ".\"}"),
    "if (ns > 1) {",
    paste0("  plot_catch(x, f = ", f, ", by = \"stock\", prop = TRUE)"),
    "}",
    "```",
    "",
    paste0("```{r catch-stock-prop-annual-", f, ", fig.cap=\"Proportion catch by stock from ", fname, ", catch aggregated to year.\"}"),
    "if (nm > 1 && ns > 1) {",
    paste0("  plot_catch(x, f = ", f, ", by = \"stock\", prop = TRUE, annual = TRUE)"),
    "}",
    "```",
    "",
    paste0("```{r VB-", f, ", fig.cap=\"Vulnerable biomass.\"}"),
    paste0("plot_V(x, f = ", f, ", by = \"stock\")"),
    "```",
    "",
    paste0("```{r VB-stock-prop-", f, ", fig.cap=\"Proportion vulnerable biomass by stock.\"}"),
    "if (ns > 1) {",
    paste0("  plot_V(x, f = ", f, ", by = \"stock\", prop = TRUE)"),
    "}",
    "```",
    "",
    #paste0("```{r VB-region-", f, ", fig.cap=\"Vulnerable biomass by region.\"}"),
    #"if (nr > 1) {",
    #paste0("  plot_V(x, f = ", f, ", by = \"region\")"),
    #"}",
    #"```",
    #"",
    paste0("```{r VB-region-prop-", f, ", fig.cap=\"Proportion vulnerable biomass by region.\"}"),
    "if (nr > 1) {",
    paste0("  plot_V(x, f = ", f, ", by = \"region\", prop = TRUE)"),
    "}",
    "```",
    ""
  )

  return(rmd)
}



make_rmd_survey <- function(i, iname) {

  rmd <- c(
    ifelse(i == 1, "## Surveys {.tabset}\n", ""),
    paste("###", iname, "{.tabset}"),
    "",
    paste0("```{r isel-", i, ", fig.cap=\"Selectivity for ", iname, ".\"}"),
    paste0("plot_seli(x, i = ", i, ")"),
    "```",
    "",
    paste0("```{r index-", i, ", fig.cap=\"Predicted (red) and observed (black) values for ", iname, ".\"}"),
    paste0("plot_index(x, i = ", i, ")"),
    "```",
    "",
    paste0("```{r index-zoom-", i, ", fig.cap=\"Predicted (red) and observed (black) values for ", iname, ", predicted values reported for years and seasons with data points.\"}"),
    paste0("plot_index(x, i = ", i, ", zoom = TRUE)"),
    "```",
    "",
    paste0("```{r IAA-", i, ", fig.cap=\"Age composition from ", iname, ".\"}"),
    paste0("plot_IAA(x, i = ", i, ")"),
    "```",
    "",
    paste0("```{r IAA-resid-", i, ", fig.cap=\"Z-score residuals for the age composition from ", iname, ".\"}"),
    paste0("plot_resid_IAA(x, i = ", i, ")"),
    "```",
    "",
    paste0("```{r IAA-hist-resid-", i, ", fig.cap=\"Residual histogram for the age composition from ", iname, ".\"}"),
    paste0("plot_resid_IAA(x, i = ", i, ", do_hist = TRUE)"),
    "```",
    "",
    paste0("```{r IAL-", i, ", fig.cap=\"Length composition from ", iname, ".\"}"),
    paste0("plot_IAL(x, i = ", i, ")"),
    "```",
    "",
    paste0("```{r IAL-resid-", i, ", fig.cap=\"Z-score residuals for the length composition from ", iname, ".\"}"),
    paste0("plot_resid_IAL(x, i = ", i, ")"),
    "```",
    "",
    paste0("```{r IAL-hist-resid-", i, ", fig.cap=\"Residual histogram for the length composition from ", iname, ".\"}"),
    paste0("plot_resid_IAL(x, i = ", i, ", do_hist = TRUE)"),
    "```",
    ""
  )

  return(rmd)
}

make_rmd_stock_region <- function(s, sname, ns = 1) {

  if (ns > 1) {
    rmd <- c(
      ifelse(s == 1, "### By region\n\n", ""),
      paste0("```{r SB-r", s, ", fig.cap=\"Spawning output of ", sname, " by region at the spawning season.\"}"),
      paste0("plot_S(x, by = \"region\", s = ", s, ")"),
      "```",
      "",
      paste0("```{r SB-rp", s, ", fig.cap=\"Proportion spawning output of ", sname, " by region.\"}"),
      "plot_S(x, by = \"region\", s = ", s, ", prop = TRUE)",
      "```",
      "",
      paste0("```{r B-r", s, ", fig.cap=\"Total biomass of ", sname, " by region.\"}"),
      paste0("plot_B(x, by = \"region\", s = ", s, ")"),
      "```",
      "",
      paste0("```{r B-rp", s, ", fig.cap=\"Proportion biomass of ", sname, " by region.\"}"),
      "plot_B(x, by = \"region\", s = ", s, ", prop = TRUE)",
      "```",
      ""
    )
  } else {
    rmd <- c(
      ifelse(s == 1, "### By region\n\n", ""),
      paste0("```{r SB-r", s, ", fig.cap=\"Spawning output by region at the spawning season.\"}"),
      paste0("plot_S(x, by = \"region\", s = ", s, ")"),
      "```",
      "",
      paste0("```{r SB-rp", s, ", fig.cap=\"Proportion spawning output by region.\"}"),
      "plot_S(x, by = \"region\", s = ", s, ", prop = TRUE)",
      "```",
      "",
      paste0("```{r B-r", s, ", fig.cap=\"Total biomass by region.\"}"),
      paste0("plot_B(x, by = \"region\", s = ", s, ")"),
      "```",
      "",
      paste0("```{r B-rp", s, ", fig.cap=\"Proportion biomass by region.\"}"),
      "plot_B(x, by = \"region\", s = ", s, ", prop = TRUE)",
      "```",
      ""
    )
  }



  return(rmd)
}


make_rmd_ind_stock <- function(s, sname, ns = 1) {
  if (ns > 1) {
    rmd <- c(
      paste0("```{r selstock-annual-s", s, ", fig.cap=\"Realized annual selectivity of ", sname, " from total annual catch at age and abundance at age at the beginning of the year.\"}"),
      paste0("plot_selstock(x, s = ", s, ", plot2d = \"filled.contour\")"),
      "```",
      "",
      paste0("```{r selstock-season-s", s, ", fig.cap=\"Realized seasonal selectivity of ", sname, " from total catch at age and abundance at age at the beginning of the time step.\"}"),
      paste0("plot_selstock(x, s = ", s, ", by = \"season\", plot2d = \"filled.contour\")"),
      "```",
      "",
      paste0("```{r N-s", s, ", fig.cap=\"Total abundance at age of ", sname, " at the beginning of the year.\"}"),
      paste0("plot_N(x, s = ", s, ", plot2d = \"filled.contour\")"),
      "```",
      "",
      paste0("```{r Rdev-s", s, ", fig.cap=\"Recruitment deviations of ", sname, ".\"}"),
      paste0("plot_Rdev(x, s = ", s, ")"),
      "```",
      "",
      paste0("```{r SRR-s", s, ", fig.cap=\"Stock recruit relationship, and historical stock-recruit pairs, of ", sname, ". The dotted line is the unfished replacement line.\"}"),
      paste0("plot_SRR(x, s = ", s, ")"),
      "```",
      ""
    )
  } else {
    rmd <- c(
      paste0("```{r selstock-annual-s", s, ", fig.cap=\"Realized annual selectivity from total annual catch at age and abundance at age at the beginning of the year.\"}"),
      paste0("plot_selstock(x, s = ", s, ", plot2d = \"filled.contour\")"),
      "```",
      "",
      paste0("```{r selstock-season-s", s, ", fig.cap=\"Realized seasonal selectivity from total catch at age and abundance at age at the beginning of the time step.\"}"),
      paste0("plot_selstock(x, s = ", s, ", by = \"season\", plot2d = \"filled.contour\")"),
      "```",
      "",
      paste0("```{r N-s", s, ", fig.cap=\"Total abundance at age at the beginning of the year.\"}"),
      paste0("plot_N(x, s = ", s, ", plot2d = \"filled.contour\")"),
      "```",
      "",
      paste0("```{r Rdev-s", s, ", fig.cap=\"Recruitment deviations.\"}"),
      paste0("plot_Rdev(x, s = ", s, ")"),
      "```",
      "",
      paste0("```{r SRR-s", s, ", fig.cap=\"Stock recruit relationship, and historical stock-recruit pairs. The dotted line is the unfished replacement line.\"}"),
      paste0("plot_SRR(x, s = ", s, ")"),
      "```",
      ""
    )
  }


  return(rmd)
}


make_rmd_mov <- function(s, y, a, yname, sname, header = TRUE) {
  rmd <- c(
    ifelse(header, "### Movement\n\n", ""),
    paste0("```{r fig.cap=\"Movement matrix for ", sname, " for year ", yname, ", age ", a, " and its corresponding equilibrium distribution over annual cycle (initialized with the recruitment distribution).\"}"),
    paste0("plot_mov(x, s = ", s, ", y = ", y, ", a = ", a, ")"),
    "```",
    ""
  )

  return(rmd)
}


make_rmd_SC <- function(f, a, r, fname, aname, rname, header = TRUE) {
  cap <- paste0("Predicted stock composition for ", fname, ", ", aname, ", in ", rname)
  cap2 <- paste0("Fits to stock composition (observed in black, predicted in red) for ", fname, ", ", aname, ", in ", rname)
  cap3 <- paste0("Z-score residuals of for ", fname, ", ", aname, ", in ", rname)
  cap4 <- paste0("Histogram of Z-score residuals for ", fname, ", ", aname, ", in ", rname)

  rmd <- c(
    ifelse(f == 1 && a == 1, paste0("### ", rname, " {.tabset}\n\n"), ""),
    paste0("```{r sc-prop-f", f, "a-", a, "-r", r, ", fig.cap=\"", cap, ".\"}"),
    paste0("plot_SC(x, ff = ", f, ", aa = ", a, ", r = ", r, ", prop = TRUE)"),
    "```",
    "",
    paste0("```{r sc-fit-f", f, "a-", a, "-r", r, ", fig.cap=\"", cap2, ".\"}"),
    paste0("plot_SC(x, ff = ", f, ", aa = ", a, ", r = ", r, ", prop = FALSE)"),
    "```",
    "",
    paste0("```{r sc-resid-f", f, "a-", a, "-r", r, ", fig.cap=\"", cap3, ".\"}"),
    paste0("plot_resid_SC(x, f = ", f, ", a = ", a, ", r = ", r, ")"),
    "```",
    "",
    paste0("```{r sc-resid-hist-f", f, "a-", a, "-r", r, ", fig.cap=\"", cap4, ".\"}"),
    paste0("plot_resid_SC(x, f = ", f, ", a = ", a, ", r = ", r, ", do_hist = TRUE)"),
    "```",
    ""
  )

  return(rmd)
}


make_rmd_tagmov <- function(y, a, s, yname, aname, sname, header = TRUE) {
  cap <- paste0("Fits to tag movement (observed in black, predicted in red) for ", sname, ", ", yname, ", ", aname)
  cap_resid <- paste0("Z-score residuals of tag movement for ", sname, ", ", yname, ", ", aname)

  rmd <- c(
    ifelse(header, paste0("### ", sname, " {.tabset}\n\n"), ""),
    paste0("```{r tagmov-y", y, "a-", a, "-s", s, ", fig.cap=\"", cap, ".\"}"),
    paste0("plot_tagmov(x, s = ", s, ", yy = ", y, ", aa = ", a, ")"),
    "```",
    "",
    paste0("```{r tagmov-resid-y", y, "a-", a, "-s", s, ", fig.cap=\"", cap, ".\"}"),
    paste0("plot_resid_tagmov(x, s = ", s, ", yy = ", y, ", aa = ", a, ")"),
    "```",
    ""
  )

  return(rmd)
}

#' Jitter starting values from fitted model
#'
#' Run additional model fits with jittered starting values.
#'
#' @param x [MSAassess-class] object returned by [fit_MSA()]
#' @param n Integer, number of jittered model runs
#' @param use_fitted Logical, whether to jitter from estimated parameters (`TRUE`) or original starting values (`FALSE`)
#' @param return_models Logical, whether to return fitted models of the jitter runs
#' @param amount Numeric or NULL, passed to [base::jitter()]
#' @param cores Integer, number of CPUs for parallel processing
#' @param seed Integer, for replicating the sampling function. Optional.
#' @param ... Other arguments to pass to [fit_MSA()]
#' @details The new starting parameters are: `pars + amount * runif(n, -1, 1)` where `pars` are either the fitted values or original starting
#' values depending on `use_fitted`.
#' @returns If `return_models = TRUE`, a list (length `n`) containing [MSAassess-class] objects.
#' Otherwise, a data frame of likelihood components made by [get_likelihood_components()]
#' @export
#' @importFrom parallel parLapplyLB
do_jitter <- function(x, n = 1, use_fitted = TRUE, return_models = TRUE, amount = NULL, cores = 1, seed, ...) {

  if (use_fitted) {
    pars <- x@obj$env$last.par.best
  } else {
    pars <- x@obj$par
  }

  if (!missing(seed)) set.seed(seed)
  jit <- lapply(seq_len(n), function(...) jitter(pars, amount = amount))

  jitter_fn <- function(i, fitted, return_models, ...) {

    dat <- get_MSAdata(fitted)
    parameters <- fitted@obj$env$parList(par = i)

    map <- dat@Misc$map
    random <- dat@Misc$random

    fit <- fit_MSA(dat, parameters, map, random, silent = TRUE, ...)

    if (return_models) {
      return(fit)
    } else {
      return(get_likelihood_components(fit))
    }
  }

  if (cores > 1) {
    cl <- parallel::makeCluster(cores)
    on.exit(parallel::stopCluster(cl))
    fits <- parLapplyLB(cl, X = jit, jitter_fn, fitted = x, return_models = return_models, ...)
  } else {
    fits <- lapply(cl, X = jit, jitter_fn, fitted = x, return_models = return_models, ...)
  }

  if (return_models) {
    return(fits)
  } else {
    out <- cbind(
      data.frame(Run = 1:n),
      do.call(rbind, fits)
    )
    return(out)
  }


#
  #init <- fit_MSA(
  #  dat,
  #  parameters = parameters,
  #  map = map,
  #  random = random,
  #  run_model = FALSE
  #)
#
  #init@obj$par[] <- jit
  #fit <- fit_MSA(init, ...)
  #return(fit)
}

prior_sel <- function() {

}


prior_R0x <- function() {

}

alphaconv <- function(m, sd) m * (((m * (1 - m))/(sd^2)) - 1)
betaconv <- function(m, sd) (1 - m) * (((m * (1 - m))/(sd^2)) - 1)
sdconv <- function(m, sd) (log(1 + ((sd^2)/(m^2))))^0.5

#' @name prior
#'
#' @title Priors for MSA model
#' @description
#' Priors in MSA are set by providing character strings which are then parsed into an expression and evaluated in the model environment
#' (see example). This provides flexibility to set a prior for any desired model parameter or variable. See list of parameters in the
#' documentation for `[check_parameters()]` for options (note that priors for `log_rdev_ys` and `log_initrdev_as` are not needed as they're hard-coded into the model).
#' Several functions below generate the character string for the prior for important dynamics parameters, such as natural mortality and steepness.
#'
#' @param MSAdata Data object. Class \linkS4class{MSAdata}
#' @return Character.
#' @examples
#' # Add M and steepness prior to model
#'
#' dat <- new("MSAdata")
#' dat@Dmodel@ns <- 1
#' dat@Dstock@SRR_s <- "BH"
#'
#' pr_M <- prior_M(dat, s = 1, log(0.25), 0.3)
#' pr_h <- prior_h(dat, s = 1, 0.8, 0.15)
#' dat@Dmodel@prior <- c(pr_M, pr_h)
NULL

#' @name prior
#' @details - `prior_h` returns the log prior for stock-recruit steepness. Beta distribution for the Beverton-Holt function and normal distribution for Ricker function.
#' @param s Integer for stock
#' @param m Mean in un-transformed space
#' @param stdev Standard deviation in un-transformed space
#' @export
prior_h <- function(MSAdata, s = 1, m, stdev) {
  SRR <- match.arg(MSAdata@Dstock@SRR_s[s], choices = c("BH", "Ricker"))

  if (SRR == "BH") {

    m_beta <- 1.25 * m - 0.25
    stdev_beta <- 1.25 * stdev

    a <- alphaconv(m_beta, stdev_beta) |> round(3)
    b <- betaconv(1.25 * m - 0.25, 1.25 * stdev) |> round(3)

    if (a <= 0) stop("Beta distribution alpha parameter < 0. Try reducing the prior SD.", call. = FALSE)
    if (b <= 0) stop("Beta distribution beta parameter < 0. Try reducing the prior SD.", call. = FALSE)

    txt <- paste0("dbeta(plogis(p$t_h_s[", s, "]), ", a, ", ", b, ", log = TRUE) + log(plogis(p$t_h_s[", s, "]) - plogis(p$t_h_s[", s, "])^2)")

  } else {
    txt <- paste0("dnorm(exp(p$t_h_s[", s, "]) + 0.2, ", round(m, 3), ", ", round(stdev, 3), ", log = TRUE) + p$t_h_s[", s, "]")
  }
  return(txt)
}

#' @name prior
#' @details - `prior_M` returns the log prior for natural mortality. Lognormal distribution.
#' @aliases prior_M
#' @param meanlog Mean of the lognormal distribution on the log scale
#' @param sdlog Standard of the lognormal distribution on the log scale
#' @export
prior_M <- function(MSAdata, s = 1, meanlog, sdlog) {
  txt <- paste0("dnorm(p$log_M_s[", s, "], ", round(meanlog, 3), ", ", round(sdlog, 3), ", log = TRUE)")
  return(txt)
}

#' @name prior
#' @details - `prior_q` returns the log prior for index catchability. Lognormal distribution.
#' @aliases prior_q
#' @param i Integer for the corresponding index
#' @export
prior_q <- function(MSAdata, i = 1, meanlog, sdlog) {
  txt <- paste0("dnorm(log(q_i[", i, "]), ", round(meanlog, 3), ", ", round(sdlog, 3), ", log = TRUE) - log(q_i[", i, "])")
  return(txt)
}

#' @importFrom graphics grid matplot abline par
plot.default <- function(..., zero_line = FALSE) {
  if (zero_line) {
    graphics::plot.default(..., panel.first = {graphics::grid(); abline(h = 0, col = "grey60")})
  } else {
    graphics::plot.default(..., panel.first = graphics::grid())
  }
}

matplot <- function(..., zero_line = FALSE) {
  if (zero_line) {
    graphics::matplot(..., panel.first = {graphics::grid(); abline(h = 0, col = "grey60")})
  } else {
    graphics::matplot(..., panel.first = graphics::grid())
  }
}

#' @importFrom grDevices hcl.colors
make_color <- function(n, type = c("fleet", "region", "stock"), alpha = 1) {
  if (n > 1) {
    type <- match.arg(type)
    pal <- switch(
      type,
      "fleet" = "Set2",
      "region" = "Sunset",
      "stock" = "Plasma"
    )
    grDevices::hcl.colors(n, palette = pal, alpha = alpha)
  } else {
    "black"
  }
}

#' @importFrom tinyplot tinyplot
make_tinyplot <- function(year, x, ylab, name, color, type = "o",
                          leg = if (ncol(x) > 1) substitute(legend(title = NULL)) else "none",
                          ylim = c(0, 1.1) * range(x, na.rm = TRUE)) {

  df <- structure(x, dimnames = list(year = year, by = name)) |>
    reshape2::melt()

  tinyplot_args <- list(
    x = df$year, y = df$value, by = factor(df$by), xlab = "Year", ylab = ylab,
    type = type,
    legend = leg,
    col = color,
    pch = 16,
    ylim = ylim,
    grid = TRUE
  )
  do.call(tinyplot, tinyplot_args)
  abline(h = 0, col = "grey60")
  box()

  invisible()
}

#' @importFrom graphics box legend
#' @importFrom tinyplot tinyplot type_barplot type_lines
#' @importFrom reshape2 melt
barplot2 <- function(x, cols, leg.names, facet.names = NULL, xval, ylab = ifelse(prop, "Proportion", "Value"),
                     border = ifelse(nrow(x) > 60, NA, "grey60"), prop = TRUE, facet.free = FALSE) {

  if (length(x) == dim(x)[1]) {
    if (prop) x[] <- 1
    plot(xval, as.numeric(x), xlab = "Year", ylab = ylab, ylim = c(0, 1.1) * range(x), pch = 16, type = "o", zero_line = TRUE)
    return(invisible())
  }

  ndim <- length(dim(x))
  if (ndim == 2) {
    if (prop) {
      p <- apply(x, 1, function(x) x/sum(x, na.rm = TRUE))
      if (is.null(dim(p))) p <- matrix(p, 1, length(xval))
      ylim <- c(0, 1)
    } else {
      p <- t(x)
      ylim <- c(0, 1.1) * range(rowSums(x))
    }

    df <- structure(p, dimnames = list(by = leg.names, xval = xval)) |>
      reshape2::melt()

  } else if (ndim == 3) {
    if (prop) {
      p <- apply(x, c(1, 3), function(x) x/sum(x, na.rm = TRUE))
      if (is.null(dim(p))) p <- matrix(p, 1, length(xval))
      ylim <- c(0, 1)
    } else {
      p <- aperm(x, c(2, 1, 3))
      ylim <- c(0, 1.1) * range(apply(p, 1:2, sum))
    }

    df <- structure(p, dimnames = list(by = leg.names, xval = xval, facet = facet.names)) |>
      reshape2::melt()

  } else {
    stop("Dimension not usable by barplot2")
  }

  do.facet <- ndim == 3 && length(unique(df$facet)) > 1
  do.group <- dim(p)[1] > 1

  if (missing(cols)) {
    ncat <- nrow(p)
    cols <- make_color(ncat)
  }

  if (do.group) {

    tinyplot_args <- list(
      x = df$xval, y = df$value,
      by = df$by,
      grid = TRUE,
      legend = substitute(legend(title = NULL)),
      facet = if (do.facet) df$facet else NULL,
      facet.args = if (do.facet) list(free = facet.free) else NULL,
      xlab = "Year", ylab = ylab,
      type = type_barplot(width = 1),
      palette = cols,
      col = border,
      ylim = NULL
    )

  } else {

    tinyplot_args <- list(
      x = df$xval, y = df$value,
      by = NULL,
      grid = TRUE,
      legend = NULL,
      facet = if (do.facet) df$facet else NULL,
      facet.args = if (do.facet) list(free = facet.free) else NULL,
      xlab = "Year", ylab = ylab,
      type = type_lines(),
      palette = NULL,
      col = cols,
      ylim = c(0, 1.1) * range(df$value)
    )

  }

  do.call(tinyplot, tinyplot_args)
  if (!do.facet) box()

  invisible()
}


# State variable plots ----
#' @name plot-MSA-state
#' @title Plotting functions for fitted MSA model
#' @description A set of functions to plot state variables (biomass, recruitment time series, etc.)
#' @return Invisible data frame of state variables that were plotted in base graphics figures
NULL

#' @rdname plot-MSA-state
#' @aliases plot_S
#'
#' @param fit [MSAassess-class] object returned by [fit_MSA()]
#' @param by Character to indicate dimension for multivariate plots
#' @param s Integer for the corresponding stock
#' @param prop Logical, whether to plot proportions (TRUE) or absolute numbers
#' @param facet_free Logical, whether to allow the y-axis limits to vary by panel in facetted plots
#' @param figure, Logical, whether to generate the plot
#' @param ylab Optional character string for custom y-axis label
#' @details
#' - `plot_S` plots spawning output by stock or region (either whole numbers or proportions for the latter)
#'
#' @export
plot_S <- function(fit, by = c("stock", "region"), r, s, prop = FALSE, facet_free = FALSE, figure = TRUE,
                   ylab) {
  by <- match.arg(by)
  var <- "S_yrs"
  output <- NULL

  d <- get_MSAdata(fit)
  Dlabel <- d@Dlabel
  Dmodel <- d@Dmodel

  year <- Dlabel@year
  ny <- length(year)

  if (missing(r)) r <- 1:Dmodel@nr
  if (missing(s)) s <- 1:Dmodel@ns
  rname <- Dlabel@region[r]
  sname <- Dlabel@stock[s]

  if (by == "stock") {
    leg.name <- sname
    facet.name <- rname
    x <- array(fit@report[[var]][, r, s, drop = FALSE], c(ny, length(rname), length(sname))) |>
      aperm(c(1, 3, 2))

    output <- structure(x, dimnames = list(year = year, stock = sname, region = rname))
  } else {
    leg.name <- rname
    facet.name <- sname
    x <- array(fit@report[[var]][, r, s, drop = FALSE], c(ny, length(rname), length(sname)))

    output <- structure(x, dimnames = list(year = year, region = rname, stock = sname))
  }

  if (figure) {
    color <- make_color(ncol(x), type = by)

    if (missing(ylab)) {
      ylab <- ifelse(prop, "Spawning fraction", "Spawning output")
    }
    barplot2(x, cols = color, leg.names = leg.name, facet.names = facet.name, xval = year, ylab = ylab, prop = prop,
             facet.free = facet_free)
  }

  invisible(reshape2::melt(output, value.name = "S"))
}


#' @rdname plot-MSA-state
#' @aliases plot_B
#' @details
#' - `plot_B` plots total biomass by stock or region (either whole numbers or proportions for the latter)
#'
#' @export
plot_B <- function(fit, by = c("stock", "region"), r, s, prop = FALSE, facet_free = FALSE, figure = TRUE) {
  by <- match.arg(by)
  var <- "B_ymrs"
  output <- NULL

  d <- get_MSAdata(fit)
  Dlabel <- d@Dlabel
  Dmodel <- d@Dmodel

  year <- Dlabel@year
  ny <- length(year)
  nm <- max(length(Dlabel@season), 1)

  if (missing(r)) r <- 1:Dmodel@nr
  if (missing(s)) s <- 1:Dmodel@ns
  rname <- Dlabel@region[r]
  sname <- Dlabel@stock[s]

  if (by == "stock") {
    leg.name <- sname
    facet.name <- rname
    x <- array(fit@report[[var]][, , r, s, drop = FALSE], c(ny, nm, length(rname), length(sname))) |>
      aperm(c(1, 2, 4, 3)) # B_ymsr
  } else {
    leg.name <- rname
    facet.name <- sname
    x <- array(fit@report[[var]][, , r, s, drop = FALSE], c(ny, nm, length(rname), length(sname))) # B_ymrs
  }

  year <- make_yearseason(year, nm)
  x <- collapse_yearseason(x)

  if (by == "stock") {
    output <- structure(x, dimnames = list(year = year, stock = sname, region = rname))
  } else {
    output <- structure(x, dimnames = list(year = year, region = rname, stock = sname))
  }

  if (figure) {
    color <- make_color(ncol(x), type = by)

    if (prop) {
      ylab <- "Biomass fraction"
    } else {
      ylab <- "Total biomass"
    }

    barplot2(x, cols = color, leg.names = leg.name, facet.names = facet.name, xval = year, ylab = ylab, prop = prop,
             facet.free = facet_free)
  }

  invisible(reshape2::melt(output, value.name = "B"))
}



#' @rdname plot-MSA-state
#' @aliases plot_R
#' @details
#' - `plot_R` plots recruitment by stock
#'
#' @export
plot_R <- function(fit, s, figure = TRUE) {
  var <- "R_ys"

  Dlabel <- get_MSAdata(fit)@Dlabel
  year <- Dlabel@year

  if (missing(s)) {
    x <- fit@report[[var]]
    name <- Dlabel@stock
    ylab <- "Recruitment"
  } else {
    x <- fit@report[[var]][, s, drop = FALSE]
    name <- Dlabel@stock[s]
    ylab <- paste(name, "recruitment")
  }

  if (figure) {
    color <- make_color(ncol(x), "stock")
    make_tinyplot(year, x, ylab, name, color)
  }

  x <- structure(x, dimnames = list(year = year, stock = name))
  invisible(array2DF(x, "R"))
}


#' @rdname plot-MSA-state
#' @aliases plot_SRR
#' @param phi Logical, whether to plot unfished replacement line
#' @details
#' - `plot_SRR` plots the stock-recruitment relationship and history (realized recruitment) by stock
#' @importFrom graphics points lines
#' @export
plot_SRR <- function(fit, s = 1, phi = TRUE) {
  dat <- get_MSAdata(fit)
  Dlabel <- get_MSAdata(fit)@Dlabel

  S_y <- apply(fit@report$S_yrs[, , s, drop = FALSE], 1, sum)
  R_y <- fit@report$R_ys[, s]

  Rpred_y <- R_y/fit@report$Rdev_ys[, s]

  a <- fit@report$sralpha_s[s]
  b <- fit@report$srbeta_s[s]

  S_SRR <- seq(0, max(S_y), length.out = 100)
  R_SRR <- calc_recruitment(S_SRR, SRR = dat@Dstock@SRR_s[s], a = a, b = b)

  S2 <- S_y[-1]
  R2 <- Rpred_y[-1]

  plot(S_SRR, R_SRR, type = "l", lwd = 1, lty = 3,
       xlab = "Spawning output", ylab = "Recruitment",
       xaxs = "i", yaxs = "i", xlim = c(0, 1.1) * range(S_y), ylim = c(0, 1.1) * range(R_y))
  lines(S2[order(S2)], R2[order(R2)], lwd = 2)
  points(S_y, R_y, pch = 16)

  if (phi) {
    phi_s <- fit@report$phi_s[s]
    abline(a = 0, b = 1/phi_s, lty = 2)
  }
  invisible()
}


#' @rdname plot-MSA-state
#' @aliases plot_Rdev
#' @param log Logical, whether to plot the natural logarithm of the response variable
#' @details
#' - `plot_Rdev` plots recruitment deviations by stock
#' @importFrom graphics arrows
#' @export
plot_Rdev <- function(fit, s = 1, log = TRUE, figure = TRUE) {

  Dlabel <- get_MSAdata(fit)@Dlabel
  year <- Dlabel@year

  if (log) {
    if (length(fit@SD) > 1) {
      x <- as.list(fit@SD, what = "Estimate")$log_rdev_ys[, s]
      std <- as.list(fit@SD, what = "Std. Error")$log_rdev_ys[, s]
      std[is.na(std)] <- 0
    } else {
      x <- fit@obj$env$parList(par = fit@obj$env$last.par.best)$log_rdev_ys[, s]
      std <- numeric(length(x))
    }

    upper <- x + 1.96 * std
    lower <- x - 1.96 * std

    if (figure) {
      plot(year, x, xlab = "Year", ylab = "log Recruitment deviations", type = "o", pch = 16,
           ylim = range(lower, upper), lty = 3)
      arrows(x0 = year, y0 = lower, y1 = upper, length = 0)
      abline(h = 0, lty = 2)
    }

  } else {
    x <- fit@report$Rdev_ys[, s]

    if (figure) {
      plot(year, x, xlab = "Year", ylab = "Recruitment deviations", type = "o", pch = 16,
           ylim = c(0, 1.1) * range(x, na.rm = TRUE), zero_line = TRUE)
      abline(h = 1, lty = 2)
    }
  }
  output <- data.frame(
    year = year,
    dev = x,
    stock = Dlabel@stock[s]
  )

  if (log) {
    output$lwr <- lower
    output$upr <- upper
  }
  invisible(output)
}

#' @rdname plot-MSA-state
#' @aliases plot_Fstock
#' @details
#' - `plot_Fstock` plots apical instantaneous fishing mortality (per year or per season) by stock
#'
#' @export
plot_Fstock <- function(fit, s, by = c("annual", "season"), figure = TRUE) {
  by <- match.arg(by)

  dat <- get_MSAdata(fit)
  year <- dat@Dlabel@year
  nm <- dat@Dmodel@nm

  unit <- ifelse(by == "annual" || nm == 1, "year", "season")

  if (missing(s)) s <- seq(1, dat@Dmodel@ns)

  if (by == "annual" || nm == 1) {
    var <- "F_yas"
    x <- fit@report[[var]][, , s, drop = FALSE] |>
      apply(c(1, 3), max) |>
      structure(dimnames = list(year = year, stock = dat@Dlabel@stock[s]))

  } else {

    F_ymas <- sapply2(s, function(ss) {
      sapply2(1:dat@Dmodel@na, function(a) {
        sapply(1:nm, function(m) {
          sapply(1:dat@Dmodel@ny, function(y) {
            N <- sum(fit@report$N_ymars[y, m, a, , ss])
            CN <- sum(fit@report$CN_ymafrs[y, m, a, , , ss])
            calc_summary_F(M = fit@report$M_yas[y, a, ss]/nm, N = N, CN = CN, Fmax = 100)
          })
        })
      })
    })

    year <- make_yearseason(year, nm)

    x <- apply(F_ymas, c(1, 2, 4), max) |>
      collapse_yearseason() |>
      structure(dimnames = list(year = year, stock = dat@Dlabel@stock[s]))

  }

  if (figure) {
    x[is.infinite(x)] <- NA

    if (length(s) > 1) {
      name <- dat@Dlabel@stock
      ylab <- paste0("Apical fishing mortality (per ", unit, ")")
    } else {
      name <- dat@Dlabel@stock[s]
      ylab <- paste0(name, " apical fishing mortality (per ", unit, ")")
    }

    color <- make_color(ncol(x), "stock")
    make_tinyplot(year, x, ylab, name, color)
    #matplot(year, x, xlab = "Year", ylab = ylab, type = "o", col = color, pch = 16, lty = 1,
    #        ylim = c(0, 1.1) * range(x, na.rm = TRUE), zero_line = TRUE)
    #if (ncol(x) > 1) legend("topleft", legend = name, col = color, lwd = 1, pch = 16, horiz = TRUE)

  }

  invisible(reshape2::melt(x, value.name = "F"))
}



#' @rdname plot-MSA-state
#' @aliases plot_self
#' @param f Integer for the corresponding fleet
#' @param type For `plot_self`, indicates whether to plot the selectivity by age or length.
#' @details
#' - `plot_self` plots fishery selectivity
#' @export
plot_self <- function(fit, f = 1, type = c("length", "age"), figure = TRUE) {
  type <- match.arg(type)

  dat <- get_MSAdata(fit)
  Dfishery <- dat@Dfishery

  sel_block <- Dfishery@sel_block_yf[, f]
  sel_b <- Dfishery@sel_f[unique(sel_block)]
  fname <- dat@Dlabel@fleet[f]

  year <- dat@Dlabel@year

  output <- NULL

  if (type == "length" && all(grepl("length", sel_b))) {
    lmid <- dat@Dmodel@lmid
    x <- fit@report$sel_lf[, unique(sel_block), drop = FALSE]

    if (ncol(x) > 1) {
      name <- sapply(unique(sel_block), function(i) {
        y <- year[sel_block == i]
        if (length(y) == 1) {
          return(y)
        } else {
          return(paste(range(y), collapse = "-"))
        }
      })
    } else {
      name <- "all"
    }

    if (figure) {
      color <- make_color(ncol(x), "fleet")
      matplot(lmid, x, xlab = "Length", ylab = paste(fname, "selectivity"),
              type = "o", col = color, pch = 16,
              ylim = c(0, 1), lty = 1, zero_line = TRUE)
      if (ncol(x) > 1) legend("topright", legend = name, col = color, lwd = 1, pch = 16)
    }
    output <- structure(t(x), dimnames = list(year = name, length = lmid)) |>
      reshape2::melt(value.name = "sel")
    output$fleet <- fname

  } else if (type == "age") {

    m <- 1
    s <- 1

    x <- fit@report$sel_ymafs[, m, , f, s]
    xx <- apply(x, 2, diff)

    age <- dat@Dlabel@age

    if (any(xx != 0)) {
      ybreak <- c(1, which(rowSums(xx) > 0) + 1)
      name <- sapply(1:length(ybreak), function(i) {
        if (i == length(ybreak)) {
          y <- c(year[ybreak[i]], year[length(year)])
        } else {
          y <- year[c(ybreak[i], ybreak[i+1] - 1)]
        }
        paste(range(y), collapse = "-")
      })
      x <- x[ybreak, , drop = FALSE]

      output <- structure(x, dimnames = list(year = name, age = age)) |>
        reshape2::melt(value.name = "sel")
      output$fleet <- fname

    } else {
      x <- x[1, , drop = FALSE]

      output <- data.frame(
        year = "all",
        age = age,
        sel = x[1, ],
        fleet = fname
      )
    }

    if (figure) {
      color <- make_color(nrow(x), "fleet")
      matplot(age, t(x), xlab = "Age", ylab = paste(fname, "selectivity"),
              type = "o", col = color, pch = 16,
              ylim = c(0, 1), lty = 1, zero_line = TRUE)
      if (nrow(x) > 1) legend("topright", legend = name, col = color, lwd = 1, pch = 16)
    }

  }
  invisible(output)
}

#' @rdname plot-MSA-state
#' @aliases plot_seli
#' @param i Integer for the corresponding survey
#' @details
#' - `plot_seli` plots index selectivity
#' @export
plot_seli <- function(fit, i = 1, figure = TRUE) {
  dat <- get_MSAdata(fit)
  output <- NULL

  sel_i <- dat@Dsurvey@sel_i[i]
  mirror_f <- suppressWarnings(as.numeric(sel_i))

  iname <- dat@Dlabel@index[i]

  if (!is.na(mirror_f)) {
    output <- plot_self(fit, f = mirror_f, figure = figure)
    output <- output[, 1:3]
    output$name <- iname
  } else {

    age_sel <- grepl("age", sel_i) || sel_i %in% c("B", "SB", "total", "mature")
    len_sel <- grepl("length", sel_i) || all(!is.na(fit@report$sel_li[, i]))

    if (!age_sel && len_sel) {
      lmid <- dat@Dmodel@lmid
      x <- fit@report$sel_li[, i]

      if (figure) {
        plot(lmid, x, xlab = "Length", ylab = paste(iname, "selectivity"),
             type = "o", pch = 16,
             ylim = c(0, 1), lty = 1, zero_line = TRUE)
      }
      output <- data.frame(
        year = "all",
        length = lmid,
        sel = x,
        name = iname
      )

    } else {

      m <- 1
      s <- 1

      x <- fit@report$sel_ymais[, m, , i, s]
      xx <- apply(x, 2, diff)

      age <- dat@Dlabel@age

      if (any(xx != 0)) {
        year <- dat@Dlabel@year
        ybreak <- c(1, which(rowSums(xx) > 0) + 1)
        name <- sapply(1:length(ybreak), function(i) {
          if (i == length(ybreak)) {
            y <- c(year[ybreak[i]], year[length(year)])
          } else {
            y <- year[c(ybreak[i], ybreak[i+1] - 1)]
          }
          paste(range(y), collapse = "-")
        })
        x <- x[ybreak, , drop = FALSE]

        output <- structure(x, dimnames = list(year = name, age = age)) |>
          reshape2::melt(value.name = "sel")
        output$name <- iname

      } else {
        x <- x[1, , drop = FALSE]

        output <- data.frame(
          year = "all",
          age = age,
          sel = x[1, ],
          name = iname
        )
      }

      if (figure) {
        color <- make_color(nrow(x), "fleet")

        matplot(age, t(x), xlab = "Age", ylab = paste(iname, "selectivity"),
                type = "o", col = color, pch = 16,
                ylim = c(0, 1), lty = 1, zero_line = TRUE)
        if (nrow(x) > 1) legend("topright", legend = name, col = color, lwd = 1, pch = 16)
      }
    }

  }
  invisible(output)
}


#' @rdname plot-MSA-state
#' @aliases plot_selstock
#' @param plot2d Character, plotting function for either a [contour()] or [filled.contour()] plot
#' @param by Character to indicate whether to calculate selectivity from F per year or per season
#' @param ... Other arguments to the base graphics function
#' @details
#' - `plot_selstock` plots the realized selectivity from total catch and total abundance at age
#' @export
#' @importFrom graphics contour filled.contour
plot_selstock <- function(fit, s = 1, by = c("annual", "season"), plot2d = c("contour", "filled.contour"), ...) {

  by <- match.arg(by)

  plot2d <- match.arg(plot2d)
  plot2d <- match.fun(plot2d)

  dat <- get_MSAdata(fit)
  year <- dat@Dlabel@year
  age <- dat@Dlabel@age

  nm <- max(length(dat@Dlabel@season), 1)

  if (by == "annual") {
    sel_ya <- fit@report$F_yas[, , s] |>
      apply(1, function(x) x/max(x)) |>
      t()
  } else if (by == "season" && nm > 1) {

    year <- make_yearseason(year, nm)
    F_yma <- sapply2(1:dat@Dmodel@na, function(a) {
      sapply(1:nm, function(m) {
        sapply(1:dat@Dmodel@ny, function(y) {
          N <- sum(fit@report$N_ymars[y, m, a, , s])
          CN <- sum(fit@report$CN_ymafrs[y, m, a, , , s])
          calc_summary_F(M = fit@report$M_yas[y, a, s]/nm, N = N, CN = CN, Fmax = 100)
        })
      })
    })
    sel_ya <- collapse_yearseason(F_yma) |>
      apply(1, function(x) x/max(x)) |>
      t()
  }

  if (exists("sel_ya", inherits = FALSE)) {
    plot2d(x = year, y = age, xlab = "Year", ylab = "Age", z = sel_ya, levels = seq(0, 1, 0.1), ...)
  }

  invisible()
}

#' @rdname plot-MSA-state
#' @aliases plot_N
#' @param m Integer for the corresponding season
#' @param r Integer for the corresponding region
#' @param ... Other argument to the base graphics function
#' @details
#' - `plot_N` reports total abundance at age
#' @export
#' @importFrom graphics contour filled.contour
plot_N <- function(fit, m = 1, r, s = 1, plot2d = c("contour", "filled.contour"), ...) {
  plot2d <- match.arg(plot2d)
  plot2d <- match.fun(plot2d)

  dat <- get_MSAdata(fit)
  if (missing(r)) r <- 1:dat@Dmodel@nr
  if (length(m) > 1) stop("length(m) should be one")
  ny <- dat@Dmodel@ny
  year <- dat@Dlabel@year
  age <- dat@Dlabel@age

  N_ya <- apply(fit@report$N_ymars[1:ny, m, , r, s, drop = FALSE], c(1, 3), sum)

  dots <- list(...)
  if (!length(dots$nlevels)) dots$nlevels <- 10
  if (!length(dots$levels)) dots$levels <- exp(pretty(log(range(N_ya)), 10))

  dots$x <- year
  dots$y <- age
  dots$xlab = "Year"
  dots$ylab <- "Age"
  dots$z <- N_ya

  do.call(plot2d, dots)

  invisible()
}


#' @rdname plot-MSA-state
#' @aliases plot_V
#' @details
#' - `plot_V` plots vulnerable biomass, availability to the fishery
#' @export
plot_V <- function(fit, f = 1, by = c("stock", "region"), prop = FALSE, facet_free = FALSE) {
  by <- match.arg(by)
  var <- "VB_ymfrs"

  d <- get_MSAdata(fit)
  Dlabel <- d@Dlabel
  Dmodel <- d@Dmodel

  year <- Dlabel@year
  ny <- length(year)
  nm <- max(length(Dlabel@season), 1)

  r <- 1:Dmodel@nr
  s <- 1:Dmodel@ns
  rname <- Dlabel@region[r]
  sname <- Dlabel@stock[s]

  if (by == "stock") {
    leg.name <- sname
    facet.name <- rname
    x <- array(fit@report[[var]][, , f, , , drop = FALSE], c(ny, nm, length(rname), length(sname))) |>
      aperm(c(1, 2, 4, 3)) # B_ymsr
  } else {
    leg.name <- rname
    facet.name <- sname
    x <- array(fit@report[[var]][, , f, , , drop = FALSE], c(ny, nm, length(rname), length(sname))) # B_ymrs
  }

  year <- make_yearseason(year, nm)
  x <- collapse_yearseason(x)

  color <- make_color(ncol(x), type = by)

  fname <- Dlabel@fleet[f]

  if (prop) {
    ylab <- paste("Proportion biomass available to", fname)
  } else {
    ylab <- paste("Biomass available to", fname)
  }

  barplot2(x, cols = color, leg.names = leg.name, facet.names = facet.name, xval = year, ylab = ylab, prop = prop,
           facet.free = facet_free)

  invisible(array2DF(x, responseName = "V"))
}


#' @rdname plot-MSA-state
#' @aliases plot_Ffleet
#' @details
#' - `plot_Ffleet` plots apical instantaneous fishing mortality (per season) by fleet
#'
#' @export
plot_Ffleet <- function(fit, f = 1) {
  var <- "F_ymfr"

  Dlabel <- get_MSAdata(fit)@Dlabel
  year <- Dlabel@year
  nm <- max(length(Dlabel@season), 1)

  x <- apply(fit@report[[var]][, , f, , drop = FALSE], c(1, 2, 4), identity)
  name <- Dlabel@region

  year <- make_yearseason(year, nm)
  x <- collapse_yearseason(x)

  color <- make_color(ncol(x), type = "region")

  fname <- Dlabel@fleet[f]
  ylab <- paste(fname, "fishing mortality")

  make_tinyplot(year, x, ylab, name, color)
  #matplot(year, x, xlab = "Year", ylab = ylab, type = "o", col = color, pch = 16, lty = 1,
  #        ylim = c(0, 1.1) * range(x, na.rm = TRUE), zero_line = TRUE)
  #if (ncol(x) > 1) legend("topleft", legend = name, col = color, lwd = 1, pch = 16, horiz = TRUE)

  invisible()





}

#' @rdname plot-MSA-state
#' @aliases plot_mov
#' @param y Integer, year for plotting the movement matrix (last model year is the default)
#' @param a Integer, corresponding age for plotting the movement matrix (age 1 is the default)
#' @param palette Character, palette name to pass to [grDevices::hcl.colors()]. See [grDevices::hcl.pals()] for options.
#' @details
#' - `plot_mov` plots movement matrices and the corresponding equilibrium distribution in multi-area models
#' @export
#' @importFrom tinyplot tinyplot type_text
plot_mov <- function(fit, s = 1, y, a, palette = "Peach", figure = TRUE) {

  dat <- get_MSAdata(fit)
  output <- NULL

  nm <- dat@Dmodel@nm
  nr <- dat@Dmodel@nr
  if (missing(y)) y <- dat@Dmodel@ny
  if (missing(a)) a <- 1
  rname <- dat@Dlabel@region
  mname <- dat@Dlabel@season

  mov <- array(fit@report$mov_ymarrs[y, , a+1, , , s], c(nm, nr, nr))

  dist_eq <- calc_eqdist(mov, start = fit@report$recdist_rs[, s], m_start = dat@Dstock@m_spawn)

  df_mov <- structure(mov, dimnames = list(Season = mname, Origin = 1:nr, Destination = 1:nr)) |>
    reshape2::melt(value.name = "proportion")

  df_eq <- structure(dist_eq, dimnames = list(Season = mname, Origin = 1:nr)) |>
    reshape2::melt(value.name = "proportion") |>
    cbind("Destination" = nr + 1.5)

  df <- rbind(df_mov, df_eq[, c("Season", "Origin", "Destination", "proportion")])
  output <- local({

    df2_mov <- df_mov
    df2_mov$Origin <- rname[df2_mov$Origin]
    df2_mov$Destination <- rname[df2_mov$Destination]

    df2_eq <- df_eq
    df2_eq$Origin <- rname[df2_eq$Origin]
    df2_eq$Destination <- "Equilibrium"

    df2 <- rbind(df2_mov, df2_eq[, c("Season", "Origin", "Destination", "proportion")])
    df2$stock <- dat@Dlabel@stock[s]
    df2$y <- y
    df2$a <- a

    df2
  })

  if (figure) {
    #if (nm > 1) {
    #  old_mar <- par()$mar
    #  old_mfrow = par()$mfrow
    #  par(mar = c(4, 4, 1, 1))
    #  on.exit(par(mar = old_mar, mfrow = old_mfrow))
    #  par(mfrow = c(2, ceiling(nm/2)))
    #}

    df$label <- round(df$proportion, 2) |> format()

    tick_fn <- function(i) ifelse(i > nr, "Eq.", as.character(rname[i]))
    tinyplot_args <- list(
      xmin = df$Destination - 0.5, xmax = df$Destination + 0.5, ymin = df$Origin - 0.5, ymax = df$Origin + 0.5,
      by = df$proportion,
      facet = df$Season, xlab = "Destination", ylab = "Origin",
      bg = "by", col = "black",
      legend = substitute(legend(title = "Proportion")),
      yaxs = "i", xaxs = "i",
      yaxl = tick_fn, xaxl = tick_fn,
      yaxb = 1:nr, xaxb = c(1:nr, nr + 1.5),
      type = "rect", palette = palette,
      main = if (dat@Dmodel@ns > 1) dat@Dlabel@stock[s] else NULL
    )
    do.call(tinyplot, tinyplot_args)

    tinyplot(
      x = df$Destination, y = df$Origin, facet = df$Season,
      type = type_text(labels = df$label, adj = 0.5),
      add = TRUE
    )
  }

  invisible(output)
}

#' @rdname plot-MSA-state
#' @aliases plot_recdist
#' @details
#' - `plot_recdist` plots the distribution of recruitment for each stock
#' @export
plot_recdist <- function(fit, palette = "Peach", figure = TRUE) {
  dat <- get_MSAdata(fit)
  output <- NULL

  nr <- dat@Dmodel@nr

  if (nr > 1) {
    ns <- dat@Dmodel@ns
    rname <- dat@Dlabel@region
    sname <- dat@Dlabel@stock

    recdist <- fit@report$recdist_rs

    output <- structure(recdist, dimnames = list(region = rname, stock = sname)) |>
      reshape2::melt(value.name = "proportion")

    if (figure) {


      #vcol <- hcl.colors(100, palette)

      #graphics::plot.default(
      #  NULL, xlab = "Stock", ylab = "Region", xaxs = "i", yaxs = "i",
      #  xaxt = "n", yaxt = "n", xlim = c(1, ns+1), ylim = c(1, nr+1)
      #)
      #for(x in 1:ns) {
      #  for(y in 1:nr) {
      #    m_yx <- round(recdist[y, x], 2)
      #    rect(xleft = x, ybottom = y, xright = x+1, ytop = y+1, col = vcol[100 * m_yx])
      #    text(x + 0.5, y + 0.5, m_yx)
      #  }
      #}

      #axis(1, at = 1:ns + 0.5, labels = as.character(sname), font = 2, cex.axis = 0.75)
      #axis(2, at = 1:nr + 0.5, labels = as.character(rname), font = 2, cex.axis = 0.75)

      df <- structure(recdist, dimnames = list(r = 1:nr, s = 1:ns)) |>
        reshape2::melt()
      df$label <- round(df$value, 2)

      tick_fnx <- function(i) sname[i]
      tick_fny <- function(i) rname[i]

      tinyplot_args <- list(
        xmin = df$s - 0.5, xmax = df$s + 0.5, ymin = df$r - 0.5, ymax = df$r + 0.5,
        by = df$value, xlab = "Stock", ylab = "Region",
        bg = "by", col = "black", border = "black",
        legend = substitute(legend(title = "Proportion")),
        yaxs = "i", xaxs = "i",
        yaxl = tick_fny, xaxl = tick_fnx,
        yaxb = 1:nr, xaxb = 1:ns,
        type = "rect", palette = palette
      )
      do.call(tinyplot, tinyplot_args)

      tinyplot(
        x = df$s, y = df$r,
        type = type_text(labels = df$label, adj = 0.5),
        add = TRUE
      )
    }
  }

  invisible(output)
}

# #' @importFrom grDevices hcl.colors
# #' @importFrom graphics rect text
# .plot_mov <- function(m, p, xlab = "Destination", ylab = "Origin",
#                       nr = length(p), rname = paste("Region", 1:nr), palette = "Peach") {
#
#   vcol <- hcl.colors(100, palette)
#
#   graphics::plot.default(
#     NULL, xlab = xlab, ylab = ylab, xaxs = "i", yaxs = "i",
#     xaxt = "n", yaxt = "n", xlim = c(1, nr+3), ylim = c(1, nr+1)
#   )
#   for(x in 1:nr) {
#     for(y in 1:nr) {
#       m_yx <- round(m[y, x], 2)
#       rect(xleft = x, ybottom = y, xright = x+1, ytop = y+1, col = vcol[100 * m_yx])
#       text(x + 0.5, y + 0.5, m_yx)
#     }
#   }
#   eq_val <- round(p, 2)
#   eq_col <- rep(NA, length(p))
#   eq_col[eq_val > 0] <- vcol[100 * eq_val]
#   rect(nr + 2, ybottom = 1:nr, xright = nr + 3, ytop = 1:nr + 1, col = eq_col)
#   text(nr + 2.5, 1:nr + 0.5, eq_val)
#
#   axis(2, at = 1:nr + 0.5, labels = as.character(rname), font = 2, cex.axis = 0.75)
#   axis(1, at = c(1:nr, nr+2) + 0.5, labels = c(as.character(rname), "Eq."), font = 2, cex.axis = 0.75)
#
#   invisible()
# }

#' @name residuals
#' @aliases residuals,MSAassess-method
#'
#' @title Calculate model residuals
#'
#' @description Extract residuals from fitted model
#'
#' @param object [MSAassess-class] object returned by `fit_MSA()`
#' @param vars Character vector to indicate which residuals will be calculated.
#' Available choices from [MSAdata-class] object are:
#' "Cinit_mfr", "Cobs_ymfr", "CAAobs_ymafr", "CALobs_ymlfr",
#' "Iobs_ymi", "IAAobs_ymai", "IALobs_ymli", "SC_ymafrs"
#' @param type Character, 'response' for the `log(observed/predicted)` values or 'pearson' for calculating Z-scores.
#' Composition data always use 'pearson'.
#' @param ... Not used
#'
#' @return A named list based on `vars` argument.
#' @aliases residuals
#' @importFrom stats residuals
#' @export
setMethod("residuals", signature(object = "MSAassess"),
          function(object, vars, type = c("response", "pearson"), ...) {

  vars_choices <- c("Cinit_mfr", "Cobs_ymfr", "CAAobs_ymafr", "CALobs_ymlfr",
                    "Iobs_ymi", "IAAobs_ymai", "IALobs_ymli", "SC_ymafrs", "tag_ymarrs")
  if (missing(vars)) vars <- vars_choices
  vars <- match.arg(vars, choices = vars_choices, several.ok = TRUE)
  type <- match.arg(type)
  dat <- get_MSAdata(object)

  res <- list()

  # Init catch ----
  if (any(vars == "Cinit_mfr") && !is.null(object@report$initCB_mfrs)) {
    Cinitres <- log(dat@Dfishery@Cinit_mfr/apply(object@report$initCB_mfrs, 1:3, sum))
    if (type == "pearson") {
      Cinitres <- Cinitres/0.01
    }
    res$Cinit_mfr <- Cinitres
  }

  # Catch ----
  if (any(vars == "Cobs_ymfr")) {
    Cres <- log(dat@Dfishery@Cobs_ymfr/apply(object@report$CB_ymfrs, 1:4, sum))
    if (type == "pearson") {
      if (length(dat@Dfishery@Csd_ymfr)) {
        Csd <- dat@Dfishery@Csd_ymfr
      } else {
        Csd <- 0.01
      }
      Cres <- Cres/Csd
    }
    res$Cobs_ymfr <- Cres
  }


  # Catch at age ----
  if (any(vars == "CAAobs_ymafr") && length(dat@Dfishery@CAAobs_ymafr)) {
    res$CAAobs_ymafr <- array(NA, dim(dat@Dfishery@CAAobs_ymafr))
    pred <- apply(object@report$CN_ymafrs, 1:5, sum)
    for(y in 1:dat@Dmodel@ny) {
      for(m in 1:dat@Dmodel@nm) {
        for(f in 1:dat@Dfishery@nf) {
          for(r in 1:dat@Dmodel@nr) {
            pred_i <- pred[y, m, , f, r]
            res$CAAobs_ymafr[y, m, , f, r] <- resid_comp(
              obs = dat@Dfishery@CAAobs_ymafr[y, m, , f, r],
              pred = pred_i,
              like = dat@Dfishery@fcomp_like,
              N = dat@Dfishery@CAAN_ymfr[y, m, f, r],
              theta = dat@Dfishery@CAAtheta_f[f],
              stdev = sum(pred_i)/pred_i
            )
          }
        }
      }
    }
  }

  # Catch at length ----
  if (any(vars == "CALobs_ymlfr") && length(dat@Dfishery@CALobs_ymlfr)) {
    res$CALobs_ymlfr <- array(NA, dim(dat@Dfishery@CALobs_ymlfr))
    pred <- apply(object@report$CN_ymlfrs, 1:5, sum)
    for(y in 1:dat@Dmodel@ny) {
      for(m in 1:dat@Dmodel@nm) {
        for(f in 1:dat@Dfishery@nf) {
          for(r in 1:dat@Dmodel@nr) {
            pred_i <- pred[y, m, , f, r]
            res$CALobs_ymlfr[y, m, , f, r] <- resid_comp(
              obs = dat@Dfishery@CALobs_ymlfr[y, m, , f, r],
              pred = pred_i,
              like = dat@Dfishery@fcomp_like,
              N = dat@Dfishery@CALN_ymfr[y, m, f, r],
              theta = dat@Dfishery@CALtheta_f[f],
              stdev = sum(pred_i)/pred_i
            )
          }
        }
      }
    }
  }

  # Indices ----
  if (any(vars == "Iobs_ymi") && length(dat@Dsurvey@Iobs_ymi)) {
    res$Iobs_ymi <- log(dat@Dsurvey@Iobs_ymi/object@report$I_ymi)
    if (type == "pearson") {
      res$Iobs_ymi <- res$Iobs_ymi/dat@Dsurvey@Isd_ymi
    }
  }

  # Indices at age ----
  if (any(vars == "IAAobs_ymai") && length(dat@Dsurvey@IAAobs_ymai)) {
    res$IAAobs_ymai <- array(NA, dim(dat@Dsurvey@IAAobs_ymai))
    pred <- apply(object@report$IN_ymais, 1:4, sum)
    for(y in 1:dat@Dmodel@ny) {
      for(m in 1:dat@Dmodel@nm) {
        for(i in 1:dat@Dsurvey@ni) {
          pred_i <- pred[y, m, , i]
          res$IAAobs_ymai[y, m, , i] <- resid_comp(
            obs = dat@Dsurvey@IAAobs_ymai[y, m, , i],
            pred = pred_i,
            like = dat@Dsurvey@icomp_like,
            N = dat@Dsurvey@IAAN_ymi[y, m, i],
            theta = dat@Dsurvey@IAAtheta_i[i],
            stdev = sum(pred_i)/pred_i
          )
        }
      }
    }
  }

  # Indices at length ----
  if (any(vars == "IALobs_ymli") && length(dat@Dsurvey@IALobs_ymli)) {
    res$IALobs_ymli <- array(NA, dim(dat@Dsurvey@IALobs_ymli))
    pred <- apply(object@report$IN_ymlis, 1:4, sum)
    for(y in 1:dat@Dmodel@ny) {
      for(m in 1:dat@Dmodel@nm) {
        for(i in 1:dat@Dsurvey@ni) {
          pred_i <- pred[y, m, , i]
          res$IALobs_ymli[y, m, , i] <- resid_comp(
            obs = dat@Dsurvey@IALobs_ymli[y, m, , i],
            pred = pred_i,
            like = dat@Dsurvey@icomp_like,
            N = dat@Dsurvey@IALN_ymi[y, m, i],
            theta = dat@Dsurvey@IALtheta_i[i],
            stdev = sum(pred_i)/pred_i
          )
        }
      }
    }
  }

  # Stock composition (at age) ----
  if (any(vars == "SC_ymafrs") && length(dat@Dfishery@SC_ymafrs)) {
    SC_ymafrs <- dat@Dfishery@SC_ymafrs
    res$SC_ymafrs <- array(NA, dim(dat@Dfishery@SC_ymafrs))
    pred <- object@report$SCpred_ymafrs
    for(y in 1:dat@Dmodel@ny) {
      for(m in 1:dat@Dmodel@nm) {
        for(a in 1:dim(SC_ymafrs)[3]) {
          for(f in 1:dim(SC_ymafrs)[4]) {
            for(r in 1:dat@Dmodel@nr) {
              res$SC_ymafrs[y, m, a, f, r, ] <- resid_comp(
                obs = SC_ymafrs[y, m, a, f, r, ],
                pred = pred[y, m, a, f, r, ],
                like = dat@Dfishery@SC_like,
                N = dat@Dfishery@SCN_ymafr[y, m, a, f, r],
                theta = dat@Dfishery@SCtheta_f[f],
                stdev = dat@Dfishery@SCstdev_ymafrs[y, m, a, f, r, ]
              )
            }
          }
        }
      }
    }
  }


  # Stock composition (at age) ----
  if (any(vars == "SC_ymafrs") && length(dat@Dfishery@SC_ymafrs)) {
    SC_ymafrs <- dat@Dfishery@SC_ymafrs
    res$SC_ymafrs <- array(NA, dim(SC_ymafrs))
    pred <- object@report$SCpred_ymafrs
    for(y in 1:dat@Dmodel@ny) {
      for(m in 1:dat@Dmodel@nm) {
        for(a in 1:dim(SC_ymafrs)[3]) {
          for(f in 1:dim(SC_ymafrs)[4]) {
            for(r in 1:dat@Dmodel@nr) {
              res$SC_ymafrs[y, m, a, f, r, ] <- resid_comp(
                obs = SC_ymafrs[y, m, a, f, r, ],
                pred = pred[y, m, a, f, r, ],
                like = dat@Dfishery@SC_like,
                N = dat@Dfishery@SCN_ymafr[y, m, a, f, r],
                theta = dat@Dfishery@SCtheta_f[f],
                stdev = dat@Dfishery@SCstdev_ymafrs[y, m, a, f, r, ]
              )
            }
          }
        }
      }
    }
  }

  # Tag movement ----
  if (any(vars == "tag_ymarrs") && length(dat@Dtag@tag_ymarrs)) {
    tag_ymarrs <- dat@Dtag@tag_ymarrs
    res$tag_ymarrs <- array(NA, dim(tag_ymarrs))
    pred <- object@report$tagpred_ymarrs
    for(y in 1:dim(tag_ymarrs)[1]) {
      for(m in 1:dat@Dmodel@nm) {
        for(a in 1:dim(tag_ymarrs)[3]) {
          for(rf in 1:dat@Dmodel@nr) {
            for(s in 1:dat@Dmodel@ns) {
              res$tag_ymarrs[y, m, a, rf, , s] <- resid_comp(
                obs = tag_ymarrs[y, m, a, rf, , s],
                pred = pred[y, m, a, rf, , s],
                like = dat@Dtag@tag_like,
                N = dat@Dtag@tagN_ymars[y, m, a, rf, s],
                theta = dat@Dtag@tagtheta_s[s],
                stdev = dat@Dtag@tagstdev_s[s]
              )
            }
          }
        }
      }
    }
  }

  return(res)
}
)

# Vectors
resid_comp <- function(obs, pred, like, ...) {
  like <- match.arg(like, choices = eval(formals(like_comp)$type))

  dots <- list(...)

  obs_prob <- obs/sum(obs, na.rm = TRUE)
  pred_prob <- pred/sum(pred, na.rm = TRUE)

  num <- denom <- rep(NA, length(obs))

  # Observed minus predicted
  num[] <- switch(
    like,
    "multinomial" = obs_prob - pred_prob,
    "dirmult1" = obs_prob - pred_prob,
    "dirmult2" = obs_prob - pred_prob,
    "lognormal" = ifelse(obs > 0, log(obs_prob/pred_prob), NA),
    NA
  )

  if (like == "logitnormal") {
    i_fit <- obs > 0
    if (any(i_fit)) {
      i_ref <- rep(FALSE, length(obs))
      i_ref[which(i_fit)[1]] <- TRUE
      num[] <- log(obs_prob/obs_prob[i_ref]) - log(pred_prob/pred_prob[i_ref])
    }
  }

  # Variance
  # Multinomial variance of obs is Np(1-p), variance of pred is p(1-p)/N
  denom[] <- switch(
    like,
    "multinomial" = pred_prob * (1 - pred_prob)/dots$N,
    "dirmult1" = dots$N * pred_prob * (1 - pred_prob) * dots$N * (1 + dots$theta) / (1 + dots$theta * dots$N),
    "dirmult2" = dots$N * pred_prob * (1 - pred_prob) * (dots$N + dots$theta) / (1 + dots$theta),
    "lognormal" = dots$stdev * dots$stdev,
    "logitnormal" = dots$stdev * dots$stdev,
    NA
  )

  res <- num/sqrt(denom)
  return(res)
}

plot_resid_Cobs <- function(fit, f = 1, ...) {
  vars <- "Cobs_ymfr"

  dat <- get_MSAdata(fit)

  res <- residuals(fit, vars = vars, ...)
  x <- apply(res[[vars]][, , f, , drop = FALSE], c(1, 2, 4), identity)

  name <- dat@Dlabel@region

  year <- make_yearseason(dat@Dlabel@year, dat@Dmodel@nm)
  x <- collapse_yearseason(x)

  color <- make_color(ncol(x), type = "region")
  fname <- dat@Dlabel@fleet[f]

  if (!all(is.na(x))) make_tinyplot(year, x, ylab = paste(fname, "catch residual"), name, color, ylim = NULL)
  #matplot(year, x, xlab = "Year", ylab = paste(fname, "catch residual"), type = "o",
  #        col = color, pch = 16, lty = 1)
  #abline(h = 0, lty = 2, col = "grey60")
  #if (ncol(x) > 1) legend("topleft", legend = name, col = color, lwd = 1, pch = 16, horiz = TRUE)

  invisible()
}

plot_resid_Iobs <- function(fit, i = 1, ...) {
  vars <- "Iobs_ymi"

  dat <- get_MSAdata(fit)

  res <- residuals(fit, vars = vars, ...)[[vars]]
  if (is.null(res)) return(invisible())

  x <- apply(res[, , i, drop = FALSE], 1:2, identity)

  name <- dat@Dlabel@index[i]

  year <- make_yearseason(dat@Dlabel@year, dat@Dmodel@nm)
  x <- collapse_yearseason(x)

  if (!all(is.na(x))) {
    plot(year, x, type = "n", xlab = "Year", ylab = paste(name, "residual"))
    lines(year[!is.na(x)], x[!is.na(x)], col = "grey70", lty = 2)
    points(year, x, type = "o", pch = 16)
    abline(h = 0, lty = 1, col = "grey60")
  }


  invisible()
}

plot_resid_CAA <- function(fit, f = 1, r = 1, do_hist = FALSE, ...) {
  vars <- "CAAobs_ymafr"

  dat <- get_MSAdata(fit)

  res <- residuals(fit, vars = vars, ...)[[vars]]
  if (is.null(res)) return(invisible())

  x <- apply(res[, , , f, r, drop = FALSE], 1:3, identity)

  year <- make_yearseason(dat@Dlabel@year, dat@Dmodel@nm)
  x <- collapse_yearseason(x)

  ind <- rowSums(x, na.rm = TRUE) != 0
  yeardiff <- c(diff(year), diff(year)[1])

  .plot_resid_comp(
    year[ind], dat@Dlabel@age, x[ind, , drop = FALSE],
    xlab = "Year", ylab = "Age", do_hist = do_hist,
    xdiff = yeardiff[ind]
  )
}

plot_resid_CAL <- function(fit, f = 1, r = 1, do_hist = FALSE, ...) {
  vars <- "CALobs_ymlfr"

  dat <- get_MSAdata(fit)

  res <- residuals(fit, vars = vars, ...)[[vars]]
  if (is.null(res)) return(invisible())

  x <- apply(res[, , , f, r, drop = FALSE], 1:3, identity)

  year <- make_yearseason(dat@Dlabel@year, dat@Dmodel@nm)
  x <- collapse_yearseason(x)

  ind <- rowSums(x, na.rm = TRUE) != 0
  yeardiff <- c(diff(year), diff(year)[1])

  .plot_resid_comp(
    year[ind], dat@Dmodel@lmid, x[ind, , drop = FALSE],
    xlab = "Year", ylab = "Length", do_hist = do_hist,
    xdiff = yeardiff[ind]
  )
}

plot_resid_IAA <- function(fit, i = 1, do_hist = FALSE, ...) {
  vars <- "IAAobs_ymai"

  dat <- get_MSAdata(fit)

  res <- residuals(fit, vars = vars, ...)[[vars]]
  if (is.null(res)) return(invisible())

  x <- apply(res[, , , i, drop = FALSE], 1:3, identity)

  year <- make_yearseason(dat@Dlabel@year, dat@Dmodel@nm)
  x <- collapse_yearseason(x)

  ind <- rowSums(x, na.rm = TRUE) != 0
  yeardiff <- c(diff(year), diff(year)[1])

  .plot_resid_comp(
    year[ind], dat@Dlabel@age, x[ind, , drop = FALSE],
    xlab = "Year", ylab = "Age", do_hist = do_hist,
    xdiff = yeardiff[ind]
  )
}

plot_resid_IAL <- function(fit, i = 1, do_hist = FALSE, ...) {
  vars <- "IALobs_ymli"

  dat <- get_MSAdata(fit)

  res <- residuals(fit, vars = vars, ...)[[vars]]
  if (is.null(res)) return(invisible())

  x <- apply(res[, , , i, drop = FALSE], 1:3, identity)

  year <- make_yearseason(dat@Dlabel@year, dat@Dmodel@nm)
  x <- collapse_yearseason(x)

  ind <- rowSums(x, na.rm = TRUE) != 0
  yeardiff <- c(diff(year), diff(year)[1])

  .plot_resid_comp(
    year[ind], dat@Dmodel@lmid, x[ind, , drop = FALSE],
    xlab = "Year", ylab = "Length", do_hist = do_hist,
    xdiff = yeardiff[ind]
  )
}

plot_resid_SC <- function(fit, a = 1, f = 1, r = 1, do_hist = FALSE, ...) {
  vars <- "SC_ymafrs"
  dat <- get_MSAdata(fit)

  res <- residuals(fit, vars = vars, ...)[[vars]]
  if (is.null(res)) return(invisible())

  x <- apply(res[, , a, f, r, , drop = FALSE], c(1, 2, 6), identity)

  year <- make_yearseason(dat@Dlabel@year, dat@Dmodel@nm)
  x <- collapse_yearseason(x)

  ind <- rowSums(x, na.rm = TRUE) != 0
  yeardiff <- c(diff(year), diff(year)[1])

  .plot_resid_comp(
    year[ind], dat@Dlabel@stock, x[ind, , drop = FALSE],
    xlab = "Year", ylab = "Stock", do_hist = do_hist,
    xdiff = yeardiff[ind]
  )
}

plot_resid_tagmov <- function(fit, yy = 1, aa = 1, s = 1, ...) {
  vars <- "tag_ymarrs"
  dat <- get_MSAdata(fit)

  res <- residuals(fit, vars = vars, ...)[[vars]]
  if (is.null(res)) return(invisible())

  x <- apply(res[yy, , aa, , , s, drop = FALSE], c(2, 4, 5), identity)

  .plot_resid_comp(z = x, do_hist = TRUE)
}

#' @importFrom graphics hist box
.plot_resid_comp <- function(x = 1:nrow(z), y = 1:ncol(z), z, xlab = "Year", ylab = "Age", zmax = 2,
                             do_hist = FALSE, ydiff, xdiff) {

  if (all(is.na(z))) return(invisible())
  if (do_hist) return(hist(z, xlab = "Residuals", main = ""))

  #if (is.character(y)) {
  #  yval <- y
  #  y <- 1:ncol(z)
  #  yaxt <- "n"
  #} else {
  #  yaxt <- "s"
  #}

  if (is.character(y)) {
    ynum <- 1:length(y)
    tick_y <- function(i) y[i]
    yaxb <- 1:length(y)
  } else {
    ynum <- y
    yaxb <- NULL
    tick_y <- NULL
  }

  #zmax <- min(zmax, max(abs(z), na.rm = TRUE))
  zz <- pmin(z, zmax) |> pmax(-zmax) |> round(2)
  #zlegend <- seq(-zmax, zmax, 0.01) |> round(2)
  #cols <- hcl.colors(length(zlegend), palette = "Blue-Red 2", alpha = 1) |>
  #  structure(names = zlegend)

  if (missing(ydiff)) {
    ydiff <- diff(ynum)
    ydiff <- c(ydiff, ydiff[1])
  }

  if (missing(xdiff)) {
    xdiff <- diff(x)
    xdiff <- c(xdiff, xdiff[1])
  }

  border <- ifelse(any(xdiff < 0.5, na.rm = TRUE), NA, "grey60")
  rect_diff <- ifelse(any(xdiff < 0.5, na.rm = TRUE), 0.475, 0.5)

  df <- lapply(1:nrow(z), function(i) {
    data.frame(
      xleft = x[i] - rect_diff * xdiff[i],
      xright = x[i] + rect_diff * xdiff[i],
      ybottom = ynum - rect_diff * ydiff,
      ytop = ynum + rect_diff * ydiff,
      value = zz[i, ]
      #col = cols[match(zz[i, ], zlegend)]
    )
  })
  df <- do.call(rbind, df)
  df <- df[!is.na(df$value), ]

  #legend_val <- zlegend[c(1, c(0.25, 0.5, 0.75, 1) * length(zlegend))]
  #legend_col <-
  #round(seq(-1, 1, 0.5) * zmax, 2)

  tinyplot(xmin = df$xleft, xmax = df$xright, ymin = df$ybottom, ymax = df$ytop,
           by = df$value, xlab = xlab, ylab = ylab, #yaxs = "i", xaxs = "i",
           bg = "by",
           col = border,
           yaxl = tick_y, yaxb = yaxb,
           #legend = bquote(legend(title = NULL, legend = .(legend_va), 2), pt.bg = c(cols))),
           legend = legend(title = NULL),
           palette = "Blue-Red 2",
           #palette = function(n) hcl.colors(100, palette = "Blue-Red 2", alpha = 0.75, rev = TRUE),
           type = "rect")
  box()

  #plot(NULL, type = "n", xlab = xlab, ylab = ylab, xlim = range(x), ylim = range(y), yaxt = yaxt)
  #sapply(1:nrow(z), function(i) {
  #  if (any(!is.na(zz[i, ]))) {
  #    rect(xleft = x[i] - rect_diff * xdiff[i], xright = x[i] + rect_diff * xdiff[i],
  #         ybottom = y - rect_diff * ydiff, ytop = y + rect_diff * ydiff,
  #         col = cols[match(zz[i, ], zlegend)],
  #         border = border)
  #  }
  #})
  #legend("topleft", legend = c(-zmax, -zmax/2, 0, zmax/2, zmax),
  #       col = "grey60", pt.cex = 1, pt.bg = cols[zlegend %in% c(-zmax, -zmax/2, 0, zmax/2, zmax)],
  #       pch = 22, horiz = TRUE)
  #if (yaxt == "n") {
  #  axis(2, at = y, labels = yval)
  #}
  #box()

  invisible()
}

# Data plots ----
#' @name plot-MSA-data
#' @title Plotting functions for data in MSA model
#' @description A set of functions to plot data variables and predicted values (catch, age composition, etc.)
#' @return Invisible data frame of observed and predicted values plotted in base graphics figures
NULL

#' @rdname plot-MSA-data
#' @aliases plot_catch
#'
#' @param fit [MSAassess-class] object returned by [fit_MSA()]
#' @param by Character to indicate dimension for multivariate plots
#' @param f Integer for the corresponding fleet
#' @param prop Logical, whether to plot proportions (TRUE) or absolute numbers
#' @param annual Logical, whether to plot annual values (summed over seasons)
#' @param figure Logical, whether to generate the plot
#' @details
#' - `plot_catch` plots the fishery catch by stock or region (either whole numbers or proportions)
#'
#' @export
plot_catch <- function(fit, f = 1, by = c("region", "stock"), prop = FALSE, annual = FALSE, figure = TRUE) {
  by <- match.arg(by)
  var <- "CB_ymfrs"

  Dlabel <- get_MSAdata(fit)@Dlabel
  year <- Dlabel@year
  nm <- max(length(Dlabel@season), 1)

  if (by == "stock") {
    x <- apply(fit@report[[var]][, , f, , , drop = FALSE], c(1, 2, 5), sum) # C_yms
    name <- Dlabel@stock
  } else {
    x <- apply(fit@report[[var]][, , f, , , drop = FALSE], c(1, 2, 4), sum)  # C_ymr
    name <- Dlabel@region
  }

  if (annual) {
    x <- apply(x, c(1, 3), sum)
  } else {
    year <- make_yearseason(year, nm)
    x <- collapse_yearseason(x)
  }

  fname <- Dlabel@fleet[f]
  if (figure) {
    color <- make_color(ncol(x), type = by)
    ylab <- if (prop) paste(fname, "catch proportion") else paste(fname, "catch")
    barplot2(x, cols = color, leg.names = name, xval = year, ylab = ylab, prop = prop)
  }

  output <- data.frame(
    year = year,
    catch = x,
    fleet = fname
  )

  invisible(output)
}

#' @rdname plot-MSA-data
#' @aliases plot_index
#' @param i Integer, indexes the survey
#' @param zoom Logical, for `plot_index()`. If \code{TRUE}, plots a subset of years with observed data points. Otherwise, plots
#' predicted values over all model years.
#' @param figure Logical, whether to generate the plot
#' @details
#' - `plot_index` plots indices of abundance
#' @export
plot_index <- function(fit, i = 1, zoom = FALSE, figure = TRUE) {
  dat <- get_MSAdata(fit)
  Dlabel <- dat@Dlabel
  year <- Dlabel@year
  ny <- dat@Dmodel@ny
  nm <- dat@Dmodel@nm

  iname <- Dlabel@index[i]

  iobs <- dat@Dsurvey@Iobs_ymi[, , i, drop = FALSE]
  isd <- dat@Dsurvey@Isd_ymi[, , i, drop = FALSE]

  ni_plot <- length(i)

  ipred <- fit@report$I_ymi[, , i, drop = FALSE]

  year <- make_yearseason(year, nm)
  ipred <- collapse_yearseason(ipred)
  iobs <- collapse_yearseason(iobs)
  isd <- collapse_yearseason(isd)

  ind_all <- apply(iobs, 1, function(x) !all(is.na(x)))

  output <- NULL
  if (any(!is.na(iobs))) {
    iupper <- exp(log(iobs) + 1.96 * isd)
    ilower <- exp(log(iobs) - 1.96 * isd)

    output <- lapply(1:ni_plot, function(ii) {
      .output <- data.frame(
        year = year, obs = iobs[, ii], pred = ipred[, ii], lwr = ilower[, ii], upr = iupper[, ii], name = iname[ii]
      )
      if (zoom) {
        mind <- rep(1:nm, ny)
        mobs <- unique(mind[!is.na(.output$obs)])
        ind <- mind %in% mobs
        return(.output[ind, ])
      } else {
        return(.output)
      }
    })
    output <- do.call(rbind, output)

    if (figure) {
      irange <- lapply(1:ni_plot, function(ii) {
        data.frame(year = range(year[ind_all]), y = c(0, 1.1) * max(iupper[, ii], ipred[, ii], na.rm = TRUE), name = iname[ii])
      })
      irange <- do.call(rbind, irange)

      irange$name <- factor(irange$name, iname)
      output$name <- factor(output$name, iname)

      tinyplot(
        irange$year, irange$y,
        facet = irange$name, facet.args = list(free = TRUE),
        type = "n", grid = TRUE,
        xlab = "Year",  ylab = "Index"
      )
      tinyplot(
        output$year, output$pred,
        facet = output$name, facet.args = list(free = TRUE),
        type = "l", col = "red",
        add = TRUE
      )
      tinyplot(
        output$year, output$obs,
        facet = output$name, facet.args = list(free = TRUE),
        ymin = output$lwr, ymax = output$upr,
        type = "pointrange", pch = 16,
        add = TRUE
      )

      #plot(year, iobs, xlab = "Year", ylab = iname, type = "p", pch = 16,
      #     ylim = c(0, 1.1) * range(ipred, iupper, na.rm = TRUE), zero_line = TRUE)
      #arrows(year, y0 = ilower, y1 = iupper, length = 0)
      #lines(year, ipred, lwd = 2, col = 2, type = ifelse(length(year) > 10, "l", "o"))
    }
  }

  invisible(output)
}


#' @rdname plot-MSA-data
#' @aliases plot_CAA
#' @param f Integer, indexes the fleet
#' @param r Integer, indexes the region
#' @param do_mean Logical, whether to plot full compositions or time series of mean length or mean age
#' @details
#' - `plot_CAA` plots the fishery catch at age
#' @importFrom stats weighted.mean
#' @export
plot_CAA <- function(fit, f = 1, r = 1, do_mean = FALSE, figure = TRUE) {
  dat <- get_MSAdata(fit)
  output <- NULL

  if (sum(dat@Dfishery@CAAN_ymfr, na.rm = TRUE)) {
    N <- apply(dat@Dfishery@CAAN_ymfr[, , f, r, drop = FALSE], 1:2, identity) |> t() |> as.numeric()
    N[is.na(N)] <- 0

    if (any(N > 0)) {
      Dlabel <- dat@Dlabel
      nm <- max(length(Dlabel@season), 1)

      year <- Dlabel@year
      year <- make_yearseason(year, nm)

      pred <- apply(fit@report$CN_ymafrs[, , , f, r, , drop = FALSE], 1:3, sum)
      pred <- collapse_yearseason(pred) |> apply(1, function(x) x/sum(x, na.rm = TRUE)) |> t()
      pred[is.na(pred)] <- 0

      obs <- apply(dat@Dfishery@CAAobs_ymafr[, , , f, r, drop = FALSE], 1:3, identity)
      obs <- collapse_yearseason(obs) |> apply(1, function(x) x/sum(x, na.rm = TRUE)) |> t()

      include <- rowSums(obs, na.rm = TRUE) > 0
      if (do_mean) {
        age <- seq(1, dat@Dmodel@na) - 1
        mpred <- apply(pred, 1, function(w) weighted.mean(x = age, w = w))
        mobs <- apply(obs, 1, function(w) weighted.mean(x = age, w = w))

        if (figure) {
          ylim <- c(0.9, 1.1) * range(c(mobs, mpred), na.rm = TRUE)

          plot(year[include], mobs[include], xlab = "Year", ylab = "Mean age", pch = 1, type = "o", ylim = ylim)
          lines(year[include], mpred[include], col = 2, lwd = 2)
        }

        output <- data.frame(
          year = year,
          pred = mpred,
          obs = mobs,
          fleet = dat@Dlabel@fleet[f],
          region = dat@Dlabel@region[r]
        )
      } else {
        if (figure) {
          plot_composition(obs[include, , drop = FALSE], pred[include, , drop = FALSE], xval = Dlabel@age, ylab = "Proportion",
                           zval = year[include], N = N[include])
        }

        obs_df <- structure(obs, dimnames = list(year = year, age = Dlabel@age)) |>
          reshape2::melt(value.name = "obs")
        pred_df <- structure(pred, dimnames = list(year = year, age = Dlabel@age)) |>
          reshape2::melt(value.name = "pred")
        N_df <- data.frame(
          year = year,
          fleet = dat@Dlabel@fleet[f],
          region = dat@Dlabel@region[r],
          N = N
        )
        output <- merge(N_df, merge(obs_df, pred_df))
      }
    }
  }

  invisible(output)
}

#' @rdname plot-MSA-data
#' @aliases plot_CAL
#' @param agg Logical, whether to aggregate composition across all time steps
#' @details
#' - `plot_CAL` plots the catch at length
#' @importFrom stats aggregate
#' @export
plot_CAL <- function(fit, f, r, agg = FALSE, do_mean = FALSE, figure = TRUE) {
  dat <- get_MSAdata(fit)
  output <- NULL
  if (missing(f)) f <- 1:dat@Dfishery@nf
  if (missing(r)) r <- 1:dat@Dmodel@nr
  nf_plot <- length(f)
  nr_plot <- length(r)

  if (sum(dat@Dfishery@CALN_ymfr, na.rm = TRUE)) {
    N <- dat@Dfishery@CALN_ymfr[, , f, r, drop = FALSE]
    N[is.na(N)] <- 0

    if (any(N > 0)) {
      Dlabel <- dat@Dlabel
      nm <- max(length(Dlabel@season), 1)

      year <- Dlabel@year
      year <- make_yearseason(year, nm)

      pred <- apply(fit@report$CN_ymlfrs[, , , f, r, , drop = FALSE], 1:5, sum)
      pred <- collapse_yearseason(pred) |> apply(c(1, 3, 4), function(x) x/sum(x, na.rm = TRUE)) |> aperm(c(2, 1, 3, 4))
      pred[is.na(pred)] <- 0

      obs <- dat@Dfishery@CALobs_ymlfr[, , , f, r, drop = FALSE]
      obs <- collapse_yearseason(obs) |> apply(c(1, 3, 4), function(x) x/sum(x, na.rm = TRUE)) |> aperm(c(2, 1, 3, 4))

      N <- collapse_yearseason(N)

      if (do_mean) {
        mpred <- apply(pred, c(1, 3, 4), function(w) weighted.mean(x = dat@Dmodel@lmid, w = w))
        mobs <- apply(obs, c(1, 3, 4), function(w) weighted.mean(x = dat@Dmodel@lmid, w = w))
        Nobs <- apply(mobs, 2:3, sum, na.rm = TRUE)

        output <- lapply(1:nf_plot, function(ff) {
          .m <- lapply(1:nr_plot, function(rr) {
            .output <- output[output$fleet == dat@Dlabel@fleet[ff] & output$region == dat@Dlabel@region[rr], ]
            if (Nobs[ff, rr]) {
              data.frame(
                year = year,
                obs = mobs[, ff, rr],
                pred = mpred[, ff, rr],
                fleet = dat@Dlabel@fleet[ff],
                region = dat@Dlabel@region[rr]
              )
            } else {
              data.frame()
            }
          })
          do.call(rbind, .m)
        })
        output <- do.call(rbind, output)

        if (figure) {

          mrange <- lapply(1:nf_plot, function(ff) {
            .m <- lapply(1:nr_plot, function(rr) {
              if (Nobs[ff, rr]) {
                .output <- output[output$fleet == dat@Dlabel@fleet[ff] & output$region == dat@Dlabel@region[rr], ]
                data.frame(
                  year = range(year),
                  y = c(0.9, 1.1) * range(.output$obs, .output$pred, na.rm = TRUE),
                  fleet = dat@Dlabel@fleet[ff],
                  region = dat@Dlabel@region[rr]
                )
              } else {
                data.frame()
              }
            })
            do.call(rbind, .m)
          })
          mrange <- do.call(rbind, mrange)

          tinyplot(
            mrange$year, mrange$y,
            facet = paste(mrange$fleet, mrange$region, sep = ", "), facet.args = list(free = TRUE),
            type = "n", grid = TRUE,
            xlab = "Year", ylab = "Mean length"
          )

          out_dat <- output[!is.na(output$obs), ]
          tinyplot(
            out_dat$year, out_dat$obs,
            facet = paste(out_dat$fleet, out_dat$region, sep = ", "),
            facet.args = list(free = TRUE),
            type = "o",
            add = TRUE
          )

          out_pred <- output[!is.na(output$pred), ]
          tinyplot(
            out_pred$year, out_pred$pred,
            facet = paste(out_pred$fleet, out_pred$region, sep = ", "), facet.args = list(free = TRUE),
            type = "l", col = "red", lwd = 2, add = TRUE
          )
        }


      } else {
        obs_df <- structure(
          obs,
          dimnames = list(year = year, lmid = dat@Dmodel@lmid,
                          fleet = dat@Dlabel@fleet[f], region = dat@Dlabel@region[r])
        ) |>
          reshape2::melt(value.name = "obs")
        pred_df <- structure(
          pred,
          dimnames = list(year = year, lmid = dat@Dmodel@lmid,
                          fleet = dat@Dlabel@fleet[f], region = dat@Dlabel@region[r])
        ) |>
          reshape2::melt(value.name = "pred")
        N_df <- structure(
          N,
          dimnames = list(year = year, fleet = dat@Dlabel@fleet[f], region = dat@Dlabel@region[r])
        ) |>
          reshape2::melt(value.name = "N")

        output <- merge(N_df, merge(obs_df, pred_df))

        if (agg) {

          .obs_agg <- aggregate(
            output$N * output$obs, by = list(lmid = output$lmid, fleet = output$fleet),
            FUN = sum, na.rm = TRUE
          )
          names(.obs_agg)[3] <- "obs"
          .pred_agg <- aggregate(
            output$N * output$pred, by = list(lmid = output$lmid, fleet = output$fleet),
            FUN = sum, na.rm = TRUE
          )
          names(.pred_agg)[3] <- "pred"

          output <- merge(.obs_agg, .pred_agg, sort = FALSE)

          if (figure) {
            frange <- lapply(unique(output$fleet), function(ff) {
              .output <- output[output$fleet == ff, ]
              data.frame(
                lmid = range(output$lmid),
                y = c(0, 1.1) * max(.output$obs, .output$pred, na.rm = TRUE),
                fleet = ff
              )
            })
            frange <- do.call(rbind, frange)
            tinyplot(
              frange$lmid, frange$y,
              facet = frange$fleet, facet.args = list(free = TRUE),
              type = "n",
              xlab = "Length", ylab = "N"
            )
            tinyplot(
              output$lmid, output$obs,
              facet = output$fleet, facet.args = list(free = TRUE),
              type = "area", pch = 16,
              add = TRUE
            )
            tinyplot(
              output$lmid, output$pred,
              facet = output$fleet, facet.args = list(free = TRUE),
              type = "l", col = 2, lwd = 3,
              add = TRUE
            )
          }

        } else if (figure) {

          if (length(f) > 1 || length(r) > 1) stop("Specify one value each for f and r")

          include <- apply(obs, 1, sum, na.rm = TRUE) > 0
          plot_composition(
            matrix(obs[include, , , ], sum(include), dat@Dmodel@nl),
            matrix(pred[include, , , ], sum(include), dat@Dmodel@nl),
            xval = dat@Dmodel@lmid,
            xlab = "Length", ylab = "Proportion",
            zval = year[include], N = as.numeric(N[include, , ])
          )
        }
      }
    }
  }

  invisible(output)
}

#' @rdname plot-MSA-data
#' @aliases plot_IAA
#' @details
#' - `plot_IAA` plots the index age composition
#' @export
plot_IAA <- function(fit, i = 1, do_mean = FALSE, figure = TRUE) {
  dat <- get_MSAdata(fit)
  output <- NULL

  if (sum(dat@Dsurvey@IAAN_ymi, na.rm = TRUE)) {
    N <- apply(dat@Dsurvey@IAAN_ymi[, , i, drop = FALSE], 1:2, identity) |> t() |> as.numeric()
    N[is.na(N)] <- 0

    if (any(N > 0)) {
      Dlabel <- dat@Dlabel
      nm <- max(length(Dlabel@season), 1)

      year <- Dlabel@year
      year <- make_yearseason(year, nm)

      pred <- apply(fit@report$IN_ymais[, , , i, , drop = FALSE], 1:3, sum)
      pred <- collapse_yearseason(pred) |> apply(1, function(x) x/sum(x, na.rm = TRUE)) |> t()
      pred[is.na(pred)] <- 0

      obs <- apply(dat@Dsurvey@IAAobs_ymai[, , , i, drop = FALSE], 1:3, identity)
      obs <- collapse_yearseason(obs) |> apply(1, function(x) x/sum(x, na.rm = TRUE)) |> t()

      include <- rowSums(obs, na.rm = TRUE) > 0
      if (do_mean) {
        age <- seq(1, dat@Dmodel@na) - 1
        mpred <- apply(pred, 1, function(w) weighted.mean(x = age, w = w))
        mobs <- apply(obs, 1, function(w) weighted.mean(x = age, w = w))

        if (figure) {
          ylim <- c(0.9, 1.1) * range(c(mobs, mpred), na.rm = TRUE)

          plot(year[include], mobs[include], xlab = "Year", ylab = "Mean age", pch = 1, type = "o", ylim = ylim)
          lines(year[include], mpred[include], col = 2, lwd = 2)
        }

        output <- data.frame(
          year = year,
          pred = mpred,
          obs = mobs,
          name = dat@Dlabel@index[i]
        )

      } else {
        if (figure) {
          plot_composition(obs[include, , drop = FALSE], pred[include, , drop = FALSE], xval = Dlabel@age, ylab = "Proportion",
                           zval = year[include], N = N[include])
        }

        obs_df <- structure(obs, dimnames = list(year = year, age = Dlabel@age)) |>
          reshape2::melt(value.name = "obs")
        pred_df <- structure(pred, dimnames = list(year = year, age = Dlabel@age)) |>
          reshape2::melt(value.name = "pred")
        N_df <- data.frame(
          year = year,
          name = dat@Dlabel@index[i],
          N = N
        )
        output <- merge(N_df, merge(obs_df, pred_df))
      }

    }
  }

  invisible(output)
}

#' @rdname plot-MSA-data
#' @aliases plot_IAL
#' @details
#' - `plot_IAL` plots the index length composition
#' @export
plot_IAL <- function(fit, i = 1, do_mean = FALSE, figure = TRUE) {
  dat <- get_MSAdata(fit)
  output <- NULL

  if (sum(dat@Dsurvey@IALN_ymi, na.rm = TRUE)) {
    N <- apply(dat@Dsurvey@IALN_ymi[, , i, drop = FALSE], 1:2, identity) |> t() |> as.numeric()
    N[is.na(N)] <- 0

    if (any(N > 0)) {
      Dlabel <- dat@Dlabel
      nm <- max(length(Dlabel@season), 1)

      year <- Dlabel@year
      year <- make_yearseason(year, nm)

      pred <- apply(fit@report$IN_ymlis[, , , i, , drop = FALSE], 1:3, sum)
      pred <- collapse_yearseason(pred) |> apply(1, function(x) x/sum(x, na.rm = TRUE)) |> t()
      pred[is.na(pred)] <- 0

      obs <- apply(dat@Dsurvey@IALobs_ymli[, , , i, drop = FALSE], 1:3, identity)
      obs <- collapse_yearseason(obs) |> apply(1, function(x) x/sum(x, na.rm = TRUE)) |> t()

      include <- rowSums(obs, na.rm = TRUE) > 0
      if (do_mean) {
        mpred <- apply(pred, 1, function(w) weighted.mean(x = dat@Dmodel@lmid, w = w))
        mobs <- apply(obs, 1, function(w) weighted.mean(x = dat@Dmodel@lmid, w = w))

        if (figure) {
          ylim <- c(0.9, 1.1) * range(c(mobs, mpred), na.rm = TRUE)

          plot(year[include], mobs[include], xlab = "Year", ylab = "Mean length", pch = 1, type = "o", ylim = ylim)
          lines(year[include], mpred[include], col = 2, lwd = 2)
        }

        output <- data.frame(
          year = year,
          pred = mpred,
          obs = mobs,
          name = dat@Dlabel@index[i]
        )

      } else {
        if (figure) {
          plot_composition(obs[include, ], pred[include, ], xval = dat@Dmodel@lmid,
                           xlab = "Length", ylab = "Proportion",
                           zval = year[include > 0], N = N[include])
        }

        obs_df <- structure(obs, dimnames = list(year = year, lmid = dat@Dmodel@lmid)) |>
          reshape2::melt(value.name = "obs")
        pred_df <- structure(pred, dimnames = list(year = year, lmid = dat@Dmodel@lmid)) |>
          reshape2::melt(value.name = "pred")
        N_df <- data.frame(
          year = year,
          name = dat@Dlabel@index[i],
          N = N
        )
        output <- merge(N_df, merge(obs_df, pred_df))
      }

    }
  }

  invisible()
}

#' @rdname plot-MSA-data
#' @param ff Integer, indexes the aggregate fleet (for stock composition data)
#' @param aa Integer, indexes the aggregate age class (for stock composition and tag data)
#' @aliases plot_SC
#' @details
#' - `plot_SC` plots the stock composition
#' @importFrom graphics matlines
#' @export
plot_SC <- function(fit, ff = 1, aa = 1, r = 1, prop = FALSE, figure = TRUE) {
  dat <- get_MSAdata(fit)
  output <- NULL

  if (dat@Dmodel@ns == 1) stop("Stock composition figure not needed.")

  if (sum(dat@Dfishery@SC_ymafrs, na.rm = TRUE)) {
    N <- apply(dat@Dfishery@SC_ymafrs[, , aa, ff, r, , drop = FALSE], 1:2, sum)
    N[is.na(N)] <- 0

    if (any(N > 0)) {
      Dlabel <- dat@Dlabel
      nm <- max(length(Dlabel@season), 1)

      year <- Dlabel@year
      year <- make_yearseason(year, nm)

      pred <- apply(fit@report$SCpred_ymafrs[, , aa, ff, r, , drop = FALSE], c(1, 2, 6), identity)
      pred <- collapse_yearseason(pred) |> apply(1, function(x) x/sum(x, na.rm = TRUE)) |> t()
      pred[is.na(pred)] <- 0

      N <- collapse_yearseason(N)

      obs <- apply(dat@Dfishery@SC_ymafrs[, , aa, ff, r, , drop = FALSE], c(1, 2, 6), identity)
      obs <- collapse_yearseason(obs) |> apply(1, function(x) x/sum(x, na.rm = TRUE)) |> t()

      if (figure) {
        if (prop) {
          color <- make_color(ncol(pred), type = "stock")
          barplot2(pred, cols = color, leg.names = Dlabel@stock, xval = year)

          if (dat@Dmodel@ns == 2) {
            obs_cumsum <- apply(obs, 1, cumsum) |> t()
            matlines(obs_cumsum[, -dat@Dmodel@ns, drop = FALSE], type = "o", col = 2, pch = 16, ylim = c(0, 1))
          }
        } else {
          include <- rowSums(obs, na.rm = TRUE) > 0

          plot_composition(obs[include, , drop = FALSE], pred[include, , drop = FALSE], xval = 1:dat@Dmodel@ns,
                           xlab = "Stock", ylab = "Proportion",
                           zval = year[include], N = N[include],
                           xaxislab = Dlabel@stock)
        }
      }

      obs_df <- structure(obs, dimnames = list(year = year, stock = Dlabel@stock)) |>
        reshape2::melt(value.name = "obs")
      pred_df <- structure(pred, dimnames = list(year = year, stock = Dlabel@stock)) |>
        reshape2::melt(value.name = "pred")

      N_df <- data.frame(
        year = year,
        ff = ff,
        aa = aa,
        region = dat@Dlabel@region[r]
      )
      output <- merge(N_df, merge(obs_df, pred_df))

      if (length(dat@Dfishery@SCstdev_ymafrs)) {
        se_df <- apply(dat@Dfishery@SCstdev_ymafrs[, , aa, ff, r, , drop = FALSE], c(1, 2, 6), identity) |>
          collapse_yearseason() |>
          structure(dimnames = list(year = year, stock = Dlabel@stock)) |>
          reshape2::melt(value.name = "se")
        output <- merge(output, se_df)
      }
    }
  }

  invisible(output)
}

#' @importFrom graphics lines mtext axis
plot_composition <- function(obs, pred = NULL, xval = 1:ncol(obs), xlab = "Age",
                             ylab = "Value", zval = 1:nrow(obs), N = rowSums(obs),
                             xaxislab = xval, ncol = 4, nrow = 4) {

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))
  par(mfcol = c(nrow, ncol), mar = rep(0, 4), oma = c(5.1, 5.1, 2.1, 2.1))

  ylim <- c(0, 1.1) * range(pred, obs, na.rm = TRUE)

  yaxp <- c(0, max(pretty(ylim, n = 4)), 4)
  if (max(obs, pred, na.rm = TRUE) == 1) yaxp <- c(0, 1, 4)
  las <- 1
  nplot <- nrow(pred)
  nset <- nrow * ncol
  xaxt_manual <- any(xaxislab != xval)
  N <- round(N, 2)

  for(i in 1:nplot) {
    yaxt <- ifelse(i %% nset %in% c(1:nrow), "s", "n") # TRUE = first column
    xaxt <- ifelse(i < nplot && i %% nrow %in% c(1:(nrow-1)), "n", "s") # TRUE = first three rows

    plot(xval, obs[i, ], type = "o", ylim = ylim, yaxp = yaxp, xaxt = ifelse(xaxt_manual, "n", xaxt), yaxt = yaxt, las = las)
    if (xaxt_manual && xaxt == "s") axis(1, at = xval, labels = xaxislab)
    if (!is.null(pred)) lines(xval, pred[i, ], lwd = 2, col = 2)
    legend("topright", legend = c(zval[i], ifelse(is.null(N), "", paste0("N = ", N[i]))), bty = "n", xjust = 1)

    if (i %% nset == 1) {
      mtext(xlab, side = 1, line = 3, outer = TRUE)
      mtext(ylab, side = 2, line = 3.5, outer = TRUE)
    }
  }

  invisible()
}

#' @rdname plot-MSA-data
#' @param yy Integer, indexes the aggregate years (for the tag data)
#' @param s Integer, indexes the stock
#' @aliases plot_tagmov
#' @details
#' - `plot_tagmov` plots the tag movements
#' @export
plot_tagmov <- function(fit, s = 1, yy = 1, aa = 1, figure = TRUE) {
  dat <- get_MSAdata(fit)
  output <- NULL

  if (dat@Dmodel@nr == 1) stop("Tag movement figure not needed.")

  if (sum(dat@Dtag@tag_ymarrs, na.rm = TRUE)) {
    N <- apply(dat@Dtag@tag_ymarrs[yy, , aa, , , s, drop = FALSE], c(2, 4), sum)
    N[is.na(N)] <- 0

    if (any(N > 0)) {
      Dlabel <- dat@Dlabel

      pred <- apply(fit@report$tagpred_ymarrs[yy, , aa, , , s, drop = FALSE], c(2, 4, 5), identity)
      pred2 <- collapse_yearseason(pred)

      obs <- apply(dat@Dtag@tag_ymarrs[yy, , aa, , , s, drop = FALSE], c(2, 4, 5), identity)
      obs2 <- apply(obs, c(1, 2), function(x) x/sum(x, na.rm = TRUE)) |> aperm(c(2, 3, 1)) |>
        collapse_yearseason() # Season + region of origin
      obs2[is.na(obs2)] <- 0

      N <- collapse_yearseason(N)

      if (figure) {
        nrow <- dat@Dmodel@nr
        ncol <- dat@Dmodel@nm

        z <- paste(
          paste("Origin:", rep(Dlabel@region, dat@Dmodel@nm)),
          paste(rep(paste("\nSeason", 1:dat@Dmodel@nm), each = dat@Dmodel@nr))
        )

        plot_composition(obs2, pred2, xval = 1:dat@Dmodel@nr,
                         xlab = "Destination", ylab = "Proportion",
                         zval = z, N = N,
                         xaxislab = Dlabel@region, ncol = ncol, nrow = nrow)
      }

      obs_df <- structure(
        obs,
        dimnames = list(season = Dlabel@season, from = Dlabel@region, to = Dlabel@region)
      ) |>
        reshape2::melt(value.name = "obs")
      pred_df <- structure(
        pred,
        dimnames = list(season = Dlabel@season, from = Dlabel@region, to = Dlabel@region)
      ) |>
        reshape2::melt(value.name = "pred")

      N_df <- data.frame(
        year = yy,
        yy = yy,
        aa = aa,
        stock = dat@Dlabel@stock[s]
      )
      output <- merge(N_df, merge(obs_df, pred_df))
    }
  }

  invisible(output)
}

#' If statements compatible with RTMB
#'
#' Convenience functions that allow taping of gradients in RTMB with if expressions,
#' following the corresponding `CppAD` functions.
#'
#' @param left Numeric on left hand side of the evaluation
#' @param right Numeric on right hand side of the evaluation
#' @param if_true Numeric if expression is true
#' @param if_false Numeric if expression is false
#' @return Numeric
#' @details Functions should be vectorized.
#'
#' `CondExpLt` evaluates whether `left < right`
#' @aliases CondExpLe CondExpGt CondExpGe CondExpEq
#'
#' @examples
#' library(RTMB)
#' TapeConfig(comparison = "tape")
#' f <- function(x) CondExpLt(x, 3, 0, x^2)
#' g <- MakeTape(f, numeric(1))
#' x <- seq(0, 5)
#'
#' # Does not work!
#' f2 <- function(x) if (x < 3) 0 else x^2
#' g2 <- MakeTape(f2, numeric(1))
#'
#' # Compare the real answer (deriv) with various values returned by RTMB
#' data.frame(
#'   x = x,
#'   deriv = ifelse(x < 3, 0, 2 * x),
#'   deriv_f = sapply(x, g$jacobian),
#'   deriv_f2 = sapply(x, g2$jacobian)
#' )
#' @export
CondExpLt <- function(left, right, if_true, if_false) {
  (left < right) * if_true + (left >= right) * if_false
}

#' @rdname CondExpLt
#' @details `CondExpLe` evaluates whether `left <= right`
#' @export
CondExpLe <- function(left, right, if_true, if_false) {
  (left <= right) * if_true + (left > right) * if_false
}

#' @rdname CondExpLt
#' @details `CondExpGt` evaluates whether `left > right`
#' @export
CondExpGt <- function(left, right, if_true, if_false) {
  (left > right) * if_true + (left <= right) * if_false
}

#' @rdname CondExpLt
#' @details `CondExpGe` evaluates whether `left >= right`
#' @export
CondExpGe <- function(left, right, if_true, if_false) {
  (left >= right) * if_true + (left < right) * if_false
}

#' @rdname CondExpLt
#' @details `CondExpEq` evaluates whether `left == right`
#' @export
CondExpEq <- function(left, right, if_true, if_false) {
  (left == right) * if_true + (left != right) * if_false
}

#' Make list of parameters for RTMB
#'
#' @description
#' Sets up the list of parameters, map of parameters (see `map` argument in [TMB::MakeADFun()]), and identifies some random effects parameters
#' based on the input data and some user choices on model configuration.
#'
#' These functions provide a template for the parameter and map setup that can be adjusted for alternative configurations. [check_parameters()]
#' checks whether custom made parameter lists are of the correct dimension.
#' @seealso [MSAdata-class]
#'
#' @section Parameters:
#'
#' Generally parameter names will have up to three components, separated by underscores.
#' For example, `log_M_s` represents the natural logarithm of natural mortality, and is a vector by stock `s`.
#'
#' The first component describes the transformation from the estimated parameter space to the normal parameter space,
#' frequently, `log` or `logit`. Prefix `t` indicates some other custom transformation that is described below.
#'
#' Second is the parameter name, e.g., `M` for natural mortality, `rdev` for recruitment deviates, etc.
#'
#' Third is the dimension of the parameter variable and the indexing for the vectors, matrices, and arrays, e.g., `y` for year, `s` for stock.
#' See [MSAdata-class]. Here, an additional index `p` represents some other number of parameters that is described below.
#'
#' \describe{
#' \item{`t_R0_s`}{Vector by `s`. Unfished recruitment, i.e., intersection of unfished replacement line and stock recruit function,
#' is represented as: `R0_s <- exp(t_R0_s) * MSAdata@Dmodel@scale_s`. By default, `t_R0_s = 3`}
#' \item{`t_h_s`}{Vector by `s`. Steepness of the stock-recruit function. Logit space for Beverton-Holt (`h = 0.8 * plogis(t_h_s) + 0.2`) and log space for Ricker function (`h = exp(t_h_s) + 0.2`).
#' Default steepness value of 0.8}
#' \item{`mat_ps`}{Matrix `[2, s]`. Maturity parameters (can be estimated or specified in data object). Logistic functional form. The
#' parameter in the first row is the age of 50 percent maturity in logit space: `a50_s <- plogis(mat_ps[1, ] * na)`.
#' In the second row is the age of 95 percent maturity as a logarithmic offset: `a95_s <- a50_s + exp(mat_ps[2, ])`.
#' Default `a50_s <- 0.5 * na` and `a95_s <- a50_s + 1`}
#' \item{`log_M_s`}{Vector by `s`. Natural logarithm of natural mortality (can be estimated or specified in data object).
#' Default parameter value for all stocks: `M <- -log(0.05)/MSAdata@Dmodel@na`}
#' \item{`log_rdev_ys`}{Matrix `[y, s]`. Log recruitment deviations. By default, all start values are at zero.}
#' \item{`log_sdr_s`}{Vector by `s`. log-Standard deviation of the log recruitment deviations. Default SD = 0.4}
#' \item{`log_q_fs`}{Matrix `[f, s]`. The natural logarithm of `q_fs`, the relative fishing efficiency of `f` for stock `s`.
#' Equal values imply equal catchability of all stocks. See equations in [calc_F()]. Default sets all values to zero.}
#' \item{`log_Fdev_ymfr`}{Array `[y, m, f, r]`. Fishing mortality parameters. For each fleet, the log of F is estimated directly for the
#' reference year, season, region. For other strata, F is an offset from this value:
#' \deqn{
#' F_{y,m,f,r} = \begin{cases}
#' \exp(x^{\textrm{Fmult}}_f) \quad & y = y_{\textrm{ref}}, m = m_{\textrm{ref}}, r = r_{\textrm{ref}}\\
#' \exp(x^{\textrm{Fmult}}_f + x^{\textrm{Fdev}}_{y,m,r}) \quad & \textrm{otherwise}
#' \end{cases}
#' }
#' }
#' \item{`sel_pf`}{Matrix `[3, f]`. Fishery selectivity parameters in logit or log space. See equations in [conv_selpar()], where `sel_pf` is the `x` matrix.}
#' \item{`sel_pi`}{Matrix `[3, i]`. Index selectivity parameters in logit or log space. See equations in [conv_selpar()], where `sel_pi` is the `x` matrix.}
#' \item{`mov_x_marrs`}{Array `[m, a, r, r, s]`. Base movement matrix. Set to -1000 to effectively exclude movements from region pairs.
#' See equations in [conv_mov()]}
#' \item{`mov_g_ymars`}{Array `[y, m, a, r, s]`. Attractivity term in gravity model for movement. If `x` and `v` are zero,
#' this matrix specifies the distribution of total stock abundance into the various regions. See equations in [conv_mov()]}
#' \item{`mov_v_ymas`}{Array `[y, m, a, s]`. Viscosity term in gravity model for movement. See equations in [conv_mov()]}
#' \item{`log_sdg_rs`}{Array `[r, s]`. Marginal log standard deviation in the stock distribution (`mov_g_ymars`) among regions for stock `s`.
#' Only used when `est_mov = "dist_random"`. Default SD of 0.1.}
#' \item{`t_corg_ps`}{Array `[sum(1:(nr - 1)), s]`. Lower triangle of the correlation matrix for `mov_g_ymars`, to be obtained with the
#' Cholesky factorization. Only used when `est_mov = dist_random`. Default values of zero.}
#' \item{`log_initF_mfr`}{Array `[m, f, r]`. Initial F corresponding to the equilibrium catch. Default is `log(0.1)` for seasons, fleets, and regions where `Dfishery@Cinit_mfr > 1e-8`, otherwise, not estimated (Finit = 0).}
#' \item{`log_initrdev_as`}{Array `[na - 1, s]`. Recruitment deviations for the initial abundance-at-age vector. Default is zero.}
#' }
#'
#' @section Start list:
#' Users can provide `R0_s` and `h_s` in the start list. [make_parameters()] will make the appropriate transformation for the starting values
#' of `t_R0_s` and `t_h_s`, respectively.
#'
#' @param MSAdata S4 data object
#' @param start An optional list of parameters. Named list of parameters with the associated dimensions and transformations below.
#' Overrides default values created by [make_parameters()].
#' @param silent Logical, whether [make_map()] reports messages to the console
#' @param ... Various arguments for [make_map()] (could be important!)
#' @return
#' [make_parameters()] returns a list of parameters (`"p"`) concatenated with the output of [make_map()].
#' @importFrom stats approx
#' @export
make_parameters <- function(MSAdata, start = list(), map = list(),
                            est_mov = c("none", "dist_random", "gravity_fixed"),
                            silent = FALSE, ...) {
  est_mov <- match.arg(est_mov)

  Dmodel <- MSAdata@Dmodel
  Dstock <- MSAdata@Dstock
  Dfishery <- MSAdata@Dfishery

  ny <- Dmodel@ny
  nm <- Dmodel@nm
  na <- Dmodel@na
  nl <- Dmodel@nl
  nr <- Dmodel@nr
  ns <- Dmodel@ns

  nf <- Dfishery@nf

  p <- start

  # Stock parameters ----
  if (!is.null(start$R0_s)) {
    p$t_R0_s <- log(start$R0_s/Dmodel@scale_s)
    p$R0_s <- NULL
  } else if (is.null(start$t_R0_s)) {
    p$t_R0_s <- rep(3, ns)
  }

  if (is.null(start$h_s)) {
    start$h_s <- rep(0.8, ns)
  } else {
    p$h_s <- NULL
  }
  p$t_h_s <- ifelse(Dstock@SRR_s == "BH", qlogis((start$h_s - 0.2)/0.8), log(start$h_s - 0.2))

  if (is.null(p$mat_ps)) {
    p$mat_ps <- sapply(1:ns, function(s) {
      a50 <- 0.5*na
      a95 <- a50 + 1
      logit_a50 <- qlogis(a50/na)
      log_diff <- log(a95 - a50)
      c(logit_a50, log_diff)
    })
  }

  if (is.null(p$log_M_s)) p$log_M_s <- rep(log(-log(0.05)/na), ns)
  if (is.null(p$log_rdev_ys)) p$log_rdev_ys <- matrix(0, ny, ns)
  if (is.null(p$log_sdr_s)) p$log_sdr_s <- rep(log(0.4), ns)

  if (is.null(p$log_recdist_rs)) p$log_recdist_rs <- matrix(0, nr, ns)

  if (is.null(p$mov_x_marrs)) {
    if (est_mov == "none" && nr > 1) {
      p$mov_x_marrs <- array(-1000, c(nm, na, nr, nr, ns))
      mov_ind <- as.matrix(expand.grid(m = 1:nm, a = 1:na, rf = 1:nr, rt = 1:nr, s = 1:ns))
      mov_ind_stay <- mov_ind[, "rf"] == mov_ind[, "rt"]
      p$mov_x_marrs[mov_ind[mov_ind_stay, ]] <- 0
    } else {
      p$mov_x_marrs <- array(0, c(nm, na, nr, nr, ns))
    }
    if (any(!Dstock@presence_rs)) {
      for(s in 1:ns) {
        presence_r <- Dstock@presence_rs[, s]
        if (any(!presence_r)) p$mov_x_marrs[, , , !presence_r, s] <- -1000 # No fish go here
      }
    }
  }
  if (is.null(p$mov_g_ymars)) p$mov_g_ymars <- array(0, c(ny, nm, na, nr, ns))
  if (is.null(p$mov_v_ymas)) p$mov_v_ymas <- array(0, c(ny, nm, na, ns))
  if (is.null(p$log_sdg_rs)) p$log_sdg_rs <- array(log(0.1), c(nr, ns))
  if (is.null(p$t_corg_ps)) p$t_corg_ps <- array(0, c(sum(1:(nr - 1)), ns))

  # Fleet parameters ----
  if (is.null(p$log_q_fs)) {
    p$log_q_fs <- matrix(0, nf, ns)
  }
  if (is.null(p$log_Fdev_ymfr)) {
    if (Dmodel@condition == "F") {
      p$log_Fdev_ymfr <- sapply2(1:nr, function(r) {
        sapply2(1:nf, function(f) {
          sapply(1:nm, function(m) {
            sapply(1:ny, function(y) {
              Fmult_y <- y == Dmodel@y_Fmult_f[f]
              Fmult_m <- m == Dmodel@m_Fmult_f[f]
              Fmult_r <- r == Dmodel@r_Fmult_f[f]
              if (Fmult_y && Fmult_m && Fmult_r) {
                log(-log(0.05)/na/nm)
              } else {
                0
              }
            })
          })
        })
      })
      p$log_Fdev_ymfr[Dfishery@Cobs_ymfr <= 1e-8] <- -1000
    } else {
      p$log_Fdev_ymfr <- array(0, c(ny, nm, nf, nr))
    }
  }

  if (is.null(p$sel_pf)) {
    nb <- unique(as.numeric(Dfishery@sel_block_yf)) |> sort()

    if (length(Dfishery@CALobs_ymlfr)) {
      CAL_lb <- sapply(nb, function(b) {
        out <- rep(0, nl)
        sel_b <- Dfishery@sel_f[b]

        if (grepl("length", sel_b)) {
          out[] <- sapply2(1:nf, function(f) {
            sapply2(1:ny, function(y) {
              out <- rep(0, nl)
              if (Dfishery@sel_block_yf[y, f] == b) {
                vec_lrm <- sapply2(1:nm, function(m) {
                  sapply(1:nr, function(r) {
                    obs <- Dfishery@CALobs_ymlfr[y, m, , f, r]
                    if (sum(obs, na.rm = TRUE)) {
                      if (length(Dfishery@CALN_ymfr)) {
                        out <- Dfishery@CALN_ymfr[y, m, f, r] * obs/sum(obs, na.rm = TRUE)
                      } else {
                        out <- obs
                      }
                    } else {
                      out <- rep(0, nl)
                    }
                    return(out)
                  })
                })
                out[] <- apply(vec_lrm, 1, sum)
              }
              return(out)
            })
          }) |> apply(1, sum)
        }
        return(out)
      })
    } else {
      CAL_lb <- matrix(0, nl, nb)
    }

    if (length(Dfishery@CAAobs_ymafr)) {
      CAA_ab <- sapply(nb, function(b) {
        out <- rep(0, na)
        sel_b <- Dfishery@sel_f[b]

        if (grepl("age", sel_b)) {
          out[] <- sapply2(1:nf, function(f) {
            sapply(1:ny, function(y) {
              out <- rep(0, na)
              if (Dfishery@sel_block_yf[y, f] == b) {
                vec_arm <- sapply2(1:nm, function(m) {
                  sapply(1:nr, function(r) {
                    obs <- Dfishery@CAAobs_ymafr[y, m, , f, r]
                    if (sum(obs, na.rm = TRUE)) {
                      if (length(Dfishery@CAAN_ymfr)) {
                        out <- Dfishery@CAAN_ymfr[y, m, f, r] * obs/sum(obs, na.rm = TRUE)
                      } else {
                        out <- obs
                      }
                    } else {
                      out <- rep(0, na)
                    }
                    return(out)
                  })
                })
                out[] <- apply(vec_arm, 1, sum)
              }
              return(out)
            })
          }) |> apply(1, sum)
        }
        return(out)
      })
    } else {
      CAA_ab <- matrix(0, na, nb)
    }

    sel_pars <- sapply(nb, function(b) { # For reporting
      val <- rep(NA_real_, 2)
      sel_b <- Dfishery@sel_f[b]
      if (grepl("length", sel_b) && sum(CAL_lb[, b])) {
        full_sel <- min(Dmodel@lmid[which.max(CAL_lb[, b])], 0.94 * max(Dmodel@lmid))
        sel5 <- approx(cumsum(CAL_lb[, b]/sum(CAL_lb[, b])), Dmodel@lmid, 0.05)$y
      } else if (grepl("age", sel_b) && sum(CAA_ab[, b])) {
        age <- seq(1, na) - 1
        full_sel <- min(age[which.max(CAA_ab[, b])], 0.99 * max(age))
        sel5 <- approx(cumsum(CAA_ab[, b])/sum(CAA_ab[, b]), age, 0.05)$y
      }
      val[1] <- full_sel
      val[2] <- sel5
      return(val)
    })

    p$sel_pf <- sapply(nb, function(b) {
      val <- numeric(3)

      sel_b <- Dfishery@sel_f[b]
      if (grepl("length", sel_b) && sum(CAL_lb[, b])) {
        LFS <- min(Dmodel@lmid[which.max(CAL_lb[, b])], 0.94 * max(Dmodel@lmid))
        L5 <- approx(cumsum(CAL_lb[, b]/sum(CAL_lb[, b])), Dmodel@lmid, 0.05)$y
        if (is.na(L5) || L5 > 0.99 * LFS) L5 <- 0.5 * LFS

        sigma_asc <- min(0.5 * (LFS - L5), 0.5 * diff(range(Dmodel@lmid)))
        val[2] <- log(sigma_asc)
        val[1] <- qlogis(LFS/(0.95 * max(Dmodel@lmid)))
        if (grepl("dome", sel_b)) val[3] <- val[2]

      } else if (grepl("age", sel_b) && sum(CAA_ab[, b])) {
        age <- seq(1, na) - 1
        AFS <- min(age[which.max(CAA_ab[, b])], 0.99 * max(age))
        A5 <- approx(cumsum(CAA_ab[, b])/sum(CAA_ab[, b]), age, 0.05)$y
        if (is.na(A5) || A5 > 0.99 * AFS) A5 <- 0.5 * AFS

        sigma_asc <- min(0.5 * (AFS - A5), 0.5 * diff(range(age)))
        val[2] <- log(sigma_asc)
        val[1] <- qlogis(AFS/max(age))
        if (grepl("dome", sel_b)) val[3] <- val[2]
      }
      if (all(!val)) {
        # Apical sel halfway between 0 and max age/length
        # Sloping ascending selectivity limbs (25% of max age or three length bins)
        val[1] <- 0
        sel_limb <- if (grepl("length", sel_b)) log(3 * min(diff(Dmodel@lmid))) else log(0.25 * Dmodel@na)
        val[2] <- sel_limb
        if (grepl("dome", sel_b)) val[3] <- sel_limb
      }
      return(val)
    })
  }

  # Index parameters ----
  Dsurvey <- MSAdata@Dsurvey
  ni <- Dsurvey@ni
  if (ni > 0 && is.null(p$sel_pi)) {
    p$sel_pi <- sapply(1:ni, function(i) {
      sel_i <- Dsurvey@sel_i[i]
      val <- numeric(3)
      if (grepl("length", sel_i) && length(Dsurvey@IALobs_ymli)) {
        if (length(Dsurvey@IALN_ymi) && sum(Dsurvey@IALN_ymi[, , i])) {
          IAL <- sapply2(1:ny, function(y) {
            sapply(1:nm, function(m) {
              obs <- Dsurvey@IALobs_ymli[y, m, , i]
              Dsurvey@IALN_ymi[y, m, i] * obs/sum(obs, na.rm = TRUE)
            })
          }) |> apply(1, sum)
        } else {
          IAL <- apply(Dsurvey@IALobs_ymli[, , , i, drop = FALSE], 3, sum)
        }

        if (sum(IAL)) {
          LFS <- min(Dmodel@lmid[which.max(IAL)], 0.9 * max(Dmodel@lmid))
          L5 <- approx(cumsum(IAL)/sum(IAL), Dmodel@lmid, 0.05)$y
          if (is.na(L5) || L5 > 0.99 * LFS) L5 <- 0.5 * LFS

          sigma_asc <- min(0.5 * (LFS - L5), 0.25 * diff(range(Dmodel@lmid)))
          val[2] <- log(sigma_asc)
          val[1] <- qlogis(min(0.95, LFS/max(Dmodel@lmid)))
          if (grepl("dome", sel_i)) val[3] <- val[2]
        }
      } else if (grepl("age", sel_i) && length(Dsurvey@IAAobs_ymai)) {
        if (length(Dsurvey@IAAN_ymi) && sum(Dsurvey@IAAN_ymi[, , i])) {
          IAA <- sapply2(1:ny, function(y) {
            sapply(1:nm, function(m) {
              obs <- Dsurvey@IAAobs_ymai[y, m, , i]
              Dsurvey@IAAN_ymi[y, m, i] * obs/sum(obs, na.rm = TRUE)
            })
          }) |> apply(1, sum)
        } else {
          IAA <- apply(Dsurvey@IAAobs_ymai[, , , i, drop = FALSE], 3, sum)
        }

        if (sum(IAA)) {
          age <- seq(1, na) - 1
          AFS <- min(age[which.max(IAA)], 0.99 * max(age))
          A5 <- approx(cumsum(IAA)/sum(IAA), Dmodel@lmid, 0.05)$y
          if (is.na(A5) || A5 > 0.99 * AFS) A5 <- 0.5 * AFS

          sigma_asc <- min(0.5 * (AFS - A5), 0.5 * diff(range(age)))
          val[2] <- log(sigma_asc)
          val[1] <- qlogis(AFS/max(age))
          if (grepl("dome", sel_i)) val[3] <- val[2]
        }
      }
      if (all(!val)) {
        # Apical sel halfway between 0 and max age/length
        # Sloping ascending selectivity limbs (25% of max age or three length bins)
        val[1] <- 0
        sel_limb <- if (grepl("length", sel_i)) log(3 * min(diff(Dmodel@lmid))) else log(0.25 * Dmodel@na)
        val[2] <- sel_limb
        if (grepl("dome", sel_i)) val[3] <- sel_limb
      }
      return(val)
    })
  }

  # Initial conditions ----
  if (is.null(p$log_initF_mfr)) {
    if (any(Dfishery@Cinit_mfr > 1e-8)) {
      if ((nm == 1 && nr == 1) || Dmodel@condition == "F") {
        p$log_initF_mfr <- ifelse(Dfishery@Cinit_mfr <= 1e-8, -1000, log(0.1))
      }
    }
  }
  if (is.null(p$log_initF_mfr)) p$log_initF_mfr <- array(-1000, c(nm, nf, nr))

  if (is.null(p$log_initrdev_as)) {
    na_init <- ifelse(Dstock@m_advanceage > 1, Dmodel@na, Dmodel@na-1)
    p$log_initrdev_as <- matrix(0, na_init, ns)
  }

  do_map <- make_map(p, MSAdata, map = map, est_mov = est_mov, silent = silent, ...)
  out <- c(list(p = p), do_map)
  out$p <- check_parameters(out$p, out$map, MSAdata, silent)

  return(out)
}

#' @rdname make_parameters
#' @aliases make_map
#' @param p List of parameters, e.g., returned by [make_parameters()]
#' @param map List of mapped parameters. Overrides following `est_*` arguments
#' @param est_M Logical, estimate natural mortality? Only used if `map$log_M_s` is NULL
#' @param est_h Logical, estimate steepness? Only used if `map$t_h_s` is NULL
#' @param est_mat Logical, estimate maturity? Only used if `map$mat_ps` is NULL
#' @param est_sdr Logical, estimate standard deviation of recruitment deviates? Only used if `map$log_sdr_s` is NULL
#' @param est_mov Character describing structure of stock movement parameters. Only used if map arguments for the movement parameters is NULL. See details below.
#' @param est_qfs Logical, estimate relative catchability of stocks by each fleet? Fix `log_q_fs` for the first stock if `TRUE`. Only used if `map$log_q_fs` is NULL
#' @section Movement setup for `make_map()`:
#' If a single region model or `est_mov = "none"`: no movement parameters are estimated.
#'
#' If `est_mov = "dist_random"`: fix all values for `mov_x_marrs` and `mov_v_ymas`. Fix `mov_g_ymars` for the first region for each year,
#' season, age, and stock. `mov_g_ymars` are random effects.
#'
#' If `est_mov = "gravity_fixed"`: fix all values for `mov_x_marrs`. Fix `mov_g_ymars` for the first region for each year,
#' season, age, and stock. Estimate all `mov_v_ymas`. Both `mov_g_ymars` and `mov_v_ymas` are fixed effects.
#'
#' By default `p$mov_x_marrs` is zero. Set to -1000 for areas for which there is no abundance of a particular stock.
#'
#' @return
#' [make_map()] returns a named list containing parameter mappings (`"map"`) and a character vector of random effects (`"random"`).
#' @export
make_map <- function(p, MSAdata, map = list(),
                     est_M = FALSE, est_h = FALSE, est_mat = FALSE, est_sdr = FALSE,
                     est_mov = c("none", "dist_random", "gravity_fixed"),
                     est_qfs = FALSE,
                     silent = FALSE) {

  est_mov <- match.arg(est_mov)

  Dmodel <- MSAdata@Dmodel
  Dstock <- MSAdata@Dstock
  Dfishery <- MSAdata@Dfishery
  Dsurvey <- MSAdata@Dsurvey
  Dlabel <- MSAdata@Dlabel

  ny <- Dmodel@ny
  nm <- Dmodel@nm
  na <- Dmodel@na
  nl <- Dmodel@nl
  nr <- Dmodel@nr
  ns <- Dmodel@ns

  nf <- Dfishery@nf

  random <- NULL

  # Stock parameters ----
  if (!silent) {
    R0_s <- signif(exp(p$t_R0_s) * Dmodel@scale_s, 4)
    if (is.null(map$t_R0_s)) {
      message_info("Estimating t_R0_s, starting R0 = ", paste(R0_s, collapse = ", "))
    } else {
      message_info("Starting R0 = ", paste(R0_s, collapse = ", "))
      if (any(is.na(map$t_R0_s))) {
        message_info("Fixed for stock ", paste(which(is.na(map$t_R0_s)), collapse = ", "))
      }
    }
  }

  if (!est_h && is.null(map$t_h_s)) map$t_h_s <- rep(NA, ns)
  if (!silent) {
    h_s <- sapply(1:ns, function(s) conv_steepness(p$t_h_s[s], Dstock@SRR_s[s])) |> signif(4)
    if (is.null(map$t_h_s)) {
      message_info("Estimating t_h_s (steepness), starting value = ", paste(h_s, collapse = ", "))
    } else {
      message_info("Starting steepness = ", paste(h_s, collapse = ", "))
      if (any(is.na(map$t_h_s))) {
        message_info("Steepness fixed for stock: ", paste(which(is.na(map$t_h_s)), collapse = ", "))
      }
    }
  }

  if (is.null(map$mat_ps)) {
    if (est_mat) {
      if (!silent) message_info("Estimating maturity ogive parameters")
    } else {
      map$mat_ps <- array(NA, dim(p$mat_ps))
      if (!silent) message_info("Fixed maturity to values in data slot 'mat_yas'")
      if (!length(Dstock@mat_yas)) stop("Maturity ogive is not estimated. Need maturity at age values in the data slot 'mat_yas'.")
    }
  } else if (!silent && any(!is.na(map$mat_ps))) {

    message_info("Estimating maturity ogive parameters. Start values:")
    sapply(1:ns, function(s) {
      map_s <- matrix(map$mat_ps, 2, ns)[, s]

      if (any(!is.na(map_s))) {
        a50 <- signif(na * plogis(p$mat_ps[1, s]), 3)
        a95 <- signif(a50 + exp(p$mat_ps[2, s]), 3)
        message_info("Stock ", s, ": a50 = ", a50, ", a95 = ", a95)
      }
    })
  }

  if (is.null(map$log_M_s)) {
    if (est_M) {
      if (!silent) message_info("Estimating natural mortality for all stocks")
    } else {
      map$log_M_s <- rep(NA, ns)
      if (!silent) message_info("Fixed natural mortality to values in data slot 'M_yas'")
      if (!length(Dstock@M_yas)) stop("Natural mortality is not estimated. Need M values in the data slot 'M_yas'.")
    }
  } else if (!silent && any(!is.na(map$log_M_s))) {
    message_info("Estimating natural mortality for stock: ", paste(which(!is.na(map$log_M_s)), collapse = ", "))
  }

  if (!silent) {
    if (is.null(map$log_rdev_ys)) {
      message_info("Estimating recruitment deviates for all years and stocks")
    } else if (all(is.na(map$log_rdev_ys))) {
      message_info("No recruitment deviates are estimated")
    } else {
      message_info("Estimating recruitment deviates for:")
      sapply(1:ns, function(s) {
        map_s <- matrix(map$log_rdev_ys, ny, ns)[, s]
        message_info("Stock ", s, ": ", sum(!is.na(map_s)), " out of ", ny, " years")
      })
    }
  }

  if (is.null(map$log_sdr_s)) {
    if (est_sdr) {
      if (!silent) message_info("Estimating sigma_R (SD of recruitment deviates) for all stocks")
    } else {
      map$log_sdr_s <- rep(NA, ns)
      if (!silent) message_info("Fixed sigma_R (SD of recruitment deviates) for all stocks")
    }
  } else if (!silent && any(!is.na(map$log_sdr_s))) {
    message_info("Estimating sigma_R (SD of recruitment deviates) for stock ", paste(which(!is.na(map$log_sdr_s)), collapse = ", "))
  }
  if (!silent) {
    message_info("sigma_R = ", paste(round(exp(p$log_sdr_s), 4), collapse = ", "))
  }

  if (nr == 1 || est_mov == "none") {
    if (is.null(map$mov_x_marrs)) map$mov_x_marrs <- array(NA, dim(p$mov_x_marrs))
    if (is.null(map$mov_g_ymars)) map$mov_g_ymars <- array(NA, dim(p$mov_g_ymars))
    if (is.null(map$mov_v_ymas)) map$mov_v_ymas <- array(NA, dim(p$mov_v_ymas))

    if (is.null(map$log_sdg_rs)) map$log_sdg_rs <- array(NA, dim(p$log_sdg_rs))
    if (is.null(map$t_corg_ps)) map$t_corg_ps <- array(NA, dim(p$t_corg_ps))

    if (!silent && nr > 1) message_info("No stock movement parameters are estimated")

  } else {

    Dtag <- MSAdata@Dtag

    if (is.null(map$mov_x_marrs)) map$mov_x_marrs <- array(NA, dim(p$mov_x_marrs))

    if (is.null(map$mov_g_ymars)) {
      map$mov_g_ymars <- array(NA, c(ny, nm, na, nr, ns))

      # Group parameters based on data stratification
      if (length(Dtag@tag_yy) && length(Dtag@tag_aa)) {
        for (s in 1:ns) { # Estimate parameters for r = 2, ..., nr (softmax transformation)
          r_eff <- which(Dstock@presence_rs[, s])[-1]
          if (length(r_eff)) {
            for (i in 1:nrow(Dtag@tag_yy)) {
              yy <- which(Dtag@tag_yy[i, ] > 0)
              for (j in 1:nrow(Dtag@tag_aa)) {
                aa <- which(Dtag@tag_aa[j, ] > 0)

                gind_strat <- expand.grid(m = 1:nm, r = r_eff, s = s, yc = i, ac = j)
                gind_strat$par_no <- 1:nrow(gind_strat)

                gind <- expand.grid(y = yy, m = 1:nm, a = aa, r = r_eff, s = s, yc = i, ac = j) |>
                  merge(gind_strat) |>
                  as.matrix()

                if (all(is.na(map$mov_g_ymars))) {
                  parmin <- 0
                } else {
                  parmin <- max(map$mov_g_ymars, na.rm = TRUE)
                }
                map$mov_g_ymars[gind[, c("y", "m", "a", "r", "s")]] <- parmin + gind[, "par_no"]
              }
            }
          }
        }
      } else {
        for (s in 1:ns) { # Estimate parameters for r = 2, ..., nr (softmax transformation)
          r_eff <- which(Dstock@presence_rs[, s])[-1]
          if (length(r_eff)) {
            gind <- as.matrix(expand.grid(y = 1:ny, m = 1:nm, a = 1:na, r = r_eff, s = s))
            if (all(is.na(map$mov_g_ymars))) {
              parmin <- 0
            } else {
              parmin <- max(map$mov_g_ymars, na.rm = TRUE)
            }
            map$mov_g_ymars[gind] <- parmin + seq(1, nrow(gind))
          }
        }
      }
      map$mov_g_ymars <- factor(map$mov_g_ymars)
    }

    if (est_mov == "dist_random") {
      if (!"mov_g_ymars" %in% random) random <- c(random, "mov_g_ymars")
      if (is.null(map$mov_v_ymas)) map$mov_v_ymas <- array(NA, dim(p$mov_v_ymas))
      if (!silent) message_info("Movement estimated as random effects")
    } else {

      if (is.null(map$mov_v_ymas)) {
        map$mov_v_ymas <- array(NA, dim(p$mov_v_ymas))

        # Group parameters based on data stratification
        if (length(Dtag@tag_yy) && length(Dtag@tag_aa)) {
          for (s in 1:ns) { # Estimate parameters for r = 2, ..., nr (softmax transformation)
            nr_eff <- sum(Dstock@presence_rs[, s])
            if (nr_eff > 1) {
              for (i in 1:nrow(Dtag@tag_yy)) {
                yy <- which(Dtag@tag_yy[i, ] > 0)
                for (j in 1:nrow(Dtag@tag_aa)) {
                  aa <- which(Dtag@tag_aa[j, ] > 0)

                  vind_strat <- expand.grid(m = 1:nm, yc = i, ac = j)
                  vind_strat$par_no <- 1:nrow(vind_strat)

                  vind <- expand.grid(y = yy, m = 1:nm, a = aa, s = s, yc = i, ac = j) |>
                    merge(vind_strat) |>
                    as.matrix()

                  if (all(is.na(map$mov_v_ymas))) {
                    parmin <- 0
                  } else {
                    parmin <- max(map$mov_v_ymas, na.rm = TRUE)
                  }
                  map$mov_v_ymas[vind[, c("y", "m", "a", "s")]] <- parmin + vind[, "par_no"]
                }
              }
            }
          }
        } else {
          for (s in 1:ns) {  # Estimate parameters if nr_effective > 1
            nr_eff <- sum(Dstock@presence_rs[, s])
            if (nr_eff > 1) {
              vind <- as.matrix(expand.grid(y = 1:ny, m = 1:nm, a = 1:na, s = s))
              if (all(is.na(map$mov_v_ymas))) {
                parmin <- 0
              } else {
                parmin <- max(map$mov_v_ymas, na.rm = TRUE)
              }
              map$mov_v_ymas[vind] <- parmin + seq(1, nrow(vind))
            }
          }
        }

        map$mov_v_ymas <- factor(map$mov_v_ymas)
      }

      if (is.null(map$log_sdg_rs)) map$log_sdg_rs <- factor(array(NA, dim(p$log_sdg_rs)))
      if (is.null(map$t_corg_ps)) map$t_corg_ps <- factor(array(NA, dim(p$t_corg_ps)))

      if (!silent) message_info("Stock movement is estimated as fixed effects")
    }
  }

  if (nr > 1) {
    if (is.null(map$log_recdist_rs)) {
      recdist_rs <- sapply(1:ns, function(s) {
        x <- ifelse(Dstock@presence_rs[, s], TRUE, NA)
        if (sum(x, na.rm = TRUE) > 1) x[which(x)[1]] <- NA
        return(x)
      })
      recdist_rs[!is.na(recdist_rs)] <- 1:sum(recdist_rs, na.rm = TRUE)
      map$log_recdist_rs <- recdist_rs
    }

    if (!silent && any(!is.na(map$log_recdist_rs))) {
      message_info("Recruitment distribution will be estimated")
    }
  } else {
    map$log_recdist_rs <- matrix(NA, nr, ns)
  }

  # Fleet parameters ----
  if (is.null(map$log_q_fs)) {
    if (!est_qfs || ns == 1) {
      map$log_q_fs <- matrix(NA, nf, ns)
      if (!silent && ns > 1) message_info("Fishery fleet catchability equal for all stocks")
    } else { # Fix q for first stock
      map$log_q_fs <- local({
        np <- (ns - 1) * nf
        m <- matrix(NA, nf, ns)
        m[, -1] <- 1:np
        factor(m)
      })
      if (!silent) message_info("Fishery catchability to be estimated for all fleets (relative to stock 1)")
    }
  }

  if (is.null(map$log_Fdev_ymfr)) {
    if (Dmodel@condition == "F" && any(Dfishery@Cobs_ymfr <= 1e-8)) {
      map$log_Fdev_ymfr <- local({
        m <- ifelse(Dfishery@Cobs_ymfr <= 1e-8, NA, TRUE)
        m[!is.na(m)] <- 1:sum(m, na.rm = TRUE)
        factor(m)
      })
    } else if (Dmodel@condition == "catch") {
      map$log_Fdev_ymfr <- array(NA, c(ny, nm, nf, nr))
    }
    if (!silent && Dmodel@condition == "F") {
      message_info("F is an estimated parameter for all corresponding catches greater than 1e-8")
    }
  }

  ## Fix dome parameter if selectivity is logistic or all parameters if mirrored to maturity
  if (is.null(map$sel_pf) && any(!grepl("dome", Dfishery@sel_f))) {
    nsel <- max(Dfishery@sel_block_yf)
    sel_pf <- sapply(1:nsel, function(f) {
      sel_f <- Dfishery@sel_f[f]
      vec <- rep(TRUE, 3)
      if (sel_f %in% c("logistic_age", "logistic_length")) vec[3] <- NA
      if (sel_f %in% c("B", "SB", "total", "mature") || startsWith(sel_f, "length") || startsWith(sel_f, "age")) {
        vec[] <- NA
      }
      return(vec)
    })
    sel_pf[!is.na(sel_pf)] <- 1:sum(sel_pf, na.rm = TRUE)
    map$sel_pf <- sel_pf
  }

  if (!silent) {
    message_info("Fishery selectivity setup:")

    fsel_start <- conv_selpar(p$sel_pf, type = Dfishery@sel_f, maxage = Dmodel@na - 1, maxL = 0.95 * max(Dmodel@lmid))
    y <- if (length(Dlabel@year)) {
      Dlabel@year
    } else {
      1:ny
    }
    no_blocks <- apply(Dfishery@sel_block_yf, 2, function(x) length(unique(x)) == 1) |> all()
    for (bb in unique(as.numeric(Dfishery@sel_block_yf))) {

      if (no_blocks) {
        if (length(Dfishery@CAAobs_ymafr) > 0) {
          nage <- sum(
            apply(Dfishery@CAAobs_ymafr[, , , bb, ], 1, function(x) sum(x, na.rm = TRUE)) > 0,
            na.rm = TRUE
          )
        } else {
          nage <- 0
        }
        if (length(Dfishery@CALobs_ymlfr)) {
          nlen <- sum(
            apply(Dfishery@CALobs_ymlfr[, , , bb, ], 1, function(x) sum(x, na.rm = TRUE)) > 0,
            na.rm = TRUE
          )
        } else {
          nlen <- 0
        }
        age <- paste(nage, "years age composition")
        len <- paste(nlen, "years length composition")
        if (length(Dlabel@fleet)) {
          fname <- paste0(" (", Dlabel@fleet[bb], ")")
        } else {
          fname <- ""
        }

        message_info("Fleet ", bb, fname, ":")
        message_info("  ", Dfishery@sel_f[bb], "")
        if (nage || nlen) {
          message_info("  ", age)
          message_info("  ", len)
        } else {
          message_info("  No composition data")
        }
      } else {
        fleet <- lapply(1:nf, function(ff) {
          yy <- y[Dfishery@sel_block_yf[, ff] == bb]
          if (length(yy)) {
            if (all(diff(y) == 1)) {
              paste0(ff, " (", range(yy) |> paste(collapse = "-"), ")")
            } else {
              paste0(ff, " (", range(yy) |> paste(collapse = "-"), ", with gaps)")
            }
          } else {
            NULL
          }
        })
        message_info("Block ", bb, " (", Dfishery@sel_f[bb], ") assigned to fleet:\n", do.call(c, fleet) |> paste(collapse = "\n"))
      }

      if (grepl("logistic", Dfishery@sel_f[bb])) {
        sel5 <- -sqrt(-2 * log(0.05)) * fsel_start[2, bb] + fsel_start[1, bb]
        message_info("  Selectivity start values: full sel = ", round(fsel_start[1, bb], 2),
                     ", ascending limb SD = ", round(fsel_start[2, bb], 2), " (5% sel = ", round(sel5, 2), ")")
      } else if (grepl("dome", Dfishery@sel_f[bb])) {
        sel5 <- -sqrt(-2 * log(0.05)) * fsel_start[2, bb] + fsel_start[1, bb]
        message_info("  Selectivity start values: full sel = ", round(fsel_start[1, bb], 2),
                     ", ascending limb SD = ", round(fsel_start[2, bb], 2), " (5% sel = ", round(sel5, 2), ")",
                     ", descending limb SD = ", round(fsel_start[3, bb], 2))
      }
      message_info("\n")
    }
  }

  # Survey parameters ----
  ## Fix dome parameter or all parameters or all parameters if mirrored to fleet or maturity
  if (is.null(map$sel_pi) && any(!grepl("dome", Dsurvey@sel_i))) {
    sel_pi <- sapply(1:Dsurvey@ni, function(i) {
      sel_i <- Dsurvey@sel_i[i]
      vec <- rep(TRUE, 3)
      if (sel_i %in% c("logistic_age", "logistic_length")) vec[3] <- NA

      int_sel_i <- suppressWarnings(as.integer(sel_i))
      if (sel_i %in% c("B", "SB", "total", "mature") || !is.na(int_sel_i) || startsWith(sel_i, "length") || startsWith(sel_i, "age")) {
        vec[] <- NA
      } else {
        sel_char <- strsplit(sel_i, "_")[[1]]
        ff <- suppressWarnings(is.numeric(sel_char[1]))
        if (length(sel_char) == 3 && !is.na(ff)) {
          vec[] <- NA
        }
      }
      return(vec)
    })
    sel_pi[!is.na(sel_pi)] <- 1:sum(sel_pi, na.rm = TRUE)

    map$sel_pi <- sel_pi
  }

  if (!silent) {
    message_info("Index selectivity setup:")
    isel_start <- conv_selpar(p$sel_pi, type = Dsurvey@sel_i, maxage = Dmodel@na - 1, maxL = 0.95 * max(Dmodel@lmid))
    for (i in 1:Dsurvey@ni) {
      sel_i <- Dsurvey@sel_i[i]
      if (length(Dlabel@index)) {
        iname <- paste0(" (", Dlabel@index[i], ")")
      } else {
        iname <- ""
      }

      if (Dmodel@nr > 1) {
        r <- which(apply(Dsurvey@samp_irs[i, , , drop = FALSE], 2, sum) > 0)
        rtext <- paste(" in region", paste(r, collapse = ", "))
      } else {
        rtext <- character(0)
      }
      if (Dmodel@ns > 1) {
        s <- which(apply(Dsurvey@samp_irs[i, , , drop = FALSE], 3, sum) > 0)
        stext <- paste("; surveys stock", paste(s, collapse = ", "))
      } else {
        stext <- character(0)
      }

      message_info("Index ", i, iname, ":")

      if (!is.na(as.integer(sel_i))) {
        message_info("  Mirrors fleet ", sel_i, rtext, stext)
      } else {
        message_info("  ", sel_i, rtext, stext)
      }

      if (grepl("logistic", sel_i)) {
        sel5 <- -sqrt(-2 * log(0.05)) * isel_start[2, i] + isel_start[1, i]
        message_info("   Selectivity start values: full sel = ", round(isel_start[1, i], 2),
                     ", ascending limb SD = ", round(isel_start[2, i], 2), " (5% sel = ", round(sel5, 2), ")")
      } else if (grepl("dome", sel_i)) {
        sel5 <- -sqrt(-2 * log(0.05)) * isel_start[2, i] + isel_start[1, i]
        message_info("   Selectivity start values: full sel = ", round(isel_start[1, i], 2),
                     ", ascending limb SD = ", round(isel_start[2, i], 2),  " (5% sel = ", round(sel5, 2), ")",
                     ", descending limb SD = ", round(isel_start[3, i], 2))
      }
      message_info("\n")
    }
  }

  # Initial conditions ----
  if (is.null(map$log_initF_mfr)) {

    if (any(Dfishery@Cinit_mfr <= 1e-8)) {
      map$log_initF_mfr <- local({
        m <- ifelse(Dfishery@Cinit_mfr <= 1e-8, NA, TRUE)
        m[!is.na(m)] <- 1:sum(m, na.rm = TRUE)
        factor(m)
      })
    }
    if (any(Dfishery@Cinit_mfr > 1e-8)) {
      simple_model <- nm == 1 && nr == 1
      if (!simple_model && Dmodel@condition == "catch") {
        map$log_initF_mfr <- factor(array(NA, c(nm, nf, nr)))
      }
    }
  }

  if (!silent && (is.null(map$log_initF_mfr) || any(!is.na(map$log_initF_mfr)))) {
    message_info("Initial equilibrium F will be estimated")
  }

  if (is.null(map$log_initrdev_as)) {
    na_init <- ifelse(Dstock@m_advanceage > 1, na, na-1)
    map$log_initrdev_as <- factor(matrix(NA, na_init, ns))
  }
  if (!silent && (is.null(map$log_initrdev_as) || any(!is.na(map$log_initrdev_as)))) {
    message_info("Year 1 recruitment deviations will be estimated")
  }

  # Retrospective adjustments
  ymax <- Dmodel@ny - Dmodel@nyret
  if (Dmodel@nyret > 0) {
    if (!silent) message_info("Making adjustments to map list for retrospective analysis")
    yout <- seq(Dmodel@ny - Dmodel@nyret, ny)

    if (is.null(map$log_rdev_ys)) map$log_rdev_ys <- matrix(1:(ny * ns), ny, ns)
    map$log_rdev_ys[yout, ] <- NA

    if (is.null(map$mov_g_ymars)) map$mov_g_ymars <- array(c(1:(ny * nm * na * nr * ns), c(ny, nm, na, nr, ns)))
    map$mov_g_ymars[yout, , , , ] <- NA

    if (is.null(map$mov_v_ymas)) map$mov_v_ymas <- array(c(1:(ny * nm * na * ns)), c(ny, nm, na, ns))
    map$mov_v_ymas[yout, , , ] <- NA

    if (is.null(map$log_Fdev_ymfr)) map$log_Fdev_ymfr <- array(c(1:(ny * nm * nf * nr)), c(ny, nm, nf, nr))
    map$log_Fdev_ymfr[yout, , , ] <- NA
  }

  map <- lapply(map, factor)

  return(list(map = map, random = random))
}

#' @rdname make_parameters
#' @param map List of mapped parameters. Used by [check_parameters()] only to count parameters.
#' @return
#' [check_parameters()] invisibly returns the parameter list if no problems are encountered.
#' @export
check_parameters <- function(p = list(), map, MSAdata, silent = FALSE) {

  if (!silent && !missing(map)) {
    vars <- names(p)
    npar <- lapply(vars, function(x) {
      if (is.null(map[[x]])) {
        nparam <- length(p[[x]])
      } else {
        m_x <- map[[x]]
        nparam <- length(unique(m_x[!is.na(m_x)]))
      }
      out <- NULL
      if (nparam) {
        out <- try(data.frame(Parameter = par_df[x], Number = nparam), silent = TRUE)
        if (is.character(out)) out <- NULL
      }
      return(out)
    })
    output <- do.call(rbind, npar)

    message_info("Total estimated parameters: ", sum(output[, "Number"]))
    print(do.call(rbind, npar))
  }

  return(invisible(p))
}

par_df = c(
  "t_R0_s" = "Unfished recruitment",
  "t_h_s" = "Steepness",
  "mat_ps" = "Maturity ogive",
  "log_M_s" = "Natural mortality",
  "log_rdev_ys" = "Recruitment deviations",
  "log_sdr_s" = "Recruitment deviation standard deviation",
  "mov_x_marrs" = "Base movement",
  "mov_g_ymars" = "Movement, gravity to regions",
  "mov_v_ymas" = "Movement, region viscosity/retention",
  "log_sdg_rs" = "Region distribution standard deviation",
  "t_corg_ps" = "Region distribution correlation matrix",
  "log_recdist_rs" = "Region distribution of recruitment",
  "log_q_fs" = "Relative catchability of stocks by fleet",
  "log_Fdev_ymfr" = "F deviations",
  "sel_pf" = "Fishery selectivity",
  "sel_pi" = "Index selectivity",
  "log_initF_mfr" = "Equilibrium (year 1) F",
  "log_initrdev_as" = "Initial (year 1) recruitment deviations"
)

#' Newton-Raphson search for fishing mortality
#'
#' Performs a root finding routine to find the index of F that minimizes the difference between
#' observed catch and the value predicted by the Baranov equation.
#'
#' @param Cobs Observed catch. Matrix `[f, r]`
#' @param N Stock abundance at the beginning of the time step. Array `[a, r, s]`
#' @param sel Selectivity. Array `[a, f, s]`
#' @param wt Fishery weight at age. Array `[a, f, s]`
#' @param M Instantaneous natural mortality. Units of per year `[a, s]`
#' @param q_fs Relative catchability of stock `s` for fleet `f`. Defaults to 1 if missing. Matrix `[f, s]`
#' @param delta Numeric, the duration of time in years corresponding to the observed catch, e.g., 0.25 is a quarterly time step.
#' @param na Integer, number of age classes
#' @param nr Integer, number of regions
#' @param nf Integer, number of fleets
#' @param ns Integer, number of stocks
#' @param Fmax Numeric, the maximum Findex value
#' @param nitF Integer, number of iterations for the Newton-Raphson routine
#' @param trans Whether to perform the search in log or logit space
#' @return
#' A list containing:
#'
#' - `F_afrs` Fishing mortality array
#' - `F_ars` Fishing mortality array (summed across fleets)
#' - `Z_ars` Total mortality array
#' - `F_index` Index of fishing mortality. Matrix `[f, r]`
#' - `CB_frs` Catch (biomass) array
#' - `CN_afrs` Catch (abundance) array
#' - `VB_afrs` Vulnerable biomass at the beginning of the time step. Array
#' - `penalty` Penalty term returned by [posfun()] when `F_index` exceeds `Fmax`
#' - `fn` Difference between predicted and observed catch at the last iteration. Matrix `[f, r]`
#' - `gr` Gradient of `fn` with respect to `F_index` in either log or logit space at the last iteration. Vector by `[f, r]`
#'
#' @details
#' The predicted catch for fleet `f` in region `r` is
#' \deqn{
#' C^{\textrm{pred}}_{f,r} = \sum_s \sum_a v_{a,f,s} q_{f,s} F_{f,r} \dfrac{1 - \exp(-Z_{a,r,s})}{Z_{a,r,s}} N_{a,r,s} w_{a,f,s}
#' }
#'
#' The Newton-Raphson routine minimizes \eqn{f(x_{f,r}) = C_{f,r}^{\textrm{pred}} - C_{f,r}^{\textrm{obs}}}.
#'
#' If `trans = "log"`, \eqn{F_{f,r} = \exp(x_{f,r})}.
#'
#' If `trans = "logit"`, \eqn{F_{f,r} = F_{\textrm{max}}/(1 + \exp(x_{f,r}))}.
#'
#' The gradient with respect to \eqn{\vec{x}} is
#' \deqn{
#' f'(x_{f,r}) = \sum_s \sum_a v_{a,f,s} q_{f,s} N_{a,r,s} w_{a,f,s} \left(\dfrac{\alpha\gamma}{\beta}\right)'
#' }
#'
#' \deqn{
#' \left(\dfrac{\alpha\gamma}{\beta}\right)' = \dfrac{(\alpha\gamma' + \alpha'\gamma)\beta - \alpha\gamma\beta'}{\beta^2}
#' }
#'
#' where
#'
#' \tabular{l}{
#' \eqn{\alpha_{f,r} = F_{f,r}} \cr
#' \eqn{\beta_{a,r,s} = Z_{a,r,s} = M_{a,s} + \sum_f v_{a,f,s} q_{f,s} F_{f,r}} \cr
#' \eqn{\gamma_{a,r,s} = 1 - \exp(-Z_{a,r,s})} \cr
#' \eqn{\beta'_{a,f,r,s} = v_{a,f} q_{f,s} \alpha'_{f,r}} \cr
#' \eqn{\gamma'_{a,f,r,s} = \exp(-Z_{a,r,s})\beta'_{a,f,r,s}}
#' }
#'
#' If `trans = "log"`, \eqn{\alpha'_{f,r} = \alpha_{f,r}}.
#'
#' If `trans = "logit"`, \eqn{\alpha'_{f,r} = F_{\textrm{max}}\exp(-x_{f,r})/(1 + \exp(-x_{f,r}))^2}.
#'
#' This function solves for \eqn{\vec{x}} by iterating until \eqn{f(\vec{x})} approaches zero, where the vector arrow
#' indexes over fleet and region. In iteration \eqn{i+1}:
#' \deqn{\vec{x}_{i+1} = \vec{x}_i - \dfrac{f(\vec{x}_i)}{f'(\vec{x}_i)}}.
#' @author Q. Huynh
#' @export
calc_F <- function(Cobs, N, sel, wt, M, q_fs, delta = 1,
                   na = dim(N)[1], nr = dim(N)[2], ns = dim(N)[3], nf = length(Cobs),
                   Fmax = 2, nitF = 5L, trans = c("log", "logit")) {

  `[<-` <- RTMB::ADoverload("[<-")

  trans <- match.arg(trans)
  if (missing(q_fs)) q_fs <- 1

  Cobs <- matrix(Cobs, nf, nr)
  N <- array(N, c(na, nr, ns))
  sel <- array(sel, c(na, nf, ns))
  wt <- array(wt, c(na, nf, ns))
  M <- matrix(M, na, ns)
  q_fs <- matrix(q_fs, nf, ns)

  # Initialize search ----
  fn <- gr <- x_loop <- list()

  ind_afrs <- as.matrix(expand.grid(a = 1:na, f = 1:nf, r = 1:nr, s = 1:ns))
  ind_ars <- as.matrix(expand.grid(a = 1:na, r = 1:nr, s = 1:ns))

  frs_afrs <- ind_afrs[, c("f", "r", "s")]
  afs_afrs <- ind_afrs[, c("a", "f", "s")]
  ars_afrs <- ind_afrs[, c("a", "r", "s")]
  fr_afrs <- ind_afrs[, c("f", "r")]
  fs_afrs <- ind_afrs[, c("f", "s")]

  as_ars <- ind_ars[, c("a", "s")]

  VB_afrs <- array(N[ars_afrs] * sel[afs_afrs] * wt[afs_afrs], c(na, nf, nr, ns))
  VB_fr <- matrix(0, nf, nr)
  for (f in 1:nf) {
    for (r in 1:nr) VB_fr[f, r] <- sum(VB_afrs[, f, r, ])
  }

  if (inherits(Cobs, "advector")) {
    Cobs_loop <- CondExpLe(Cobs, 1e-8, 1e-8, Cobs)
  } else {
    Cobs_loop <- ifelse(Cobs <= 1e-8, 1e-8, Cobs)
  }
  F_init <- Cobs_loop/(Cobs_loop + VB_fr)

  if (trans == "log") {
    x_loop[[1]] <- log(F_init)
  } else {
    x_loop[[1]] <- qlogis(F_init/Fmax)
  }

  # Run search for Findex ----
  F_frs <- array(NA_real_, c(nf, nr, ns))
  F_afrs <- array(NA_real_, c(na, nf, nr, ns))
  F_ars <- Z_ars <- gamma_ars <- array(NA_real_, c(na, nr, ns))
  CN_afrs <- CB_afrs <- deriv_Z_afrs <- deriv_gamma_afrs <-
    deriv1_afrs <- deriv2_afrs <- deriv3_afrs <- deriv_afrs <- array(NA_real_, c(na, nf, nr, ns))

  CB_fr <- matrix(NA_real_, nf, nr)

  constants_afrs <- array(q_fs[fs_afrs] * sel[afs_afrs] * N[ars_afrs] * wt[afs_afrs], c(na, nf, nr, ns))

  penalty <- 0 # posfun penalty if F_index > Fmax
  ln_Fmax <- log(Fmax)
  for(i in seq(1, nitF + 1)) {
    if (trans == "log") {
      x_loop_i <- CondExpGt(x_loop[[i]], ln_Fmax, ln_Fmax, x_loop[[i]])
      F_loop <- exp(x_loop_i)
      if (i == nitF + 1) { # Last iteration used to calculate corresponding f and g with F and penalty
        F_loop <- CondExpLe(Cobs, 1e-8, 0, F_loop)
        penalty <- penalty + sum(posfun(Fmax, F_loop))
      }
    } else {
      F_loop <- Fmax * plogis(x_loop[[i]])
    }

    F_frs[] <- sapply2(1:ns, function(s) q_fs[, s] * F_loop)
    F_afrs[] <- F_frs[frs_afrs] * sel[afs_afrs]
    F_ars[] <- apply(F_afrs, c(1, 3, 4), sum)

    Z_ars[] <- F_ars + delta * M[as_ars]
    gamma_ars[] <- 1 - exp(-Z_ars)

    CN_afrs[] <- F_afrs * gamma_ars[ars_afrs] * N[ars_afrs]/Z_ars[ars_afrs]
    CB_afrs[] <- CN_afrs * wt[afs_afrs]
    CB_fr[] <- apply(CB_afrs, 2:3, sum)

    fn[[i]] <- CB_fr - Cobs

    if (trans == "log") {
      deriv_F <- F_loop # f r
    } else {
      deriv_F <- Fmax * exp(-x_loop[[i]])/(1 + exp(-x_loop[[i]]))/(1 + exp(-x_loop[[i]]))
    }

    deriv_Z_afrs[] <- q_fs[fs_afrs] * deriv_F[fr_afrs] * sel[afs_afrs]
    deriv_gamma_afrs[] <- exp(-Z_ars[ars_afrs]) * deriv_Z_afrs

    deriv1_afrs[] <- deriv_F[fr_afrs] * gamma_ars[ars_afrs] + F_loop[fr_afrs] * deriv_gamma_afrs
    deriv2_afrs[] <- deriv1_afrs * Z_ars[ars_afrs]
    deriv3_afrs[] <- deriv2_afrs - gamma_ars[ars_afrs] * F_loop[fr_afrs] * deriv_Z_afrs
    deriv_afrs[] <- deriv3_afrs/Z_ars[ars_afrs]/Z_ars[ars_afrs]

    gr[[i]] <- apply(constants_afrs * deriv_afrs, 2:3, sum)

    if (i <= nitF) x_loop[[i+1]] <- x_loop[[i]] - fn[[i]]/gr[[i]]
  }

  CB_frs <- array(0, c(nf, nr, ns))
  for (a in 1:na) CB_frs[] <- CB_frs[] + array(CB_afrs[a, , , ], c(nf, nr, ns))

  list(F_afrs = F_afrs, F_ars = F_ars, F_index = F_loop, Z_ars = Z_ars,
       CB_frs = CB_frs, CN_afrs = CN_afrs, VB_afrs = VB_afrs,
       penalty = penalty, fn = fn[[nitF + 1]], gr = gr[[nitF + 1]])
}

calc_Baranov <- function(FM, Z, N) FM/Z * (1 - exp(-Z)) * N

.calc_summary_F <- function(FM, M, N, CN) calc_Baranov(FM, FM + M, N) - CN
calc_summary_F <- function(M, N, CN, Fmax) {

  if (is.na(CN) || is.na(N)) {
    out <- NA_real_
  } else if (CN > N) {
    out <- Inf
  } else {
    out <- try(uniroot(.calc_summary_F, interval = c(0, Fmax), M = M, N = N, CN = CN)$root, silent = TRUE)
    if (is.character(out)) out <- Fmax
  }
  return(out)
}

#' Calculate recruitment from stock-recruit function
#'
#' @param x Numeric, either the spawning output or the equilibrium spawners per recruit, from which
#' the recruitment will be calculated. See argument `eq`.
#' @param SRR Character to indicate the functional form of the stock recruit function
#' @param eq Logical, indicates whether `x` is the spawning output (`FALSE`) or equilibrium spawners per recruit (`TRUE`)
#' @param ... Parameters of the SRR function. Provide one of two sets of variables:
#' 1. `h`, `R0` and `phi0`, or
#' 2. `a` and `b` (alpha, beta values)
#'
#' @examples
#' calc_recruitment(10, SRR = "Ricker", a = 2, b = 0.5)
#' calc_recruitment(10, SRR = "Ricker", h = 0.9, R0 = 1, phi0 = 1)
#' @return
#' Numeric of length `x`
#' @export
calc_recruitment <- function(x, SRR = c("BH", "Ricker"), eq = FALSE, ...) {
  SRR <- match.arg(SRR)
  dots <- list(...)

  if (all(names(dots) %in% c("h", "R0", "phi0"))) {
    h <- dots$h
    R0 <- dots$R0
    phi0 <- dots$phi0

    if (SRR == "BH") {
      a <- 4*h/(1-h)/phi0
      b <- (5*h)/(1-h)/phi0/R0
    } else {
      a <- (5*h)^1.25/phi0
      b <- log((5*h)^1.25)/phi0/R0
    }

  } else if (all(names(dots) %in% c("a", "b"))) {
    a <- dots$a
    b <- dots$b
  } else {
    stop("No stock recruit parameters found")
  }

  if (SRR == "BH") {
    val <- if (eq) (a*x - 1)/b/x else a*x/(1 + b*x)
  } else {
    val <- if (eq) log(a*x)/b/x else a*x*exp(-b*x)
  }
  return(val)
}

SRalphaconv <- function(h, phi, SRR = c("BH", "Ricker")) {
  SRR <- match.arg(SRR)
  k <- SRkconv(h, SRR)
  k/phi
}

SRbetaconv <- function(h, R0, phi, SRR = c("BH", "Ricker")) {
  SRR <- match.arg(SRR)
  switch(
    SRR,
    "BH" = (5*h-1)/(1-h)/phi/R0,
    "Ricker" = log((5*h)^1.25)/phi/R0
  )
}

SRhconv <- function(k, SRR = c("BH", "Ricker")) {
  SRR <- match.arg(SRR)
  switch(
    SRR,
    "BH" = k/(4+k),
    "Ricker" = 0.2*k^0.8
  )
}

SRkconv <- function(h, SRR = c("BH", "Ricker")) {
  SRR <- match.arg(SRR)
  switch(
    SRR,
    "BH" = 4*h/(1-h),
    "Ricker" = (5*h)^1.25
  )
}

#' Length-at-age key
#'
#' Calculates the probability distribution of length-at-age using the normal probability density function
#'
#' @param len_a Vector of length-at-age
#' @param sd_la Vector of standard deviation in length-at-age
#' @param lbin Vector of the lower boundary of the length bins
#' @param nl Integer, number of length bins (default is `length(lbin)`)
#' @return Matrix by age (rows) and length (columns)
#' @export
calc_LAK <- function(len_a, sd_la, lbin, nl = length(lbin)) {
  stopifnot(length(sd_la) == length(len_a))

  lak_al <- sapply(1:nl, function(j) {
    if (j == nl) {
      1 - pnorm(lbin[j], len_a, sd_la)
    } else if (j == 1) {
      pnorm(lbin[j+1], len_a, sd_la)
    } else {
      pnorm(lbin[j+1], len_a, sd_la) - pnorm(lbin[j], len_a, sd_la)
    }
  })

  return(lak_al)
}

#' Calculate von Bertalanffy length-at-age
#'
#' Returns an array of length-at-age with seasonal dimension.
#' Useful for [MSAdata-class] inputs.
#'
#' @param Linf_s Vector by stock `s` of asymptotic length
#' @param K_s Vector by stock `s` of the growth coefficient
#' @param t0_s Vector by `s` of the age at length zero.
#' @param ns Integer, number of stocks
#' @param nm Integer, number of seasons
#' @param ny Integer, number of years
#' @param a Integer vector of ages
#' @return Array `[y, m, a, s]`
#' @examples
#' len_ymas <- calc_growth(c(30, 40), c(0.4, 0.2), c(-1, -1))
#'
#' # Calculate stock weight at age
#' a_s <- rep(1e-6, 2)
#' b_s <- c(3, 3.1)
#'
#' ns <- length(a_s)
#' swt_ymas <- sapply(1:ns, function(s) {
#'   a_s[s]*len_ymas[, , , s]^b_s[s]
#' }, simplify = "array")
#' @export
calc_growth <- function(Linf_s, K_s, t0_s, ns = length(Linf_s), nm = 4, ny = 20, a = seq(1, 10) - 1) {
  len_ymas <- sapply2(1:ns, function(s) {
    sapply2(a, function(aa) {
      sapply(1:nm, function(m) {
        tt <- aa + (m - 1)/nm
        len_y <- Linf_s[s] * (1 - exp(-K_s[s] * (tt - t0_s[s])))
        rep(len_y, ny)
      })
    })
  })
  return(len_ymas)
}


#' Project stock abundance to the next time step
#'
#' This function applies survival of the current abundance, advances
#' age classes, re-distributes the stock using the movement matrix.
#'
#' @param N Abundance at current time step. Array `[a, r, s]`
#' @param surv Survival during the current time step. Array `[a, r, s]`
#' @param na Integer, number of age classes
#' @param nr Integer, number of regions
#' @param ns Integer, number of stocks
#' @param advance_age Logical, whether the animals advance to their next age class
#' @param mov Movement array in the next time step. Array `[a, r, r, s]`. Rows denote region of origin and columns denote region of destination.
#' @param plusgroup Logical, whether the last age class is an accumulator plus group.
#' @return Abundance at the next time step. Array `[a, r, s]`
#' @export
calc_nextN <- function(N, surv, na = dim(N)[1], nr = dim(N)[2], ns = dim(N)[3],
                       advance_age = TRUE, mov = array(1/nr, c(na, nr, nr, ns)), plusgroup = TRUE) {

  is_ad <- inherits(N, "advector") || inherits(surv, "advector") || inherits(mov, "advector")
  if (is_ad) {
    `[<-` <- RTMB::ADoverload("[<-")
  }

  N <- array(N, c(na, nr, ns))
  surv <- array(surv, c(na, nr, ns))
  mov <- array(mov, c(na, nr, nr, ns))

  # Apply survival and advance age class ----
  Nsurv_ars_temp <- N * surv
  if (advance_age) {
    Nsurv_a_rs <- array(list(), na)
    Nsurv_a_rs[[1]] <- matrix(0, nr, ns)
    Nsurv_a_rs[seq(2, na)] <- lapply(seq(1, na - 1), function(a) matrix(Nsurv_ars_temp[a, , ], nr, ns))
    if (plusgroup) Nsurv_a_rs[[na]] <- Nsurv_a_rs[[na]] + Nsurv_ars_temp[na, , ]

    Nsurv_ars <- do.call(c, Nsurv_a_rs) |> array(c(nr, ns, na)) |> aperm(c(3, 1:2))
  } else {
    Nsurv_ars <- Nsurv_ars_temp
  }

  # Distribute stock ----
  Nnext_ars <- array(NA_real_, c(na, nr, ns))
  if (nr > 1) {
    ind_arrs <- as.matrix(expand.grid(a = 1:na, rf = 1:nr, rt = 1:nr, s = 1:ns))
    arfs_arrs <- ind_arrs[, c("a", "rf", "s")]

    Nnext_arrs <- array(Nsurv_ars[arfs_arrs] * mov, c(na, nr, nr, ns))
    Nnext_ars[] <- apply(Nnext_arrs, c(1, 3, 4), sum)
  } else {
    Nnext_ars[] <- Nsurv_ars
  }

  return(Nnext_ars)
}


#' Calculate index at age
#'
#' For indices of abundance, the function calculates the numbers vulnerable to the survey.
#'
#' @param N Stock abundance at the beginning of the time step. Array `[a, r, s]`
#' @param Z Instantaneous total mortality. Array `[a, r, s]`
#' @param sel Index selectivity. Array `[a, i, s]`
#' @param na Integer, number of age classes
#' @param nr Integer, number of regions
#' @param ns Integer, number of stocks
#' @param ni Integer, number of indices
#' @param samp Boolean indicates which regions and stocks are sampled by the index. Array `[i, r, s]`
#' @param delta Fraction of time step when the index samples the population. Vector by `i`. Set to a negative number (-1) to sample the average
#' population over the course of the time step, i.e. `N * (1 - exp(-Z))/Z`.
#' @return Index at age. Array `[a, i, s]`
#' @details
#' The index is calculated as
#' \deqn{
#' I_{a,i,s} = v_{a,i,s} \sum_r N_{a,r,s} d_{a,r,s} \times \mathbb{1}(r \in R_i) \mathbb{1}(s \in S_i)
#' }
#'
#' If the survey samples at a moment in time, then
#' \deqn{
#' d_{a,r,s} = \exp(-\delta_i Z_{a,r,s})
#' }
#'
#' Otherwise, if the index samples the population over the duration of the time step, then
#' \deqn{
#' d_{a,r,s} = (1 - \exp(-Z_{a,r,s}))/Z_{a,r,s}
#' }
#'
#' where \eqn{R_i} and \eqn{S_i} denote the regions and stocks, respectively, sampled by index \eqn{i}. For example,
#' \eqn{R_2 = 1} denotes that the second index of abundance only samples region 1. These are informed by array `samp` where
#' `samp[i, r, s] = 1` indicates that stock `s` in region `r` is sampled by index `i`.
#' @export
calc_index <- function(N, Z, sel, na = dim(N)[1], nr = dim(N)[2], ns = dim(N)[3], ni = dim(sel)[2],
                       samp = array(1, c(ni, nr, ns)), delta = rep(0, ni)) {

  is_ad <- inherits(N, "advector") || inherits(Z, "advector") || inherits(sel, "advector")
  if (is_ad) {
    `[<-` <- RTMB::ADoverload("[<-")
  }

  N <- array(N, c(na, nr, ns))
  Z <- array(Z, c(na, nr, ns))
  sel <- array(sel, c(na, ni, ns))

  ind_arsi <- as.matrix(expand.grid(a = 1:na, r = 1:nr, s = 1:ns, i = 1:ni))
  irs_arsi <- ind_arsi[, c("i", "r", "s")]
  ars_arsi <- ind_arsi[, c("a", "r", "s")]
  ais_arsi <- ind_arsi[, c("a", "i", "s")]

  duration_arsi <- sapply2(1:ni, function(i) {
    if (delta[i] < 0) {
      (1 - exp(-Z))/Z
    } else {
      exp(-delta[i] * Z)
    }
  })

  IN_airs <- array(
    N[ars_arsi] * samp[irs_arsi] * sel[ais_arsi] * duration_arsi,
    c(na, nr, ns, ni)
  ) |>
    aperm(c(1, 4, 2, 3))
  IN_ais <- array(0, c(na, ni, ns))
  for (r in 1:nr) IN_ais[] <- IN_ais[] + array(IN_airs[, , r, ], c(na, ni, ns))
  return(IN_ais)
}

#calc_index2 <- function(N_ymars, Z_ymars, sel_ymais,
#                        ny = dim(N_ymars)[1], nm = dim(N_ymars[2]),
#                        na = dim(N_ymars)[3], nr = dim(N_ymars)[4], ns = dim(N_ymars)[5],
#                        ni = dim(sel_ymais)[4],
#                        samp_irs = array(1, c(ni, nr, ns)), delta_i = rep(0, ni)) {
#
#  `[<-` <- RTMB::ADoverload("[<-")
#
#  ind_ymarsi <- as.matrix(expand.grid(y = 1:ny, m = 1:nm, a = 1:na, r = 1:nr, s = 1:ns, i = 1:ni))
#  ymars_ymarsi <- ind_ymarsi[, c("y", "m", "a", "r", "s")]
#  irs_ymarsi <- ind_ymarsi[, c("i", "r", "s")]
#  ymais_ymarsi <- ind_ymarsi[, c("y", "m", "a", "i", "s")]
#
#  duration_ymarsi <- sapply2(1:ni, function(i) {
#    if (delta_i[i] < 0) {
#      (1 - exp(-Z_ymars))/Z_ymars
#    } else {
#      exp(-delta_i[i] * Z_ymars)
#    }
#  })
#  IN_ymarsi <- array(NA_real_, c(ny, nm, na, nr, ns, ni))
#  IN_ymarsi[] <- N_ymars[ymars_ymarsi] * samp_irs[irs_ymarsi] * sel_ymais[ymais_ymarsi] * duration_ymarsi
#  IN_ymais <- apply(IN_ymarsi, c(1:4, 6, 5), sum)
#  return(IN_ymais)
#}

calc_q <- function(Iobs, B) {
  i <- !is.na(Iobs) & Iobs > 0
  n <- sum(i)
  num <- log(Iobs[i]/B[i]) |> sum()
  q <- exp(num/n)
  return(q)
}

#' Calculate movement matrix for all age classes
#'
#' Movement matrices are calculated for all age classes from a base matrix and a gravity model formulation
#' (Carruthers et al. 2016).
#'
#' @param x Base log-movement parameters. See details. Array `[a, r, r]`
#' @param g Gravity model attractivity term. Tendency to move to region `r`. Matrix `[a, r]`
#' @param v Gravity model viscosity term. Tendency to stay in same region. Vector by `a`
#' @param na Integer, number of ages
#' @param nr Integer, number of regions
#' @param aref Integer, reference age class
#' @details
#' Rows index region of origin and columns denote region of destination.
#'
#' In log space, the movement matrix \eqn{m_a} for age class \eqn{a} from region \eqn{r} to \eqn{r'} is the sum of base matrix \eqn{x} and
#' gravity matrix \eqn{G}:
#' \deqn{m_{a,r,r'} = x_{a,r,r'} + G_{a,r,r'}}
#'
#' To essentially exclude movement from \eqn{r} to \eqn{r'}, set \eqn{x_{a,r,r'} = -1000}.
#'
#' Gravity matrix \eqn{G} includes an attractivity term \eqn{g} and viscosity term \eqn{v}:
#'
#' \deqn{G_{a,r,r'} =
#' \begin{cases}
#' g'_{a,r'} + v_a \quad & r = r'\\
#' g'_{a,r'} \quad & \textrm{otherwise}
#' \end{cases}
#' }
#'
#' Vector \eqn{g'} are offset terms relative to the value for the reference age class:
#' \deqn{g'_{a,r'} =
#' \begin{cases}
#' g_{a,r} \quad & a = a_{ref}\\
#' g_{a,r} + g_{a=aref,r} \quad & \textrm{otherwise}
#' \end{cases}
#' }
#'
#' The movement matrix in normal space is obtained by the softmax transformation:
#' \deqn{M_{a,r,r'} = \dfrac{\exp(m_{a,r,r'})}{\sum_{r'}\exp(m_{a,r,r'})}}
#'
#' If \eqn{x} and \eqn{v} are zero, then the movement matrix simply distributes the total stock
#' abundance into the various regions as specified in \eqn{g'}.
#' @references
#' Carruthers, T.R., et al. 2015. Modelling age-dependent movement: an application to red and
#' gag groupers in the Gulf of Mexico. CJFAS 72: 1159-1176. \doi{10.1139/cjfas-2014-0471}
#' @return Movement array `[a, r, r]`
#' @export
conv_mov <- function(x, g, v, na = dim(x)[1], nr = dim(x)[2], aref = ceiling(0.5 * na)) {
  x <- array(x, c(na, nr, nr))
  g <- matrix(g, na, nr)

  mov_rra <- sapply2(1:na, function(a) {
    gg <- g[a, ] + (a != aref) * g[aref, ]
    ln_mov <- x[a, , ] + v[a] * diag(nr) + matrix(gg, nr, nr, byrow = TRUE)
    matrix(ln_mov, nr, nr) |>
      apply(1, softmax) # This operation flips matrix orientation. FROM = col, TO = row
  }) |>
    array(c(nr, nr, na))

  return(aperm(mov_rra, 3:1))
}

#' Equilibrium distribution from movement matrix
#'
#' Applies the seasonal movement matrices several times in order to obtain the equilibrium spatial distribution over the course of a year.
#' Not used in the model but useful for reporting.
#'
#' @return Matrix by season and region `[m, r]`
#' @param x Movement array `[m, r, r]`. The second dimension corresponds to origin (sum to 1), and third dimension corresponds to destination
#' @param nm Number of seasons
#' @param nr Number of regions
#' @param start The initial distribution. Vector of length `nr`
#' @param m_start Integer, the season in which to apply the initial distribution and start the projection
#' @param nit Integer, the number of times the movement matrix will be applied
#' @export
calc_eqdist <- function(x, nm = dim(x)[1], nr = dim(x)[2], start = rep(1/nr, nr),
                        m_start = 1L, nit = 20) {
  if (inherits(start, "advector") || inherits(x, "advector")) {
    `[<-` <- RTMB::ADoverload("[<-")
  }

  N <- array(NA_real_, c(nit, nm, nr))
  N[1, m_start, ] <- start

  if (nm > m_start) {
    for (m in seq(m_start + 1, nm)) N[1, m, ] <- colSums(N[1, m-1, ] * x[m-1, , ])
  }
  for (i in seq(2, nit)) {
    N[i, 1, ] <- colSums(N[i-1, nm, ] * x[nm, , ])
    if (nm > 1) {
      for (m in 2:nm) N[i, m, ] <- colSums(N[i, m-1, ] * x[m-1, , ])
    }
  }
  return(matrix(N[nit, , ], nm, nr))
}

#' Predict the probability of CKMR kinship pairs
#'
#' Calculate the probability of observing a parent-offspring pair (`calc_POP`) and
#' half-sibling pair (`calc_HSP`) for closed-kin mark recapture (CKMR) for an age-structured
#' model.
#'
#' @param t Vector, capture year of parent `i`
#' @param a Vector, age at capture of parent `i`
#' @param y Vector, birth year of offspring `j`
#' @param N Abundance of mature spawners. Matrix by `[y, a]`
#' @param fec Fecundity schedule of mature spawners. Matrix by `[y, a]`
#' @return Numeric, vector of probabilities
#' @seealso [like_CKMR()]
#' @section Parent-offspring pairs:
#' The parent-offspring probability is calculated from Bravington et al. 2016, eq 3.4:
#'
#' \deqn{p_{\textrm{POP}} = 2 \times \dfrac{f(y_j,y_j - (t_i - a_i))}{\sum_a f(y_j,a) N(y_j,a)}}
#'
#' where \eqn{y_j - (t_i - a_i)} is the parental age in year \eqn{y_j}. Scalar 2 accounts for the fact
#' that the parent could be either a mother or a father.
#' `calc_POP` is vectorized with respect to `t`, `a`, and `y`.
#' @references
#' Bravington, M.V. et al. 2016. Close-Kin Mark-Recapture. Stat. Sci. 31: 259-274.
#' \doi{10.1214/16-STS552}
#' @author Q. Huynh with contribution from Y. Tsukahara (Fisheries Research Institute, Japan)
#' @export
calc_POP <- function(t, a, y, N, fec) {
  if (is.null(t)) return(0)
  a_yj <- y - t + a # Age of parent in birth year of offspring, Bravington 2016, eq 3.4
  if (!length(a_yj) == length(y)) stop("Vectors t, a, y need to be the same length")

  rel_RO <- sapply(1:length(a_yj), function(j) {
    RO <- sum(N[y[j], ] * fec[y[j], ])
    fec[y[j], a_yj[j]]/RO
  })
  p <- 2 * rel_RO
  return(p)
}

#' @name calc_POP
#' @section Half-sibling pairs:
#' The half-sibling probability is calculated from Bravington et al. 2016, eq 3.10, and expanded
#' by Hillary et al. 2018, Supplement S2.8.1 for age-specific survival and fecundity of the parent:
#'
#' \deqn{p_{\textrm{HSP}} = 4 \times
#' \sum_a\left(
#' \dfrac{N(y_i, a)f(y_i, a)}{\sum_{a'} N(y_i, a')f(y_i,a')}\times
#' \exp(-\sum_{t = 0}^{y_j - y_i - 1} Z(y_i + t,a + t))\times
#' \dfrac{f(y_j,a+y_j-y_i)}{\sum_{a'} N(y_j,a')f(y_j,a')}
#' \right)
#' }
#'
#' - The first ratio is the probability that a fish at age \eqn{a} in year \eqn{y_i} is the parent of \eqn{i}.
#' - The exponential term is that fish's survival from year \eqn{y_i} to \eqn{y_j}.
#' - The second ratio is the probability that the parent of \eqn{i}, age \eqn{a+y_j-y_i} in year \eqn{y_j}, is the parent of \eqn{j}.
#'
#' The parent is not observed in the HSP, so we sum the probabilities over all potential ages in year \eqn{y_i}.
#' `calc_HSP` is vectorized with respect to `yi` and `yj`.
#' @references
#' Hillary, R.M. et al. 2018. Genetic relatedness reveals total population size of white sharks in eastern Australia and
#' New Zealand. Sci. Rep. 8: 2661. \doi{10.1038/s41598-018-20593-w}
#' @param yi Vector, birth year of sibling `i`. Must be older than sibling `j`.
#' @param yj Vector, birth year of sibling `j`.
#' @param Z Instantaneous total mortality rate. Matrix by `[y, a]`
#' @export
calc_HSP <- function(yi, yj, N, fec, Z) {
  if (is.null(yi)) return(0)
  stopifnot(length(yi) == length(yj))
  sapply(1:length(yi), function(x) {
    .calc_HSP(yi[x], yj[x], N = N, fec = fec, Z = Z)
  })
}

.calc_HSP <- function(yi, yj, N, fec, Z, na = ncol(N)) {
  delta_ij <- yj - yi

  RO_yi <- sum(N[yi, ] * fec[yi, ])
  RO_yj <- sum(N[yj, ] * fec[yj, ])

  p_ai <- sapply(1:na, function(ai) {
    relRO_di <- fec[yi, ai]/RO_yi
    relRO_dj <- fec[yj, min(ai + delta_ij, na)]/RO_yj

    Z_ij <- sapply(seq(0, delta_ij - 1), function(t) Z[yi + t, min(ai + t, na)])
    surv_dij <- exp(-sum(Z_ij))

    N[yi, ai] * relRO_di * relRO_dj * surv_dij
  })
  p <- 4 * sum(p_ai)
  return(p)
}

#' @importFrom methods slot setClass setMethod setOldClass slot<- setClassUnion
#' @importFrom utils packageVersion
init_fn <- function(.Object, dots = list()) {
  if (length(dots)) {
    for(i in names(dots)) slot(.Object, i) <- dots[[i]]
  }
  attr(.Object, "version") <- paste("multiSA", packageVersion("multiSA"))
  attr(.Object, "date") <- date()
  attr(.Object, "R.version") <- getRversion()

  return(.Object)
}

setClassUnion("num.char", c("numeric", "character"))

#' Dmodel S4 object
#' @template MSAdata-template
#' @template Dmodel-slot
#' @export
setClass(
  "Dmodel",
  slots = c(ny = "numeric", nm = "numeric", na = "numeric", nl = "numeric", nr = "numeric", ns = "numeric",
            lbin = "numeric", lmid = "numeric", Fmax = "numeric",
            y_phi = "numeric", scale_s = "numeric", nyinit = "numeric",
            condition = "character", nitF = "numeric",
            y_Fmult_f = "numeric", m_Fmult_f = "numeric", r_Fmult_f = "numeric",
            pbc_rdev_ys = "matrix", pbc_initrdev_as = "matrix",
            prior = "character", nyret = "numeric")
)

#' Dstock S4 object
#' @template MSAdata-template
#' @template Dstock-slot
#' @export
setClass(
  "Dstock",
  slots = c(m_spawn = "numeric", m_advanceage = "numeric",
            len_ymas = "array", sdlen_ymas = "array",
            LAK_ymals = "array", mat_yas = "array", swt_ymas = "array", fec_yas = "array",
            M_yas = "array", SRR_s = "character",
            delta_s = "numeric", presence_rs = "matrix", natal_rs = "matrix")
)

#' Dfishery S4 object
#' @template MSAdata-template
#' @template Dfishery-slot
#' @export
setClass(
  "Dfishery",
  slots = c(nf = "numeric", Cobs_ymfr = "array", Csd_ymfr = "array", fwt_ymafs = "array", CAAobs_ymafr = "array", CALobs_ymlfr = "array",
            fcomp_like = "character", CAAN_ymfr = "array", CALN_ymfr = "array", CAAtheta_f = "numeric", CALtheta_f = "numeric",
            sel_block_yf = "array", sel_f = "character", Cinit_mfr = "array",
            SC_ymafrs = "array", SC_aa = "matrix", SC_ff = "matrix",
            SC_like = "character", SCN_ymafr = "array", SCtheta_f = "numeric", SCstdev_ymafrs = "array",
            lambdaCobs_f = "numeric", lambdaCAA_f = "numeric", lambdaCAL_f = "numeric", lambdaSC_f = "numeric")
)

#' Dsurvey S4 object
#' @template MSAdata-template
#' @template Dsurvey-slot
#' @export
setClass(
  "Dsurvey",
  slots = c(ni = "numeric", Iobs_ymi = "array", Isd_ymi = "array", unit_i = "character",
            IAAobs_ymai = "array", IALobs_ymli = "array",
            icomp_like = "character", IAAN_ymi = "array", IALN_ymi = "array", IAAtheta_i = "numeric", IALtheta_i = "numeric",
            samp_irs = "array", sel_i = "vector", delta_i = "numeric", qest_i = "num.char",
            lambdaI_i = "numeric", lambdaIAA_i = "numeric", lambdaIAL_i = "numeric")
)

#' DCKMR S4 object
#'
#' Store lists of data frames for parent offspring pairs and half-sibling pairs
#' @template DCKMR-slot
#' @export
setClass(
  "DCKMR",
  slots = c(POP_s = "list", HSP_s = "list", CKMR_like = "character")
)

#' Dtag S4 object
#' @template MSAdata-template
#' @template Dtag-slot
#' @export
setClass(
  "Dtag",
  slots = c(tag_ymarrs = "array", tag_ymars = "array", tag_yy = "matrix", tag_aa = "matrix", tag_like = "character",
            tagN_ymars = "array", tagN_ymas = "array", tagtheta_s = "numeric", tagstdev_s = "numeric",
            lambdaTag_s = "numeric")
)

#' Dlabel S4 object
#'
#' Vectors for labeling plots.
#'
#' @template Dlabel-slot
#' @export
setClass(
  "Dlabel",
  slots = c(year = "vector", season = "vector", age = "vector", region = "vector",
            stock = "vector", fleet = "vector", index = "vector")
)


#' MSAdata S4 object
#' @keywords MSAdata
#' @template MSAdata-template
#' @slot Dmodel Class [Dmodel-class] containing parameters for model structure (number of years, ages, etc.)
#' @slot Dstock Class [Dstock-class] containing stock parameters (growth, natural mortality, etc.)
#' @slot Dfishery Class [Dfishery-class] containing fishery data (catch, size and stock composition, etc.)
#' @slot Dsurvey Class [Dsurvey-class] containing survey data (indices of abundance)
#' @slot DCKMR Class [DCKMR-class] containing genetic close-kin data
#' @slot Dtag Class [Dtag-class] containing tagging data
#' @slot Dlabel Class [Dlabel-class] containing names for various dimensions. Used for plotting.
#' @slot Misc List for miscellaneous inputs as needed
#' @template Dmodel-slot
#' @template Dstock-slot
#' @template Dfishery-slot
#' @template Dsurvey-slot
#' @template DCKMR-slot
#' @template Dtag-slot
#' @template Dlabel-slot
#' @export
setClass(
  "MSAdata",
  slots = c(Dmodel = "Dmodel", Dstock = "Dstock", Dfishery = "Dfishery", Dsurvey = "Dsurvey",
            DCKMR = "DCKMR", Dtag = "Dtag", Dlabel = "Dlabel", Misc = "list")
)
setMethod("initialize", "MSAdata", function(.Object, ...) init_fn(.Object, list(...)))
setMethod("initialize", "Dmodel", function(.Object, ...) init_fn(.Object, list(...)))
setMethod("initialize", "Dstock", function(.Object, ...) init_fn(.Object, list(...)))
setMethod("initialize", "Dfishery", function(.Object, ...) init_fn(.Object, list(...)))
setMethod("initialize", "Dsurvey", function(.Object, ...) init_fn(.Object, list(...)))
setMethod("initialize", "DCKMR", function(.Object, ...) init_fn(.Object, list(...)))
setMethod("initialize", "Dtag", function(.Object, ...) init_fn(.Object, list(...)))
setMethod("initialize", "Dlabel", function(.Object, ...) init_fn(.Object, list(...)))



setOldClass("sdreport")

#' MSAassess S4 object
#'
#' S4 object that returns output from MSA model
#'
#' @slot obj RTMB object returned by [RTMB::MakeADFun()]
#' @slot opt List returned by [stats::nlminb()]
#' @slot SD List returned by [RTMB::sdreport()]
#' @slot report List of model output at the parameter estimates, returned by `obj$report(obj$env$last.par.best)`
#' @slot Misc List, miscellaneous items
#'
#' @keywords MSAassess
#'
#' @export
setClass(
  "MSAassess",
  slots = c(obj = "list", opt = "list", SD = "sdreport", report = "list", Misc = "list")
)
setMethod("initialize", "MSAassess", function(.Object, ...) init_fn(.Object, list(...)))

#' @name summary
#' @title Summarize MSA model output
#' @description Report parameter estimates, gradients, and standard errors
#' @param object Output from MSA
#' @param ... Not used
#' @returns Matrix
#' @aliases summary,MSAassess-method
#' @export
setMethod("summary", signature(object = "MSAassess"), function(object, ...) {
  if (length(object@SD) > 1) {
    sdreport_int(object@SD, ...)
  } else {
    invisible()
  }
})

#' @name report
#' @title Generate markdown reports
#'
#' @description Generate a markdown report of model fits and estimates.
#'
#' @param object An object from MSA.
#' @param ... Additional arguments to render reports.
#'
#' @export
setGeneric("report", function(object, ...) standardGeneric("report"))

#' @param filename Character string for the name of the markdown and HTML files.
#' @param dir The directory in which the markdown and HTML files will be saved.
#' @param open_file Logical, whether the HTML document is opened after it is rendered.
#' @param render_args List of arguments to pass to [rmarkdown::render()].
#' @param name Optional character string for the model name to include in the report, e.g., model run number. Default
#' uses `substitute(object)`
#' @return
#' `report` generic for MSAassess invisibly returns the output of [rmarkdown::render()]: character of the path of the rendered HTML markdown report.
#' @rdname report
#' @aliases report.MSAassess report,MSAassess-method
#' @export
setMethod("report", signature(object = "MSAassess"),
          function(object, name, filename = "MSA", dir = tempdir(), open_file = TRUE, render_args = list(), ...) {

            if (missing(name)) name <- substitute(object) |> as.character()

            dots <- list(...)
            x <- object # Needed for markdown file

            dat <- get_MSAdata(object)

            nm <- dat@Dmodel@nm

            nr <- dat@Dmodel@nr
            rname <- dat@Dlabel@region
            if (!length(rname)) rname <- "Region 1"

            ns <- dat@Dmodel@ns
            sname <- dat@Dlabel@stock
            if (!length(sname)) sname <- "Stock 1"

            rmd <- system.file("include", "MSAreport.Rmd", package = "multiSA") |> readLines()
            rmd_split <- split(rmd, 1:length(rmd))

            name_ind <- grep("NAME", rmd)
            rmd_split[[name_ind]] <- paste("#", name, "{.tabset}")

            hessian_ind <- grep("*ADD HESSIAN RMD*", rmd)
            if (length(x@SD) > 1 && !x@SD$pdHess && !is.null(x@SD$env$hessian)) {
              rmd_split[[hessian_ind]] <- c(
                "### Hessian",
                "```{r hessian}",
                "as.data.frame(round(x@SD$env$hessian, 4))",
                "```",
                ""
              )
            } else {
              rmd_split[[hessian_ind]] <- ""
            }

            fname <- dat@Dlabel@fleet
            nf <- dat@Dfishery@nf
            fishery_ind <- grep("*ADD FISHERY RMD*", rmd)
            rmd_split[[fishery_ind]] <- mapply(make_rmd_fishery, f = 1:nf, fname = fname,
                                               MoreArgs = list(nm = nm, rname = rname)) |>
              as.character()

            ni <- dat@Dsurvey@ni
            iname <- dat@Dlabel@index
            survey_ind <- grep("*ADD SURVEY RMD*", rmd)
            if (ni > 0) {
              rmd_split[[survey_ind]] <- mapply(make_rmd_survey, i = 1:ni, iname = iname) |>
                as.character()
            } else {
              rmd_split[[survey_ind]] <- ""
            }

            sc_ind <- grep("*ADD SC RMD*", rmd)
            if (ns > 1 && any(dat@Dfishery@SC_ymafrs > 0, na.rm = TRUE)) {

              r_plot <- apply(dat@Dfishery@SC_ymafrs, 5, sum) > 0

              rmd_split[[sc_ind]] <- local({

                sc_txt <- lapply(1:nr, function(r) {
                  if (r_plot[r]) {
                    out <- lapply(1:nrow(dat@Dfishery@SC_ff), function(ff) {
                      lapply(1:nrow(dat@Dfishery@SC_aa), function(aa) {
                        fname <- ifelse(nrow(dat@Dfishery@SC_ff) != ncol(dat@Dfishery@SC_ff), "aggregate fleet", "fleet")
                        aname <- ifelse(nrow(dat@Dfishery@SC_aa) != ncol(dat@Dfishery@SC_aa), "age class", "age")
                        make_rmd_SC(ff, aa, r, paste(fname, ff), paste(aname, aa), rname[r])
                      }) |> unlist()
                    })
                    do.call(c, out)
                  } else {
                    ""
                  }
                }) |>
                  unlist() |>
                  as.character()

                c("## Stock composition {.tabset}\n\n", sc_txt)
              })

            } else {
              rmd_split[[sc_ind]] <- ""
            }

            tagmov_ind <- grep("*ADD TAG MOV RMD*", rmd)
            if (nr > 1 && any(dat@Dtag@tag_ymarrs > 0, na.rm = TRUE)) {

              r_plot <- apply(dat@Dfishery@SC_ymafrs, 5, sum) > 0

              rmd_split[[tagmov_ind]] <- local({

                tagmov_txt <- lapply(1:ns, function(s) {
                  out <- lapply(1:nrow(dat@Dtag@tag_yy), function(yy) {
                    lapply(1:nrow(dat@Dtag@tag_aa), function(aa) {
                      yname <- ifelse(nrow(dat@Dtag@tag_yy) != ncol(dat@Dtag@tag_yy), "aggregate year", "year")
                      aname <- ifelse(nrow(dat@Dtag@tag_aa) != ncol(dat@Dtag@tag_aa), "age class", "age")
                      make_rmd_tagmov(yy, aa, s, paste(yname, yy), paste(aname, aa), sname[s], header = yy == 1 && aa == 1)
                    }) |> unlist()
                  })
                  do.call(c, out)
                }) |>
                  unlist() |>
                  as.character()

                c("## Tag movement {.tabset}\n\n", tagmov_txt)
              })

            } else {
              rmd_split[[tagmov_ind]] <- ""
            }

            summarystock_ind <- grep("*ADD IND STOCK RMD*", rmd)
            rmd_split[[summarystock_ind]] <- mapply(make_rmd_ind_stock, s = 1:ns, sname = sname, MoreArgs = list(ns = ns)) |>
              as.character()

            stock_ind <- grep("*ADD STOCK REGION RMD*", rmd)
            mov_ind <- grep("*ADD MOVEMENT RMD*", rmd)
            if (nr > 1) {
              year <- dat@Dlabel@year
              rmd_split[[stock_ind]] <- mapply(make_rmd_stock_region, s = 1:ns, sname = sname, MoreArgs = list(ns = ns)) |>
                as.character()

              if (is.null(dots$ymov)) dots$ymov <- dat@Dmodel@ny
              if (is.null(dots$amov)) dots$amov <- 2

              mov <- expand.grid(a = dots$amov, y = dots$ymov, s = 1:ns)
              rmd_split[[mov_ind]] <- sapply(1:nrow(mov), function(i) {
                make_rmd_mov(s = mov$s[i], y = mov$y[i], a = mov$a[i], yname = dat@Dlabel@year[mov$y[i]],
                             sname = sname[mov$s[i]], header = i == 1)
              }) |> as.character()

            } else {
              rmd_split[[stock_ind]] <- rmd_split[[mov_ind]] <- ""
            }

            ####### Function arguments for rmarkdown::render
            filename_rmd <- paste0(filename, ".Rmd")

            render_args$input <- file.path(dir, filename_rmd)
            if (is.null(render_args$quiet)) render_args$quiet <- TRUE

            # Generate markdown report
            if (!dir.exists(dir)) {
              message_info("Creating directory: ", dir)
              dir.create(dir)
            }
            write(unlist(rmd_split), file = file.path(dir, filename_rmd))

            # Rendering markdown file
            message_info("Rendering markdown file: ", file.path(dir, filename_rmd))
            output_filename <- do.call(rmarkdown::render, render_args)
            message("Rendered file: ", output_filename)

            if (open_file) browseURL(output_filename)
            invisible(output_filename)
          })

#' @import RTMB
NULL

#' @importFrom methods callNextMethod
setMethod(
  "sapply", signature(X="integer"),
  function (X, FUN, ..., simplify = TRUE, USE.NAMES = TRUE) {
    ans <- callNextMethod()
    if (is.complex(ans))
      class(ans) <- "advector"
    ans
  }
)


#' `sapply2` function
#'
#' An alternate `sapply` function with argument `simplify = "array"` for convenience.
#'
#' @param X,FUN,...,USE.NAMES Same arguments as [sapply()]
#' @returns Output of `simplify2array()`, typically an array
#' @export
#' @keywords internal
sapply2 <- function(X, FUN, ..., USE.NAMES = TRUE) {
  sapply(X, FUN, ..., simplify = "array", USE.NAMES = USE.NAMES)
}

#' @name AD
#' @title Additional methods for AD types
#'
#' @description Methods for RTMB AD class
NULL

#' @describeIn AD Matrix product function implemented for mixed AD and non-AD objects with `colSums(x * y)`. See \link[RTMB]{ADmatrix}.
#'
#' @param x AD object
#' @param y Non-AD matrix
#' @aliases %*%,ad,matrix-method matmul
#' @keywords internal
setMethod("%*%",
          signature("ad", "matrix"),
          function(x, y) {
            colSums(x * y)
          })



show <- function(object) methods::show(object)
setMethod("show",
          signature = "MSAassess",
          function(object) {

            res <- list()
            res$npar <- paste("Number of parameters:", length(object@obj$par))

            cat(res$npar)
            if (length(object@SD) > 1 && !is.null(object@SD$gradient.fixed)) {
              gr <- abs(object@SD$gradient.fixed)
              gr_max <- ifelse(all(is.na(gr)), NA, round(max(gr, na.rm = TRUE), 4))

              res$max_grad <- paste("Maximum gradient:", gr_max)
            } else {
              res$max_grad <- "Run model to view gradient report"
            }
            cat("\n", res$max_grad)

            if (length(object@SD) > 1 && !is.null(object@SD$gradient.fixed)) {
              gr_na <- is.na(gr)
              if (sum(gr_na)) {
                if (sum(gr_na) < length(gr_na)) {
                  gr_names <- make_unique_names(object@SD)[gr_na]

                  res$grad_na <- paste(gr_names, collapse = ", ")

                  cat("\nParameters with gradient = NA:")
                } else {
                  res$grad_na <- "Gradient of NA for all parameters"
                }
                cat("\n", res$grad_na)
              }

              gr_large <- !is.na(gr) & gr > 0.1
              if (sum(gr_large)) {
                gr_report <- gr[gr_large]
                gr_names <- make_unique_names(object@SD)[gr_large]

                cat("\nParameters with large gradients (> 0.1):\n")

                gr_order <- order(gr_report, decreasing = TRUE)

                x <- data.frame(
                  Estimate = round(object@SD$par.fixed[gr_large], 4),
                  Gradient = round(gr_report, 4)
                )
                rownames(x) <- gr_names
                res$grad_large <- x[gr_order, ]
                print(res$grad_large)
              }
            }

            if (length(object@SD) > 1 && !is.null(object@SD$env$hessian)) {
              h <- object@SD$env$hessian

              res$det_h <- signif(det(h), 5)
              cat(paste("\nDeterminant of Hessian:"), res$det_h)

              zero_rows <- apply(h, 1, function(x) all(x == 0, na.rm = TRUE))
              na_rows <- apply(h, 1, function(x) all(is.na(x)))
              if (any(zero_rows)) {
                cat("\nParameters with all zeros in Hessian:\n")
                par_zero <- names(zero_rows)[zero_rows]
                res$hess_zero <- par_zero
                for(i in par_zero) cat(i, "\n")
              }
              if (any(na_rows)) {
                cat("\nParameters with all NAs in Hessian:\n")
                par_na <- names(na_rows)[na_rows]
                res$hess_na <- par_na
                for(i in par_na) cat(i, "\n")
              }
            }

            invisible(res)
          })

#' @name retrospective
#' @title Retrospective analysis
#'
#' @description Perform a retrospective analysis, successive removals of most recent years of data to evaluate
#' consistency in model estimates of biomass, recruitment, etc.
#'
#' The `summary` method returns Mohn's rho and the plot method generates a markdown report.
#' @return A `MSAretro` object containing a named lists of arrays generated by the retrospective analysis:
#' - `S_yst` Spawning output array `[y, s, t]` where `t` indexes the retrospective peel
#' - `R_yst` Recruitment array `[y, s, t]`
#' - `F_yst` Apical fishing mortality `[y, s, t]`
#' - `VB_ymft` Vulnerable biomass available to each fishery `[y, m, f, t]`
#' @param MSAassess [MSAassess-class] object
#' @param yret Vector specifying the years (positive integers and include zero) to remove for the retrospective analysis
#' @param cores Integer for the number of cores to use for parallel processing
#' @export
#' @importFrom parallel makeCluster stopCluster
retrospective <- function(MSAassess, yret = 0:5, cores = 1) {

  if (cores > 1) {
    cl <- parallel::makeCluster(cores)
    on.exit(parallel::stopCluster(cl), add = TRUE)
  } else {
    cl <- NULL
  }

  old_opts <- pbapply::pboptions()
  on.exit(pbapply::pboptions(old_opts), add = TRUE)

  pbapply::pboptions(use_lb = TRUE)

  ret <- pbapply::pblapply(yret, .ret, MSAassess, cl = cl)

  MSAdata <- get_MSAdata(MSAassess)
  ny <- MSAdata@Dmodel@ny
  nm <- MSAdata@Dmodel@nm
  nf <- MSAdata@Dfishery@nf
  ns <- MSAdata@Dmodel@ns

  nt <- length(yret)

  F_yst <-
    R_yst <-
    S_yst <-
    log_rdev_yst <- array(NA_real_, c(ny, ns, nt))

  VB_ymft <- array(NA_real_, c(ny, nm, nf, nt))

  F_yst[] <- sapply2(ret, function(x) apply(x@report$F_yas, c(1, 3), max))
  R_yst[] <- sapply2(ret, function(x) x@report$R_ys)
  S_yst[] <- sapply2(ret, function(x) apply(x@report$S_yrs, c(1, 3), sum))
  log_rdev_yst[] <- sapply2(ret, function(x) x@obj$env$parList(x@obj$env$last.par.best)$log_rdev_ys)
  VB_ymft[] <- sapply2(ret, function(x) apply(x@report$VB_ymfrs, 1:3, sum))

  ret_out <- list(S_yst = S_yst, R_yst = R_yst, F_yst = F_yst, VB_ymft = VB_ymft,
                  log_rdev_yst = log_rdev_yst) |>
    structure(class = "MSAretro")
  attr(ret_out, "Dlabel") <- MSAdata@Dlabel
  attr(ret_out, "yret") <- yret
  return(ret_out)
}


.ret <- function(y, MSAassess) {

  MSAdata <- get_MSAdata(MSAassess)
  if (y == MSAdata@Dmodel@nyret) return(MSAassess)

  MSAdata@Dmodel@nyret <- y

  parameters <- MSAassess@obj$env$parList(par = MSAassess@obj$par)
  map <- MSAassess@obj$env$map
  if (length(map)) {
    map_dim <- lapply(names(map), function(i) {
      dim_i <- dim(parameters[[i]])
      if (length(dim_i)) {
        array(map[[i]], dim_i)
      } else {
        map[[i]]
      }
    }) |>
      structure(names = names(map))
  } else {
    map_dim <- list()
  }
  random <- MSAassess@obj$env$random

  data_new <- check_data(MSAdata, silent = TRUE)
  parameters_new <- make_parameters(data_new, start = parameters, map = map_dim, silent = TRUE)
  fit <- fit_MSA(data_new, parameters_new$p, parameters_new$map, random, do_sd = FALSE, silent = TRUE)

  return(fit)
}

#' @rdname retrospective
#' @param x,object Output of `retrospective` function
#' @param var Character to indicate the metric, the item in the `MSAretro` list to be plotted. See details below.
#' @param s Integer for the stock index to plot
#' @param f Integer for the fleet index to plot
#' @param ... Not used
#' @importFrom gplots rich.colors
#' @importFrom graphics legend
#' @returns
#' `plot` generic for MSAretro returns individual figures using base graphics.
#' @export
#' @method plot MSAretro
plot.MSAretro <- function(x, var = c("S_yst", "R_yst", "F_yst", "log_rdev_yst", "VB_ymft"), s = 1, f = 1, ...) {
  var <- match.arg(var)
  Dlabel <- attr(x, "Dlabel")
  peel <- attr(x, "yret")
  ny <- length(Dlabel@year)
  nm <- max(length(Dlabel@season), 1)

  year <- Dlabel@year
  ylab <- retro_label(var)

  if (grepl("yst", var)) {
    y <- yannual <- x[[var]][, s, ]
    denom <- 1
  } else if (grepl("ymft", var)) {
    yannual <- x[[var]][, , f, ]
    if (length(dim(yannual)) > 2) { # nm = 1
      y <- collapse_yearseason(yannual)
      year <- make_yearseason(year, nm)
      ylab <- paste("Seasonal", tolower(ylab))
    } else {
      y <- yannual
    }
    denom <- nm
  }

  if (all(y >= 0, na.rm = TRUE)) {
    ylim <- c(0, 1.1) * range(y, na.rm = TRUE)
  } else {
    ylim <- 1.1 * range(y, na.rm = TRUE)
  }

  rcolor <- rich.colors(ncol(y))
  for (i in 1:length(peel)) {
    if (peel[i] > 0) y[seq(ny - peel[i] + 1, ny), i] <- NA_real_
  }

  make_tinyplot(year, y, ylab, name = peel/denom, rcolor, type = "l", leg = substitute(legend(title = "Years\nremoved:")))
  points(year[ny - peel], y[cbind(ny - peel, 1:ncol(y))], pch = 16, col = rcolor)

  invisible()
}

#' @rdname retrospective
#' @param by Character indicating whether to calculate to Mohn's rho on stock or fleet-based time series
#' @return
#' `summary` generic for MSAretro returns a matrix of Mohn's rho.
#' @export
#' @method summary MSAretro
summary.MSAretro <- function(object, by = c("stock", "fleet"), ...) {
  by <- match.arg(by)
  peel <- attr(object, "yret")

  if (by == "stock") {
    ns <- dim(object$S_yst)[2]
    sname <- attr(object, "Dlabel")@stock
    if (length(sname) <= 1) sname <- "rho"

    x <- object[grepl("yst", names(object))]

    rho <- sapply(x, function(i) apply(i, 2, Mohn_rho, peel)) |>
      t() |>
      matrix(length(x), ns) |>
      structure(dimnames = list(sapply(names(x), retro_label), sname))
  } else {
    fname <- attr(object, "Dlabel")@fleet
    if (length(fname) <= 1) fname <- "rho"

    x <- object[grepl("ymft", names(object))]
    ny <- dim(x[[1]])[1]
    nf <- dim(x[[1]])[3]
    nt <- dim(x[[1]])[4]

    # First season
    rho <- sapply(x, function(i) {
      array(i[, 1, , ], c(ny, nf, nt)) |> apply(2, Mohn_rho, peel)
    }) |>
      t() |>
      matrix(length(x), nf) |>
      structure(dimnames = list(sapply(names(x), retro_label), fname))
  }
  return(rho)
}

Mohn_rho <- function(x, peel) {
  ny <- dim(x)[1]
  for (i in 1:length(peel)) { # Not necessary but good practice
    if (peel[i] > 0) x[seq(ny - peel[i] + 1, ny), i] <- NA_real_
  }
  num <- x[cbind(ny - peel, 1:ncol(x))]
  denom <- x[ny - peel, peel == 0]
  rho <- num/denom - 1
  mean(rho[peel > 0])
}



#' @inheritParams report
#' @return
#' `report` generic for MSAretro invisibly returns the output of [rmarkdown::render()]: character of the path of the rendered HTML markdown report.
#' @rdname retrospective
#' @importFrom rmarkdown render
#' @importFrom utils browseURL
#' @export
#' @method report MSAretro
report.MSAretro <- function(object, filename = "retro", dir = tempdir(), open_file = TRUE, render_args = list(), ...) {

  sname <- attr(object, "Dlabel")@stock
  fname <- attr(object, "Dlabel")@fleet

  rmd <- system.file("include", "retrospective.Rmd", package = "multiSA") |> readLines()
  rmd_split <- split(rmd, 1:length(rmd))

  fleet_ind <- grep("*ADD FLEET RMD*", rmd)
  stock_ind <- grep("*ADD STOCK RMD*", rmd)
  rmd_split[[fleet_ind]] <- make_rmd_ret_fleet(fname)
  rmd_split[[stock_ind]] <- make_rmd_ret_stock(sname)

  ####### Function arguments for rmarkdown::render
  filename_rmd <- paste0(filename, ".Rmd")

  render_args$input <- file.path(dir, filename_rmd)
  if (is.null(render_args$quiet)) render_args$quiet <- TRUE

  # Generate markdown report
  if (!dir.exists(dir)) {
    message_info("Creating directory: ", dir)
    dir.create(dir)
  }
  write(do.call(c, rmd_split), file = file.path(dir, filename_rmd))

  # Rendering markdown file
  message_info("Rendering markdown file: ", file.path(dir, filename_rmd))
  output_filename <- do.call(rmarkdown::render, render_args)
  message("Rendered file: ", output_filename)

  if (open_file) browseURL(output_filename)
  invisible(output_filename)
}

make_rmd_ret_stock <- function(sname) {
  ns <- max(length(sname), 1)

  rmd <- lapply(1:ns, function(s) {
    header <- if (!length(sname) || length(sname) == 1) NULL else paste("###", sname[s], "\n")
    out <- c("```{r}",
             paste0("plot(object, \"S_yst\", s = ", s, ")"),
             "```\n",
             "```{r}",
             paste0("plot(object, \"R_yst\", s = ", s, ")"),
             "```\n",
             "```{r}",
             paste0("plot(object, \"F_yst\", s = ", s, ")"),
             "```\n",
             "```{r}",
             paste0("plot(object, \"log_rdev_yst\", s = ", s, ")"),
             "```\n")
    c(header, out)
  })
  do.call(c, rmd)
}

make_rmd_ret_fleet <- function(fname) {
  nf <- max(length(fname), 1)

  rmd <- lapply(1:nf, function(f) {
    out <- c("```{r}",
             paste0("plot(object, \"VB_ymft\", f = ", f, ")"),
             "```\n")
  })
  do.call(c, rmd)
}

retro_label <- function(var) {
  switch(
    var,
    "F_yst" = "Fishing mortality",
    "R_yst" = "Recruitment",
    "S_yst" = "Spawning output",
    "VB_ymft" = "Vulnerable biomass",
    "log_rdev_yst" = "log Recruitment deviations"
  )
}

---
title: "multiSA assessment summary"
subtitle: Tables and Figures
date: "`r Sys.Date()`"
output:
  html_document:
    highlight: rstudio
    df_print: paged
---

<style type="text/css">
h1 { /* Header 1 */
  font-size: 24px;
}
</style>

```{r setup, include = FALSE, echo = FALSE}
library(dplyr)
knitr::opts_chunk$set(
  collapse = TRUE, echo = FALSE, message = FALSE,
  fig.width = 6, fig.height = 4.5, out.width = "650px", comment = "#>"
)
```

# NAME {.tabset}

## Parameters

```{r print-summary}
Summary <- utils::capture.output(x)
as.data.frame(Summary)
```

### Parameter estimates 

```{r sdreport}
if (length(x@SD) > 1) summary(x, select = c("fixed", "random")) %>% round(4) %>% as.data.frame()
```


### Derived quantities

```{r sdreport-derived}
if (length(x@SD) > 1) summary(x, select = "report") %>% round(4) %>% as.data.frame()
```

### Correlation

```{r sdcorr}
if (length(x@SD) > 1 && length(x@SD$env$corr.fixed)) as.data.frame(x@SD$env$corr.fixed)
```

<!-- *ADD HESSIAN RMD* -->

## Fishery {.tabset}

<!-- *ADD FISHERY RMD* -->

<!-- *ADD SURVEY RMD* -->

<!-- *ADD SC RMD* -->

<!-- *ADD CK RMD* -->

<!-- *ADD TAG MOV RMD* -->

## Stock dynamics {.tabset}

### Summary

```{r SB, fig.cap="Spawning output."}
plot_S(x, by = "region")
```

```{r B, fig.cap="Total biomass."}
plot_B(x, by = "region")
```

```{r FF, fig.cap="Instantaneous fishing mortality (per year)."}
plot_Fstock(x)
```

```{r R, fig.cap="Recruitment."}
plot_R(x)
```

```{r recdist, fig.cap="Distribution of recruitment by region per stock."}
plot_recdist(x)
```

<!-- *ADD IND STOCK RMD* -->

<!-- *ADD STOCK REGION RMD* -->

<!-- *ADD MOVEMENT RMD* -->


## About

This report was generated on: `r Sys.time()`<br />
multiSA version `r packageVersion("multiSA")`<br />
`r R.version.string`

---
title: "multiSA assessment"
subtitle: Tables and Figures
date: "`r Sys.Date()`"
output:
  html_document:
    highlight: rstudio
    df_print: paged
---

<style type="text/css">
h1 { /* Header 1 */
  font-size: 24px;
}
</style>

```{r setup, include = FALSE, echo = FALSE}
library(dplyr)
knitr::opts_chunk$set(
  collapse = TRUE, echo = FALSE, message = FALSE,
  fig.width = 6, fig.height = 4.5, out.width = "650px", comment = "#>"
)
```

# Retrospective analysis {.tabset}

## By fleet {.tabset}

```{r}
summary(object, by = "fleet") %>% as.data.frame()
```

<!-- *ADD FLEET RMD* -->

## By stock {.tabset}

```{r}
summary(object, by = "stock") %>% as.data.frame()
```

<!-- *ADD STOCK RMD* -->

## About

This report was generated on: `r Sys.time()`<br />
multiSA version `r packageVersion("multiSA")`<br />
`r R.version.string`

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{optimize_RTMB}
\alias{optimize_RTMB}
\title{Optimize RTMB model}
\usage{
optimize_RTMB(
  obj,
  hessian = FALSE,
  restart = 0,
  do_sd = TRUE,
  control = list(iter.max = 2e+05, eval.max = 4e+05),
  lower = -Inf,
  upper = Inf,
  silent = FALSE
)
}
\arguments{
\item{obj}{The list returned by \code{\link[RTMB:TMB-interface]{RTMB::MakeADFun()}}}

\item{hessian}{Logical, whether to pass the Hessian function \code{obj$he} to \code{\link[stats:nlminb]{stats::nlminb()}}. Only used if
there are no random effects in the model.}

\item{restart}{Deprecated.}

\item{do_sd}{Deprecated.}

\item{control}{List of options passed to \code{\link[stats:nlminb]{stats::nlminb()}}}

\item{lower}{Lower bounds of parameters passed to \code{\link[stats:nlminb]{stats::nlminb()}}}

\item{upper}{Upper bounds of parameters passed to \code{\link[stats:nlminb]{stats::nlminb()}}}

\item{silent}{Logical, whether to report progress to console}
}
\value{
A named list, output of \code{\link[stats:nlminb]{stats::nlminb()}}
}
\description{
A convenient function to fit a RTMB model with \code{\link[stats:nlminb]{stats::nlminb()}}
}
\seealso{
\code{\link[=get_sdreport]{get_sdreport()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/profile.R
\name{profile}
\alias{profile}
\alias{profile,MSAassess-method}
\alias{profile.MSAassess}
\alias{get_likelihood_components}
\alias{plot.MSAprof}
\title{Profile parameters of MSA model}
\usage{
\S4method{profile}{MSAassess}(
  fitted,
  p1,
  v1,
  p2,
  v2,
  use_fitted = TRUE,
  return_models = TRUE,
  cores = 1,
  ...
)

get_likelihood_components(fitted)

\method{plot}{MSAprof}(
  x,
  component = "objective",
  rel = TRUE,
  xlab,
  ylab,
  main,
  plot2d = c("contour", "filled.contour"),
  ...
)
}
\arguments{
\item{fitted}{\linkS4class{MSAassess} object returned by \code{\link[=fit_MSA]{fit_MSA()}}}

\item{p1}{Character string that represents the first parameter to be profiled,
including the parameter name and index of the vector/array. See "Parameters" section of \code{\link[=make_parameters]{make_parameters()}}.
Additionally, this function allows users to specify \code{R0_s} and \code{h_s} (in normal units).}

\item{v1}{Vector of values corresponding to \code{p1}}

\item{p2}{Character string that represents the optional second parameter to be profiled}

\item{v2}{Vector of values corresponding to \code{p2}}

\item{use_fitted}{Logical, whether to use estimated parameters or the starting values in \code{fitted} to start the profile}

\item{return_models}{Logical, whether to return fitted models in the profile}

\item{cores}{Integer for the number of cores to use for parallel processing}

\item{...}{Other argument to the base graphics function, i.e., either plot() or contour()}

\item{x}{Output from \code{\link[=profile.MSAassess]{profile.MSAassess()}}}

\item{component}{Character for the column in \code{x} to be plotted}

\item{rel}{Logical, whether the relative change in \code{component} is plotted (TRUE) or the raw values (FALSE)}

\item{xlab}{Optional character for the x-axis label}

\item{ylab}{Optional character for the y-axis label}

\item{main}{Optional character for the plot title}

\item{plot2d}{Character, plotting function for two-dimensional profiling (either a \code{\link[=contour]{contour()}} or \code{\link[=filled.contour]{filled.contour()}} plot)}
}
\value{
Named list (length 2).

First, \code{profile} contains a data frame of the likelihood values that correspond to
fixed values of \code{p1} and \code{p2}. Other columns:
\itemize{
\item Likelihood \code{loglike} refers to maximizing the probability of the observed data (higher values for better fit)
\item Prior \code{logprior} refers to maximizing the probability of a parameter to their prior distribution (higher values are closer to the prior mode)
\item Penalty \code{penalty} are values added to the objective function when parameters exceed model bounds (lower values are better)
\item \code{fn} is the objective function returned by RTMB (lower values are better)
\item \code{objective} is the objective function returned by the optimizer (lower values are better)
}

Second, \code{fits} contains a list of the \code{MSAassess} objects if \code{return_models = TRUE}.

\code{get_likelihood_components()} returns a data.frame of the components to the objective function (log-likelihoods, log-priors, etc.)
as well as some diagnostic information: maximum gradient (\code{maxgrad}) and convergence (\code{conv})

The accompanying plot function returns a line plot for a 1-dimensional profile or a contour plot for a two
dimensional profile. Will plot the negative log likelihood or negative log prior (better fit with lower values).

Relative values are obtained by subtracting from the fitted value. See \code{attr(x$profile, "fitted")}
}
\description{
Evaluate change in objective function and likelihood components for up to 2 parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiSA-package.R
\docType{package}
\name{multiSA-package}
\alias{multiSA}
\alias{multiSA-package}
\title{multiSA: Multi-Stock Assessment}
\description{
Implementation of a next-generation, multi-stock age-structured fisheries assessment model. 'multiSA' is intended for use in mixed fisheries where stock composition can not be readily identified in fishery data alone, e.g., from catch and age/length composition. Models can be fitted to genetic data, e.g., stock composition of catches and close-kin pairs, with seasonal stock availability and movement.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://blue-matter.github.io/multiSA/}
  \item \url{https://github.com/Blue-Matter/multiSA}
  \item Report bugs at \url{https://github.com/Blue-Matter/multiSA/issues}
}

}
\author{
\strong{Maintainer}: Quang Huynh \email{quang@bluematterscience.com} (\href{https://orcid.org/0000-0001-7835-4376}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtmb-methods.R
\name{sapply2}
\alias{sapply2}
\title{\code{sapply2} function}
\usage{
sapply2(X, FUN, ..., USE.NAMES = TRUE)
}
\arguments{
\item{X, FUN, ..., USE.NAMES}{Same arguments as \code{\link[=sapply]{sapply()}}}
}
\value{
Output of \code{simplify2array()}, typically an array
}
\description{
An alternate \code{sapply} function with argument \code{simplify = "array"} for convenience.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\docType{class}
\name{MSAassess-class}
\alias{MSAassess-class}
\title{MSAassess S4 object}
\description{
S4 object that returns output from MSA model
}
\section{Slots}{

\describe{
\item{\code{obj}}{RTMB object returned by \code{\link[RTMB:TMB-interface]{RTMB::MakeADFun()}}}

\item{\code{opt}}{List returned by \code{\link[stats:nlminb]{stats::nlminb()}}}

\item{\code{SD}}{List returned by \code{\link[RTMB:TMB-interface]{RTMB::sdreport()}}}

\item{\code{report}}{List of model output at the parameter estimates, returned by \code{obj$report(obj$env$last.par.best)}}

\item{\code{Misc}}{List, miscellaneous items}
}}

\keyword{MSAassess}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{get_sdreport}
\alias{get_sdreport}
\title{Calculate standard errors}
\usage{
get_sdreport(
  obj,
  par.fixed,
  exact = FALSE,
  getReportCovariance = FALSE,
  silent = FALSE,
  ...
)
}
\arguments{
\item{obj}{The list returned by \code{\link[RTMB:TMB-interface]{RTMB::MakeADFun()}}}

\item{par.fixed}{Numeric vector of parameters from which to calculate covariance matrix. Optional}

\item{exact}{Logical, whether to use autodiff or finite-difference approximation for the hessian. See details.}

\item{getReportCovariance}{Logical, passed to \code{\link[RTMB:TMB-interface]{RTMB::sdreport()}}}

\item{silent}{Logical, whether to report progress to console. See details.}

\item{...}{Other arguments to \code{\link[RTMB:TMB-interface]{RTMB::sdreport()}} besides \verb{par.fixed, hessian.fixed, getReportCovariance}}
}
\value{
Object returned by \code{\link[RTMB:TMB-interface]{RTMB::sdreport()}}.

A correlation matrix is generated and stored in: \code{get_sdreport(obj)$env$corr.fixed}

The hessian is stored in \code{get_sdreport(obj)$env$hessian}
}
\description{
A wrapper function to return standard errors. Various numerical techniques are employed to obtain
a positive-definite covariance matrix in marginal cases.
}
\details{
Uses \code{\link[stats:optim]{stats::optimHess()}} if \code{exact = FALSE}.
Autodiff with \code{exact = TRUE} is only available for TMB models without random effects, but is also memory-intensive.

In numerically marginal cases where the determinant of the Hessian matrix is less than 0.1, the function will attempt
to calculate the hessian with \code{numDeriv::jacobian()} and the gradient from TMB.

Finally, in other marginal cases where \code{\link[=chol]{chol()}} identifies a positive-definite Hessian but \code{\link[=solve]{solve()}} fails to
invert the matrix, the covariance matrix will be updated with \code{chol2inv(chol(h))}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{conv_Sigma}
\alias{conv_Sigma}
\title{Calculate covariance matrix}
\usage{
conv_Sigma(sigma, lower_diag)
}
\arguments{
\item{sigma}{Numeric vector of marginal standard deviations (all greater than zeros)}

\item{lower_diag}{Numeric vector to populate the lower triangle of the correlation matrix. All real numbers.
Length \code{sum(1:(length(sigma) - 1))}}
}
\value{
Numeric
}
\description{
Uses Cholesky factorization to generate a covariance matrix (or any symmetric positive definite matrix).
}
\examples{
set.seed(23)
n <- 5
sigma <- runif(n, 0, 2)
lower_diag <- runif(sum(1:(n-1)), -10, 10)
Sigma <- conv_Sigma(sigma, lower_diag)
Sigma/t(Sigma) # Is symmetric matrix? All ones
cov2cor(Sigma)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report-int-state.R
\name{plot-MSA-state}
\alias{plot-MSA-state}
\alias{plot_S}
\alias{plot_B}
\alias{plot_R}
\alias{plot_SRR}
\alias{plot_Rdev}
\alias{plot_Fstock}
\alias{plot_self}
\alias{plot_seli}
\alias{plot_selstock}
\alias{plot_N}
\alias{plot_V}
\alias{plot_Ffleet}
\alias{plot_mov}
\alias{plot_recdist}
\title{Plotting functions for fitted MSA model}
\usage{
plot_S(
  fit,
  by = c("stock", "region"),
  r,
  s,
  prop = FALSE,
  facet_free = FALSE,
  figure = TRUE,
  ylab
)

plot_B(
  fit,
  by = c("stock", "region"),
  r,
  s,
  prop = FALSE,
  facet_free = FALSE,
  figure = TRUE
)

plot_R(fit, s, figure = TRUE)

plot_SRR(fit, s = 1, phi = TRUE)

plot_Rdev(fit, s = 1, log = TRUE, figure = TRUE)

plot_Fstock(fit, s, by = c("annual", "season"), figure = TRUE)

plot_self(fit, f = 1, type = c("length", "age"), figure = TRUE)

plot_seli(fit, i = 1, figure = TRUE)

plot_selstock(
  fit,
  s = 1,
  by = c("annual", "season"),
  plot2d = c("contour", "filled.contour"),
  ...
)

plot_N(fit, m = 1, r, s = 1, plot2d = c("contour", "filled.contour"), ...)

plot_V(fit, f = 1, by = c("stock", "region"), prop = FALSE, facet_free = FALSE)

plot_Ffleet(fit, f = 1)

plot_mov(fit, s = 1, y, a, palette = "Peach", figure = TRUE)

plot_recdist(fit, palette = "Peach", figure = TRUE)
}
\arguments{
\item{fit}{\linkS4class{MSAassess} object returned by \code{\link[=fit_MSA]{fit_MSA()}}}

\item{by}{Character to indicate whether to calculate selectivity from F per year or per season}

\item{r}{Integer for the corresponding region}

\item{s}{Integer for the corresponding stock}

\item{prop}{Logical, whether to plot proportions (TRUE) or absolute numbers}

\item{facet_free}{Logical, whether to allow the y-axis limits to vary by panel in facetted plots}

\item{figure, }{Logical, whether to generate the plot}

\item{ylab}{Optional character string for custom y-axis label}

\item{phi}{Logical, whether to plot unfished replacement line}

\item{log}{Logical, whether to plot the natural logarithm of the response variable}

\item{f}{Integer for the corresponding fleet}

\item{type}{For \code{plot_self}, indicates whether to plot the selectivity by age or length.}

\item{i}{Integer for the corresponding survey}

\item{plot2d}{Character, plotting function for either a \code{\link[=contour]{contour()}} or \code{\link[=filled.contour]{filled.contour()}} plot}

\item{...}{Other argument to the base graphics function}

\item{m}{Integer for the corresponding season}

\item{y}{Integer, year for plotting the movement matrix (last model year is the default)}

\item{a}{Integer, corresponding age for plotting the movement matrix (age 1 is the default)}

\item{palette}{Character, palette name to pass to \code{\link[grDevices:palettes]{grDevices::hcl.colors()}}. See \code{\link[grDevices:palettes]{grDevices::hcl.pals()}} for options.}
}
\value{
Invisible data frame of state variables that were plotted in base graphics figures
}
\description{
A set of functions to plot state variables (biomass, recruitment time series, etc.)
}
\details{
\itemize{
\item \code{plot_S} plots spawning output by stock or region (either whole numbers or proportions for the latter)
}

\itemize{
\item \code{plot_B} plots total biomass by stock or region (either whole numbers or proportions for the latter)
}

\itemize{
\item \code{plot_R} plots recruitment by stock
}

\itemize{
\item \code{plot_SRR} plots the stock-recruitment relationship and history (realized recruitment) by stock
}

\itemize{
\item \code{plot_Rdev} plots recruitment deviations by stock
}

\itemize{
\item \code{plot_Fstock} plots apical instantaneous fishing mortality (per year or per season) by stock
}

\itemize{
\item \code{plot_self} plots fishery selectivity
}

\itemize{
\item \code{plot_seli} plots index selectivity
}

\itemize{
\item \code{plot_selstock} plots the realized selectivity from total catch and total abundance at age
}

\itemize{
\item \code{plot_N} reports total abundance at age
}

\itemize{
\item \code{plot_V} plots vulnerable biomass, availability to the fishery
}

\itemize{
\item \code{plot_Ffleet} plots apical instantaneous fishing mortality (per season) by fleet
}

\itemize{
\item \code{plot_mov} plots movement matrices and the corresponding equilibrium distribution in multi-area models
}

\itemize{
\item \code{plot_recdist} plots the distribution of recruitment for each stock
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtmb-methods.R
\name{AD}
\alias{AD}
\alias{\%*\%,ad,matrix-method}
\alias{matmul}
\title{Additional methods for AD types}
\usage{
\S4method{\%*\%}{ad,matrix}(x, y)
}
\arguments{
\item{x}{AD object}

\item{y}{Non-AD matrix}
}
\description{
Methods for RTMB AD class
}
\section{Functions}{
\itemize{
\item \code{x \%*\% y}: Matrix product function implemented for mixed AD and non-AD objects with \code{colSums(x * y)}. See \link[RTMB]{ADmatrix}.

}}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\docType{class}
\name{Dsurvey-class}
\alias{Dsurvey-class}
\title{Dsurvey S4 object}
\description{
S4 class that organizes the various data inputs for the MSA model. \code{MSAdata} simply inherits the slots from 6 component classes:
\code{Dmodel}, \code{Dstock}, \code{Dfishery}, \code{Dsurvey} \code{DCKMR}, and \code{Dtag}, where the \code{D}- prefix denotes an object for data inputs (or model configuration).
}
\details{
For convenience, most arrays and matrices have the associated dimensions in the variable name. For example, \code{Cobs_ymfr} represents
observed catch with the dimension following the underscore, following this template:

\tabular{ll}{
\code{y} \tab Year \cr
\code{m} \tab Season \cr
\code{a} \tab Age \cr
\code{r} \tab Region \cr
\code{f} \tab Fishery \cr
\code{i} \tab Index \cr
\code{s} \tab Stock
}
}
\section{Slots inherited from Dsurvey}{

\describe{
\item{\code{ni}}{Integer, number of indices of abundance. Zero is possible.}
\item{\code{Iobs_ymi}}{Observed indices}
\item{\code{Isd_ymi}}{Lognormal standard deviation of the observed indices}
\item{\code{unit_i}}{Character vector, units of the index. Set to \code{"B"} to use stock weight at age (default) or \code{"N"} for abundance (numbers).}
\item{\code{IAAobs_ymai}}{Array, survey age composition}
\item{\code{IALobs_ymli}}{Array, survey length composition}
\item{\code{icomp_like}}{Character, likelihood for the composition data. See \code{\link[=like_comp]{like_comp()}} for options}
\item{\code{IAAN_ymi}}{Array, sample size of the index age composition by season if using the multinomial or Dirichlet-multinomial likelihoods}
\item{\code{IALN_ymi}}{Array, sample size of the index length composition by season if using the multinomial or Dirichlet-multinomial likelihoods}
\item{\code{IAAtheta_i}}{Numeric vector, survey age composition dispersion parameter if using the Dirichlet-multinomial likelihood}
\item{\code{IALtheta_i}}{Numeric vector, index length composition dispersion parameter if using the Dirichlet-multinomial likelihood}
\item{\code{samp_irs}}{Boolean array that specifies the regions and stocks sampled by the index. \code{samp[i, r, s]} indicates whether index \code{i} operates in region \code{r} and catches stock \code{s}.}
\item{\code{sel_i}}{Character vector, functional forms for selectivity. See \code{"type"} argument in \code{\link[=conv_selpar]{conv_selpar()}} for options.}
\item{\code{delta_i}}{Numeric vector, the elapsed fraction of time in the seasonal time step (between 0 - 1) when the index samples the population. Set to a negative number (-1) to sample over the duration of the timestep, i.e., \code{(1 - exp(-Z))/Z}.}
\item{\code{qest_i}}{Vector, either a character \code{"est"} to allow the model to estimate the catchability for index \code{i} (default) or a numeric to fix the parameter.}
\item{\code{lambdaI_i}}{Likelihood weight (by survey) for the indices. Default is 1.}
\item{\code{lambdaIAA_i}}{Likelihood weight (by fleet) for the index age composition. Default is 1.}
\item{\code{lambdaIAL_i}}{Likelihood weight (by fleet) for the index length composition. Default is 1.}
}
}

\seealso{
\link{MSAdata-class} \code{\link[=check_data]{check_data()}} \link{Dmodel-class} \link{Dstock-class} \link{Dfishery-class} \link{Dsurvey-class} \link{DCKMR-class} \link{Dtag-class}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\docType{class}
\name{DCKMR-class}
\alias{DCKMR-class}
\title{DCKMR S4 object}
\description{
Store lists of data frames for parent offspring pairs and half-sibling pairs
}
\section{Slots inherited from DCKMR}{

\describe{
\item{\code{POP_s}}{A list by stock of data frames for parent-offspring pairs. Each row in the data frame corresponds to a "sampling unit" defined by the columns:
\tabular{ll}{
\code{a} \tab Capture year of parent \cr
\code{t} \tab Age at capture of parent \cr
\code{y} \tab Birth year of offspring \cr
\code{n} \tab Number of pairwise comparisons \cr
\code{m} \tab Number of POPs \cr
}
}
\item{\code{HSP_s}}{A list by stock of data frames for half-sibling pairs. Each row in the data frame corresponds to a "sampling unit" defined by the columns:
\tabular{ll}{
\code{yi} \tab Birth year of older sibling \cr
\code{yj} \tab Birth year of younger sibling \cr
\code{n} \tab Number of pairwise comparisons \cr
\code{m} \tab Number of HSPs
}
}
\item{\code{CKMR_like}}{Character, likelihood for the POP and HSP sampling units. See \code{type} argument in \code{\link[=like_CKMR]{like_CKMR()}} for options.}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/if-statement.R
\name{CondExpLt}
\alias{CondExpLt}
\alias{CondExpLe}
\alias{CondExpGt}
\alias{CondExpGe}
\alias{CondExpEq}
\title{If statements compatible with RTMB}
\usage{
CondExpLt(left, right, if_true, if_false)

CondExpLe(left, right, if_true, if_false)

CondExpGt(left, right, if_true, if_false)

CondExpGe(left, right, if_true, if_false)

CondExpEq(left, right, if_true, if_false)
}
\arguments{
\item{left}{Numeric on left hand side of the evaluation}

\item{right}{Numeric on right hand side of the evaluation}

\item{if_true}{Numeric if expression is true}

\item{if_false}{Numeric if expression is false}
}
\value{
Numeric
}
\description{
Convenience functions that allow taping of gradients in RTMB with if expressions,
following the corresponding \code{CppAD} functions.
}
\details{
Functions should be vectorized.

\code{CondExpLt} evaluates whether \code{left < right}

\code{CondExpLe} evaluates whether \code{left <= right}

\code{CondExpGt} evaluates whether \code{left > right}

\code{CondExpGe} evaluates whether \code{left >= right}

\code{CondExpEq} evaluates whether \code{left == right}
}
\examples{
library(RTMB)
TapeConfig(comparison = "tape")
f <- function(x) CondExpLt(x, 3, 0, x^2)
g <- MakeTape(f, numeric(1))
x <- seq(0, 5)

# Does not work!
f2 <- function(x) if (x < 3) 0 else x^2
g2 <- MakeTape(f2, numeric(1))

# Compare the real answer (deriv) with various values returned by RTMB
data.frame(
  x = x,
  deriv = ifelse(x < 3, 0, 2 * x),
  deriv_f = sapply(x, g$jacobian),
  deriv_f2 = sapply(x, g2$jacobian)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_int.R
\name{calc_POP}
\alias{calc_POP}
\alias{calc_HSP}
\title{Predict the probability of CKMR kinship pairs}
\usage{
calc_POP(t, a, y, N, fec)

calc_HSP(yi, yj, N, fec, Z)
}
\arguments{
\item{t}{Vector, capture year of parent \code{i}}

\item{a}{Vector, age at capture of parent \code{i}}

\item{y}{Vector, birth year of offspring \code{j}}

\item{N}{Abundance of mature spawners. Matrix by \verb{[y, a]}}

\item{fec}{Fecundity schedule of mature spawners. Matrix by \verb{[y, a]}}

\item{yi}{Vector, birth year of sibling \code{i}. Must be older than sibling \code{j}.}

\item{yj}{Vector, birth year of sibling \code{j}.}

\item{Z}{Instantaneous total mortality rate. Matrix by \verb{[y, a]}}
}
\value{
Numeric, vector of probabilities
}
\description{
Calculate the probability of observing a parent-offspring pair (\code{calc_POP}) and
half-sibling pair (\code{calc_HSP}) for closed-kin mark recapture (CKMR) for an age-structured
model.
}
\section{Parent-offspring pairs}{

The parent-offspring probability is calculated from Bravington et al. 2016, eq 3.4:

\deqn{p_{\textrm{POP}} = 2 \times \dfrac{f(y_j,y_j - (t_i - a_i))}{\sum_a f(y_j,a) N(y_j,a)}}

where \eqn{y_j - (t_i - a_i)} is the parental age in year \eqn{y_j}. Scalar 2 accounts for the fact
that the parent could be either a mother or a father.
\code{calc_POP} is vectorized with respect to \code{t}, \code{a}, and \code{y}.
}

\section{Half-sibling pairs}{

The half-sibling probability is calculated from Bravington et al. 2016, eq 3.10, and expanded
by Hillary et al. 2018, Supplement S2.8.1 for age-specific survival and fecundity of the parent:

\deqn{p_{\textrm{HSP}} = 4 \times
\sum_a\left(
\dfrac{N(y_i, a)f(y_i, a)}{\sum_{a'} N(y_i, a')f(y_i,a')}\times
\exp(-\sum_{t = 0}^{y_j - y_i - 1} Z(y_i + t,a + t))\times
\dfrac{f(y_j,a+y_j-y_i)}{\sum_{a'} N(y_j,a')f(y_j,a')}
\right)
}
\itemize{
\item The first ratio is the probability that a fish at age \eqn{a} in year \eqn{y_i} is the parent of \eqn{i}.
\item The exponential term is that fish's survival from year \eqn{y_i} to \eqn{y_j}.
\item The second ratio is the probability that the parent of \eqn{i}, age \eqn{a+y_j-y_i} in year \eqn{y_j}, is the parent of \eqn{j}.
}

The parent is not observed in the HSP, so we sum the probabilities over all potential ages in year \eqn{y_i}.
\code{calc_HSP} is vectorized with respect to \code{yi} and \code{yj}.
}

\references{
Bravington, M.V. et al. 2016. Close-Kin Mark-Recapture. Stat. Sci. 31: 259-274.
\doi{10.1214/16-STS552}

Hillary, R.M. et al. 2018. Genetic relatedness reveals total population size of white sharks in eastern Australia and
New Zealand. Sci. Rep. 8: 2661. \doi{10.1038/s41598-018-20593-w}
}
\seealso{
\code{\link[=like_CKMR]{like_CKMR()}}
}
\author{
Q. Huynh with contribution from Y. Tsukahara (Fisheries Research Institute, Japan)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jitter.R
\name{do_jitter}
\alias{do_jitter}
\title{Jitter starting values from fitted model}
\usage{
do_jitter(
  x,
  n = 1,
  use_fitted = TRUE,
  return_models = TRUE,
  amount = NULL,
  cores = 1,
  seed,
  ...
)
}
\arguments{
\item{x}{\linkS4class{MSAassess} object returned by \code{\link[=fit_MSA]{fit_MSA()}}}

\item{n}{Integer, number of jittered model runs}

\item{use_fitted}{Logical, whether to jitter from estimated parameters (\code{TRUE}) or original starting values (\code{FALSE})}

\item{return_models}{Logical, whether to return fitted models of the jitter runs}

\item{amount}{Numeric or NULL, passed to \code{\link[base:jitter]{base::jitter()}}}

\item{cores}{Integer, number of CPUs for parallel processing}

\item{seed}{Integer, for replicating the sampling function. Optional.}

\item{...}{Other arguments to pass to \code{\link[=fit_MSA]{fit_MSA()}}}
}
\value{
If \code{return_models = TRUE}, a list (length \code{n}) containing \linkS4class{MSAassess} objects.
Otherwise, a data frame of likelihood components made by \code{\link[=get_likelihood_components]{get_likelihood_components()}}
}
\description{
Run additional model fits with jittered starting values.
}
\details{
The new starting parameters are: \code{pars + amount * runif(n, -1, 1)} where \code{pars} are either the fitted values or original starting
values depending on \code{use_fitted}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_int.R
\name{calc_recruitment}
\alias{calc_recruitment}
\title{Calculate recruitment from stock-recruit function}
\usage{
calc_recruitment(x, SRR = c("BH", "Ricker"), eq = FALSE, ...)
}
\arguments{
\item{x}{Numeric, either the spawning output or the equilibrium spawners per recruit, from which
the recruitment will be calculated. See argument \code{eq}.}

\item{SRR}{Character to indicate the functional form of the stock recruit function}

\item{eq}{Logical, indicates whether \code{x} is the spawning output (\code{FALSE}) or equilibrium spawners per recruit (\code{TRUE})}

\item{...}{Parameters of the SRR function. Provide one of two sets of variables:
\enumerate{
\item \code{h}, \code{R0} and \code{phi0}, or
\item \code{a} and \code{b} (alpha, beta values)
}}
}
\value{
Numeric of length \code{x}
}
\description{
Calculate recruitment from stock-recruit function
}
\examples{
calc_recruitment(10, SRR = "Ricker", a = 2, b = 0.5)
calc_recruitment(10, SRR = "Ricker", h = 0.9, R0 = 1, phi0 = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/like.R
\name{like_CKMR}
\alias{like_CKMR}
\title{Likelihood for CKMR}
\usage{
like_CKMR(n, m, p, type = c("binomial", "poisson"))
}
\arguments{
\item{n}{The number of pairwise comparisons}

\item{m}{The number of kinship matches}

\item{p}{The probability of kinship match}

\item{type}{The statistical distribution for the likelihood calculation}
}
\value{
Numeric representing the log-likelihood.
}
\description{
Returns the log-likelihood for a set of pairwise comparisons. For a parent-offspring pair, a comparison
is defined by the capture year of parent, capture age of parent, and birth year of offspring.
For a half-sibling pair, a comparison is defined by the cohort year of each sibling.
Binomial and Poisson distributions are supported (Conn et al. 2020).
}
\references{
Conn, P.B. et al. 2020. Robustness of close-kin mark-recapture estimators to dispersal
limitation and spatially varying sampling probabilities. Ecol. Evol. 10: 5558-5569. \doi{10.1002/ece3.6296}
}
\seealso{
\code{\link[=calc_POP]{calc_POP()}} \code{\link[=calc_HSP]{calc_HSP()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_int.R
\name{calc_growth}
\alias{calc_growth}
\title{Calculate von Bertalanffy length-at-age}
\usage{
calc_growth(
  Linf_s,
  K_s,
  t0_s,
  ns = length(Linf_s),
  nm = 4,
  ny = 20,
  a = seq(1, 10) - 1
)
}
\arguments{
\item{Linf_s}{Vector by stock \code{s} of asymptotic length}

\item{K_s}{Vector by stock \code{s} of the growth coefficient}

\item{t0_s}{Vector by \code{s} of the age at length zero.}

\item{ns}{Integer, number of stocks}

\item{nm}{Integer, number of seasons}

\item{ny}{Integer, number of years}

\item{a}{Integer vector of ages}
}
\value{
Array \verb{[y, m, a, s]}
}
\description{
Returns an array of length-at-age with seasonal dimension.
Useful for \linkS4class{MSAdata} inputs.
}
\examples{
len_ymas <- calc_growth(c(30, 40), c(0.4, 0.2), c(-1, -1))

# Calculate stock weight at age
a_s <- rep(1e-6, 2)
b_s <- c(3, 3.1)

ns <- length(a_s)
swt_ymas <- sapply(1:ns, function(s) {
  a_s[s]*len_ymas[, , , s]^b_s[s]
}, simplify = "array")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retrospective.R
\name{retrospective}
\alias{retrospective}
\alias{plot.MSAretro}
\alias{summary.MSAretro}
\alias{report.MSAretro}
\title{Retrospective analysis}
\usage{
retrospective(MSAassess, yret = 0:5, cores = 1)

\method{plot}{MSAretro}(
  x,
  var = c("S_yst", "R_yst", "F_yst", "log_rdev_yst", "VB_ymft"),
  s = 1,
  f = 1,
  ...
)

\method{summary}{MSAretro}(object, by = c("stock", "fleet"), ...)

\method{report}{MSAretro}(
  object,
  filename = "retro",
  dir = tempdir(),
  open_file = TRUE,
  render_args = list(),
  ...
)
}
\arguments{
\item{MSAassess}{\linkS4class{MSAassess} object}

\item{yret}{Vector specifying the years (positive integers and include zero) to remove for the retrospective analysis}

\item{cores}{Integer for the number of cores to use for parallel processing}

\item{x, object}{Output of \code{retrospective} function}

\item{var}{Character to indicate the metric, the item in the \code{MSAretro} list to be plotted. See details below.}

\item{s}{Integer for the stock index to plot}

\item{f}{Integer for the fleet index to plot}

\item{...}{Not used}

\item{by}{Character indicating whether to calculate to Mohn's rho on stock or fleet-based time series}

\item{filename}{Character string for the name of the markdown and HTML files.}

\item{dir}{The directory in which the markdown and HTML files will be saved.}

\item{open_file}{Logical, whether the HTML document is opened after it is rendered.}

\item{render_args}{List of arguments to pass to \code{\link[rmarkdown:render]{rmarkdown::render()}}.}
}
\value{
A \code{MSAretro} object containing a named lists of arrays generated by the retrospective analysis:
\itemize{
\item \code{S_yst} Spawning output array \verb{[y, s, t]} where \code{t} indexes the retrospective peel
\item \code{R_yst} Recruitment array \verb{[y, s, t]}
\item \code{F_yst} Apical fishing mortality \verb{[y, s, t]}
\item \code{VB_ymft} Vulnerable biomass available to each fishery \verb{[y, m, f, t]}
}

\code{plot} generic for MSAretro returns individual figures using base graphics.

\code{summary} generic for MSAretro returns a matrix of Mohn's rho.

\code{report} generic for MSAretro invisibly returns the output of \code{\link[rmarkdown:render]{rmarkdown::render()}}: character of the path of the rendered HTML markdown report.
}
\description{
Perform a retrospective analysis, successive removals of most recent years of data to evaluate
consistency in model estimates of biomass, recruitment, etc.

The \code{summary} method returns Mohn's rho and the plot method generates a markdown report.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_int.R
\name{calc_index}
\alias{calc_index}
\title{Calculate index at age}
\usage{
calc_index(
  N,
  Z,
  sel,
  na = dim(N)[1],
  nr = dim(N)[2],
  ns = dim(N)[3],
  ni = dim(sel)[2],
  samp = array(1, c(ni, nr, ns)),
  delta = rep(0, ni)
)
}
\arguments{
\item{N}{Stock abundance at the beginning of the time step. Array \verb{[a, r, s]}}

\item{Z}{Instantaneous total mortality. Array \verb{[a, r, s]}}

\item{sel}{Index selectivity. Array \verb{[a, i, s]}}

\item{na}{Integer, number of age classes}

\item{nr}{Integer, number of regions}

\item{ns}{Integer, number of stocks}

\item{ni}{Integer, number of indices}

\item{samp}{Boolean indicates which regions and stocks are sampled by the index. Array \verb{[i, r, s]}}

\item{delta}{Fraction of time step when the index samples the population. Vector by \code{i}. Set to a negative number (-1) to sample the average
population over the course of the time step, i.e. \code{N * (1 - exp(-Z))/Z}.}
}
\value{
Index at age. Array \verb{[a, i, s]}
}
\description{
For indices of abundance, the function calculates the numbers vulnerable to the survey.
}
\details{
The index is calculated as
\deqn{
I_{a,i,s} = v_{a,i,s} \sum_r N_{a,r,s} d_{a,r,s} \times \mathbb{1}(r \in R_i) \mathbb{1}(s \in S_i)
}

If the survey samples at a moment in time, then
\deqn{
d_{a,r,s} = \exp(-\delta_i Z_{a,r,s})
}

Otherwise, if the index samples the population over the duration of the time step, then
\deqn{
d_{a,r,s} = (1 - \exp(-Z_{a,r,s}))/Z_{a,r,s}
}

where \eqn{R_i} and \eqn{S_i} denote the regions and stocks, respectively, sampled by index \eqn{i}. For example,
\eqn{R_2 = 1} denotes that the second index of abundance only samples region 1. These are informed by array \code{samp} where
\code{samp[i, r, s] = 1} indicates that stock \code{s} in region \code{r} is sampled by index \code{i}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\name{summary}
\alias{summary}
\alias{summary,MSAassess-method}
\title{Summarize MSA model output}
\usage{
\S4method{summary}{MSAassess}(object, ...)
}
\arguments{
\item{object}{Output from MSA}

\item{...}{Not used}
}
\value{
Matrix
}
\description{
Report parameter estimates, gradients, and standard errors
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/like.R
\name{like_comp}
\alias{like_comp}
\title{Likelihood for composition vectors}
\usage{
like_comp(
  obs,
  pred,
  type = c("multinomial", "dirmult1", "dirmult2", "lognormal", "logitnormal"),
  N = sum(obs),
  theta,
  stdev
)
}
\arguments{
\item{obs}{A vector of observed values. Internally converted to proportions.}

\item{pred}{A vector of predicted values. Same length as \code{obs}. Internally converted to proportions.}

\item{type}{Character for the desired distribution}

\item{N}{Numeric, the sample size corresponding to \code{obs} for multinomial or Dirichlet multinomial distributions.}

\item{theta}{Numeric, the linear (\code{type = "dirmult1"}) or saturating (\code{type = "dirmult2"}) Dirichlet-multinomial parameter, respectively. See Thorson et al. (2017)}

\item{stdev}{Numeric or vectorized for \code{obs}, the likelihood standard deviation for lognormal distribution.}
}
\value{
Numeric representing the log-likelihood.
}
\description{
Returns the log-likelihood for composition data, e.g., length, age, or stock composition,
with various statistical distributions supported.
}
\details{
Observed and predicted vectors are internally converted to proportions.

For \code{type = "lognormal"} or \code{"logitnormal"}, zero observations are removed from the likelihood calculation.
}
\examples{
M <- 0.1
age <- seq(1:10)
pred <- exp(-M * age)
obs <- pred * rlnorm(10, sd = 0.05)
like_comp(obs, pred, N = 10, type = "multinomial")
like_comp(obs, pred, N = 100, type = "multinomial")
like_comp(obs, pred, N = 10, type = "dirmult1", theta = 1)
like_comp(obs, pred, N = 10, type = "dirmult1", theta = 20)
}
\references{
Thorson et al. 2017. Model-based estimates of effective sample size in stock assessment models using the
Dirichlet-multinomial distribution. Fish. Res. 192:84-93. \doi{10.1016/j.fishres.2016.06.005}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_population.R
\name{calc_init_population}
\alias{calc_init_population}
\title{Initial population projection}
\usage{
calc_init_population(
  ny = 10,
  nm = 4,
  na = 20,
  nf = 1,
  nr = 4,
  ns = 1,
  initN_ars = array(1, c(na, nr, ns)),
  condition = c("catch", "F"),
  C_mfr = array(0, c(nm, nf, nr)),
  F_mfr = array(0, c(nm, nf, nr)),
  sel_mafs = array(1, c(nm, na, nf, ns)),
  fwt_mafs = array(1, c(nm, na, nf, ns)),
  q_fs = matrix(1, nf, ns),
  M_as,
  mov_marrs,
  mat_as,
  fec_as,
  SRR_s,
  sralpha_s,
  srbeta_s,
  m_spawn = 1,
  m_advanceage = 1,
  delta_s = rep(0, ns),
  natal_rs = matrix(1, nr, ns),
  recdist_rs = matrix(1/nr, nr, ns),
  Fmax,
  nitF
)
}
\arguments{
\item{ny}{Integer, number of years for the projection}

\item{nm}{Integer, number of seasons}

\item{na}{Integer, number of age classes}

\item{nf}{Integer, number of fleets}

\item{nr}{Integer, number of regions}

\item{ns}{Integer, number of stocks}

\item{initN_ars}{Abundance in the first year, first season. Array \verb{[a, r, s]}}

\item{condition}{Whether the fishing mortality is conditioned on the catch or specified F argument.}

\item{C_mfr}{Equilibrium catch. Matrix \verb{[m, f, r]}}

\item{F_mfr}{Equilibrium fishing mortality (per season). Matrix \verb{[m, f, r]}}

\item{sel_mafs}{Selectivity by season, age, fleet, stock. Array \verb{[m, a, f, s]}}

\item{fwt_mafs}{Fishery weight array by season, age, fleet, stock. Array \verb{[m, a, r, r]}. Can be used
calculate yield per recruit.}

\item{q_fs}{Relative catchability of stock \code{s} for fleet \code{f}. Defaults to 1 if missing. Matrix \verb{[f, s]}}

\item{M_as}{Natural mortality. Matrix \verb{[a, s]}}

\item{mov_marrs}{Movement array \verb{[m, a, r, r, s]}. If missing, uses a diagonal matrix (no movement among areas).}

\item{mat_as}{Maturity at age. Matrix \verb{[a, s]}}

\item{fec_as}{Fecundity at age. Matrix \verb{[a, s]}}

\item{SRR_s}{Character vector by \code{s} for the stock recruit relationship. See \code{\link[=calc_recruitment]{calc_recruitment()}} for options}

\item{sralpha_s}{Numeric vector by \code{s} for the stock recruit alpha parameter}

\item{srbeta_s}{Numeric vector by \code{s} for the stock recruit beta parameter}

\item{m_spawn}{Integer, season of spawning}

\item{m_advanceage}{Integer, season at which to advance integer year age classes}

\item{delta_s}{Numeric vector by \code{s}. Fraction of season that elapses when spawning occurs, e.g., midseason spawning when \code{delta_s = 0.5}.}

\item{natal_rs}{Matrix \verb{[r, s]}. The fraction of the mature stock \code{s} in region \code{r} that spawns at
time of spawning. See example in \linkS4class{Dstock}.}

\item{recdist_rs}{Matrix \verb{[r, s]}. The fraction of the incoming recruitment of stock \code{s} that settles in region \code{r}.}

\item{Fmax}{Numeric, the maximum Findex value}

\item{nitF}{Integer, number of iterations for the Newton-Raphson routine}
}
\value{
A named list returned by \code{\link[=calc_population]{calc_population()}}.
}
\description{
Project a population forward in time with constant parameters (biology and F) with \code{\link[=calc_population]{calc_population()}},
an alternative to \code{\link[=calc_phi_project]{calc_phi_project()}} to establish initial age structure.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\docType{class}
\name{Dmodel-class}
\alias{Dmodel-class}
\title{Dmodel S4 object}
\description{
S4 class that organizes the various data inputs for the MSA model. \code{MSAdata} simply inherits the slots from 6 component classes:
\code{Dmodel}, \code{Dstock}, \code{Dfishery}, \code{Dsurvey} \code{DCKMR}, and \code{Dtag}, where the \code{D}- prefix denotes an object for data inputs (or model configuration).
}
\details{
For convenience, most arrays and matrices have the associated dimensions in the variable name. For example, \code{Cobs_ymfr} represents
observed catch with the dimension following the underscore, following this template:

\tabular{ll}{
\code{y} \tab Year \cr
\code{m} \tab Season \cr
\code{a} \tab Age \cr
\code{r} \tab Region \cr
\code{f} \tab Fishery \cr
\code{i} \tab Index \cr
\code{s} \tab Stock
}
}
\section{Slots inherited from Dmodel}{

\describe{
\item{\code{ny}}{Integer, number of years}
\item{\code{nm}}{Integer, number of seasons}
\item{\code{na}}{Integer, number of ages. The first age class is zero and the last age class (plus group is age \code{na - 1}).}
\item{\code{nl}}{Integer, number of length bins. Set to zero if lengths are not modeled.}
\item{\code{nr}}{Integer, number of spatial regions}
\item{\code{ns}}{Integer, number of stocks}
\item{\code{lbin}}{Vector of lower boundary of length bins. Length \code{nl + 1}}
\item{\code{lmid}}{Vector of midpoint of length bins. Length \code{nl}}
\item{\code{Fmax}}{Numeric, maximum allowable instantaneous fishing mortality rate (units of per season). Defaults to 3.}
\item{\code{y_phi}}{Integer, the year from which to obtain values of natural mortality and fecundity for the unfished stock-recruit replacement line (\code{phi}). Relevant if natural mortality or fecundity are time-varying. Defaults to 1.}
\item{\code{scale_s}}{Vector, length \code{ns}. Multiplicative scaling factor that informs relative stock size to aid parameter estimation. Larger values implies larger stocks. Default set to 1. See \code{\link[=make_parameters]{make_parameters()}}.}
\item{\code{nyinit}}{Integer, number of years of spool-up to calculate equilibrium unfished and starting conditions for the population model to account for seasonal and spatial dynamics. The numerical spool-up is not needed when both \code{nm = 1} and \code{nr = 1}, i.e., \code{nyinit = 1}. Otherwise, set to \code{2 * na} by default.}
\item{\code{condition}}{Character, either to specify the model estimates fishing mortality as a parameter (\code{"F"}, default) or equal to the catch (\code{"catch"}).}
\item{\code{nitF}}{Integer, number of iterations to solve Baranov catch equation from observed catch if \code{condition = "catch"}. Defaults to 5.}
\item{\code{y_Fmult_f}}{Integer vector by fleet, the year in which to directly estimate F. Choose a year/season/region combination when the catch is average relative to the time series. Only used if \code{condition = "F"}.}
\item{\code{m_Fmult_f}}{Integer vector by fleet, the season in which to directly estimate F. Choose a year/season/region combination when the catch is average relative to the time series. Only used if \code{condition = "F"}.}
\item{\code{r_Fmult_f}}{Integer vector by fleet, the region in which to directly estimate F. Choose a year/season/region combination when the catch is average relative to the time series. Only used if \code{condition = "F"}.}
\item{\code{pbc_rdev_ys}}{Numeric matrix, for the fraction of lognormal bias correction (\code{-0.5 * sd_r^2}) applied to the recruitment estimates in the model. Typically between 0-1, with default of 1.}
\item{\code{pbc_initrdev_as}}{Numeric matrix, for the fraction of lognormal bias correction (\code{-0.5 * sd_r^2}) applied to the initial abundance vector in the model. Typically between 0-1, with default of 1.}
\item{\code{prior}}{Character vector to be evaluated in the model to return the log prior for a parameter. See example in documentation for \link{prior}.}
\item{\code{nyret}}{Integer, number of recent years of data to remove from the likelihood for retrospective analysis (positive numbers). Default is zero.}
}
}

\seealso{
\link{MSAdata-class} \code{\link[=check_data]{check_data()}} \link{Dmodel-class} \link{Dstock-class} \link{Dfishery-class} \link{Dsurvey-class} \link{DCKMR-class} \link{Dtag-class}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{get_MSAdata}
\alias{get_MSAdata}
\title{Retrieve data object used to fit model}
\usage{
get_MSAdata(MSAassess)
}
\arguments{
\item{MSAassess}{\linkS4class{MSAassess} object returned by \code{fit_MSA()}}
}
\value{
\linkS4class{MSAdata} object
}
\description{
A convenient function to retrieve the data object used to fit the model. The object is embedded in an environment
within the RTMB object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_population.R
\name{calc_population}
\alias{calc_population}
\title{Multi-fleet, multi-area, multi-stock population dynamics model}
\usage{
calc_population(
  ny = 10,
  nm = 4,
  na = 20,
  nf = 1,
  nr = 4,
  ns = 2,
  initN_ars = array(1, c(na, nr, ns)),
  mov_ymarrs,
  M_yas = array(0.3, c(ny, na, ns)),
  SRR_s = rep("BH", ns),
  sralpha_s = rep(1e+16, ns),
  srbeta_s = rep(1e+16, ns),
  mat_yas = array(1, c(ny, na, ns)),
  fec_yas = array(1, c(ny, na, ns)),
  Rdev_ys = matrix(1, ny, ns),
  m_spawn = 1,
  m_advanceage = 1,
  delta_s = rep(0, ns),
  natal_rs = matrix(1, nr, ns),
  recdist_rs = matrix(1/nr, nr, ns),
  fwt_ymafs = array(1, c(ny, nm, na, nf, ns)),
  q_fs = matrix(1, nf, ns),
  sel_ymafs = array(1, c(ny, nm, na, nf, ns)),
  condition = c("F", "catch"),
  F_ymfr = array(0, c(ny, nm, nf, nr)),
  Cobs_ymfr = array(1e-08, c(ny, nm, nf, nr)),
  Fmax = 2,
  nitF = 5L
)
}
\arguments{
\item{ny}{Integer, number of years for the projection}

\item{nm}{Integer, number of seasons}

\item{na}{Integer, number of age classes}

\item{nf}{Integer, number of fleets}

\item{nr}{Integer, number of regions}

\item{ns}{Integer, number of stocks}

\item{initN_ars}{Abundance in the first year, first season. Array \verb{[a, r, s]}}

\item{mov_ymarrs}{Movement array \verb{[y, m, a, r, r, s]}. If missing, uses a diagonal matrix (no movement among areas).}

\item{M_yas}{Natural mortality (per year). Array \verb{[y, a, s]}}

\item{SRR_s}{Character vector by \code{s} for the stock recruit relationship. See \code{\link[=calc_recruitment]{calc_recruitment()}} for options}

\item{sralpha_s}{Numeric vector by \code{s} for the stock recruit alpha parameter}

\item{srbeta_s}{Numeric vector by \code{s} for the stock recruit beta parameter}

\item{mat_yas}{Maturity ogive. Array \verb{[y, a, s]}}

\item{fec_yas}{Fecundity schedule (spawning output of mature individuals). Array \verb{[y, a, s]}}

\item{Rdev_ys}{Recruitment deviations. Matrix \verb{[y, s]}}

\item{m_spawn}{Integer, season of spawning}

\item{m_advanceage}{Integer, season at which to advance integer year age classes}

\item{delta_s}{Numeric vector by \code{s}. Fraction of season that elapses when spawning occurs, e.g., midseason spawning when \code{delta_s = 0.5}.}

\item{natal_rs}{Matrix \verb{[r, s]}. The fraction of the mature stock \code{s} in region \code{r} that spawns at
time of spawning. See example in \linkS4class{Dstock}.}

\item{recdist_rs}{Matrix \verb{[r, s]}. The fraction of the incoming recruitment of stock \code{s} that settles in region \code{r}.}

\item{fwt_ymafs}{Fishery weight at age. Array \verb{[y, m, a, f, s]}}

\item{q_fs}{Relative catchability of stock \code{s} for fleet \code{f}. Defaults to 1 if missing. Matrix \verb{[f, s]}}

\item{sel_ymafs}{Fishery selectivity. Array \verb{[y, m, a, f, s]}}

\item{condition}{Whether the fishing mortality is conditioned on the catch or specified F argument.}

\item{F_ymfr}{Fishing mortality (per season). Array \verb{[y, m, f, r]}. Only used if \code{condition = "F"}.}

\item{Cobs_ymfr}{Fishery catch (weight). Array \verb{[y, m, f, r]}. Only used if \code{condition = "catch"} to solve for F (see \code{\link[=calc_F]{calc_F()}}).}

\item{Fmax}{Numeric, the maximum Findex value}

\item{nitF}{Integer, number of iterations for the Newton-Raphson routine}
}
\value{
A named list containing:
\itemize{
\item \code{N_ymars} Stock abundance
\item \code{F_ymars} Fishing mortality (summed across fleets)
\item \code{F_ymfr} Fishing mortality (by fleet and region)
\item \code{Z_ymars} Total mortality
\item \code{F_ymafrs} Fishing mortality (disaggregated by fleet)
\item \code{CN_ymafrs} Catch at age (abundance)
\item \code{CB_ymfrs} Fishery catch (weight)
\item \code{VB_ymfrs} Vulnerable biomass available to the fishing fleets
\item \code{Nsp_yars} Spawning abundance (in the spawning season)
\item \code{Npsp_yars} Potentail spawners, mature animals in the spawning season that do not spawn if outside natal regions
\item \code{S_yrs} Spawning output
\item \code{R_ys} Recruitment
\item \code{penalty} Numeric quadratic penalty if apical fishing mortality (by fleet) exceeds \code{Fmax}. See \code{\link[=calc_F]{calc_F()}}.
}
}
\description{
Project age-structured populations forward in time. Also used by \code{\link[=calc_phi_project]{calc_phi_project()}} to calculate
equilibrium abundance and biomass for which there is no analytic solution
due to seasonal movement.
}
\examples{
unfished_pop <- calc_population()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_population.R
\name{calc_phi_simple}
\alias{calc_phi_simple}
\alias{calc_NPR}
\title{Simple spawners per recruit calculation}
\usage{
calc_phi_simple(Z, fec_a, mat_a, delta = 0)

calc_NPR(Z, na = length(Z), plusgroup = TRUE)
}
\arguments{
\item{Z}{Total mortality at age}

\item{fec_a}{Fecundity at age. Vector}

\item{mat_a}{Maturity at age. Vector}

\item{delta}{Fraction of season that elapses when spawning occurs, e.g., midseason spawning when \code{delta = 0.5}.}

\item{na}{Integer, number of age classes}

\item{plusgroup}{Logical, whether the largest age class is a plusgroup accumulator age}
}
\value{
\code{\link[=calc_phi_simple]{calc_phi_simple()}} returns a numeric for the spawning biomass per recruit.

\code{\link[=calc_NPR]{calc_NPR()}} returns a numeric, numbers per recruit at age
}
\description{
Calculate spawners per recruit by individual stock. Appropriate for a population model with a single
spatial area and an annual time steps, i.e. single season.
}
\seealso{
\code{\link[=calc_phi_project]{calc_phi_project()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_int.R
\name{calc_nextN}
\alias{calc_nextN}
\title{Project stock abundance to the next time step}
\usage{
calc_nextN(
  N,
  surv,
  na = dim(N)[1],
  nr = dim(N)[2],
  ns = dim(N)[3],
  advance_age = TRUE,
  mov = array(1/nr, c(na, nr, nr, ns)),
  plusgroup = TRUE
)
}
\arguments{
\item{N}{Abundance at current time step. Array \verb{[a, r, s]}}

\item{surv}{Survival during the current time step. Array \verb{[a, r, s]}}

\item{na}{Integer, number of age classes}

\item{nr}{Integer, number of regions}

\item{ns}{Integer, number of stocks}

\item{advance_age}{Logical, whether the animals advance to their next age class}

\item{mov}{Movement array in the next time step. Array \verb{[a, r, r, s]}. Rows denote region of origin and columns denote region of destination.}

\item{plusgroup}{Logical, whether the last age class is an accumulator plus group.}
}
\value{
Abundance at the next time step. Array \verb{[a, r, s]}
}
\description{
This function applies survival of the current abundance, advances
age classes, re-distributes the stock using the movement matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_int.R
\name{calc_LAK}
\alias{calc_LAK}
\title{Length-at-age key}
\usage{
calc_LAK(len_a, sd_la, lbin, nl = length(lbin))
}
\arguments{
\item{len_a}{Vector of length-at-age}

\item{sd_la}{Vector of standard deviation in length-at-age}

\item{lbin}{Vector of the lower boundary of the length bins}

\item{nl}{Integer, number of length bins (default is \code{length(lbin)})}
}
\value{
Matrix by age (rows) and length (columns)
}
\description{
Calculates the probability distribution of length-at-age using the normal probability density function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\docType{class}
\name{Dfishery-class}
\alias{Dfishery-class}
\title{Dfishery S4 object}
\description{
S4 class that organizes the various data inputs for the MSA model. \code{MSAdata} simply inherits the slots from 6 component classes:
\code{Dmodel}, \code{Dstock}, \code{Dfishery}, \code{Dsurvey} \code{DCKMR}, and \code{Dtag}, where the \code{D}- prefix denotes an object for data inputs (or model configuration).
}
\details{
For convenience, most arrays and matrices have the associated dimensions in the variable name. For example, \code{Cobs_ymfr} represents
observed catch with the dimension following the underscore, following this template:

\tabular{ll}{
\code{y} \tab Year \cr
\code{m} \tab Season \cr
\code{a} \tab Age \cr
\code{r} \tab Region \cr
\code{f} \tab Fishery \cr
\code{i} \tab Index \cr
\code{s} \tab Stock
}
}
\section{Slots inherited from Dfishery}{

\describe{
\item{\code{nf}}{Integer, number of fleets}
\item{\code{Cobs_ymfr}}{Total fishery catch}
\item{\code{Csd_ymfr}}{Lognormal standard deviation of the fishery catch. Only used if \code{Dmodel@condition = "F"}. Default of 0.01.}
\item{\code{fwt_ymafs}}{Fishery weight at age. Set to 1 when fleet catch is in units of abundance. Set to stock weight at age by default (values at beginning of time step).}
\item{\code{CAAobs_ymafr}}{Fishery catch at age composition}
\item{\code{CALobs_ymlfr}}{Fishery catch at length composition}
\item{\code{fcomp_like}}{Character, likelihood for the fishery composition data. See \code{type} argument of \code{\link[=like_comp]{like_comp()}} for options}
\item{\code{CAAN_ymfr}}{Sample size of the catch at age vector by season if using the multinomial or Dirichlet-multinomial likelihoods}
\item{\code{CALN_ymfr}}{Sample size of the catch at length vector by season if using the multinomial or Dirichlet-multinomial likelihoods}
\item{\code{CAAtheta_f}}{Catch at age dispersion parameter if using the Dirichlet-multinomial likelihood. Default set to 1.}
\item{\code{CALtheta_f}}{Catch at length dispersion parameter if using the Dirichlet-multinomial likelihood. Default set to 1.}
\item{\code{sel_block_yf}}{Index of dummy fleets to model time blocks of selectivity}
\item{\code{sel_f}}{Character vector of the functional form for selectivity for each fleet. See \code{type} argument in \code{\link[=conv_selpar]{conv_selpar()}} for all options, for example, \verb{"logistic_length", "dome_length", "logistic_age", "dome_age", "total", "mature"}}
\item{\code{Cinit_mfr}}{Equilibrium seasonal catch prior to the first year. One way to initialize the abundance at the start of the first year
in the model. Default of zero.}
\item{\code{SC_ymafrs}}{Stock composition data.}
\item{\code{SC_aa}}{Boolean matrix that aggregates age classes for the stock composition data. See example.}
\item{\code{SC_ff}}{Boolean matrix that aggregates fleets for the stock composition data. See example.}
\item{\code{SC_like}}{Character, likelihood for the stock composition data. See \code{type} argument of \code{\link[=like_comp]{like_comp()}} for options}
\item{\code{SCN_ymafr}}{Sample size of the stock composition vector if using the multinomial or Dirichlet-multinomial likelihoods}
\item{\code{SCtheta_f}}{Stock composition dispersion parameter if using the Dirichlet-multinomial likelihood. Default set to 1.}
\item{\code{SCstdev_ymafrs}}{Stock composition standard deviation if using the lognormal likelihood. Default set to 0.1.}
\item{\code{lambdaCobs_f}}{Likelihood weight (by fleet) for the catch. Default is 1.}
\item{\code{lambdaCAA_f}}{Likelihood weight (by fleet) for the catch at age. Default is 1.}
\item{\code{lambdaCAL_f}}{Likelihood weight (by fleet) for the catch at length. Default is 1.}
\item{\code{lambdaSC_f}}{Likelihood weight (by fleet) for the stock composition. Default is 1.}
}
}

\examples{
# Aggregate stock composition for ages 1-4 and 5-10 across all fleets
na <- 10
na_SC <- 2
SC_aa <- matrix(0, na_SC, na) # Assumes dim(SC_ymafrs)[3] = na_SC
SC_aa[1, 1:4] <- SC_aa[2, 5:10] <- 1

nf <- 3
nf_SC <- 1
SC_ff <- matrix(1, nf_SC, nf) # Assumes dim(SC_ymafrs)[4] <- nf_SC

}
\seealso{
\link{MSAdata-class} \code{\link[=check_data]{check_data()}} \link{Dmodel-class} \link{Dstock-class} \link{Dfishery-class} \link{Dsurvey-class} \link{DCKMR-class} \link{Dtag-class}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_population.R
\name{calc_phi_project}
\alias{calc_phi_project}
\title{Equilibrium spawners per recruit by projection}
\usage{
calc_phi_project(
  ny,
  nm,
  na,
  nf = 1,
  nr,
  ns = 1,
  F_mfr = array(0, c(nm, nf, nr)),
  sel_mafs = array(1, c(nm, na, nf, ns)),
  fwt_mafs = array(1, c(nm, na, nf, ns)),
  q_fs = matrix(1, nf, ns),
  M_as,
  mov_marrs,
  mat_as,
  fec_as,
  m_spawn = 1,
  m_advanceage = 1,
  delta_s = rep(0, ns),
  natal_rs = matrix(1, nr, ns),
  recdist_rs = matrix(1/nr, nr, ns)
)
}
\arguments{
\item{ny}{Integer, number of years for the projection}

\item{nm}{Integer, number of seasons}

\item{na}{Integer, number of age classes}

\item{nf}{Integer, number of fleets}

\item{nr}{Integer, number of regions}

\item{ns}{Integer, number of stocks}

\item{F_mfr}{Equilibrium fishing mortality (per season). Matrix \verb{[m, f, r]}}

\item{sel_mafs}{Selectivity by season, age, fleet, stock. Array \verb{[m, a, f, s]}}

\item{fwt_mafs}{Fishery weight array by season, age, fleet, stock. Array \verb{[m, a, r, r]}. Can be used
calculate yield per recruit.}

\item{q_fs}{Relative catchability of stock \code{s} for fleet \code{f}. Defaults to 1 if missing. Matrix \verb{[f, s]}}

\item{M_as}{Natural mortality. Matrix \verb{[a, s]}}

\item{mov_marrs}{Movement array \verb{[m, a, r, r, s]}. If missing, uses a diagonal matrix (no movement among areas).}

\item{mat_as}{Maturity at age. Matrix \verb{[a, s]}}

\item{fec_as}{Fecundity at age. Matrix \verb{[a, s]}}

\item{m_spawn}{Integer, season of spawning}

\item{m_advanceage}{Integer, season at which to advance integer year age classes}

\item{delta_s}{Numeric vector by \code{s}. Fraction of season that elapses when spawning occurs, e.g., midseason spawning when \code{delta_s = 0.5}.}

\item{natal_rs}{Matrix \verb{[r, s]}. The fraction of the mature stock \code{s} in region \code{r} that spawns at
time of spawning. See example in \linkS4class{Dstock}.}

\item{recdist_rs}{Matrix \verb{[r, s]}. The fraction of the incoming recruitment of stock \code{s} that settles in region \code{r}.}
}
\value{
A named list returned by \code{\link[=calc_population]{calc_population()}}.
}
\description{
Project a population forward in time using \code{\link[=calc_population]{calc_population()}} with constant recruitment and
seasonal dynamics (growth, movement-by-season) to obtain per recruit parameters. Note that the fishing
mortality among fleets and stocks remain linked by matrix \code{q_fs}.
}
\details{
The initial population vector will be the survival at age evenly divided by the number of regions \code{nr}.
}
\seealso{
\code{\link[=calc_phi_simple]{calc_phi_simple()}} \code{\link[=calc_init_population]{calc_init_population()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{simulate}
\alias{simulate}
\alias{simulate,MSAassess-method}
\alias{simulate.MSAassess}
\title{Simulate data}
\usage{
\S4method{simulate}{MSAassess}(object, nsim = 1, seed = NULL, ...)
}
\arguments{
\item{object}{\linkS4class{MSAassess} object returned by \code{\link[=fit_MSA]{fit_MSA()}}}

\item{nsim}{Integer, number of simulations}

\item{seed}{Random number generator seed}

\item{...}{Not used}
}
\value{
A list of \code{nsim} length with data observations
}
\description{
Simulate data observations from fitted MSA model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\docType{class}
\name{Dtag-class}
\alias{Dtag-class}
\title{Dtag S4 object}
\description{
S4 class that organizes the various data inputs for the MSA model. \code{MSAdata} simply inherits the slots from 6 component classes:
\code{Dmodel}, \code{Dstock}, \code{Dfishery}, \code{Dsurvey} \code{DCKMR}, and \code{Dtag}, where the \code{D}- prefix denotes an object for data inputs (or model configuration).
}
\details{
For convenience, most arrays and matrices have the associated dimensions in the variable name. For example, \code{Cobs_ymfr} represents
observed catch with the dimension following the underscore, following this template:

\tabular{ll}{
\code{y} \tab Year \cr
\code{m} \tab Season \cr
\code{a} \tab Age \cr
\code{r} \tab Region \cr
\code{f} \tab Fishery \cr
\code{i} \tab Index \cr
\code{s} \tab Stock
}
}
\section{Slots inherited from Dtag}{

\describe{
\item{\code{tag_ymarrs}}{Array. Number of tags that move between regions. Informs movement matrices of stocks between time steps.}
\item{\code{tag_ymars}}{Array. Number of tags distributed among regions. Informs stock distribution (within time step).}
\item{\code{tag_yy}}{Boolean matrix that aggregates years for the tag data. Only used for the tag movement array \code{tag_ymarrs}.}
\item{\code{tag_aa}}{Boolean matrix that aggregates ages for the tag data.}
\item{\code{tag_like}}{Character. Likelihood for the tagging data, either the vector of proportions by region of origin for \code{tag_ymarrs}, or
by region of stock distribution for \code{tag_ymars}. See \code{type} argument of \code{\link[=like_comp]{like_comp()}} for options}
\item{\code{tagN_ymars}}{Array. Sample size of the tag movement vectors if using the multinomial or Dirichlet-multinomial likelihoods.}
\item{\code{tagN_ymas}}{Array. Sample size of the tag distribution vectors if using the multinomial or Dirichlet-multinomial likelihoods.}
\item{\code{tagtheta_s}}{Array. Tag dispersion parameter (by stock) if using the Dirichlet-multinomial likelihoods. Default set to 1.}
\item{\code{tagstdev_s}}{Array. Tag standard deviation (by stock) if using the lognormal likelihood. Default set to 0.1.}
\item{\code{lambdaTag_s}}{Array. Likelihood weight for the tag data (by stock). Default is 1.}
}
}

\seealso{
\link{MSAdata-class} \code{\link[=check_data]{check_data()}} \link{Dmodel-class} \link{Dstock-class} \link{Dfishery-class} \link{Dsurvey-class} \link{DCKMR-class} \link{Dtag-class}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_int.R
\name{calc_eqdist}
\alias{calc_eqdist}
\title{Equilibrium distribution from movement matrix}
\usage{
calc_eqdist(
  x,
  nm = dim(x)[1],
  nr = dim(x)[2],
  start = rep(1/nr, nr),
  m_start = 1L,
  nit = 20
)
}
\arguments{
\item{x}{Movement array \verb{[m, r, r]}. The second dimension corresponds to origin (sum to 1), and third dimension corresponds to destination}

\item{nm}{Number of seasons}

\item{nr}{Number of regions}

\item{start}{The initial distribution. Vector of length \code{nr}}

\item{m_start}{Integer, the season in which to apply the initial distribution and start the projection}

\item{nit}{Integer, the number of times the movement matrix will be applied}
}
\value{
Matrix by season and region \verb{[m, r]}
}
\description{
Applies the seasonal movement matrices several times in order to obtain the equilibrium spatial distribution over the course of a year.
Not used in the model but useful for reporting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{softmax}
\alias{softmax}
\title{Softmax function}
\usage{
softmax(eta, log = FALSE)
}
\arguments{
\item{eta}{Vector}

\item{log}{Logical, whether to return the value of the logarithm}
}
\value{
Numeric, vector length of \code{eta}: \eqn{\exp(\eta)/\sum\exp(\eta)}
}
\description{
Takes a vector of real numbers and returns the corresponding vector of probabilities
}
\details{
Uses \code{multiSA:::logspace.add} for numerical stability
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{posfun}
\alias{posfun}
\title{Quadratic penalty function}
\usage{
posfun(x, eps)
}
\arguments{
\item{x}{Numeric, the parameter}

\item{eps}{Numeric, the threshold below which a penalty will be applied}
}
\value{
The penalty value is

\deqn{
\textrm{penalty} =
\begin{cases}
0.1 (x - \varepsilon)^2 & x \le \varepsilon\\
0 & x > \varepsilon
\end{cases}
}

Numeric
}
\description{
Taped penalty function if \code{x < eps}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\docType{class}
\name{MSAdata-class}
\alias{MSAdata-class}
\title{MSAdata S4 object}
\description{
S4 class that organizes the various data inputs for the MSA model. \code{MSAdata} simply inherits the slots from 6 component classes:
\code{Dmodel}, \code{Dstock}, \code{Dfishery}, \code{Dsurvey} \code{DCKMR}, and \code{Dtag}, where the \code{D}- prefix denotes an object for data inputs (or model configuration).
}
\details{
For convenience, most arrays and matrices have the associated dimensions in the variable name. For example, \code{Cobs_ymfr} represents
observed catch with the dimension following the underscore, following this template:

\tabular{ll}{
\code{y} \tab Year \cr
\code{m} \tab Season \cr
\code{a} \tab Age \cr
\code{r} \tab Region \cr
\code{f} \tab Fishery \cr
\code{i} \tab Index \cr
\code{s} \tab Stock
}
}
\section{Slots}{

\describe{
\item{\code{Dmodel}}{Class \linkS4class{Dmodel} containing parameters for model structure (number of years, ages, etc.)}

\item{\code{Dstock}}{Class \linkS4class{Dstock} containing stock parameters (growth, natural mortality, etc.)}

\item{\code{Dfishery}}{Class \linkS4class{Dfishery} containing fishery data (catch, size and stock composition, etc.)}

\item{\code{Dsurvey}}{Class \linkS4class{Dsurvey} containing survey data (indices of abundance)}

\item{\code{DCKMR}}{Class \linkS4class{DCKMR} containing genetic close-kin data}

\item{\code{Dtag}}{Class \linkS4class{Dtag} containing tagging data}

\item{\code{Dlabel}}{Class \linkS4class{Dlabel} containing names for various dimensions. Used for plotting.}

\item{\code{Misc}}{List for miscellaneous inputs as needed}
}}

\section{Slots inherited from Dmodel}{

\describe{
\item{\code{ny}}{Integer, number of years}
\item{\code{nm}}{Integer, number of seasons}
\item{\code{na}}{Integer, number of ages. The first age class is zero and the last age class (plus group is age \code{na - 1}).}
\item{\code{nl}}{Integer, number of length bins. Set to zero if lengths are not modeled.}
\item{\code{nr}}{Integer, number of spatial regions}
\item{\code{ns}}{Integer, number of stocks}
\item{\code{lbin}}{Vector of lower boundary of length bins. Length \code{nl + 1}}
\item{\code{lmid}}{Vector of midpoint of length bins. Length \code{nl}}
\item{\code{Fmax}}{Numeric, maximum allowable instantaneous fishing mortality rate (units of per season). Defaults to 3.}
\item{\code{y_phi}}{Integer, the year from which to obtain values of natural mortality and fecundity for the unfished stock-recruit replacement line (\code{phi}). Relevant if natural mortality or fecundity are time-varying. Defaults to 1.}
\item{\code{scale_s}}{Vector, length \code{ns}. Multiplicative scaling factor that informs relative stock size to aid parameter estimation. Larger values implies larger stocks. Default set to 1. See \code{\link[=make_parameters]{make_parameters()}}.}
\item{\code{nyinit}}{Integer, number of years of spool-up to calculate equilibrium unfished and starting conditions for the population model to account for seasonal and spatial dynamics. The numerical spool-up is not needed when both \code{nm = 1} and \code{nr = 1}, i.e., \code{nyinit = 1}. Otherwise, set to \code{2 * na} by default.}
\item{\code{condition}}{Character, either to specify the model estimates fishing mortality as a parameter (\code{"F"}, default) or equal to the catch (\code{"catch"}).}
\item{\code{nitF}}{Integer, number of iterations to solve Baranov catch equation from observed catch if \code{condition = "catch"}. Defaults to 5.}
\item{\code{y_Fmult_f}}{Integer vector by fleet, the year in which to directly estimate F. Choose a year/season/region combination when the catch is average relative to the time series. Only used if \code{condition = "F"}.}
\item{\code{m_Fmult_f}}{Integer vector by fleet, the season in which to directly estimate F. Choose a year/season/region combination when the catch is average relative to the time series. Only used if \code{condition = "F"}.}
\item{\code{r_Fmult_f}}{Integer vector by fleet, the region in which to directly estimate F. Choose a year/season/region combination when the catch is average relative to the time series. Only used if \code{condition = "F"}.}
\item{\code{pbc_rdev_ys}}{Numeric matrix, for the fraction of lognormal bias correction (\code{-0.5 * sd_r^2}) applied to the recruitment estimates in the model. Typically between 0-1, with default of 1.}
\item{\code{pbc_initrdev_as}}{Numeric matrix, for the fraction of lognormal bias correction (\code{-0.5 * sd_r^2}) applied to the initial abundance vector in the model. Typically between 0-1, with default of 1.}
\item{\code{prior}}{Character vector to be evaluated in the model to return the log prior for a parameter. See example in documentation for \link{prior}.}
\item{\code{nyret}}{Integer, number of recent years of data to remove from the likelihood for retrospective analysis (positive numbers). Default is zero.}
}
}

\section{Slots inherited from Dstock}{

\describe{
\item{\code{m_spawn}}{Integer, season of spawning. Defaults to 1. The progeny will enter the age structure in the same season.}
\item{\code{m_advanceage}}{Integer, season in which to advance age classes (corresponding to calendar year) to the next age class. Defaults to 1.}
\item{\code{len_ymas}}{Length-at-age. Only needed if \code{Dmodel@nl > 0} to fit length composition (you may want to calculate the growth at the middle of the time step). \code{\link[=calc_growth]{calc_growth()}} may be a helpful function.}
\item{\code{sdlen_ymas}}{Standard deviation in length-at-age}
\item{\code{LAK_ymals}}{Length-at-age probability array. If empty, values will be calculated by \code{\link[=check_data]{check_data()}} with \code{\link[=calc_LAK]{calc_LAK()}}.}
\item{\code{mat_yas}}{Proportion mature by age class. Ignored if maturity ogive is estimated, e.g., when fitting to close-kin genetic data.}
\item{\code{swt_ymas}}{Stock weight-at-age (at beginning of time step). See \code{\link[=calc_growth]{calc_growth()}} example.}
\item{\code{fec_yas}}{Fecundity, i.e., spawning output, of mature animals. Default uses stock weight at age.}
\item{\code{M_yas}}{Natural mortality. Instantaneous units per year. Ignored if M is estimated.}
\item{\code{SRR_s}}{Character vector of stock-recruit relationship by stock. See \code{SRR} argument in \code{\link[=calc_recruitment]{calc_recruitment()}} for options.}
\item{\code{delta_s}}{Fraction of season that elapses when spawning occurs, e.g., midseason spawning occurs when \code{delta_s = 0.5}. Default is zero.}
\item{\code{presence_rs}}{Logical matrix indicating presence/absence of stock \code{s} in region \code{r}. Used to constrain movement matrix.
Default is TRUE for all stocks and regions.}
\item{\code{natal_rs}}{The fraction of the mature stock \code{s} in region \code{r} that spawns at
time of spawning. See example. Default is 1 for all stocks and regions.}
}
}

\section{Slots inherited from Dfishery}{

\describe{
\item{\code{nf}}{Integer, number of fleets}
\item{\code{Cobs_ymfr}}{Total fishery catch}
\item{\code{Csd_ymfr}}{Lognormal standard deviation of the fishery catch. Only used if \code{Dmodel@condition = "F"}. Default of 0.01.}
\item{\code{fwt_ymafs}}{Fishery weight at age. Set to 1 when fleet catch is in units of abundance. Set to stock weight at age by default (values at beginning of time step).}
\item{\code{CAAobs_ymafr}}{Fishery catch at age composition}
\item{\code{CALobs_ymlfr}}{Fishery catch at length composition}
\item{\code{fcomp_like}}{Character, likelihood for the fishery composition data. See \code{type} argument of \code{\link[=like_comp]{like_comp()}} for options}
\item{\code{CAAN_ymfr}}{Sample size of the catch at age vector by season if using the multinomial or Dirichlet-multinomial likelihoods}
\item{\code{CALN_ymfr}}{Sample size of the catch at length vector by season if using the multinomial or Dirichlet-multinomial likelihoods}
\item{\code{CAAtheta_f}}{Catch at age dispersion parameter if using the Dirichlet-multinomial likelihood. Default set to 1.}
\item{\code{CALtheta_f}}{Catch at length dispersion parameter if using the Dirichlet-multinomial likelihood. Default set to 1.}
\item{\code{sel_block_yf}}{Index of dummy fleets to model time blocks of selectivity}
\item{\code{sel_f}}{Character vector of the functional form for selectivity for each fleet. See \code{type} argument in \code{\link[=conv_selpar]{conv_selpar()}} for all options, for example, \verb{"logistic_length", "dome_length", "logistic_age", "dome_age", "total", "mature"}}
\item{\code{Cinit_mfr}}{Equilibrium seasonal catch prior to the first year. One way to initialize the abundance at the start of the first year
in the model. Default of zero.}
\item{\code{SC_ymafrs}}{Stock composition data.}
\item{\code{SC_aa}}{Boolean matrix that aggregates age classes for the stock composition data. See example.}
\item{\code{SC_ff}}{Boolean matrix that aggregates fleets for the stock composition data. See example.}
\item{\code{SC_like}}{Character, likelihood for the stock composition data. See \code{type} argument of \code{\link[=like_comp]{like_comp()}} for options}
\item{\code{SCN_ymafr}}{Sample size of the stock composition vector if using the multinomial or Dirichlet-multinomial likelihoods}
\item{\code{SCtheta_f}}{Stock composition dispersion parameter if using the Dirichlet-multinomial likelihood. Default set to 1.}
\item{\code{SCstdev_ymafrs}}{Stock composition standard deviation if using the lognormal likelihood. Default set to 0.1.}
\item{\code{lambdaCobs_f}}{Likelihood weight (by fleet) for the catch. Default is 1.}
\item{\code{lambdaCAA_f}}{Likelihood weight (by fleet) for the catch at age. Default is 1.}
\item{\code{lambdaCAL_f}}{Likelihood weight (by fleet) for the catch at length. Default is 1.}
\item{\code{lambdaSC_f}}{Likelihood weight (by fleet) for the stock composition. Default is 1.}
}
}

\section{Slots inherited from Dsurvey}{

\describe{
\item{\code{ni}}{Integer, number of indices of abundance. Zero is possible.}
\item{\code{Iobs_ymi}}{Observed indices}
\item{\code{Isd_ymi}}{Lognormal standard deviation of the observed indices}
\item{\code{unit_i}}{Character vector, units of the index. Set to \code{"B"} to use stock weight at age (default) or \code{"N"} for abundance (numbers).}
\item{\code{IAAobs_ymai}}{Array, survey age composition}
\item{\code{IALobs_ymli}}{Array, survey length composition}
\item{\code{icomp_like}}{Character, likelihood for the composition data. See \code{\link[=like_comp]{like_comp()}} for options}
\item{\code{IAAN_ymi}}{Array, sample size of the index age composition by season if using the multinomial or Dirichlet-multinomial likelihoods}
\item{\code{IALN_ymi}}{Array, sample size of the index length composition by season if using the multinomial or Dirichlet-multinomial likelihoods}
\item{\code{IAAtheta_i}}{Numeric vector, survey age composition dispersion parameter if using the Dirichlet-multinomial likelihood}
\item{\code{IALtheta_i}}{Numeric vector, index length composition dispersion parameter if using the Dirichlet-multinomial likelihood}
\item{\code{samp_irs}}{Boolean array that specifies the regions and stocks sampled by the index. \code{samp[i, r, s]} indicates whether index \code{i} operates in region \code{r} and catches stock \code{s}.}
\item{\code{sel_i}}{Character vector, functional forms for selectivity. See \code{"type"} argument in \code{\link[=conv_selpar]{conv_selpar()}} for options.}
\item{\code{delta_i}}{Numeric vector, the elapsed fraction of time in the seasonal time step (between 0 - 1) when the index samples the population. Set to a negative number (-1) to sample over the duration of the timestep, i.e., \code{(1 - exp(-Z))/Z}.}
\item{\code{qest_i}}{Vector, either a character \code{"est"} to allow the model to estimate the catchability for index \code{i} (default) or a numeric to fix the parameter.}
\item{\code{lambdaI_i}}{Likelihood weight (by survey) for the indices. Default is 1.}
\item{\code{lambdaIAA_i}}{Likelihood weight (by fleet) for the index age composition. Default is 1.}
\item{\code{lambdaIAL_i}}{Likelihood weight (by fleet) for the index length composition. Default is 1.}
}
}

\section{Slots inherited from DCKMR}{

\describe{
\item{\code{POP_s}}{A list by stock of data frames for parent-offspring pairs. Each row in the data frame corresponds to a "sampling unit" defined by the columns:
\tabular{ll}{
\code{a} \tab Capture year of parent \cr
\code{t} \tab Age at capture of parent \cr
\code{y} \tab Birth year of offspring \cr
\code{n} \tab Number of pairwise comparisons \cr
\code{m} \tab Number of POPs \cr
}
}
\item{\code{HSP_s}}{A list by stock of data frames for half-sibling pairs. Each row in the data frame corresponds to a "sampling unit" defined by the columns:
\tabular{ll}{
\code{yi} \tab Birth year of older sibling \cr
\code{yj} \tab Birth year of younger sibling \cr
\code{n} \tab Number of pairwise comparisons \cr
\code{m} \tab Number of HSPs
}
}
\item{\code{CKMR_like}}{Character, likelihood for the POP and HSP sampling units. See \code{type} argument in \code{\link[=like_CKMR]{like_CKMR()}} for options.}
}
}

\section{Slots inherited from Dtag}{

\describe{
\item{\code{tag_ymarrs}}{Array. Number of tags that move between regions. Informs movement matrices of stocks between time steps.}
\item{\code{tag_ymars}}{Array. Number of tags distributed among regions. Informs stock distribution (within time step).}
\item{\code{tag_yy}}{Boolean matrix that aggregates years for the tag data. Only used for the tag movement array \code{tag_ymarrs}.}
\item{\code{tag_aa}}{Boolean matrix that aggregates ages for the tag data.}
\item{\code{tag_like}}{Character. Likelihood for the tagging data, either the vector of proportions by region of origin for \code{tag_ymarrs}, or
by region of stock distribution for \code{tag_ymars}. See \code{type} argument of \code{\link[=like_comp]{like_comp()}} for options}
\item{\code{tagN_ymars}}{Array. Sample size of the tag movement vectors if using the multinomial or Dirichlet-multinomial likelihoods.}
\item{\code{tagN_ymas}}{Array. Sample size of the tag distribution vectors if using the multinomial or Dirichlet-multinomial likelihoods.}
\item{\code{tagtheta_s}}{Array. Tag dispersion parameter (by stock) if using the Dirichlet-multinomial likelihoods. Default set to 1.}
\item{\code{tagstdev_s}}{Array. Tag standard deviation (by stock) if using the lognormal likelihood. Default set to 0.1.}
\item{\code{lambdaTag_s}}{Array. Likelihood weight for the tag data (by stock). Default is 1.}
}
}

\section{Slots inherited from Dlabel}{

\describe{
\item{\code{year}}{Vector of years. Length \code{Dmodel@ny}}
\item{\code{season}}{Vector of season names. Length \code{Dmodel@nm}}
\item{\code{age}}{Vector of ages. Length \code{Dmodel@na}}
\item{\code{region}}{Vector of region names. Length \code{Dmodel@nr}}
\item{\code{stock}}{Vector of stock names. Length \code{Dmodel@ns}}
\item{\code{fleet}}{Vector of fleet names. Length \code{Dfishery@nf}}
\item{\code{index}}{Vector of index of abundance names. Length \code{Dsurvey@ni}}
}
}

\examples{
# Set natal_rs matrix so that the spawning output of stock 1 is
# calculated from mature animals present in regions 1, 2.
# Similarly for stock 2, spawning output from areas 2 and 3.
nr <- 4
ns <- 2
natal_rs <- matrix(0, nr, ns)
natal_rs[1:2, 1] <- natal_rs[2:3, 2] <- 1

# Aggregate stock composition for ages 1-4 and 5-10 across all fleets
na <- 10
na_SC <- 2
SC_aa <- matrix(0, na_SC, na) # Assumes dim(SC_ymafrs)[3] = na_SC
SC_aa[1, 1:4] <- SC_aa[2, 5:10] <- 1

nf <- 3
nf_SC <- 1
SC_ff <- matrix(1, nf_SC, nf) # Assumes dim(SC_ymafrs)[4] <- nf_SC

}
\seealso{
\link{MSAdata-class} \code{\link[=check_data]{check_data()}} \link{Dmodel-class} \link{Dstock-class} \link{Dfishery-class} \link{Dsurvey-class} \link{DCKMR-class} \link{Dtag-class}
}
\keyword{MSAdata}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resid.R
\name{residuals}
\alias{residuals}
\alias{residuals,MSAassess-method}
\title{Calculate model residuals}
\usage{
\S4method{residuals}{MSAassess}(object, vars, type = c("response", "pearson"), ...)
}
\arguments{
\item{object}{\linkS4class{MSAassess} object returned by \code{fit_MSA()}}

\item{vars}{Character vector to indicate which residuals will be calculated.
Available choices from \linkS4class{MSAdata} object are:
"Cinit_mfr", "Cobs_ymfr", "CAAobs_ymafr", "CALobs_ymlfr",
"Iobs_ymi", "IAAobs_ymai", "IALobs_ymli", "SC_ymafrs"}

\item{type}{Character, 'response' for the \code{log(observed/predicted)} values or 'pearson' for calculating Z-scores.
Composition data always use 'pearson'.}

\item{...}{Not used}
}
\value{
A named list based on \code{vars} argument.
}
\description{
Extract residuals from fitted model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior.R
\name{prior}
\alias{prior}
\alias{prior_h}
\alias{prior_M}
\alias{prior_q}
\title{Priors for MSA model}
\usage{
prior_h(MSAdata, s = 1, m, stdev)

prior_M(MSAdata, s = 1, meanlog, sdlog)

prior_q(MSAdata, i = 1, meanlog, sdlog)
}
\arguments{
\item{MSAdata}{Data object. Class \linkS4class{MSAdata}}

\item{s}{Integer for stock}

\item{m}{Mean in un-transformed space}

\item{stdev}{Standard deviation in un-transformed space}

\item{meanlog}{Mean of the lognormal distribution on the log scale}

\item{sdlog}{Standard of the lognormal distribution on the log scale}

\item{i}{Integer for the corresponding index}
}
\value{
Character.
}
\description{
Priors in MSA are set by providing character strings which are then parsed into an expression and evaluated in the model environment
(see example). This provides flexibility to set a prior for any desired model parameter or variable. See list of parameters in the
documentation for \verb{[check_parameters()]} for options (note that priors for \code{log_rdev_ys} and \code{log_initrdev_as} are not needed as they're hard-coded into the model).
Several functions below generate the character string for the prior for important dynamics parameters, such as natural mortality and steepness.
}
\details{
\itemize{
\item \code{prior_h} returns the log prior for stock-recruit steepness. Beta distribution for the Beverton-Holt function and normal distribution for Ricker function.
}

\itemize{
\item \code{prior_M} returns the log prior for natural mortality. Lognormal distribution.
}

\itemize{
\item \code{prior_q} returns the log prior for index catchability. Lognormal distribution.
}
}
\examples{
# Add M and steepness prior to model

dat <- new("MSAdata")
dat@Dmodel@ns <- 1
dat@Dstock@SRR_s <- "BH"

pr_M <- prior_M(dat, s = 1, log(0.25), 0.3)
pr_h <- prior_h(dat, s = 1, 0.8, 0.15)
dat@Dmodel@prior <- c(pr_M, pr_h)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\docType{class}
\name{Dlabel-class}
\alias{Dlabel-class}
\title{Dlabel S4 object}
\description{
Vectors for labeling plots.
}
\section{Slots inherited from Dlabel}{

\describe{
\item{\code{year}}{Vector of years. Length \code{Dmodel@ny}}
\item{\code{season}}{Vector of season names. Length \code{Dmodel@nm}}
\item{\code{age}}{Vector of ages. Length \code{Dmodel@na}}
\item{\code{region}}{Vector of region names. Length \code{Dmodel@nr}}
\item{\code{stock}}{Vector of stock names. Length \code{Dmodel@ns}}
\item{\code{fleet}}{Vector of fleet names. Length \code{Dfishery@nf}}
\item{\code{index}}{Vector of index of abundance names. Length \code{Dsurvey@ni}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectivity.R
\name{conv_selpar}
\alias{conv_selpar}
\alias{calc_sel_len}
\alias{calc_fsel_age}
\alias{calc_isel_age}
\title{Selectivity at age and length}
\usage{
conv_selpar(x, type, maxage, maxL)

calc_sel_len(sel_par, lmid, type, fsel_type, fsel_len)

calc_fsel_age(
  sel_len,
  LAK,
  type,
  sel_par,
  sel_block = seq(1, length(type)),
  mat,
  a = seq(1, nrow(LAK)) - 1
)

calc_isel_age(
  sel_len,
  LAK,
  type,
  sel_par,
  fsel_age,
  maxage,
  mat,
  a = seq(1, nrow(LAK)) - 1,
  fsel_type,
  fsel_len
)
}
\arguments{
\item{x}{Estimated parameters. Matrix \verb{[3, f]}}

\item{type, fsel_type}{Character string to indicate the functional form of selectivity. See details below.}

\item{maxage}{Maximum value of the age of full selectivity}

\item{maxL}{Maximum value of the length of full selectivity}

\item{sel_par}{Matrix of parameters returned by \code{\link[=conv_selpar]{conv_selpar()}}}

\item{lmid}{Midpoints of length bins for calculating selectivity at length}

\item{fsel_len}{Selectivity at length matrix for fleets, returned by previous call to \code{calc_sel_len()}}

\item{sel_len}{Selectivity at length matrix returned by \code{\link[=calc_sel_len]{calc_sel_len()}}}

\item{LAK}{Length-at-age probability matrix. Matrix \verb{[a, length(lmid)]}}

\item{sel_block}{Integer vector. Length \code{length(type)}. See details below.}

\item{mat}{Maturity at age vector}

\item{a}{Integer vector of ages corresponding to the rows of \code{LAK} (as well as \code{mat})}

\item{fsel_age}{Matrix returned by \code{\link[=calc_fsel_age]{calc_fsel_age()}}}
}
\value{
\code{\link[=conv_selpar]{conv_selpar()}} returns a matrix of the same dimensions as \code{x}.

\code{\link[=calc_sel_len]{calc_sel_len()}} returns a matrix \verb{[l, f]}, i.e., \verb{[length(lmid), length(type)]}.

\code{\link[=calc_fsel_age]{calc_fsel_age()}} returns a matrix \verb{[a, f]}, i.e., \verb{[a, length(sel_block)]}

\code{\link[=calc_isel_age]{calc_isel_age()}} returns a matrix \verb{[a, i]}, i.e., \verb{[a, length(type)]}
}
\description{
Calculate selectivity at age and length from a matrix of parameters.
\itemize{
\item \code{\link[=conv_selpar]{conv_selpar()}} converts parameters from log or logit space to real units.
\item \code{\link[=calc_sel_len]{calc_sel_len()}} calculates selectivity at length.
\item \code{\link[=calc_fsel_age]{calc_fsel_age()}} calculates selectivity at age for fisheries, and coordinates dummy fleets.
\item \code{\link[=calc_isel_age]{calc_isel_age()}} calculates selectivity at age for indices, and can map selectivity from fisheries
or population parameters (e.g, mature or total biomass).
}
}
\details{
Options for argument \code{type} include:
\itemize{
\item functional forms with respect to length: \verb{"logistic_length", "dome_length"}
\item functional forms with respect to age: \verb{"logistic_age", "dome_age"}
\item for surveys, an integer (\code{f}) to map index selectivity at age to fleet \code{f} (will be coerced to integer)
\item \code{"mature"} to fix to maturity at age schedule (\code{"SB"} is also acceptable for backwards compatibility)
\item \code{"total"} to fix selectivity to 1 for all ages (\code{"B"} is also acceptable for backwards compatibility)
\item \code{"length_x_y"} to specify selectivity to 1 between lengths \code{x} and \code{y} (example: \code{"length_20_50"})
\item \code{"age_x_y"} to specify selectivity to 1 between age \code{x} and \code{y} (example: \code{"age_2_4"})
\item for surveys, \code{"f_x_y"} uses selectivity values from fleet \code{f} between bins \code{x} and \code{y}
(either length or age depending on definition of selectivity for \code{f}) (example: \code{"3_2_4"})
}
}
\section{Converting selectivity parameters (conv_selpar)}{

The first row of \code{x} corresponds to the length or age of full selectivity: \eqn{\mu_f = L_{max}/(1 + \exp(-x_{1,f}))}

The second row of \code{x} corresponds to the width of the ascending limb for selectivity:
\eqn{\sigma_f^{asc} = \exp(x_{2,f})}

The third row of \code{x} corresponds to the width of the descending limb for selectivity (if dome-shaped):
\eqn{\sigma_f^{des} = \exp(x_{3,f})}
}

\section{Length selectivity (calc_sel_len)}{

The selectivity at length is
\deqn{
s_{\ell} =
\begin{cases}
\exp(\alpha) & L_{\ell} < \mu_f\\
\exp(\beta) & L_{\ell} \ge \mu_f\\
\end{cases}
}
where
\eqn{
\alpha = -0.5(L_\ell - \mu_f)^2/(\sigma_f^{asc})^2
}
and
\eqn{
\beta = -0.5(L_\ell - \mu_f)^2/(\sigma_f^{des})^2
}
}

\section{Age selectivity (calc_fsel_age)}{

The equivalent selectivity at age is converted from the length values (\code{sel_len}) as
\deqn{
s_a = \sum_\ell s_\ell \times \textrm{Prob}(L_{\ell}|a)
}

If selectivity is explicitly in age units, then the values are directly calculated
from parameters in \code{sel_par}.

Vector \code{sel_block} assigns the output selectivity from a different column of the input matrix
and facilitates time-varying selectivity in time blocks. For example, \code{sel_block[1] <- 2} means
that selectivity values in the first column of the output is based on the second column of the
input matrices (\code{sel_len[, 2]} or \code{sel_par[, 2]}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\docType{class}
\name{Dstock-class}
\alias{Dstock-class}
\title{Dstock S4 object}
\description{
S4 class that organizes the various data inputs for the MSA model. \code{MSAdata} simply inherits the slots from 6 component classes:
\code{Dmodel}, \code{Dstock}, \code{Dfishery}, \code{Dsurvey} \code{DCKMR}, and \code{Dtag}, where the \code{D}- prefix denotes an object for data inputs (or model configuration).
}
\details{
For convenience, most arrays and matrices have the associated dimensions in the variable name. For example, \code{Cobs_ymfr} represents
observed catch with the dimension following the underscore, following this template:

\tabular{ll}{
\code{y} \tab Year \cr
\code{m} \tab Season \cr
\code{a} \tab Age \cr
\code{r} \tab Region \cr
\code{f} \tab Fishery \cr
\code{i} \tab Index \cr
\code{s} \tab Stock
}
}
\section{Slots inherited from Dstock}{

\describe{
\item{\code{m_spawn}}{Integer, season of spawning. Defaults to 1. The progeny will enter the age structure in the same season.}
\item{\code{m_advanceage}}{Integer, season in which to advance age classes (corresponding to calendar year) to the next age class. Defaults to 1.}
\item{\code{len_ymas}}{Length-at-age. Only needed if \code{Dmodel@nl > 0} to fit length composition (you may want to calculate the growth at the middle of the time step). \code{\link[=calc_growth]{calc_growth()}} may be a helpful function.}
\item{\code{sdlen_ymas}}{Standard deviation in length-at-age}
\item{\code{LAK_ymals}}{Length-at-age probability array. If empty, values will be calculated by \code{\link[=check_data]{check_data()}} with \code{\link[=calc_LAK]{calc_LAK()}}.}
\item{\code{mat_yas}}{Proportion mature by age class. Ignored if maturity ogive is estimated, e.g., when fitting to close-kin genetic data.}
\item{\code{swt_ymas}}{Stock weight-at-age (at beginning of time step). See \code{\link[=calc_growth]{calc_growth()}} example.}
\item{\code{fec_yas}}{Fecundity, i.e., spawning output, of mature animals. Default uses stock weight at age.}
\item{\code{M_yas}}{Natural mortality. Instantaneous units per year. Ignored if M is estimated.}
\item{\code{SRR_s}}{Character vector of stock-recruit relationship by stock. See \code{SRR} argument in \code{\link[=calc_recruitment]{calc_recruitment()}} for options.}
\item{\code{delta_s}}{Fraction of season that elapses when spawning occurs, e.g., midseason spawning occurs when \code{delta_s = 0.5}. Default is zero.}
\item{\code{presence_rs}}{Logical matrix indicating presence/absence of stock \code{s} in region \code{r}. Used to constrain movement matrix.
Default is TRUE for all stocks and regions.}
\item{\code{natal_rs}}{The fraction of the mature stock \code{s} in region \code{r} that spawns at
time of spawning. See example. Default is 1 for all stocks and regions.}
}
}

\examples{
# Set natal_rs matrix so that the spawning output of stock 1 is
# calculated from mature animals present in regions 1, 2.
# Similarly for stock 2, spawning output from areas 2 and 3.
nr <- 4
ns <- 2
natal_rs <- matrix(0, nr, ns)
natal_rs[1:2, 1] <- natal_rs[2:3, 2] <- 1

}
\seealso{
\link{MSAdata-class} \code{\link[=check_data]{check_data()}} \link{Dmodel-class} \link{Dstock-class} \link{Dfishery-class} \link{Dsurvey-class} \link{DCKMR-class} \link{Dtag-class}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_int.R
\name{calc_F}
\alias{calc_F}
\title{Newton-Raphson search for fishing mortality}
\usage{
calc_F(
  Cobs,
  N,
  sel,
  wt,
  M,
  q_fs,
  delta = 1,
  na = dim(N)[1],
  nr = dim(N)[2],
  ns = dim(N)[3],
  nf = length(Cobs),
  Fmax = 2,
  nitF = 5L,
  trans = c("log", "logit")
)
}
\arguments{
\item{Cobs}{Observed catch. Matrix \verb{[f, r]}}

\item{N}{Stock abundance at the beginning of the time step. Array \verb{[a, r, s]}}

\item{sel}{Selectivity. Array \verb{[a, f, s]}}

\item{wt}{Fishery weight at age. Array \verb{[a, f, s]}}

\item{M}{Instantaneous natural mortality. Units of per year \verb{[a, s]}}

\item{q_fs}{Relative catchability of stock \code{s} for fleet \code{f}. Defaults to 1 if missing. Matrix \verb{[f, s]}}

\item{delta}{Numeric, the duration of time in years corresponding to the observed catch, e.g., 0.25 is a quarterly time step.}

\item{na}{Integer, number of age classes}

\item{nr}{Integer, number of regions}

\item{ns}{Integer, number of stocks}

\item{nf}{Integer, number of fleets}

\item{Fmax}{Numeric, the maximum Findex value}

\item{nitF}{Integer, number of iterations for the Newton-Raphson routine}

\item{trans}{Whether to perform the search in log or logit space}
}
\value{
A list containing:
\itemize{
\item \code{F_afrs} Fishing mortality array
\item \code{F_ars} Fishing mortality array (summed across fleets)
\item \code{Z_ars} Total mortality array
\item \code{F_index} Index of fishing mortality. Matrix \verb{[f, r]}
\item \code{CB_frs} Catch (biomass) array
\item \code{CN_afrs} Catch (abundance) array
\item \code{VB_afrs} Vulnerable biomass at the beginning of the time step. Array
\item \code{penalty} Penalty term returned by \code{\link[=posfun]{posfun()}} when \code{F_index} exceeds \code{Fmax}
\item \code{fn} Difference between predicted and observed catch at the last iteration. Matrix \verb{[f, r]}
\item \code{gr} Gradient of \code{fn} with respect to \code{F_index} in either log or logit space at the last iteration. Vector by \verb{[f, r]}
}
}
\description{
Performs a root finding routine to find the index of F that minimizes the difference between
observed catch and the value predicted by the Baranov equation.
}
\details{
The predicted catch for fleet \code{f} in region \code{r} is
\deqn{
C^{\textrm{pred}}_{f,r} = \sum_s \sum_a v_{a,f,s} q_{f,s} F_{f,r} \dfrac{1 - \exp(-Z_{a,r,s})}{Z_{a,r,s}} N_{a,r,s} w_{a,f,s}
}

The Newton-Raphson routine minimizes \eqn{f(x_{f,r}) = C_{f,r}^{\textrm{pred}} - C_{f,r}^{\textrm{obs}}}.

If \code{trans = "log"}, \eqn{F_{f,r} = \exp(x_{f,r})}.

If \code{trans = "logit"}, \eqn{F_{f,r} = F_{\textrm{max}}/(1 + \exp(x_{f,r}))}.

The gradient with respect to \eqn{\vec{x}} is
\deqn{
f'(x_{f,r}) = \sum_s \sum_a v_{a,f,s} q_{f,s} N_{a,r,s} w_{a,f,s} \left(\dfrac{\alpha\gamma}{\beta}\right)'
}

\deqn{
\left(\dfrac{\alpha\gamma}{\beta}\right)' = \dfrac{(\alpha\gamma' + \alpha'\gamma)\beta - \alpha\gamma\beta'}{\beta^2}
}

where

\tabular{l}{
\eqn{\alpha_{f,r} = F_{f,r}} \cr
\eqn{\beta_{a,r,s} = Z_{a,r,s} = M_{a,s} + \sum_f v_{a,f,s} q_{f,s} F_{f,r}} \cr
\eqn{\gamma_{a,r,s} = 1 - \exp(-Z_{a,r,s})} \cr
\eqn{\beta'_{a,f,r,s} = v_{a,f} q_{f,s} \alpha'_{f,r}} \cr
\eqn{\gamma'_{a,f,r,s} = \exp(-Z_{a,r,s})\beta'_{a,f,r,s}}
}

If \code{trans = "log"}, \eqn{\alpha'_{f,r} = \alpha_{f,r}}.

If \code{trans = "logit"}, \eqn{\alpha'_{f,r} = F_{\textrm{max}}\exp(-x_{f,r})/(1 + \exp(-x_{f,r}))^2}.

This function solves for \eqn{\vec{x}} by iterating until \eqn{f(\vec{x})} approaches zero, where the vector arrow
indexes over fleet and region. In iteration \eqn{i+1}:
\deqn{\vec{x}_{i+1} = \vec{x}_i - \dfrac{f(\vec{x}_i)}{f'(\vec{x}_i)}}.
}
\author{
Q. Huynh
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_int.R
\name{conv_mov}
\alias{conv_mov}
\title{Calculate movement matrix for all age classes}
\usage{
conv_mov(x, g, v, na = dim(x)[1], nr = dim(x)[2], aref = ceiling(0.5 * na))
}
\arguments{
\item{x}{Base log-movement parameters. See details. Array \verb{[a, r, r]}}

\item{g}{Gravity model attractivity term. Tendency to move to region \code{r}. Matrix \verb{[a, r]}}

\item{v}{Gravity model viscosity term. Tendency to stay in same region. Vector by \code{a}}

\item{na}{Integer, number of ages}

\item{nr}{Integer, number of regions}

\item{aref}{Integer, reference age class}
}
\value{
Movement array \verb{[a, r, r]}
}
\description{
Movement matrices are calculated for all age classes from a base matrix and a gravity model formulation
(Carruthers et al. 2016).
}
\details{
Rows index region of origin and columns denote region of destination.

In log space, the movement matrix \eqn{m_a} for age class \eqn{a} from region \eqn{r} to \eqn{r'} is the sum of base matrix \eqn{x} and
gravity matrix \eqn{G}:
\deqn{m_{a,r,r'} = x_{a,r,r'} + G_{a,r,r'}}

To essentially exclude movement from \eqn{r} to \eqn{r'}, set \eqn{x_{a,r,r'} = -1000}.

Gravity matrix \eqn{G} includes an attractivity term \eqn{g} and viscosity term \eqn{v}:

\deqn{G_{a,r,r'} =
\begin{cases}
g'_{a,r'} + v_a \quad & r = r'\\
g'_{a,r'} \quad & \textrm{otherwise}
\end{cases}
}

Vector \eqn{g'} are offset terms relative to the value for the reference age class:
\deqn{g'_{a,r'} =
\begin{cases}
g_{a,r} \quad & a = a_{ref}\\
g_{a,r} + g_{a=aref,r} \quad & \textrm{otherwise}
\end{cases}
}

The movement matrix in normal space is obtained by the softmax transformation:
\deqn{M_{a,r,r'} = \dfrac{\exp(m_{a,r,r'})}{\sum_{r'}\exp(m_{a,r,r'})}}

If \eqn{x} and \eqn{v} are zero, then the movement matrix simply distributes the total stock
abundance into the various regions as specified in \eqn{g'}.
}
\references{
Carruthers, T.R., et al. 2015. Modelling age-dependent movement: an application to red and
gag groupers in the Gulf of Mexico. CJFAS 72: 1159-1176. \doi{10.1139/cjfas-2014-0471}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report-int-data.R
\name{plot-MSA-data}
\alias{plot-MSA-data}
\alias{plot_catch}
\alias{plot_index}
\alias{plot_CAA}
\alias{plot_CAL}
\alias{plot_IAA}
\alias{plot_IAL}
\alias{plot_SC}
\alias{plot_tagmov}
\title{Plotting functions for data in MSA model}
\usage{
plot_catch(
  fit,
  f = 1,
  by = c("region", "stock"),
  prop = FALSE,
  annual = FALSE,
  figure = TRUE
)

plot_index(fit, i = 1, zoom = FALSE, figure = TRUE)

plot_CAA(fit, f = 1, r = 1, do_mean = FALSE, figure = TRUE)

plot_CAL(fit, f, r, agg = FALSE, do_mean = FALSE, figure = TRUE)

plot_IAA(fit, i = 1, do_mean = FALSE, figure = TRUE)

plot_IAL(fit, i = 1, do_mean = FALSE, figure = TRUE)

plot_SC(fit, ff = 1, aa = 1, r = 1, prop = FALSE, figure = TRUE)

plot_tagmov(fit, s = 1, yy = 1, aa = 1, figure = TRUE)
}
\arguments{
\item{fit}{\linkS4class{MSAassess} object returned by \code{\link[=fit_MSA]{fit_MSA()}}}

\item{f}{Integer, indexes the fleet}

\item{by}{Character to indicate dimension for multivariate plots}

\item{prop}{Logical, whether to plot proportions (TRUE) or absolute numbers}

\item{annual}{Logical, whether to plot annual values (summed over seasons)}

\item{figure}{Logical, whether to generate the plot}

\item{i}{Integer, indexes the survey}

\item{zoom}{Logical, for \code{plot_index()}. If \code{TRUE}, plots a subset of years with observed data points. Otherwise, plots
predicted values over all model years.}

\item{r}{Integer, indexes the region}

\item{do_mean}{Logical, whether to plot full compositions or time series of mean length or mean age}

\item{agg}{Logical, whether to aggregate composition across all time steps}

\item{ff}{Integer, indexes the aggregate fleet (for stock composition data)}

\item{aa}{Integer, indexes the aggregate age class (for stock composition and tag data)}

\item{s}{Integer, indexes the stock}

\item{yy}{Integer, indexes the aggregate years (for the tag data)}
}
\value{
Invisible data frame of observed and predicted values plotted in base graphics figures
}
\description{
A set of functions to plot data variables and predicted values (catch, age composition, etc.)
}
\details{
\itemize{
\item \code{plot_catch} plots the fishery catch by stock or region (either whole numbers or proportions)
}

\itemize{
\item \code{plot_index} plots indices of abundance
}

\itemize{
\item \code{plot_CAA} plots the fishery catch at age
}

\itemize{
\item \code{plot_CAL} plots the catch at length
}

\itemize{
\item \code{plot_IAA} plots the index age composition
}

\itemize{
\item \code{plot_IAL} plots the index length composition
}

\itemize{
\item \code{plot_SC} plots the stock composition
}

\itemize{
\item \code{plot_tagmov} plots the tag movements
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-s4-class.R
\name{report}
\alias{report}
\alias{report,MSAassess-method}
\alias{report.MSAassess}
\title{Generate markdown reports}
\usage{
report(object, ...)

\S4method{report}{MSAassess}(
  object,
  name,
  filename = "MSA",
  dir = tempdir(),
  open_file = TRUE,
  render_args = list(),
  ...
)
}
\arguments{
\item{object}{An object from MSA.}

\item{...}{Additional arguments to render reports.}

\item{name}{Optional character string for the model name to include in the report, e.g., model run number. Default
uses \code{substitute(object)}}

\item{filename}{Character string for the name of the markdown and HTML files.}

\item{dir}{The directory in which the markdown and HTML files will be saved.}

\item{open_file}{Logical, whether the HTML document is opened after it is rendered.}

\item{render_args}{List of arguments to pass to \code{\link[rmarkdown:render]{rmarkdown::render()}}.}
}
\value{
\code{report} generic for MSAassess invisibly returns the output of \code{\link[rmarkdown:render]{rmarkdown::render()}}: character of the path of the rendered HTML markdown report.
}
\description{
Generate a markdown report of model fits and estimates.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make-parameters.R
\name{make_parameters}
\alias{make_parameters}
\alias{make_map}
\alias{check_parameters}
\title{Make list of parameters for RTMB}
\usage{
make_parameters(
  MSAdata,
  start = list(),
  map = list(),
  est_mov = c("none", "dist_random", "gravity_fixed"),
  silent = FALSE,
  ...
)

make_map(
  p,
  MSAdata,
  map = list(),
  est_M = FALSE,
  est_h = FALSE,
  est_mat = FALSE,
  est_sdr = FALSE,
  est_mov = c("none", "dist_random", "gravity_fixed"),
  est_qfs = FALSE,
  silent = FALSE
)

check_parameters(p = list(), map, MSAdata, silent = FALSE)
}
\arguments{
\item{MSAdata}{S4 data object}

\item{start}{An optional list of parameters. Named list of parameters with the associated dimensions and transformations below.
Overrides default values created by \code{\link[=make_parameters]{make_parameters()}}.}

\item{map}{List of mapped parameters. Used by \code{\link[=check_parameters]{check_parameters()}} only to count parameters.}

\item{est_mov}{Character describing structure of stock movement parameters. Only used if map arguments for the movement parameters is NULL. See details below.}

\item{silent}{Logical, whether \code{\link[=make_map]{make_map()}} reports messages to the console}

\item{...}{Various arguments for \code{\link[=make_map]{make_map()}} (could be important!)}

\item{p}{List of parameters, e.g., returned by \code{\link[=make_parameters]{make_parameters()}}}

\item{est_M}{Logical, estimate natural mortality? Only used if \code{map$log_M_s} is NULL}

\item{est_h}{Logical, estimate steepness? Only used if \code{map$t_h_s} is NULL}

\item{est_mat}{Logical, estimate maturity? Only used if \code{map$mat_ps} is NULL}

\item{est_sdr}{Logical, estimate standard deviation of recruitment deviates? Only used if \code{map$log_sdr_s} is NULL}

\item{est_qfs}{Logical, estimate relative catchability of stocks by each fleet? Fix \code{log_q_fs} for the first stock if \code{TRUE}. Only used if \code{map$log_q_fs} is NULL}
}
\value{
\code{\link[=make_parameters]{make_parameters()}} returns a list of parameters (\code{"p"}) concatenated with the output of \code{\link[=make_map]{make_map()}}.

\code{\link[=make_map]{make_map()}} returns a named list containing parameter mappings (\code{"map"}) and a character vector of random effects (\code{"random"}).

\code{\link[=check_parameters]{check_parameters()}} invisibly returns the parameter list if no problems are encountered.
}
\description{
Sets up the list of parameters, map of parameters (see \code{map} argument in \code{\link[TMB:MakeADFun]{TMB::MakeADFun()}}), and identifies some random effects parameters
based on the input data and some user choices on model configuration.

These functions provide a template for the parameter and map setup that can be adjusted for alternative configurations. \code{\link[=check_parameters]{check_parameters()}}
checks whether custom made parameter lists are of the correct dimension.
}
\section{Parameters}{


Generally parameter names will have up to three components, separated by underscores.
For example, \code{log_M_s} represents the natural logarithm of natural mortality, and is a vector by stock \code{s}.

The first component describes the transformation from the estimated parameter space to the normal parameter space,
frequently, \code{log} or \code{logit}. Prefix \code{t} indicates some other custom transformation that is described below.

Second is the parameter name, e.g., \code{M} for natural mortality, \code{rdev} for recruitment deviates, etc.

Third is the dimension of the parameter variable and the indexing for the vectors, matrices, and arrays, e.g., \code{y} for year, \code{s} for stock.
See \linkS4class{MSAdata}. Here, an additional index \code{p} represents some other number of parameters that is described below.

\describe{
\item{\code{t_R0_s}}{Vector by \code{s}. Unfished recruitment, i.e., intersection of unfished replacement line and stock recruit function,
is represented as: \code{R0_s <- exp(t_R0_s) * MSAdata@Dmodel@scale_s}. By default, \code{t_R0_s = 3}}
\item{\code{t_h_s}}{Vector by \code{s}. Steepness of the stock-recruit function. Logit space for Beverton-Holt (\code{h = 0.8 * plogis(t_h_s) + 0.2}) and log space for Ricker function (\code{h = exp(t_h_s) + 0.2}).
Default steepness value of 0.8}
\item{\code{mat_ps}}{Matrix \verb{[2, s]}. Maturity parameters (can be estimated or specified in data object). Logistic functional form. The
parameter in the first row is the age of 50 percent maturity in logit space: \code{a50_s <- plogis(mat_ps[1, ] * na)}.
In the second row is the age of 95 percent maturity as a logarithmic offset: \code{a95_s <- a50_s + exp(mat_ps[2, ])}.
Default \code{a50_s <- 0.5 * na} and \code{a95_s <- a50_s + 1}}
\item{\code{log_M_s}}{Vector by \code{s}. Natural logarithm of natural mortality (can be estimated or specified in data object).
Default parameter value for all stocks: \code{M <- -log(0.05)/MSAdata@Dmodel@na}}
\item{\code{log_rdev_ys}}{Matrix \verb{[y, s]}. Log recruitment deviations. By default, all start values are at zero.}
\item{\code{log_sdr_s}}{Vector by \code{s}. log-Standard deviation of the log recruitment deviations. Default SD = 0.4}
\item{\code{log_q_fs}}{Matrix \verb{[f, s]}. The natural logarithm of \code{q_fs}, the relative fishing efficiency of \code{f} for stock \code{s}.
Equal values imply equal catchability of all stocks. See equations in \code{\link[=calc_F]{calc_F()}}. Default sets all values to zero.}
\item{\code{log_Fdev_ymfr}}{Array \verb{[y, m, f, r]}. Fishing mortality parameters. For each fleet, the log of F is estimated directly for the
reference year, season, region. For other strata, F is an offset from this value:
\deqn{
F_{y,m,f,r} = \begin{cases}
\exp(x^{\textrm{Fmult}}_f) \quad & y = y_{\textrm{ref}}, m = m_{\textrm{ref}}, r = r_{\textrm{ref}}\\
\exp(x^{\textrm{Fmult}}_f + x^{\textrm{Fdev}}_{y,m,r}) \quad & \textrm{otherwise}
\end{cases}
}
}
\item{\code{sel_pf}}{Matrix \verb{[3, f]}. Fishery selectivity parameters in logit or log space. See equations in \code{\link[=conv_selpar]{conv_selpar()}}, where \code{sel_pf} is the \code{x} matrix.}
\item{\code{sel_pi}}{Matrix \verb{[3, i]}. Index selectivity parameters in logit or log space. See equations in \code{\link[=conv_selpar]{conv_selpar()}}, where \code{sel_pi} is the \code{x} matrix.}
\item{\code{mov_x_marrs}}{Array \verb{[m, a, r, r, s]}. Base movement matrix. Set to -1000 to effectively exclude movements from region pairs.
See equations in \code{\link[=conv_mov]{conv_mov()}}}
\item{\code{mov_g_ymars}}{Array \verb{[y, m, a, r, s]}. Attractivity term in gravity model for movement. If \code{x} and \code{v} are zero,
this matrix specifies the distribution of total stock abundance into the various regions. See equations in \code{\link[=conv_mov]{conv_mov()}}}
\item{\code{mov_v_ymas}}{Array \verb{[y, m, a, s]}. Viscosity term in gravity model for movement. See equations in \code{\link[=conv_mov]{conv_mov()}}}
\item{\code{log_sdg_rs}}{Array \verb{[r, s]}. Marginal log standard deviation in the stock distribution (\code{mov_g_ymars}) among regions for stock \code{s}.
Only used when \code{est_mov = "dist_random"}. Default SD of 0.1.}
\item{\code{t_corg_ps}}{Array \verb{[sum(1:(nr - 1)), s]}. Lower triangle of the correlation matrix for \code{mov_g_ymars}, to be obtained with the
Cholesky factorization. Only used when \code{est_mov = dist_random}. Default values of zero.}
\item{\code{log_initF_mfr}}{Array \verb{[m, f, r]}. Initial F corresponding to the equilibrium catch. Default is \code{log(0.1)} for seasons, fleets, and regions where \code{Dfishery@Cinit_mfr > 1e-8}, otherwise, not estimated (Finit = 0).}
\item{\code{log_initrdev_as}}{Array \verb{[na - 1, s]}. Recruitment deviations for the initial abundance-at-age vector. Default is zero.}
}
}

\section{Start list}{

Users can provide \code{R0_s} and \code{h_s} in the start list. \code{\link[=make_parameters]{make_parameters()}} will make the appropriate transformation for the starting values
of \code{t_R0_s} and \code{t_h_s}, respectively.
}

\section{Movement setup for \code{make_map()}}{

If a single region model or \code{est_mov = "none"}: no movement parameters are estimated.

If \code{est_mov = "dist_random"}: fix all values for \code{mov_x_marrs} and \code{mov_v_ymas}. Fix \code{mov_g_ymars} for the first region for each year,
season, age, and stock. \code{mov_g_ymars} are random effects.

If \code{est_mov = "gravity_fixed"}: fix all values for \code{mov_x_marrs}. Fix \code{mov_g_ymars} for the first region for each year,
season, age, and stock. Estimate all \code{mov_v_ymas}. Both \code{mov_g_ymars} and \code{mov_v_ymas} are fixed effects.

By default \code{p$mov_x_marrs} is zero. Set to -1000 for areas for which there is no abundance of a particular stock.
}

\seealso{
\linkS4class{MSAdata}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check-data.R
\name{check_data}
\alias{check_data}
\title{Check dimensions and inputs in MSAdata object}
\usage{
check_data(MSAdata, silent = FALSE)
}
\arguments{
\item{MSAdata}{S4 object containing data inputs. See \linkS4class{MSAdata}}

\item{silent}{Logical, whether or not to report default values to the console}
}
\value{
An updated \linkS4class{MSAdata} object.
}
\description{
Ensures that data inputs are of proper dimension. Whenever possible, default values are added to missing items.
}
\seealso{
\linkS4class{MSAdata}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{fit_MSA}
\alias{fit_MSA}
\title{Fit MSA model}
\usage{
fit_MSA(
  x,
  parameters,
  map = list(),
  random = NULL,
  run_model = TRUE,
  do_sd = TRUE,
  report = TRUE,
  silent = FALSE,
  control = list(iter.max = 2e+05, eval.max = 4e+05),
  ...
)
}
\arguments{
\item{x}{Data object. Class \linkS4class{MSAdata}, validated by \code{\link[=check_data]{check_data()}}. Alternatively, \linkS4class{MSAassess} that will be fitted again.}

\item{parameters}{List of parameters, e.g., returned by \code{\link[=make_parameters]{make_parameters()}} and validated by \code{\link[=check_parameters]{check_parameters()}}.}

\item{map}{List of parameters indicated whether they are fixed and how they are shared, e.g., returned by \code{\link[=make_parameters]{make_parameters()}}.
See \code{\link[RTMB:TMB-interface]{RTMB::MakeADFun()}}.}

\item{random}{Character vector indicating the parameters that are random effects, e.g., returned by \code{\link[=make_parameters]{make_parameters()}}.}

\item{run_model}{Logical, whether to fit the model through \code{\link[stats:nlminb]{stats::nlminb()}}.}

\item{do_sd}{Logical, whether to calculate the standard errors with \code{\link[RTMB:TMB-interface]{RTMB::sdreport()}}.}

\item{report}{Logical, whether to return the report list with \code{obj$report(obj$env$last.par.best)}.}

\item{silent}{Logical, whether to report progress to console. \strong{Not passed to \code{\link[TMB:MakeADFun]{TMB::MakeADFun()}}.} Recommend to set to \code{TRUE}
to speed up run time, e.g., when running simulations, multiple fits, etc.}

\item{control}{Passed to \code{\link[stats:nlminb]{stats::nlminb()}}}

\item{...}{Other arguments to \code{\link[RTMB:TMB-interface]{RTMB::MakeADFun()}}.}
}
\value{
A \linkS4class{MSAassess} object.
}
\description{
Wrapper function that calls RTMB to create the model and perform the numerical optimization
}
\details{
When re-fitting a model, you may want to update the starting values through \code{x@obj$par}.
}
\seealso{
\code{\link[=report]{report()}} \code{\link[=retrospective]{retrospective()}} \code{\link[=do_jitter]{do_jitter()}}
}
